#SkIndigo Ruby code is copyright Dale Martens 2009, all rights reserved.

######################
module SkIndigo

#method to collect all indigo instances from a collection of entities
def SkIndigo.get_instances(ents)

instances=ents.find_all {|e|
	((e.class==Sketchup::ComponentInstance) or (e.class==Sketchup::Group))
	#if e.class==Sketchup::ComponentInstance
	#	e.definition.indigo_instance?
	#elsif e.class==Sketchup::Group
	#	e.entities.parent.indigo_instance?
	#end
}
return instances
end

#interface for settings motion blur keyframes
def SkIndigo.set_keyframe(ents)

return if ents.length==0
prompts=["Time (0.0-1.0)"]
values=[0.0]
results=UI.inputbox(prompts,values,"Set Keyframes")
return if not results
time=results[0].to_f
if time>1.0 or time<0.0
	UI.messagebox("Time must be between 0.0 and 1.0")
	return
end
instances=self.get_instances(ents)
if instances.length==0
	UI.messagebox("No instances selected.  No keyframes added.")
else
	instances.each {|inst| inst.add_indigo_keyframe(time)}
end
UI.messagebox("Added keyframe for #{instances.length} instances.")
end

######clear all keyfreames
def SkIndigo.clear_keyframes(ents)

instances=self.get_instances(ents)
if instances.length==0
	UI.messagebox("No instances selected.")
else
	instances.each {|inst| inst.clear_indigo_keyframes}
end
UI.messagebox("Cleared keyframes for #{instances.length} instances.")
end
	

def smoothstep(min,max,input) 

r=(input.to_f-min.to_f)/(max.to_f-min.to_f)
return r*r*(3.0-2.0*r)

end

#Check for saved sketchy physics animation in the file
def SkIndigo.saved_sketchy_replay?

sr=SketchyReplay::SketchyReplay.new()
if sr.lastFrame==0
	return false
else
	return true
end
	
end
####################################################
def SkIndigo.export_animation(sphysics)  #sphysics is a flag for whether to export the sketchyphysics animation

#if on_mac?
#	UI.messagebox("Animation Export is currently not available on Mac systems.")
#	return
#end

if IndigoRenderSettings.new.halt.to_i==-1
	UI.messagebox("Warning: Halt time is set to -1. This means that each frame will render forever unless the halt time is set to a value such as 10 (s).  Halt time can be set in the SkIndigo render settings dialog.")
end

if not sphysics
	prompts=["Frame Rate","Smooth Transitions","Animate Objects"]
	values=["25.0","Off","Off"]
	enums=[nil,"On|Off","On|Off"]
	results=UI.inputbox(prompts,values,enums,"Animation Options")
	return if not results
	smooth=true if results[1]=="On"
	smooth=false if results[1]=="Off"
	objects=true if results[2]=="On"
	objects=false if results[2]=="Off"
else
    if not saved_sketchy_replay?
		UI.messagebox("SketchyReplay animation not found.  You must first save an animation using SketchyPhysics before using this feature.")
		return
	end
	 #set objects pos and camera for first frame.
	sr=SketchyReplay::SketchyReplay.new()
    sr.start()
end

model=Sketchup.active_model
model_filename = File.basename(model.path,".skp")
if model_filename!=""
	batch_name = model_filename
	batch_name += ".igq"
else
	batch_name = "Untitled.igq" 	
end
batch_file_path=UI.savepanel("Export Animation", "" , batch_name)    #path for the batch file script
return if batch_file_path==nil

export_dir=File.join(File.dirname(batch_file_path),File.basename(batch_file_path,".*"))
Dir.mkdir(export_dir) unless FileTest.exist?(export_dir)
export_path=File.join(export_dir,File.basename(batch_file_path,".*"))+".igs" 


ie=IndigoExporter.new(export_path)
ie.physics=true if sphysics  #set the sketchyphysics flag as this modifies mesh exporting by setting the mesh axis to the center of the bounding box
ie.settings.save_to_model()  #get the settings from the model, not the current scene

model = ie.model
model_name=File.basename(ie.path,".igs")
return if model_name==""
tex_path=File.join(File.dirname(export_path),"TX_"+model_name)

ie.tex_path="TX_"+model_name
meshes_path=File.join(File.dirname(ie.path),model_name+"-meshes.igs")

ents=Sketchup.active_model.entities
mb=IndigoMeshBuilder.new
mb.reset_instances()
mb.build_active_mesh(ents)
exported_entity_set = Set.new() # Used to prevent infinite loops on recursive components.
mb.get_instances(ents,Geom::Transformation.new, exported_entity_set)
mb.build_meshes2()
#mb.get_meshes(Sketchup.active_model.entities,Geom::Transformation.new, true)

#create the meshes IGS file 
ie.get_used_layers()
out_mesh = File.new(meshes_path,"w")
out_mesh.puts "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
out_mesh.puts "<scenedata>"
#export all materials
ie.export_default_mat(out_mesh,nil)
ie.model.materials.each {|mat| ie.export_material(IndigoMaterial.new(mat),out_mesh)}
ie.export_meshes2(mb,out_mesh)

if objects==false
	ie.export_instances(mb,out_mesh)
end

p "------SkIndigo Export Run------"	
p "Faces #{mb.faces}"
p "Instances #{mb.component_instances.length}"
p "Component Meshes #{mb.count_meshes()}"
p "Meshes loaded from cache #{mb.cached_meshes}"
p "New meshes exported #{mb.new_meshes}"
p "Spheres #{mb.spheres.length}"
out_mesh.puts "</scenedata>"
out_mesh.close

Sketchup.set_status_text("Exporting textures")		
if FileTest.exist? (tex_path)
else
	Dir.mkdir(tex_path)
end
indigo_textures=[]

#export all textures collected during the material export process
status=ie.export_textures(ie.textures) if ie.textures.length>0
		
#Export all collected nkata
#for path in ie.nkdata
#	nk=NKdata.new(path)
#	nk.export(tex_path)  #exports the nkdata to the textures directory
#end

#Export all collected ies profiles
#p "ies profiles #{ie.ies}"
for path in ie.ies
	profile=IESProfile.new(path)
	profile.export(tex_path) if profile.valid?  #export the IES profiles to the textures directory
end

if !sphysics
	ie.export_animation(File.dirname(export_path),results[0].to_f,smooth,objects,mb)
else #we are exporting a sketchyphysics animation
	ie.export_sketchyphysics_frame(File.dirname(export_path),mb)  #export the first frame
	 0.upto(sr.lastFrame){
        #advance object and camera positions
        sr.nextFrame()
        ie.export_sketchyphysics_frame(File.dirname(export_path),mb) 
    }
	sr.rewind()
end
ie.batch_path=batch_file_path
#ie.create_animation_batch_file()
ie.create_animation_igq_file()
UI.messagebox("Animation Exported Successfully.  Open #{batch_file_path} to render.")  

end #export_animation

######
def SkIndigo.export_scene_tabs()





end #export scene tabs

end #module skindigo

###################
class IndigoExporter  

include SkIndigo

def export_animation(igs_path,frame_rate,smooth,objects,mesh_builder)

Dir.mkdir(igs_path) unless FileTest.exist?(igs_path)    #igs_path is a directory name that stores the IGS files for each frame
return 0 unless FileTest.exist?(igs_path)    #failed to create path
pages=Sketchup.active_model.pages

#set all page delay times to 0.0 (restore after animation)
delay_times=pages.collect {|page| page.delay_time}
pages.each {|page| page.delay_time=0.0}

begin
def_trans_time=Sketchup.active_model.options["PageOptions"]["TransitionTime"]
total_time=pages.slideshow_time

time_step=1.0/frame_rate.to_f
num_frames=(total_time*frame_rate).to_i
igs_path=File.basename(self.path)       
model_name=File.basename(igs_path,".*")  #minus the file extension
mesh_path=File.join(File.dirname(self.path),model_name+"-meshes.igs")
time=0.0

page_start_time=time
page_trans=pages[0].transition_time
page_trans=def_trans_time if page_trans==-1.0
page_end_time=page_start_time+page_trans
@frame=num_frames
mb=mesh_builder

num_frames.times {|i|
	if objects==true
		#p "exporting objects"
		ents=Sketchup.active_model.entities
		mb.reset_instances()
		mb.build_active_mesh(ents)
		exported_entity_set = Set.new() # Used to prevent infinite loops on recursive components.
		mb.get_instances(ents,Geom::Transformation.new, exported_entity_set)
		#mb.build_meshes2()
	end
	Sketchup.set_status_text("Exporting Animation Frame #{i}")
	#pages.show_frame_at(time+0.01)
	get_page=pages.show_frame_at(time)
	current_page=get_page[0]
	progress=get_page[1]    #the fraction that we are into the transition
	
	
	if time>=page_end_time
		page_start_time=page_end_time
		page_trans=current_page.transition_time
		page_trans=def_trans_time if page_trans==-1.0
		page_end_time=page_start_time+page_trans
	end
	frame_num="%04d" % i
	frame_path=File.join(File.dirname(self.path),model_name+"-#{frame_num}.igs")
	
	smoothed_time=smoothstep(0.0,1.0,progress.to_f)*(page_end_time-page_start_time)+page_start_time
	pages.show_frame_at(smoothed_time) if smooth
	
	get_used_layers()
	out=File.new(frame_path,"w")
	out.puts "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
	out.puts "<scene>"
	export_metadata(out)
	export_render_settings(out)
	export_tonemapping(out)
	export_camera(model.active_view,out)
	export_environment(out)
	export_default_mat(out,nil)
	mb.spheres.each {|sphere| export_sphere(sphere,out)}
	export_fog(out) if self.model.rendering_options["DisplayFog"]

	out.puts "		<include>"
	out.puts "		<pathname>#{File.basename(mesh_path)}</pathname>"
	out.puts "		</include>"
	
	if objects==true
		export_instances(mb,out)
	end
	export_light_layer_names(out)
	out.puts "</scene>"
	out.close
	time=time+time_step
}

ensure
#now restore the scene delay times
pages.count.times {|i| pages[i].delay_time=delay_times[i]}

end #end begin

end #export animation

def export_sketchyphysics_frame(igs_path,mb)

Dir.mkdir(igs_path) unless FileTest.exist?(igs_path)    #igs_path is a directory name that stores the IGS files for each frame - move function to initialization method???
return 0 unless FileTest.exist?(igs_path)    #failed to create path


 #put this in initialization method?
igs_path=File.basename(self.path)       
model_name=File.basename(igs_path,".*")  #minus the file extension
mesh_path=File.join(File.dirname(self.path),model_name+"-meshes.igs")

#begin exporting the current frame
	ents=Sketchup.active_model.entities
	#mb=IndigoMeshBuilder.new
	mb.reset_instances
	mb.build_active_mesh(ents)
	
	exported_entity_set = Set.new() # Used to prevent infinite loops on recursive components.
	mb.get_instances(ents,Geom::Transformation.new, exported_entity_set)
	#mb.build_meshes2()
	model=self.model
	
	Sketchup.set_status_text("Exporting Animation SketchyPhysics Frame #{@frame}")
	frame_num="%04d" % @frame
	frame_path=File.join(File.dirname(self.path),model_name+"-#{frame_num}.igs")
	get_used_layers()
	out=File.new(frame_path,"w")
	out.puts "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
	out.puts "<scene>"
	export_metadata(out)
	export_render_settings(out)
	export_tonemapping(out)
	export_camera(model.active_view,out)
	export_environment(out)
	export_default_mat(out,nil)
	mb.spheres.each {|sphere| export_sphere(sphere,out)}
	
	export_fog(out) if model.rendering_options["DisplayFog"]

	out.puts "		<include>"
	out.puts "		<pathname>#{File.basename(mesh_path)}</pathname>"
	out.puts "		</include>"
	
	export_instances(mb,out)
	export_light_layer_names(out)
	out.puts "</scene>"
	out.close
	@frame+=1
end  #sketchyphysics frame


def create_animation_igq_file()

model = self.model
model_name=File.basename(self.path,".igs")

irs=IndigoRenderSettings.new

image_name=irs.image_name
halt=irs.halt

indigo_path=SkIndigo.get_indigo_path
return 0 if not indigo_path

#make sure that the 'renders' directory exists
root=SkIndigo.get_indigo_app_data_path
if not FileTest.exist?(File.join(root,"renders"))
	Dir.mkdir(File.join(root,"renders"))
end

batch_file_path=self.batch_path

batch=File.new(batch_file_path,"w")
batch.puts "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
batch.puts "<render_queue>"

@frame.times {|i|
	frame_num="%04d" % i
	export_path=File.join(File.dirname(self.path),model_name+"-#{frame_num}.igs")
	image_name=File.join(root,"renders",(File.basename(export_path,".*")+".png"))
	igi_output_path=File.join(root,"renders",(File.basename(export_path,".*")+".igi"))
	batch.puts "<item>"
	batch.puts "	<scene_path>#{export_path}</scene_path>"
	batch.puts "	<halt_time>#{halt}</halt_time>" if irs.halt_unit=="sec"
	batch.puts "	<halt_spp>#{halt}</halt_spp>" if irs.halt_unit=="spp"
	#batch.puts "	<output_path>#{image_name}</output_path>"  #indigo does not write the output filename, just to the output directory
	batch.puts "</item>"
	
}
batch.puts "</render_queue>"

batch.close
return batch_file_path


end


def create_animation_batch_file()

model = self.model
model_name=File.basename(self.path,".igs")

#Create the batch file  - move this out of the class methods
irs=IndigoRenderSettings.new

image_name=irs.image_name
halt=irs.halt
low_priority=irs.low_priority?
network_param=""
case irs.network_mode
		when 1 #master
			network_param+="-n m"
		when 2 #working master
			network_param+="-n wm"
end

indigo_path=SkIndigo.get_indigo_path
return 0 if not indigo_path

#make sure that the 'renders' directory exists
root=SkIndigo.get_indigo_app_data_path
if not FileTest.exist?(File.join(root,"renders"))
	Dir.mkdir(File.join(root,"renders"))
end

batch_file_path=self.batch_path

batch=File.new(batch_file_path,"w")
batch.puts indigo_path.split(":")[0]+":"
batch.puts "cd #{File.dirname(indigo_path)}"

@frame.times {|i|
	frame_num="%04d" % i
	export_path=File.join(File.dirname(self.path),model_name+"-#{frame_num}.igs")
	image_name=File.join(root,"renders",(File.basename(export_path,".*")+".png"))
	igi_output_path=File.join(root,"renders",(File.basename(export_path,".*")+".igi"))
	batch.puts "start \/wait \/low \/min indigo_console.exe \"#{export_path}\" -o \"#{image_name}\" -igio \"#{igi_output_path}\""+network_param if low_priority
	batch.puts "start \/wait \/min indigo_console.exe \"#{export_path}\" -o \"#{image_name}\" -igio \"#{igi_output_path}\""+network_param if !low_priority
}

batch.close
return batch_file_path

end  #create batch file

end  #end class indigo exporter


##############
class IndigoKeyframe

def initialize

end

end #class

##########methods for motion blur implementation
module IndigoKeyframes

def add_indigo_keyframe(time)
	time=time.to_f
	trans=self.transformation.to_a
	keyframes=self.get_indigo_keyframes
	keyframes.push([time,trans])
	new_keyframes=self.set_attribute('indigo07_settings','keyframes',keyframes)	
end

def get_indigo_keyframes
	self.get_attribute('indigo07_settings','keyframes',[])
end

def clear_indigo_keyframes
	dict=self.attribute_dictionaries["indigo07_settings"]
	if dict
		dict.delete_key("keyframes")
	end
end

def has_indigo_keyframes?
	return self.get_indigo_keyframes.length>0
end

end #module

class Sketchup::ComponentInstance
include IndigoKeyframes
end

class Sketchup::Group
include IndigoKeyframes
end