#SkIndigo Ruby code is copyright Dale Martens 2009, all rights reserved.
##################################Mesh Builder#############################
class IndigoInstance #an instance of a mesh

attr_reader :material, :transformation, :definition, :su_instance, :imesh
attr_writer :material, :transformation, :definition, :su_instance, :imesh

@@dict="indigo07_materials"
@@exit_portal=false

def initialize

@material=nil
@transformation=nil
@definition=nil
@su_instance=nil
@imesh=nil

end

def exit_portal?
	return false unless @su_instance
	@su_instance.get_attribute(@@dict,'exit_portal',@@exit_portal)
end

def exit_portal=(value)
	if value
		@su_instance.set_attribute(@@dict,'exit_portal',value) 
	else
		@su_instance.set_attribute(@@dict,'exit_portal',value)  
	end
end

end


class IndigoMeshBuilder

attr_reader :faces, :mesh_materials, :mesh_back_materials, :instances, :cached_meshes, :new_meshes, :cache_meshes

#these class variables keep track of all the component meshes, instances, and spheres in the model
#they are in effect global so that when an animation is exported, the meshes and instances are already saved in memory
#so they don't have to be 'collected' again by traversing all the entites in the model
@@component_meshes=Set.new  #keeps track of 'used' component definitions (in model and visible)
@@component_instances={}
@@indigo_meshes={}
@@spheres=[]
@@images=[]
@@section_planes=[]
@@dict="indigo07_materials"

def initialize

@parent_mat=[]
@parent_visible=[]
@mesh_materials={}
@mesh_back_materials={}
@faces=0
@nested_instances={}
@scale=0.0254*Indigo.scale
@instances=0
@model_instances_count=1
@cached_meshes=0
@new_meshes=0
@cache_meshes=false


end

def count_model_instances()

count=0
@@indigo_meshes.each {|defn,imesh|
	if defn.class==Sketchup::ComponentDefinition
		count=count+defn.count_instances
	end
}
@model_instances_count=count
return count

end


def count_meshes()

count=0
return @@indigo_meshes.keys.length

end
#reset the class variables
def reset

@@component_meshes=Set.new
@@component_instances={}
@@indigo_meshes={}
@@spheres=[]
@@images=[]
@@section_planes=[]



end
#####
def get_imesh(i_instance)

return @@indigo_meshes[i_instance.definition]


end

def validate_imeshes()

invalid_defns=[]
@@indigo_meshes.each {|defn,imesh|
	unless defn.valid?
		invalid_defns.push(defn)
	end
}
invalid_defns.each {|defn| @@indigo_meshes.delete(defn)}


end

def reset_instances

@@component_instances={}
#@@component_meshes=Set.new
#@@indigo_meshes={}
@@spheres=[]
@@images=[]


end

#####collects the mesh from the active entities - or selected active entities
def build_active_mesh(ents)

model=Sketchup.active_model

found_face=ents.find {|ent| ((ent.class==Sketchup::Face)and(ent.visible?)and(ent.layer.visible?))}
		
if found_face
	imesh=build_mesh(model,ents)
	@@indigo_meshes[model]=imesh
	
	inst=IndigoInstance.new
	inst.transformation=Geom::Transformation.new
	#inst.imesh=imesh
	inst.su_instance=nil
	inst.material=nil
	inst.definition=model
	mat=inst.material
	@@component_instances[mat] = [] if not @@component_instances[mat]
	(@@component_instances[mat] ||=[]) << inst
end



end

###
def spheres

return @@spheres

end

###
def component_instances

return @@component_instances

end

####
def component_meshes

return @@component_meshes

end

def indigo_meshes

return @@indigo_meshes

end

####
def images

return @@images

end

###
def section_planes

return @@section_planes

end

###
def load_external_skp_defs()


defns=Sketchup.active_model.definitions
for defn in defns
	if IndigoMesh.external_skp?(defn)
		#p "foudn external"
		instances=defn.instances
		imesh=IndigoMesh.new(defn)
		external_def=defns.load(imesh.external_mesh_path)
		instances.each {|inst| inst.definition=external_def}
	end
end


end
#####
def build_meshes2()

#self.validate_imeshes()  #deletes any invalid imeshes that are cached
mats=Sketchup.active_model.materials
mesh_dir=SkIndigo.temp_dir
#meshes_arr=@@indigo_meshes.values  #gets an array of all the cached meshes (array of comp defns)
#load_external_skp_defs()
#defns=Sketchup.active_model.definitions
defns=@@component_meshes.to_a  #this should only include 'used' and visible meshes
for defn in defns
	#if defn.count_instances>0
		
		if @cache_meshes
			#this checks of this mesh has been written already and is cached
			mesh_path=File.join(mesh_dir,defn.guid+".igmesh")
			#cached_imesh=meshes_arr.find {|imesh| imesh.definition==defn}
			if (FileTest.exist?(mesh_path)) #and (cached_imesh!=nil))
				#now check if the used material names in the IGMesh file match the names of the cached imesh
				#cached_materials=cached_imesh.used_materials.dup
				#p cached_materials
				cached_imesh=IndigoMesh.new(defn)
				cached_imesh.read_from_file(mesh_path)  #reads the material names from the igmesh into the @used_materials variable
				if (cached_imesh.valid_igmesh==true)
					@@indigo_meshes[defn]=cached_imesh
					@cached_meshes+=1
					next
				end
			
			end
		end
		#if we have got this far, the mesh must be built or rebuilt
		@new_meshes+=1
		imesh=build_mesh(defn)
		@@indigo_meshes[defn]=imesh
	#end
end

self.count_model_instances()  #stores the number of instances in a class variable

end

######
def build_mesh(defn,ents=defn.entities)

faces={}
points=[]
normals=[]
uvs=[]
uv=[]
uv0=[]
uv1=[]
uv2=[]
uv3=[]
tris=[]
has_uvs=false
has_uv0=false


face_triangles=[]
uvset0={}
uvset1={}
uvset2={}
uvset3={}
v_index = 0
mat_index=0

im=IndigoMesh.new(defn)
active_uv_set=im.uv_set
has_uv0=im.has_uvset(0)
has_uv1=im.has_uvset(1)
has_uv2=im.has_uvset(2)
has_uv3=im.has_uvset(3)

#if im.use_external_skp?
#	path=im.external_mesh_path
#	defns=Sketchup.active_model.definitions
#	defn=defns.load(path)
#	im.definition=defn
#end
#dots=""
#timer_id=UI.start_timer(1,true) {
#	dots=dots+"."
	
#}	
#ents=defn.entities

for e in ents
	if ((e.class==Sketchup::Face) and (e.visible?) and (e.layer.visible?))
		@faces=@faces+1
		mat=e.material
		back_mat=e.back_material
		if ((mat==nil) and (back_mat!=nil))
			backface=true
			mat=back_mat
		else
			backface=false
		end
		faces[mat]=[] if not faces[mat]
		(faces[mat] ||= []) << [e,backface]
	end
end
faces.each {|mat,face_arr|
	mat_name=mat ? (mat.name):"Default"
	Sketchup.set_status_text("Processing - #{im.name} - #{mat_name}...")
	
	face_triangles=[]
	face_arr.each {|face,backface|

		e=face  #the face entity
		if backface==true
			facemesh=e.mesh 7
		else
			facemesh=e.mesh 5
		end
		#check the uvsets of the face
		#might be room for further optimization directly below
		if backface==true
			uvset0[facemesh]=e.back_material
		else
			uvset0[facemesh]=mat  #get the material actually applied to this face to determine if it contains sketchup default uvs or face-level uvs
		end
					
		facemesh_points=facemesh.points
		facemesh_points.each {|e| points<<e}
			
		#get the vertex normals
		for p in (1..facemesh_points.length)
			if backface==true
				normals.push(facemesh.normal_at(p).reverse)
			else
				normals.push(facemesh.normal_at(p))
			end
		end
		
		facemesh_uvs=[]
		for p in (1..facemesh_points.length)
			if backface==true
				facemesh_uvs.push(facemesh.uv_at(p,false))
			else
				facemesh_uvs.push(facemesh.uv_at(p,1))
			end
		end
		
		#if uvset0[facemesh]==nil #no material applied to face so apply the default SketchUp uvs
		#	facemesh_uvs=facemesh_uvs.collect {|p| [p.x,p.y/te,p.z]}
		#end
		facemesh_uvs.each {|e| uv<<e.to_a}
		#p uv
		#now the stored uvs sets
		#check if this mesh uses multiple uvsets and that the current face has the corresponding stored uvset
		#uv1=uv.dup
		#uv2=uv.dup
		#uv3=uv.dup
		
		#build any other stored uv sets
		if has_uv0
			#p "has uv0"
			#p facemesh_uvs
			uvs=face.get_attribute 'uvs','uv0',facemesh_uvs
			uvs.each {|e| uv0<<e.to_a}
			#p uv0
		end
		if has_uv1
			#p "has uv1"
			uvs=face.get_attribute 'uvs','uv1',facemesh_uvs
			uvs.each {|e| uv1<<e.to_a}
		end
		if has_uv2
			#p "has uv2"
			uvs=face.get_attribute 'uvs','uv2',facemesh_uvs
			uvs.each {|e| uv2<<e.to_a}
		end
		if has_uv3
			#p "has uv3"
			uvs=face.get_attribute 'uvs','uv3',facemesh_uvs
			uvs.each {|e| uv3<<e.to_a}
		end
		
		#note that the active uv set is the only set displayed by SketchUp and accessible directly by the API
		#p "active uv set #{active_uv_set}"
		
		#now the triangles
		#face_triangles=[]
		polygons=facemesh.polygons
		for poly in polygons
			v1 = (poly[0]>=0?poly[0]:-poly[0])+v_index
			v2 = (poly[1]>=0?poly[1]:-poly[1])+v_index
			v3 = (poly[2]>=0?poly[2]:-poly[2])+v_index
			if backface==true
				face_triangles.push([v1-1,v3-1,v2-1])
			else
				face_triangles.push([v1-1,v2-1,v3-1])
			end
		end
		#im.add_triangles(tris,mat)
		v_index=v_index+facemesh_points.length
	}
	#p face_triangles.length
	im.add_triangles(face_triangles,mat)
}

case active_uv_set
		when 0
			uv0=uv
		when 1
			uv1=uv
		when 2
			uv2=uv
		when 3
			uv3=uv
end

im.add_points(points,@scale)  #takes an array of points
im.add_normals(normals)  #takes an array of normals
im.add_uvs(uv0 ,uv1,uv2,uv3)


return im #the indigomesh object

end

##################################
def get_instances(object, trans, exported_entity_set)

#NOTE: Removed this code for now as was causing not all instances in a component to get exported.  (See http://www.indigorenderer.com/forum/viewtopic.php?p=124882#p124882)
#if exported_entity_set.include?(object)
	# We have already exported this entity.  Don't export it again, or we could get stuck in an infinite loop with recursive components.  (see recursive_component.skp)
	#return
#else
	#exported_entity_set.insert(object)
#end

case object
	when Sketchup::ComponentInstance:
		definition=object.definition
		entities=definition.entities
	when Sketchup::Group:
		definition=object.entities.parent
		entities=object.entities
	when Sketchup::Entities:
		entities=object
		definition=object.parent
	when Sketchup::Selection:
		entities=object
	
end

nested_instances=@nested_instances[entities]
if @nested_instances[entities]==nil
	@nested_instances[entities]=entities.find_all {|e| (e.respond_to?(:explode))}
	nested_instances=@nested_instances[entities]
end

	for e in nested_instances
		visible=((e.visible?) and (e.layer.visible?))
		mat=e.material
		if mat!=nil
			@parent_mat.push(mat)
		else
			@parent_mat.push(@parent_mat.last)
			mat=@parent_mat.last
		end
		@parent_visible.push(visible)
		
		case e
			when Sketchup::ComponentInstance:
				if (e.definition.name.downcase.slice("indigosphere")=="indigosphere") 
					if visible
						inst=IndigoInstance.new
						inst.transformation=trans
						inst.su_instance=e
						inst.material=mat
						@@spheres.push(inst)
						@parent_visible.pop()
						@parent_mat.pop()
						next
					end
				elsif (e.definition.name[-6..-1]=="_dummy")
					dummy_defn=get_dummy_defn(e)
					definition=dummy_defn
					entities=definition.entities
				else
					definition=e.definition
					entities=definition.entities
				end
			when Sketchup::Group:
				definition=e.entities.parent
				entities=e.entities
			when Sketchup::Image:
				@@images.push([e,trans]) if visible
				next
		end
		
		
		#check if this definition has any faces and that it is visible
		has_face=entities.find {|ent| ((ent.class==Sketchup::Face)and(ent.visible?)and(ent.layer.visible?))}
		if (has_face!=nil) and (visible) #at least one face exists in this definition so add it as a valid mesh instance
			#add it as a 'used mesh'
			@@component_meshes.insert(definition)
			
			#build the instance object
			inst=IndigoInstance.new
			inst.transformation=trans*e.transformation
			inst.definition=definition
			inst.su_instance=e
			inst.material=mat
			#collect the instance as a used instance
			@@component_instances[mat] = [] if not @@component_instances[mat]
			(@@component_instances[mat] ||=[]) << inst
			@instances=@instances+1
			
		end
		get_instances(entities,trans*e.transformation, exported_entity_set) if visible
		@parent_visible.pop()
		@parent_mat.pop()
	end

end

####
def get_dummy_defn(object)

arr=object.definition.name.split("_")

if (arr.length>1) and (arr.last.upcase=="DUMMY")
    arr.pop
    dummy_def_name=arr.join("_")
    if dummy_def_name
        def_list=Sketchup.active_model.definitions
        dummy_def=def_list[dummy_def_name]
    end
end

if dummy_def
    return dummy_def
else
    return object.definition 
end

end

end  #class end


class MeshBuilder

attr_reader :faces, :mesh_materials, :mesh_back_materials

#these class variables keep track of all the component meshes, instances, and spheres in the model
#they are in effect global so that when an animation is exported, the meshes and instances are already saved in memory
#so they don't have to be 'collected' again by traversing all the entites in the model
@@component_meshes=[]
@@component_instances=[]
@@spheres=[]
@@images=[]
@@material_meshes=[]

def initialize

@parent_mat=[]
@mesh_materials={}
@mesh_back_materials={}
@faces=0

end

#reset the class variables
def reset

@@component_meshes=[]
@@component_instances=[]
@@spheres=[]
@@images=[]

end

#####
def reset_instances

@@component_instances=[]

end

###
def spheres

return @@spheres

end

###
def component_instances

return @@component_instances

end

####
def component_meshes

return @@component_meshes

end

####
def images

return @@images

end

##################################
def get_meshes_front_face(object, trans, inst)
#this method recurses through the nested entities in the model and collects faces and associated materials
#the core algorithm is borrowed from SU2KT although it really is just a simple recursion tree traversal
#if inst is false, then we are no longer searching for nested instances (i.e. we are currently in an indigo instance)

if object.class==Sketchup::ComponentInstance or object.class==Sketchup::Group
	definition=object.definition if object.class==Sketchup::ComponentInstance
	definition=object.entities.parent if object.class==Sketchup::Group
	def_name=definition.name
	text="Mesh: " + def_name
	if (definition.indigo_instance?) and (inst) 
		unless @@component_meshes.find {|i| i[0]==definition}
			mb=MeshBuilder.new
			mb.get_meshes(object,Geom::Transformation.new, false)
			@@component_meshes.push([definition,mb.mesh_materials])
			@faces+=mb.faces
			mb=nil
		end

		@@component_instances.push([object,trans]) #store instance object and transformation in array
		return
		
		
	else #if not an instance
		entity_list=SkIndigo.dummy(object)
	end

else
	entity_list=object
end

Sketchup.set_status_text("Collecting Faces - Level #{@parent_mat.size} - #{text}")  #{@faces}

	for e in entity_list
		if (e.class == Sketchup::Group and e.visible? and e.layer.visible?)
			if e.material
				mat = e.material
				@parent_mat.push(e.material)
			else
				@parent_mat.push(@parent_mat.last)
			end
			get_meshes(e, trans*e.transformation, inst)
			@parent_mat.pop
		end
		if (e.class == Sketchup::ComponentInstance and e.visible? and e.layer.visible?)
			if e.material
				mat = e.material
				@parent_mat.push(e.material)
			else
				@parent_mat.push(@parent_mat.last)
			end
			if (e.definition.name.downcase.slice("indigosphere")=="indigosphere") 
				@@spheres.push([e,trans])
			else
				get_meshes(e, trans*e.transformation, inst)
			end
			@parent_mat.pop
		end
		if (e.class == Sketchup::Face and e.visible? and e.layer.visible?)
			if e.material
				mat = e.material				
			elsif e.back_material
				mat = e.back_material				
			else
				mat=@parent_mat.last
			end
			
			
			@mesh_materials[mat] = [] if not @mesh_materials[mat]
			(@mesh_materials[mat] ||= []) << [e,trans]
			
			@faces+=1   #count of number of faces collected
		end
		###FUTURE support for image types
		#if (e.class==Sketchup::Image and e.visible? and e.layer.visible?)
			#@@images.push([e,trans])
		#end
	end

end

##################################new routine for backfaces
def get_meshes(object, trans, inst)
#this method recurses through the nested entities in the model and collects faces and associated materials
#the core algorithm is borrowed from SU2KT although it really is just a simple recursion tree traversal
#if inst is false, then we are no longer searching for nested instances (i.e. we are currently in an indigo instance)

if object.class==Sketchup::ComponentInstance or object.class==Sketchup::Group
	definition=object.definition if object.class==Sketchup::ComponentInstance
	definition=object.entities.parent if object.class==Sketchup::Group
	def_name=definition.name
	text="Mesh: " + def_name
	if (definition.indigo_instance?) and (inst) 
		unless @@component_meshes.find {|i| i[0]==definition}
			mb=MeshBuilder.new
			mb.get_meshes(object,Geom::Transformation.new, false)
			@@component_meshes.push([definition,mb.mesh_materials])
			@faces+=mb.faces
			mb=nil
		end

		@@component_instances.push([object,trans]) #store instance object and transformation in array
		return
		
		
	else #if not an instance
		entity_list=SkIndigo.dummy(object)
	end

else
	entity_list=object
end

Sketchup.set_status_text("Collecting Faces - Level #{@parent_mat.size} - #{text}")  #{@faces}

	for e in entity_list
		if (e.class == Sketchup::Group and e.visible? and e.layer.visible?)
			if e.material
				mat = e.material
				@parent_mat.push(e.material)
			else
				@parent_mat.push(@parent_mat.last)
			end
			get_meshes(e, trans*e.transformation, inst)
			@parent_mat.pop
		end
		if (e.class == Sketchup::ComponentInstance and e.visible? and e.layer.visible?)
			if e.material
				mat = e.material
				@parent_mat.push(e.material)
			else
				@parent_mat.push(@parent_mat.last)
			end
			if (e.definition.name.downcase.slice("indigosphere")=="indigosphere") 
				@@spheres.push([e,trans])
			else
				get_meshes(e, trans*e.transformation, inst)
			end
			@parent_mat.pop
		end
		if (e.class == Sketchup::Face and e.visible? and e.layer.visible?)
			if e.material
				mat = e.material
				backface=false
			elsif e.back_material
				mat=e.back_material
				backface=true
			else
				mat=@parent_mat.last
				backface=false
			end
			
			
			@mesh_materials[mat] = [] if not @mesh_materials[mat]
			(@mesh_materials[mat] ||= []) << [e,trans,backface]
			
			
			@faces+=1   #count of number of faces collected
		end
		###FUTURE support for image types
		#if (e.class==Sketchup::Image and e.visible? and e.layer.visible?)
			#@@images.push([e,trans])
		#end
	end

end

##################################
#identical to get_meshes except it ignores faces

# NOTE: This code doesn't seem to be used by anything
=begin
def get_instances(object, trans, inst)

puts "def get_instances(object, trans, inst)"

if object.class==Sketchup::ComponentInstance or object.class==Sketchup::Group
	definition=object.definition if object.class==Sketchup::ComponentInstance
	definition=object.entities.parent if object.class==Sketchup::Group
	def_name=definition.name
	text="Mesh: " + def_name
	#note that only the top-level instance is treated as an instance
	if (definition.indigo_instance?) and (inst) 
		#unless @@component_meshes.find {|i| i[0]==definition}  
			#mb=MeshBuilder.new
			#mb.get_instances(object,Geom::Transformation.new, false)
			#@@component_meshes.push([definition,mb.mesh_materials])
			#@faces+=mb.faces
			#mb=nil
		#end
		
		@@component_instances.push([object,trans]) #store instance object and transformation in array
		return
		
	else #if not an instance
		entity_list=SkIndigo.dummy(object)
	end

else
	entity_list=object
end

#Sketchup.set_status_text("Collecting Faces - Level #{@parent_mat.size} - #{text}")  #{@faces}

	for e in entity_list
		if (e.class == Sketchup::Group and e.visible? and e.layer.visible?)
			
			get_instances(e, trans*e.transformation, inst)
		end
		if (e.class == Sketchup::ComponentInstance and e.visible? and e.layer.visible?)
			
			if (e.definition.name.downcase.slice("indigosphere")=="indigosphere") 
				@@spheres.push([e,trans])
			else
				get_instances(e, trans*e.transformation, inst)
			end
			
		end
		
	end

end
=end

end  #class end
