
class IndigoSpectrumPicker

include DMcolormath

def initialize(texture,material,min_value,max_value,map_type=nil)
	@tex=texture
	@mat=material
	@min_value=min_value
	@max_value=max_value
	@min_value="null" if min_value==nil
	@max_value="null" if max_value==nil
	@su_mat=@mat.mat
	@map_type=map_type
	@dlg=UI::WebDialog.new("SkIndigo Spectrum Picker", true, "SkIndigoSpectrumPicker",385,413,419,243,true) #width,height,left,top
	@dlg.set_file(File.join(File.dirname(__FILE__),"SpectrumEditor.html"))
	color = @dlg.get_default_dialog_color 
	@dlg.set_background_color(color)
	@dlg.add_action_callback("getValues") {|d,p|
		loadValuesFromSU()
		setVisibility()
	}
	@dlg.add_action_callback("apply_settings") {|d,p|
		apply_settings(d)
		$skindigo_mat_observer.loadValuesFromSU()
		d.close
	}
	@dlg.add_action_callback("cancel_changes") {|d,p| d.close}
	@dlg.add_action_callback("updateColorPickerFromField") {|d,p|
		update_color_picker_from_field()
	}
	@dlg.add_action_callback("updateRGBFields") {|d,p|
		params=p.split("=")
		red=params[0].to_f
		green=params[1].to_f
		blue=params[2].to_f
		#red=d.get_element_value("rgb_red").to_f
		#green=d.get_element_value("rgb_green").to_f
		#blue=d.get_element_value("rgb_blue").to_f
		color_vec=Geom::Vector3d.new(red,green,blue)
		#length=d.get_element_value("magnitude").to_f
		length=1.0
		color_vec=params.collect {|e| (e.to_f/255.0*length)}
		setValue("rgb_red","%.4f" %(color_vec[0]))  #"%.4f" %((1.0-result[0]/255.0)
		setValue("rgb_green","%.4f" %(color_vec[1]))
		setValue("rgb_blue","%.4f" %(color_vec[2]))
	}
	@dlg.add_action_callback("RefreshValues") {|d,p|
		refresh_dialog()
	}
	@dlg.add_action_callback("refreshTable") {|d,p|
		refresh_table()
		set_color_swatch()
	}
	@dlg.add_action_callback("setColorSwatch") {|d,p|
		set_color_swatch()
	}
	#now display the dialog
	@dlg.show_modal {}
end

def set_color_swatch()

spectrum_type=@dlg.get_element_value("spectrumType")

case spectrum_type
	when "RGB"
		rgb_r=@dlg.get_element_value("rgb_red").to_f.abs
		rgb_g=@dlg.get_element_value("rgb_green").to_f.abs
		rgb_b=@dlg.get_element_value("rgb_blue").to_f.abs
		color=[rgb_r,rgb_g,rgb_b]
		color=color.collect {|e| (e*255).to_i}
	when "Uniform"
		uni_value=@dlg.get_element_value("uniformValue").to_f.abs
		color=[]
		3.times {|i| color[i]=(uni_value.to_f*255).to_i}
	when "Blackbody"
		bb_temperature=@dlg.get_element_value("temperature").to_i
		color=self.blackbody_to_rgb(bb_temperature)
		color=color.collect {|e| (e*255).to_i}
	when "Tabulated"
		get_table_values()
		#p @table_values
		abs_table_values=@table_values.collect {|val| val.to_f.abs}
		reg_start=@dlg.get_element_value("startWavelength").to_i
		reg_end=@dlg.get_element_value("endWavelength").to_i
		color=self.tabulated_to_rgb(abs_table_values,reg_start,reg_end)
		#p "tabulated color is #{color}"
		
end #case

#now set the color
setSwatch("colorSwatch",color)

end
######refresh the dialog values
def refresh_dialog()
	loadValuesFromSU()
	setVisibility()
	createSpinboxes()
	#update_color_picker_from_field()
end

####
def createSpinboxes()

ids=["uniformValue","blackbodyGain"] #"rgb_red","rgb_green","rgb_blue"
ids.each {|id|
	cmd="$(\"##{id}\").spinbox({min:#{@min_value},max:#{@max_value},step:0.001,bigStep:100,round:function round(num,dp) {return Math.round(num*100000)/100000;}});"
	@dlg.execute_script(cmd)
}

end


def loadValuesFromSU()
	#p "start load values"
	setValue("spectrumType",@tex.spectrum_type)
	setValue("gamma",@tex.rgb_gamma)
	setValue("rgb_red",@tex.rgb_r)
	setValue("rgb_green",@tex.rgb_g)
	setValue("rgb_blue",@tex.rgb_b)
	setValue("uniformValue",@tex.uni_value)
	setValue("temperature",@tex.bb_temperature)
	setValue("blackbodyGain",@tex.bb_gain)
	setValue("magnitude",@tex.rgb_gain)
	setValue("startWavelength",@tex.reg_start)
	setValue("endWavelength",@tex.reg_end)
	setWavelengthTable()
	update_color_picker_from_field()
	#color_vec=[@tex.rgb_r,@tex.rgb_g,@tex.rgb_b].collect {|e| (e*255).to_i}
	#setColorPicker(color_vec)
	#setSwatch("colorSwatch",@tex.color)
	#p "end load values"
end

def setWavelengthTable()
	#p @tex.reg_values
	@table_values=@tex.reg_values.split(" ")
	@table_values=@table_values.collect {|e| e.to_f}
	#tab_values=@tex.reg_values.split(" ")
	#p @table_values
	num_values=@table_values.length
	setValue("numValues",num_values.to_s)
	p num_values
	refresh_table()
end

def setVisibility()
	#p "start set vis"
	@dlg.execute_script("setVisibility()")
	#p "end set vis"
end

def update_color_picker_from_field()
	red=@dlg.get_element_value("rgb_red").to_f.abs
	green=@dlg.get_element_value("rgb_green").to_f.abs
	blue=@dlg.get_element_value("rgb_blue").to_f.abs
	color_vec=Geom::Vector3d.new(red,green,blue)
	#setValue("magnitude","%.4f" %(color_vec.length.to_f))
	color_vec=[red,green,blue]
	color_vec=color_vec.collect {|e| (e*255).to_i}
	setColorPicker(color_vec)
	set_color_swatch()
	#p color_vec
end

def setColorPicker(color)  #rucolor is an rgb triplet array
	r=color[0]
	b=color[2]
	g=color[1]
	cmd="$('#colorpicker').ColorPickerSetColor({r:#{r},g:#{g},b:#{b}});"
	@dlg.execute_script(cmd)
end

def setSwatch(id,color)

r=color[0]
g=color[1]
b=color[2]
cmd="document.getElementById('#{id}').style.backgroundColor=\"rgb(#{r},#{g},#{b})\""
#cmd="$('#{id}).css('backgroundColor','RGB(#{r},#{g},#{b})');"

@dlg.execute_script(cmd)
end

def setValue(id,value)
	new_value=value.to_s
	cmd="document.getElementById('#{id}').value=\"#{new_value}\""
	@dlg.execute_script(cmd)
end

def apply_settings(d) #d is the dialog

@tex.spectrum_type=d.get_element_value("spectrumType")
@tex.rgb_gamma=d.get_element_value("gamma")
@tex.rgb_r=d.get_element_value("rgb_red")
@tex.rgb_g=d.get_element_value("rgb_green")
@tex.rgb_b=d.get_element_value("rgb_blue")
@tex.rgb_gain=d.get_element_value("magnitude")
@tex.uni_value=d.get_element_value("uniformValue")
@tex.bb_temperature=d.get_element_value("temperature")
@tex.bb_gain=d.get_element_value("blackbodyGain")
get_table_values()
@tex.reg_start=d.get_element_value("startWavelength")
@tex.reg_end=d.get_element_value("endWavelength")
@tex.reg_values=@table_values.join(" ")

#the map used to be applied by the material editor class.  however, this does not work on Mac because web dialogs are asynchronous.  We must apply the map inside this class for it to work on Mac
case @map_type
when "albedo"
	@mat.albedo_texture=@tex
when "emission"
	@mat.emission_map=@tex
when "transmittance"
	@mat.transmittance_map=@tex
when "specular"
	@mat.specular_map=@tex
when "abs_layer_trans"
	@mat.abs_layer_trans_map=@tex
when "g_spectrum"
	@mat.g_spectrum_map=@tex
when "absorption"
	@mat.absorption_map=@tex
when "sss"
	@mat.sss_map=@tex
when "base_emission"
	@mat.base_emission=@tex
end


end

####
def refresh_table()

#get_table_values()
num_values=@dlg.get_element_value("numValues").to_i
#num_values=@table_values.length
#p num_values
#p @table_values

start_wl=@dlg.get_element_value("startWavelength").to_i
end_wl=@dlg.get_element_value("endWavelength").to_i
step_wl=((end_wl-start_wl)/(num_values.to_f-1))

#first clear the existing table
cmd="$('#wavelengthTable').empty();"
@dlg.execute_script(cmd)
cmd="$('#wavelengthTable').append('<tr><th>&nbsp;</th><th>Wavelength</th><th>Value</th></tr>');"
@dlg.execute_script(cmd)

if @table_values!=nil
	t_values=@table_values.dup
	#now create the data table
	num_values.times {|i|
			current_wavelength=(start_wl+(step_wl*i)).to_i
			#current_value=@table_values[i]
			current_value=t_values[i]
			#p i
			#p current_value
			cmd="$(\"#wavelengthTable\").append(\"<tr><td>#{i}</td><td align='center'>#{current_wavelength}</td><td align='center'><input class='formInputText' id='value#{i}' type='text' size='7' value='#{current_value}' onchange='setColorSwatch();'></td></tr>\");"
			#p cmd
			@dlg.execute_script(cmd)
			cmd="$(\"#value#{i}\").spinbox({min:#{@min_value},max:#{@max_value},step:0.001,bigStep:100});"
			@dlg.execute_script(cmd)
			#cmd="$(\"#value#{i}\").bind(\"change\",function(){setColorSwatch();});"
			#p cmd
			#@dlg.execute_script(cmd)
	}

end

end #refresh_table

###
def get_table_values()

num_values=@dlg.get_element_value("numValues").to_i
values=[]
num_values.times {|i|
		values[i]=@dlg.get_element_value("value#{i}")
	}
@table_values=values
#p "table values"
#p @table_values.join(" ")
return @table_values

end #end get_table_values

end #indigo shader editor class

