#SkIndigo Ruby code is copyright Dale Martens 2011, all rights reserved.
#-----------------------------------------------------------------------------
# Name        	: SkIndigo.rb
# Description 	: Model exporter and material editor for Indigo Renderer
# Menu Item   	: Plugins\SkIndigo\
# Author      	: Dale Martens (Whaat)


#require 'profiler'
skindigo_path=File.dirname(__FILE__)
load File.join(skindigo_path,"SkIndigoUVS.rb")
load File.join(skindigo_path,"ObjectEditor.rb")
load File.join(skindigo_path,"IndigoMeshBuilder.rb")
load File.join(skindigo_path,"IndigoRenderSettings.rb")
load File.join(skindigo_path,"DMXML.rb")
load File.join(skindigo_path,"indigoMatImport.rb")
load File.join(skindigo_path,"AnimationExport.rb")
load File.join(skindigo_path,"sun_override.rb")
load File.join(skindigo_path,"material_picker.rb")
load File.join(skindigo_path,"DMcolormath.rb")
load File.join(skindigo_path,"IndigoSpectrumPicker.rb")


module SkIndigo

######test if user is on a mac platform
def SkIndigo.on_mac?

return (Object::RUBY_PLATFORM =~ /mswin/i) ? false : ((Object::RUBY_PLATFORM =~ /darwin/i) ? true : :other)

end
	
end

#Version 3.6.25.3

#Reverted fix in 3.6.25.2 for infinite loops as was causing issue with instances in components not exporting properly.
#Fixed error when exporting animations.

#Version 3.6.25.2

#Fixed infinite loop causing SketchUp crash when exporting recursive components.

#Version 3.6.25.1

#Added proper export of image component object textures.
#Removed 'You must explode this image component' warnings.
#Removed 'Warn about non-exploded image components' checkbox from settings.

#Version 3.6.25

#Fixed issue with duplicate image components resulting in 'mesh names must be unique' error.
#Made warnings about exploding image components be combined into one dialog, which can be turned off in the settings.

#Version 3.6.24

#Forcing material editor UI mode to normal.  (Fixes getting stuck in simple material UI mode)

#Version 3.6.23

#Added backup means of launching Indigo if WIN32OLE method fails.

#Version 3.6.21.3

#SketchUp color changes now trigger automatic refresh of the SkIndigo material editor
#Reverted to original behavior (prior to 3.6.21) when a new material is created.  All SketchUp and Indigo attributes are copied from the currently selected material to the new one
#Updated included material preview scenes.
#Removed simple UI material editor mode.
#Renamed 'Search' button to 'Search Mat DB'.
#Fixed exporting of invalid scene when texture was invalid for albedo.  Just using SketchUp colour instead.

#Version 3.6.21.2

#Fixed quick export
#Changed alpha render from tracing method to foreground alpha checkbox
#Fixed clip map export for materials applied to groups and components
#Added warning message if material is Specular or Glossy Transparent material is chosen and opacity is set to 100%
#Tweaked default wavelength independent shader to use posOS().
#Added importing of coating and double-sided thin materials.
#Fixed substrate button for coating mat, and front and back buttons for double-sided-thin mats.
#Added missing export of coating interference parameter.

#Version 3.6.21

#Added 'Render Selected Entities' button to the SkIndigo toolbar.
#Removed dialog asking the user if they only want to export the selected entities after pressing render.
#Fixed problem with material attributes being copied from last material when a new material is created.

#Version 3.6.20

#Worked around error when exporting scene in some cases. (Error: #<NoMethodError: undefined method `definition' for nil:NilClass>)
#Fixed export error with displacement. (Error: #<NoMethodError: undefined method `out' for #<IndigoTexture:0xd409b710>>)
#Made all preview scenes have width and height 256, which fits the pop-up window well.
#Made all preview scenes have image save period 1, and made the thumbnail update timer to 1 second for faster mat previews.
#Fixed auto detection of Indigo path.
#Changed default image save period to 30 mins.
#Increased max value for image save period spinbox as well.
#Added detection of SketchUp 2013 to Windows installer.

#Version 3.6.19

#Fixed material preset dialog being not scrollable.
#Made material preview scenes using SSF1, single dir QMC PT instead of bidir mlt, for faster and better previews.
#Fixed thumbnails in online MatDB search for recent materials.
#Fixed XML escaping problem with emitters, resulting in invalid scenes in some cases.
#Increased frequency of updating some material preview scene images.

#Version 3.6.16

#Fixed: bug with custom preview scene directory not being automatically created
#Fixed: enabled support for GIF textures as they are now supported by Indigo

#Version 3.6.14

#Added: support for DST materials 
#Changed: moved emission map setting to 'Emitter Attributes' section in UI
#Changed: map dropdown option to say 'None' instead of 'Constant' to be consistent with Indigo UI
#Added: support for Coating material
#Added: fresnel scale for Phong materials
#Added: support for spherical camera
#Added: 1m scale Indigo preview scene - WIP
#Added: ability to easily create custom preview scenes - WIP
#Changed: Exit Portal materials now replace exit portal meshes
#Fixed: full path of textures now shown in texture editor 
#Fixed: Made Indigo preview scene render faster

#Version 3.4.12

#Fixed: IGM import of elements containing CDATA
#Fixed: absolute paths import for IGM and PIGM
#Added: option to cull geometry behind section planes (in advanced render settings)

#Version 3.4.4b

#Added: support for normal maps
#Fixed: errors which occur when a material name had leading or trailing whitespace

#Version 3.4.4

#Added: support for orthographic cameras
#Added: section plane support

#Version 3.2.14

#Changed: made GPU acceleration off by default 

#Version 3.2.12

#Fixed: PIGM unpacking on Mac
#Fixed: no longer processing meshes for each sketchy replay animation frame
#Fixed: no longer says 'Material successfully loaded' when unsupported attributes are found
#Changed: improved error message if unsupported attributes are found when loading a material
#Changed: user is now prompted to created a linked IGM material when loading a material that has unsupported attributes
#Fixed: missing icon in material database search - replaced with text only
#Fixed: improved handling of distorted textures

#Version 3.2.8

#Changed: emitting materials and Linked IGM materials cannot be previewed using the sphere model
#Fixed: default window size for mesh editor and shader editor
#Fixed: no longer forcing maximized window when launching Indigo
#Fixed: export and IGM import of texture smoothing option
#Changed: invalid paths for image components will still render but with white albedo
#Changed: warning messages for invalid paths for image components now give more information
#Fixed: issue with regular tabulated spectra data sometimes becoming corrupted
#Fixed: simple UI mode inadvertently setting default exponent value to 1

#Version 3.2.5

#Changed: no longer storing full path for nkdata (fixes nkdata path errors when transferring SKP from Mac to PC)
#Added: Environment Map Quick-Add button
#Changed; displays error message and aborts export if aperture obstacle map is not found
#Fixed: simple UI power slider to work with new emission scale system
#Changed: displays error message and aborts export if aperture image map is not found
#Fixed: bug with white space in Indigo application path on Mac
#Fixed: normal smoothing bugs
#Changed: made SkIndigo a SketchUp extension (can now be disabled or enabled without uninstalling/reinstalling)
#Changed: displays warning message if using environment map and light layer is turned off

#Version 3.2.2

#Fixed: mesh errors due to float export bug introduced in version 3.2.0

#Version 3.2.1

#Fixed: emission scale bugs
#Fixed: 'No such Material Default' errors

#Version 3.2.0

#Added: support for new Indigo sky model (to use old model, enable the legacy sky model checkbox in Render Settings)
#Fixed: Texture editor thumbnail not changing when loading new texture in certain situations
#Changed: completely reworked workflow for using multiple UV sets (UV sets created, saved, and loaded using Mesh Editor)
#Changed: throws error message and aborts rendering if not using perspective camera
#Added: support for SketchUp Image components (exported as diffuse material - cannot be edited inside SkIndigo)
#Fixed: normal smoothing precision errors introduced in version 3.0.15
#Removed: UV Mapping context menu
#Removed: 'Position Map' button in Texture Editor
#Added: UV Mapping options in Object Editor
#Changed: Object Editor now displays properties of the active mesh (group/component) not the selected one
#Added: automatic Indigo version compatibility check at SketchUp launch
#Changed: resolution preset from 1280x1024 to 1280x720
#Added: World scale setting (in advanced parameters)
#Changed: '..' button shows red background if invalid file reference is found
#Changed: emission scale of environment map now controlled by the emission scale setting of the env map material, not the render settings slider
#Changed: made it optional to export object animation
#Fixed: multiple UV sets
#Changed: no longer loading meshes from cache due to SketchUp not refreshing GUIDs when changing UVs of a mesh
#Fixed: error when applying an environment map material to normal geometry

#Version 3.0.15

#Fixed: Material picker ink dropper cursor hotspot
#Removed: quick export from right-click context menu
#Fixed: broken error messages (eg 'Blend material tried to use non-existent material named')
#Removed: instancing enable/disable command - instancing is determined automatically
#Changed: major optimizations in export routines
#Fixed: Texture editor no longer accepts loading of GIF images (they are unsupported by Indigo)
#Fixed: Quick export now works even after exporting selection only
#Fixed: permissions errors on Mac - now exporting scenes to users home directory
#Changed: Quick export no longer prompts to refresh textures - also quick export does not refresh instances
#Added: GPU acceleration checkbox
#Added: full support for emission scale
#Fixed: nested instances not inheriting parent material
#Fixed: aperture settings visibility bug
#Added: Lock aspect ratio
#Changed: Mesh export to IGMESH file format
#Added: Object editor - currently accessed by right click on group or component
#Added: Support for External OBJ, 3DS, or PLY files
#Changed: Exit Portal is now an object setting, not a material setting
#Changed: Subdivision settings are now applied via mesh, not material
#Added: Smooth option for displacement, bump, sigma, exponent, clip, and blend textures 
#Added: Support for background/sum material (allows sun and environment map in same render)
#Changed: Renamed tracing methods to match Indigo GUI
#Fixed: bug that occured when model contained scenes but none were selected

#Version 3.0.14

#Fixed: Environment map export (light layer bug introduced in 3.0.10b)

#Version 3.0.10b

#Fixed: bugs for exporting constant emission, surface color, and other wavelength dependent parameters
#Fixed: no longer exporting unused light layers
#Added: compatiblity with IndigoRT - user must specify the Indigo path
#Fixed: Bugs in RGB spectrum picker
#Added: auto-lounching of renders on Mac

#Version 3.0.10

#Added: vignetting control
#Added: material id tracer (to tracing method)
#Removed: frame upload period (no longer used by Indigo)
#Added: IGQ format support for animations (Animation export now works on Mac)
#Removed: Apply settings to scene (temporary removal until bugs are fixed)
#Removed: inifile access button from render settings

#Version 2.6.1

#Added: glass acceleration to advanced settings
#Removed: hybrid tracing methods
#Changed: meta data author tag to username, not my name

#Version 2.4.13

#Fixed: emitter checkbox in simple UI mode
#Fixed: bug with light layer names 
#Fixed: region rendering and sun position override in conjunction with save settings to scene
#Fixed: exposure compensation value saving and loading in render settings
#Fixed: bug that causes SkIndigo not load if Indigo was not installed in default location
#Added: ability to cancel Quick Export
#Changed: updated Windows installer for SketchUp 8

#Version 2.4.8

#Added: ability to save render settings to current scene
#Fixed: G spectrum negative values in spectrum picker
#Changed: using fastbox splat filter
#Changed: rounding of bump values to 5 decimal places
#Added: ability to download and load materials automatically from online material database.
#Fixed: spectrum picker spinner boxes now have different ranges of values depending on the parameter

#Version 2.4.4

#Added: Option to open Linked IGM material with Indigo Material Editor - not working on Mac
#Added: default shaders that work without errors
#Added: IGS file meta data
#Added: Invert texture button
#Fixed: IGM loading of clip maps now supported
#Added; step blend option for blend material types
#Changed: step blend automatically enabled for clip map textures if texture 'b' parameter equals 1.0
#Added: support for blackbody and tabulated spectra for all wavelength dependent parameters
#Added: spectrum picker for wavelength dependent paramters
#Added: support for absorption and sss shaders
#Added: Material picker tool
#Added: spinboxes to numeric input fields
#Changed: Naming convention for 'external' textures to 'texture'
#Changed: Naming convention for 'None' to 'Constant' - added checkboxes for optional material parameters

#Version 2.2.12

#Fixed: material editor not opening in rare situations
#Fixed: IGM loading of absorption layer transmittance parameter
#Changed: improved material previews

#Version 2.2.9

#Added: correlated color temperature to white balance preset drop-down
#Fixed: Material preview thumbnails on Mac
#Changed: made material previews halt at 5000 samples
#Added: start and stop preview buttons
#Fixed: custom UV set export bug which prevented scene export (Joel)
#Changed: low cpu priority to 'background mode'
#Changed: made 'use preset thumbnail' enabled by default when assigning presets
#Changed: turbidity range to 2-6
#Fixed: nkdata (metals) IGM loading on Mac 
#Fixed: SKP file saving bug - launching indigo in new way on PC
#Fixed: back button bug
#Changed: added icon to sun tool
#Changed: made button text pure black
#Added: Absorption layer transmittance (surface color) 
#Added: method to use external meshes (hidden from user)
#Fixed: bug with instances using different materials

#Version 2.2.3

#Added: 1000x700 resolution preset
#Fixed: PIGM saving and loading on Mac
#Added: scene tab object animation support (using 'Proper Animation' and 'SUanimate' plugins)
#Added: sun position override tool
#Added: new color picker interface (for emission color)
#Added: view dependent subdivision
#Changed: default UI to simple mode
#Removed: 'Undo' button in render settings dialog
#Added: Saving of default settings (in render settings dialog)
#Added: new interface to set absorption
#Changed: changed some default render settings (sunny 16, Bidir PT, save period 60s)
#Added: automatic SkIndigo update checker
#Changed: added zero padding for animation output filenames
#Changed: Re-added subdivision curvature threshold and changed default to 0.1 radians
#Changed: default max subdiv to 10
#Changed: exporting displacement error threshold of 0.0005m
#Added: custom light layer names (new menu item)
#Changed: UI enhancements
#Added: integrated online material database search
#Added: Camera response function to tonemapping settings
#Changed: IGI auto-saves with custom name if enabled
#Changed: maximum shutter speed to 1/3000
#Changed: No longer exporting absolute path to nkdata - all nkdata must be in the Indigo/nkdata folder
#Added: clay, wax, and orange juice presets
#Added: support for back face export (material applied to front face overides back face)
#Changed: recalibrated background color gain
#Added: network render settings (in advanced settings)
#Changed: made animation batch process occur in minimized window

#Version 2.0.12c

#Changed: Indigo, monkey, and custom preview scenes now use Indigo GUI instead of console
#Changed: removed watermark from Indigo material database preview scene
#Changed: default tracing method to Path Tracing
#Changed: throws error and aborts export if SkIndigo UV Helper material is applied when exporting 
#Changed: default emission map 'b' value to 1.0
#Fixed: emitting material previews for sphere model
#Changed: Default MNCR to 1000
#Fixed: Indigo Sphere export
#Changed: made query material types dialog scrollable
#Fixed: material previews with illegal filename characters in material name
#Fixed: Mirrored geometry export bug
#Fixed: shader editor
#Changed: now throws error when external texture not found
#Fixed: clip maps

#Version 2.0.12b

#Fixed: Mac loading errors (Stack Level too deep) caused by Indigo not being located in Applications folder
#Added: Set Indigo path menu item
#Fixed: SkIndigo loading errors on Windows Vista
#Fixed: SketchUp textures now get exported in original file format instead of all being converted to JPG

#Version 2.0.12

#Changed: Now getting Indigo path on Mac differently to avoid permissions errors
#Changed: default tracing method to MLT
#Fixed: nkdata on Mac

#Version 2.0.11d

#Fixed: Mac bug with material previews not launching

#Version 2.0.11c

#Fixed: Mac bug causing skindigo not to load
#Changed: updated and finished all HTML tooltips

#Version 2.0.11b

#Changed: Exporting frame upload period of 120 seconds for network rendering
#Fixed: Monochrome render mode (clay)
#Fixed: animation export bug with default material applied

#Version 2.0.11

#Changed: indigo path now read from registry on windows systems
#Changed: changed range of exposure compensation to +-20

#Version 2.0.10

#Added: Windows Installer
#Changed: moved preview scenes to Plugins/skindigo/supreviews
#Fixed: Custom image name
#Added: Separate layer assignment for sun, sky and environment map
#Changed: Enabled material previews for Mac (does not launch render automatically yet)
#Changed: Enabled IGM import for Mac (PIGM import still not avaialable)
#Added: Material Editor Icon to toolbar
#Fixed: more Mac fixes
#Fixed: Exponent map bug
#Fixed: IGM export of materials with external textures
#Fixed: PIGM Linking (windows only)
#Fixed: PIGM Loading (windows only)
#Added: material dropdown list at top of material editor
#Fixed: creation of Indigo Renderer directory in Appdata folder if it does not exist

#Version 2.0.8

#Added: Exporting Sun, Sky, and EnvMap layers
#Added: Motion blur - still has some bugs, needs better UI,  access by right-clicking context menu on an instanced group or component 
#Added: support for Supersample factor
#Added: support for Info overlay
#Fixed: Errors with duplicate material/mesh names
#Added: Animation to UI and support for Sketchy Physics Animation
#Fixed: Default UVs should now be properly exported even for textures applied to groups or components - should eliminate Indigo UV error messages
#Removed: Resume render menu item - use Indigo GUI instead
#Fixed: MDB preview scene
#Changed: more Mac fixes
#Changed: batch file no longer used to launch indigo
#Fixed: instancing of groups
#Changed; now export meshes file and scene file (instead of settings file and meshes/instances file)

#Version  2.0.4

#Fixed: Material Import
#Fixed: Displacment maps no longer exported for blend materials
#Added: Set displacement error threshold to 0.0
#Changed: material previews now using indigo console
#Changed: some Mac version fixes, still not fully Mac compatible yet
#Changed: messed around with preview scenes

#Version 1.1.16

#Changed: slightly improved the material types query (shows emitting materials)
#Changed: removed environment map rotation hack and added Indigo env map rotation
#Fixed: Simple UI emitter checkbox works again
#Changed: Material preview thumbnail shows SketchUp texture if no thumbnail is present
#Changed: filenames (not pathnames) are now listed in the External texture editor drop-down list
#Changed: toolbar icons
#Added: Position Map button to Texture Editor
#Changed: UV mapping workflow

#Version 1.1.15

#Fixed: Diffuse Transmitters causing Indigo to crash

#Version 1.1.12

#Changed: updated preview scenes to work with emitting materials
#Changed: re-added emitter presets
#Changed: groups of material attributes can now be collapsed by clicking on the appropriate heading
#Changed: You can now change instancing properties for all selected groups or components simultaneously
#Changed: messed around with instancing - instancing can now be enabled for groups
#Removed: Blackbody and RGB Light material types - use emitting materials instead
#Added: IES support to emitting materials
#Added: power and efficacy to emitting materials
#Changed: base emission now determined by emitter color or temperature value
#Added: support for non-uniformly scaled instances 
#Fixed: Animation - forced to use Indigo Console for animation
#Added: option to smooth animation transitions
#Changed; re-added max subdivisions to UI
#Changed: preview thumbnail background color set from material color
#Changed: Mapping system completely reworked.  External or internal textures can now be used.
#Changed: Albedo can now be any texture in your model or a custom shader
#Changed: Blend maps can now be any texture or a custom shader
#Added: Specular Reflectivity maps and custom shader support
#Added: support for clip map shaders
#Fixed: displacement maps with clip maps and blended materials
#Fixed: automatic blending of diffuse and phong materials wth diffuse transmitters when 0<SUopacity<100

#Version 1.1.8

#Changed: disabled IGM loading
#Added: support for custom shaders for bump,displacement,exponent,sigma,emission
#Changed: saved uv set options to 'SU, 1, 2, 3'
#Changed: disabled subdivision smoothing due to confusion
#Changed: removed subdivision curvature threshold from the interface
#Changed: constant 'exponent' values are set using using the texture 'b' parameter
#Changed: constant 'blend factor' values are now set using the texture 'b' parameter
#Changed: constant 'sigma' values are now set using the texture 'b' parameter
#Changed: disabled automatic blending of diffuse transmitters when 0<SUopacity<100
#Added: sigma mapping for Oren-Nayar materials
#Added: specular reflectivity mapping for Metal materials
#Added: support for emitting materials and layer blending
#Changed: removed fresnel scale
#Changed: now using new material format
#Changed: indigo embedded_02 mesh format exclusively used
#Changed: prompts user to create linked igm material if trying to load a material with unsupported indigo attributes
#Changed: made thumbnails refresh automatically when rendering a preview - old thumbnail deleted before starting new preview
#Added: Buttons to 'jump to' child materials of blend materials
#Added: back button
#Fixed: instancing and multiple uvsets should work again

#Version 1.1.5

#Added: support for lens shifting
#Added: displacement mapping for blend materials
#Changed: nkdata files not exported when saving IGMs anymore
#Added: option to pack IGM file when saving material
#Changed: When loading an IGM with nkdata, the nkdata file must be already located in the Indigo/nkdata folder
#Added: PIGM/IGM linking (new material - Linked IGM)
#Added: PIGM loading
#Added: support for IGM loading of oren-nayar materials, phong with specular reflectivity spectrum, displacement maps
#Changed: indigo preview scene to official indigo preview scene
#Changed: made default uv set be exported with name 'default' to improve compatibility with linked IGM files

#Version 1.1.1

#Removed: option to enable or disable normal smoothing
#Added: support for displacement mapping
#Added: support for indigo subdivision and smoothing
#Added: support for fractal brownian motion procedural texture


#Version 1.0.9 Official

#Fixed: sphere primitive with no material bug
#Fixed: monochrome render bug

#Version 1.0.8

#Added: support for alpha rendering
#Changed: aperture diffraction disabled by default
#Fixed: environment map rotation
#Added: support for Oren-Nayar material
#Added: support for specular reflectivity spectrum (disable nkdata checkbox to use)

#Version 1.0.5

#Added: region rendering
#Added: Advanced Texture Editor
#Added: support for a,c, and gamma texture attributes
#Changed: all textures exported in JPG format (BMP, GIF, TIF, all PNG should now work except for grayscale textures)
#Changed: replaced cube preview scene with new fast cube preview scene
#Added: Lambertian material preset
#Changed: made IGM export compliant with new proposed IGM spec
#Changed: improved IGM loading code and fixed some bugs
#Changed: SUpreviews folder moved to the SkIndigo directory
#Changed: thumbnails saved in the indigo/SUthumbnails directory (cannot be changed)
#Changed: exported textures filenames converted to material name at export time (removes texture naming conflicts)
#Fixed: export selection only

#Version 1.0.4

#Added: support for IES profiles
#Changed: changed previews folder name to SUpreviews to avoid conflicts with other exporters' preview scenes
#Added: aperture diffraction support
#Changed: made IGM import backwards compatible with older versions of Indigo (not fully tested)
#Added: Water preset (credit CoolColJ)

#Version 1.0.1

#Fixed: IGM import with arbitrary gamma values
#Changed: made Indigo always open maximized
#Fixed: made nkdata IGMs compatible with Maxigo
#Changed: NKdata files are now exported when saving IGM
#Changed: frosted glass preset exponent to 100
#Changed: environment maps can now be located in any directory
#Removed: option to change exponent of Thin Glass materials (exports as 1 million)
#Added: watermark option
#Added: auto choose threads
#Added: halt samples per pixel
#Added: custom output image name
#Changed: low priority mode can be used when using more than 1 thread
#Added: enhanced support for nkdata including custom nkdata
#Changed: Phong with opacity less than 100% exports as blended with diffuse trasmitter
#Changed: Complete UI overhaul using dhtml web dialogs
#Added: Reinhard and Camera tonemapping

#Version 0.9.1 Official

#Changed: when applying glass or frosted glass preset, opacity now gets set to zero
#Changed: auto choose threads deactivated (conflicts with present SkIndigo UI)
#Changed: default tracing method to Bidir MLT
#Added: Fast sphere preview scene (and removed previous Sphere preview scene)
#Fixed: double gamma bug with RGB Spectrum lights
#Fixed: bump map export bug with blended diffuse/diffuse transmitters
#Fixed: save IGM bug that would occur with some nested blend materials ($g_su2in_exported_mats not properly initialized)
#Fixed: export path bugs (bug if "." in directory name or if export filename is identical to directory name)

#Version 0.9test4b

#Added: advanced user function Indigo world scale (Indigo.scale)
#Changed: RGB Absorption and RGB Scattering spectra gamma exported as 1.0
#Added: Support for Henyey-Greenstein phase function
#Changed: new material preview set courtesy of drbouvierleduc
#Added: ability to rotate environment maps (does not update with quick export)
#Fixed: sun direction bug when north angle is not default
#Fixed: material preview bug with blended materials that have clip maps

#Version 0.9test4

#Changed: You do not have to export to the Indigo directory in order to auto-render
#Changed: Quick Export can now be used even after closing SketchUp and restarting
#Changed: Default height and width to 50% of viewport dimensions
#Fixed: Loading IGM files to support new albedo_spectrum tags (loading IGM files created using previous versions not supported)
#Changed: Rendering a photomatched scene should now automatically match the photo perfectly
#Fixed: instances can now use different materials
#Changed: Shadows need not be on to enable sunlight
#Added: Black environment option
#Changed: Lots of UI improvements (toolbar, menus, etc.)
#Changed: Max number of threads to 8
#Changed: Default tracing method to Standard Path Tracing
#Changed: Prompts user to switch to Bidirectional Path tracing if using exit portals
#Fixed: material previews to work with Indigo 0.9
#Removed: Render mode and auto-render options in render settings
#Fixed: Tons of sloppy code cleanup


#Version 0.9test1

#Added: support for exit portals
#Added: support for textured emitters
#Added: material previews
#Changed: messed around with the UV set loading code
#Changed: minor UI changes

#Version 0.8 beta2

#Changed: presets for lampshade and curtains to diffuse transmitter presets
#Added: support for multiple uv sets
#Added: dummy components and dummy instancing (proxies)
#Fixed: Sketchup view zooming in when setting render settings
#Changed: Now double-click a material in the paint bucket to edit it
#Added: Explode distorted texture tool
#Added: Support for export of nested blends
#Added: Support for loading and saving of IGM nested blends
#Added: Clip map channel to all materials
#Removed: Option to disable material texture


#Version 0.8 beta1

#Added: application and material observers to determine the current selected material in the Paint Bucket
#Changed: combined material browser with material editor
#Changed: render setting 'bidirectional' to 'tracing method' to allow selection of more methods including hybrid
#Added: support for diffuse transmitter material.  Change relative amount of reflectance and transmittance with the Sketchup opacity slider
#Added: Thin Glass material type

#Version 0.7.1 Official

#Added: new light presets
#Added: support for loading and saving IGM files (Blend materials not supported yet)
#Added: support for Indigo sphere primitive by naming component definition IndigoSphere (cannot be placed inside Indigo instance)
#Added: support for Indigo instancing
#Added: Sketchup Fog export
#Changed: Default Reinhard Pre-Scale to 6.0
#Added: Export selection only
#Removed: Camera is outside the sky-sphere warning message
#Changed: Indigo path is no longer saved in the Sketchup model, it is stored in a text file in the Sketchup/Plugins directory
#Fixed: command window staying open when auto-rendering
#Changed: Default image save period to 120 seconds
#Fixed: minor bugfixes


#version 0.7test7

#Changed: Re-enabled Bidirectional Path Tracing
#Changed: export file extension from XML to IGS
#Added: Option to save igi file in render settings
#Added: Advanced user functions accessible through Ruby Console
#Changed: Materials are now listed in alphabetical order in Material Editor


#Version 0.7test5

#Added: Environment Map Support
#Added: Up to 4 threads selectable
#Changed: Mirror preset is now a specular material with high IOR (perfect dielectric)
#Added: Support for most features of Indigo 0.7
#Changed: Removed tonemapping from render settings - use Indigo tonemapper GUI
#Added: Bump map support to metals (complex IOR)
#Changed: Changed default exponent to 1000
#Added: Material editor context menu
#Added: New sss material and light presets
#Changed: aperture setting is now specified as f-stop
#Changed: minor UI changes
#Added: autofocus is selectable through focus tool
#Added: reverse gamma correction to rgb meshlights
#Added: context menu to apply material presets and quick export
#Added: New Render Modes - Export All, Separate Settings, and Settings Only (Quick Export)
#Added: Option to run Indigo in Low Priority (creates batch file) - May only work with PCs
#Changed: Material Table to Query Material Types
#Changed: Only visible geometry is exported
#Changed: Auto-render only allowed when file is exported to Indigo directory
#Changed: BPT always turned off (not working in Indigo 0.7test5)
#Changed: Default MNCR to 500 from 100.  500 works better for SSS


#Version 0.6 Official

#Changes since Beta1 - Initial Test Release
#Fixed: [,,,,] characters corrupting inifile.txt on some sytems
#Changed: Auto-render now aborts if there are no lights in scene
#Changed: Changed the way NK_Data (Complex IOR) materials are selected
#Fixed: imported 3DS materials automatically being assigned Specular material type
#Fixed: applied reverse gamma correction exponent of 2.3 to diffuse color of materials
#Changed: Sketchup camera aspect ratio will change based on width and height of render.	
#Changed: Focal Length in Set Focus Tool changed to Focus Distance
#Added: Monochrome Render Modes
#Added: Dull plastic, shiny plastic, leather, car paint, complex IOR presets
#Changed: Specular Materials can now be transparent or non-transparent depending on the opacity setting
#Changed: Negative bump values are now allowed
#Changed: Changed default fresnel to 1.0
#-----------------------------------------------------------------------------

$:.push(File.join(File.dirname(__FILE__)))  #add the skindigo folder to the ruby library search list
require 'sketchup.rb'

unless SkIndigo.on_mac?
	require 'Win32API'
	require 'win32ole'
	require 'registry'
end
	
module SkIndigo

def SkIndigo.start_operation(name)

if Sketchup.version.split(".")[0].to_i>=7
	Sketchup.active_model.start_operation(name,true)
else
	Sketchup.active_model.start_operation(name)
end

end

def SkIndigo.commit_operation

Sketchup.active_model.commit_operation

end

def SkIndigo.validate_quick_export

return MF_ENABLED if SkIndigo.on_mac?
return MF_GRAYED if not $skindigo_exported_this_session
igs_path=Sketchup.active_model.get_attribute 'indigo07_settings','export_path'
if  igs_path and FileTest.exist?(igs_path)
	return MF_ENABLED
else
	return MF_GRAYED
end

end

def SkIndigo.validate_export_selected_entities_only
	return Sketchup.active_model.selection.length > 0 ? MF_ENABLED : MF_GRAYED
end

def SkIndigo.default_export_path()

return SkIndigo.get_user_app_support_path

end
####
def SkIndigo.to_ascii(txt)
  return txt.unpack("U*").map{|c|c.chr}.join
end#def

def SkIndigo.to_unicode(txt)
  return txt.unpack("C*").pack("U*")
end

def SkIndigo.file_exist?(path)

return FileTest.exist?(path.unpack("U*").map{|c|c.chr}.join)  #thanks TIG and Thomthom

end

#####export the scene with a user-given filename
def SkIndigo.export_as(mode,textures)  

model=Sketchup.active_model
model_filename = File.basename(model.path,".skp")
if model_filename!=""
	model_name = model_filename
	model_name += ".igs"
else
	model_name = "Untitled.igs" 	
end

def_path=File.join(model_name)	#default path for exporting is the indigo directory
export_path=UI.savepanel("Export Model", "" , def_path)
return if export_path==nil
export_path=File.join(File.dirname(export_path),File.basename(export_path,".*")+".igs")
SkIndigo.export(0,true,export_path,false,false)

end

###### Export and Render ###########
def SkIndigo.export(mode,textures,export_path,auto_render, should_export_selected_entities_only)

#Export Modes
#0 = Normal (exporting meshes)
#1 = Quick Export (Settings file only - materials, render settings, fog, models and instances)
#2 = Material Preview Scene
	
	is=IndigoRenderSettings.new
	indigo_path=SkIndigo.get_indigo_path
	if indigo_path==nil
		UI.messagebox "Indigo application not found.  Export aborted."
		return
	end
	if not export_path  
		last_path=is.last_path  #returns the last path that the model was exported to
		if (last_path==nil) or (!FileTest.exist?(last_path))
			exp_path=SkIndigo.default_export_path()
			export_path=File.join(exp_path,"default.igs")
			#export_path=File.join(File.dirname(indigo_path),"default.igs")
		else
			export_path=last_path
		end
	end
	export_path=SkIndigo.to_ascii(export_path)
	ie=IndigoExporter.new(export_path)
	
	ie.preview_scene=true if mode==2
	
	model = ie.model
	model_name=File.basename(ie.path,".igs")
	return if model_name==""
	tex_path=File.join(File.dirname(export_path),"TX_"+model_name)
	ie.tex_path="TX_"+model_name
	meshes_path=File.join(File.dirname(ie.path),model_name+"-meshes.igs")
	
	renderingoptions=model.rendering_options
	render_mode=renderingoptions["RenderMode"]
	
	#TODO fix and test monochrome/clay mode	
	#monochrome mode currently disabled
	if  render_mode==5 and mode!=1		#tests for monochrome render mode (but not if using quick export)	
		ie.clay_render=true	
	end

	ie.start=Time.new
	mb=IndigoMeshBuilder.new
	scene_element="scene"
	if mode==0			#normal export mode (exporting meshes)
		if should_export_selected_entities_only && Sketchup.active_model.selection.length>0
			ents=Sketchup.active_model.selection
		else
			ents=Sketchup.active_model.entities
		end
		mb.reset  #erases the stored meshes, instances, and spheres from last export
		#mb.reset_instances()
		mb.build_active_mesh(ents)
		
		#TESTING BINARY EXPORT BEGIN
		#Sketchup.active_model.start_operation("Temp Load Definition",true,true)
		
		t=Time.now
		Sketchup.set_status_text("Collecting Instances...")
		exported_entity_set = Set.new() # Used to prevent infinite loops on recursive components.
		mb.get_instances(ents,Geom::Transformation.new, exported_entity_set)
		p "Traversed model and got instances in #{Time.now-t} seconds."
		
		t=Time.now
		mb.build_meshes2()
		p "Built meshes in #{Time.now-t} seconds."	
		ie.get_used_layers()
	
	elsif mode==1   #quick export
		ie.get_used_layers()
		#mb.reset_instances
		#mb.reset
		#mb.get_instances(Sketchup.active_model.entities,Geom::Transformation.new)
		#mb.build_meshes2()
	elsif mode==2 #preview scene
		ents=Sketchup.active_model.entities
		mb.reset 		
		mb.build_active_mesh(ents)		
		exported_entity_set = Set.new() # Used to prevent infinite loops on recursive components.
		mb.get_instances(ents,Geom::Transformation.new, exported_entity_set)
		mb.build_meshes2()
		ie.get_used_layers()
		scene_element="scenedata"
	end	
	
	out=File.new(export_path,"w") 
	out.puts "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
	out.puts "<#{scene_element}>"
	ie.export_metadata(out)
	ie.export_render_settings(out)
	ie.export_section_planes(out)
	ie.export_tonemapping(out)
	ie.export_camera(model.active_view,out)
	ie.export_environment(out)
	ie.export_default_mat(out,nil)
	
	#export all materials
	#ie.export_clay(out) if ie.clay_render==true
	materials=model.materials.to_a
	
	#don't export the previewmaterial if we are creating a previewscene
	if mode==2
		materials=materials.delete_if {|m| m.name=="previewmaterial"}
	end
	
	#export all the materials
	materials.each {|mat|
		ie.export_material(IndigoMaterial.new(mat),out)
	}
	
	#export SketchUp fog
	ie.export_fog(out) if model.rendering_options["DisplayFog"]
	
	
	#create the meshes IGS file
	if (mode==0) or (mode==2)
		if mode==0  #meshes will be in an external file
			out_mesh = File.new(meshes_path,"w")
			out_mesh.puts "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
			out_mesh.puts "<scenedata>"
		elsif mode==2 #preview scene - meshes xml located in same scene file as materials and settings
			out_mesh=out #use the same scene file as materials
		end
		t=Time.now
		ie.export_meshes2(mb,out_mesh)
		p "Exported meshes in #{Time.now-t} seconds."
		t=Time.now
		Sketchup.set_status_text("Exporting Instances...")
		mb.spheres.each {|sphere| ie.export_sphere(sphere,out_mesh)}
		ie.export_instances(mb,out_mesh)
		p "Exported instances in #{Time.now-t} seconds."
		p "------SkIndigo Export Run------"	
		p "Faces #{mb.faces}"
		p "Instances #{mb.instances}"
		#p "Material Meshes #{mb.mesh_materials.length}"
		p "Component Meshes #{mb.count_meshes()}"
		p "Meshes loaded from cache #{mb.cached_meshes}"
		p "New meshes exported #{mb.new_meshes}"
		p "Spheres #{mb.spheres.length}"
		
		if mode==0
			out_mesh.puts "</scenedata>"
			out_mesh.close
			out.puts "		<include>"
			out.puts "		<pathname>#{File.basename(meshes_path)}</pathname>"
			out.puts "		</include>"
		end
	end
	
	if mode==1
		out.puts "		<include>"
		out.puts "		<pathname>#{File.basename(meshes_path)}</pathname>"
		out.puts "		</include>"
	end
	#image export 
	mb.images.each {|image| ie.export_image(image,out)}
		
	ie.export_light_layer_names(out)
	out.puts "</#{scene_element}>"

	#ie.validate_light_source()
	
	out.close
	
	is.last_path=ie.path unless mode==2 #if exporting is successful, store the IGS path as an attribute
	
	#textures is an boolean flag for exporting textures
	if (textures)
		Sketchup.set_status_text("Exporting textures")		
		if FileTest.exist? (tex_path)
		else
			Dir.mkdir(tex_path)
		end
		indigo_textures=ie.textures
			
		#export all textures collected during the material export process
		status=ie.export_textures(indigo_textures) if indigo_textures.length>0

		#Export all collected ies profiles
		for path in ie.ies
			profile=IESProfile.new(path)
			profile.export(tex_path) if profile.valid?  #export the IES profiles to the textures directory
		end

		stext="All textures and model"	
	else
		stext="Model"
	end
	
	Sketchup.set_status_text("Export Complete")	
	ie.end=Time.new
	elapsed=ie.end-ie.start
	p "exported in #{elapsed.round} seconds."
	
	return true if mode==2 #exit here if exporting a preview scene
	
	$skindigo_exported_this_session=true
	

	
	if not auto_render
		result=UI.messagebox("Model saved in file:\n" + export_path + "\nWould you like to render now?",MB_YESNO) 
		auto_render=true if result==6
	end
	#abort any external skps that were imported
	#status=Sketchup.active_model.abort_operation
	#Sketchup.undo
	#p "aborted = #{status}"
#--------------Open Indigo for rendering
	if (auto_render) and (FileTest.exist?(indigo_path))
		indigo_root=File.dirname(indigo_path)
		renders_dir=SkIndigo.get_renders_folder()
		#check if user wants to use a custom image name
		output_param=""
		case is.network_mode
		when 1 #master
			output_param+="-n m " unless SkIndigo.on_mac?
		when 2 #working master
			output_param+="-n wm " unless SkIndigo.on_mac?
		end
		if is.custom_image_name?
			image_name=is.image_name
			output_path=File.join(renders_dir,image_name+".png")
			igi_output_path=File.join(renders_dir,image_name+".igi")
			#output_param="-o \"#{image_name}\""
			output_param+="-o \"#{output_path}\" -igio \"#{igi_output_path}\"" unless SkIndigo.on_mac?
		end
		
		SkIndigo.launch_indigo(indigo_path,ie.path,output_param,is.low_priority?,"normal")		
		#command_line=create_batch_file(indigo_path,ie.path)
		#command_line=command_line.gsub(/\\/,"\/")
		#UI.openURL(command_line)
		#UI.openURL(indigo_path)  #test for mac auto-open indigo
		#UI.openURL(export_path)
		
	end

end

#####get the renders directory
def SkIndigo.get_renders_folder()

app_data=SkIndigo.get_indigo_app_data_path()
renders=File.join(app_data,"renders")
Dir.mkdir(renders) unless FileTest.exist?(renders)
return renders

end


########method to launch indigo application with optional parameters
def SkIndigo.launch_indigo(indigo_path,igs_file_path,params="",low_priority=false,window="max")

if SkIndigo.on_mac?  #we can currently only open the indigo app on a mac (cannot launch scene)
	#UI.openURL("file:///"+File.expand_path(indigo_path))
	#indigo_path=File.expand_path(indigo_path)
	#p indigo_path
	Dir.chdir(File.dirname(indigo_path))
	indigo_exe=File.basename(indigo_path)
	#export_path="\"#{File.expand_path(igs_file_path)}\""
	export_path="#{File.expand_path(igs_file_path)}"
	#/Applications/Indigo.app/Contents/MacOS/Indigo path_to_file
	if indigo_exe.downcase=~/indigoconsole/
		indigo_exe=File.expand_path(indigo_path)
		command_line="'"<<indigo_exe<<"' '"<<export_path<<"' "<<params
		#p command_line
		#Process.kill(0,preview_thread+1)
		#Process.wait
		system("kill #{$skindigo_preview_thread}") if $skindigo_preview_thread
		pid=fork {exec(command_line)}
		return pid
		#system("killall IndigoConsole")
		#Process.kill("KILL",$skindigo_preview_thread) if $skindigo_preview_thread
		#$skindigo_preview_thread=Thread.new {system(command_line)}
	else
		#command_line="open -a "<<indigo_exe<<" "<<export_path<<" "<<params
		#command_line
		#p "launching indigo"
		#system(command_line) #execute the command
		indigo_exe=File.join(File.expand_path(indigo_path),"Contents","MacOS","Indigo")
		command_line="'"<<indigo_exe<<"' '"<<export_path<<"' "<<params
		#p command_line
		#Process.kill(0,preview_thread+1)
		#Process.wait
		#system("kill #{$skindigo_preview_thread}") if $skindigo_preview_thread
		pid=fork {exec(command_line)}
	end

else   #on windows, we can launch with scene and parameters
	# TODO - test this on windows - taskkill /F /IM <processname.exe> /T
	case window 
		when "max"
			window_param=3
		when "min"
			window_param=2
		when "normal"
			window_param=1
		when "hidden"
			window_param=0
	end
	Dir.chdir(File.dirname(indigo_path))
	indigo_exe=File.basename(indigo_path)
	low=""
	if low_priority
		low="\/low " 
		params=params+" --thread_priority belownormal"
	end
	export_path="\"#{igs_file_path}\""

	begin
		############
		#launch_indigo_with_batch_file(indigo_path,igs_file_path,params,low_priority,window)
		#first, store the pids of all the instances of indigo_console that are running
		wmi = WIN32OLE.connect("winmgmts://")
		query = "select * from Win32_process where name='indigo_console.exe'"
		processes = wmi.ExecQuery(query)
		pids=[]
		processes.each {|process|
			pids.push(process.ProcessID)
		}
		
		#now create a new indigo process
		p indigo_exe
		p export_path
		p params
		shell = WIN32OLE.new('Shell.Application')
		shell.ShellExecute(indigo_exe, export_path+" "+params,'', 'open', window_param)
		
		#check if a new process of indigo_console was created and compare the pid to any previous ones
		wmi = WIN32OLE.connect("winmgmts://")
		query = "select * from Win32_process where name='indigo_console.exe'"
		processes = wmi.ExecQuery(query)
		new_pids=[]
		processes.each {|process|
			new_pids.push(process.ProcessID)
		}
		return (new_pids-pids)[0]  #return the new pid of the new instance of indigo (or nil if a new indigo_console process was not found)
	rescue Exception => msg
		puts "An exception occurred: #{msg}"
		
		# Launch Indigo.  if this was requested to be a hidden window, e.g. for material previews, use the --noninteractive flag.
		# In either case, we want to use indigo.exe, not indigo_console.exe, to avoid display of the console window.
		indigo_exe = "indigo.exe" if indigo_exe == "indigo_console.exe"
		
		noninteractive_arg = window == "hidden" ? " --noninteractive" : ""
		cmd = indigo_exe + " " + export_path + " " + params + noninteractive_arg
		# puts "cmd: '#{cmd}'"
		process = IO.popen(cmd)
		return process.pid
	end
	
	
	#i_proc=DMprocess.new
	#i_proc.create_process(indigo_exe+" "+export_path+" "+params)
	

end

end


##########Display the render settings dialog - TODO - make the settings editor a singleton class and revise this method
def SkIndigo.render_settings
	if not $g_skindigo_settings
		$g_skindigo_settings=IndigoSettingsEditor.new
	end
	$g_skindigo_settings.show
end


#####About message
def SkIndigo.about
UI.messagebox("SkIndigo " + VERSION + " Released " + RELEASE_DATE + "
SketchUp Exporter to Indigo " + VERSION + "
Author: Whaat\n\n

Please visit Indigo's website and forums if you have questions about Indigo.
http://www.indigorenderer.com" , MB_MULTILINE , "About SkIndigo") 

end

#######method to list all materials and Indigo material types
def SkIndigo.mat_query

table=""
model = Sketchup.active_model
materials = model.materials

materials.each do |m| 
        	
	im=IndigoMaterial.new(m)
  
	table+=m.name+" - "
	if im.emitter?
		table+=im.type.upcase+" (Emitter)\n"
	else
		table+=im.type.upcase+"\n"
	end
    next
end
      
       
    # Now display the results
	
	#header="MATERIAL NAME".ljust(40)
	#header+="TYPE\n"
	#UI.messagebox table,MB_OK,"Indigo Material Types"
	UI.messagebox table,MB_MULTILINE,"Indigo Material Types"
	
end




##############get the user's temp directory (platform dependent)
def SkIndigo.temp_dir

if on_mac?
	return ENV['TMPDIR']
else
	return ENV['TEMP']
end

end


#######gets the indigo path from the windows registry
def SkIndigo.get_indigo_path_from_registry

begin
	Win32::Registry::HKEY_CURRENT_USER.open('Software\Glare Technologies\Indigo Renderer') do |reg|
		reg_typ, reg_val = reg.read('InstallDirectory')
	    return reg_val
	end
rescue
	raise
	#return nil
end

end

#######gets the indigo version from the windows registry
def SkIndigo.get_indigo_version_from_registry

begin
	Win32::Registry::HKEY_CURRENT_USER.open('Software\Glare Technologies\Indigo Renderer') do |reg|
		reg_typ, reg_val = reg.read('InstalledVersion')
	    return reg_val
	end
rescue
	return nil
end

end


def SkIndigo.check_indigo_version

indigo_version=SkIndigo.get_indigo_version
reqd=SkIndigo::INDIGO_VERSION_REQUIRED
if reqd
	parsed_version=reqd.split(".")
	v1=parsed_version[0].to_i  #deletes the v character if it exists and converts to an integer
	v2=parsed_version[1].to_i
	v3=parsed_version[2].to_i
	reqd=[v1,v2,v3]
end

return if indigo_version==nil

valid=true
if indigo_version[0]<reqd[0]
	valid=false
end
#UI.messagebox(valid)
if ((indigo_version[0]==reqd[0]) and (indigo_version[1]<reqd[1]))
	valid=false
end
#UI.messagebox(valid)
if ((indigo_version[0]==reqd[0]) and (indigo_version[1]==reqd[1]) and (indigo_version[2]<reqd[2]))
	valid=false
end

#UI.messagebox(valid)
if valid==false
	UI.messagebox("Your Indigo Version is #{indigo_version.join(".")}. You need version #{reqd.join(".")} or greater in order to use this version of SkIndigo.  Please upgrade your copy of Indigo or downgrade your SkIndigo plugin to version #{indigo_version.join(".")} or lower.")
	UI.openURL("http://www.indigorenderer.com")
end


end
######
def SkIndigo.get_indigo_version

if SkIndigo.on_mac?
	data=SkIndigo.get_indigo_version_mac
else
	data=SkIndigo.get_indigo_version_pc
end

if data!=nil
	reg=/(\d+)(\.)(\d+)(\.)(\d+)/
	version=data.slice(reg)
	#UI.messagebox(version)
	if version
		parsed_version=version.split(".")
		v1=parsed_version[0].to_i  
		v2=parsed_version[1].to_i
		v3=parsed_version[2].to_i
		return [v1,v2,v3]
	else
		p "Could not parse version"
	end
end

end


def SkIndigo.get_indigo_version_mac

begin
	indigo_path=SkIndigo.get_indigo_console_path
	Dir.chdir(File.dirname(indigo_path))
	indigo_exe=File.expand_path(indigo_path)
	return nil unless (indigo_exe=~/onsole/) 
	cmd="\"#{indigo_exe}\" -v"
	data=`#{cmd}`
	return data	
rescue
	p "Could not get Indigo version on Mac"
	return nil
end


end
#######gets the indigo version
def SkIndigo.get_indigo_version_pc

begin
	indigo_exe=SkIndigo.get_indigo_console_path  #this method will return the Indigo GUI path if the console path was not found
	return nil unless (indigo_exe=~/console/)  
	log_path=File.join(SkIndigo.temp_dir,"indigo_version.txt")  #standard output will be directed to this file
	bf_path=File.join(SkIndigo.temp_dir,"get_indigo_version.bat")  #batch file that will launch 'indigo_console.exe -v'. 
	
	if FileTest.exist?(log_path)
		File.delete(log_path)
	end
	
	#create the batch file
	bf=File.new(bf_path,"w")
	bf.puts "\"#{indigo_exe}\" -v > \"#{log_path}\""  #output is directed to the log_path file
	bf.close()
	
	#execute the batch file - using ShellExecute is the only way I have found to launch indigo without opening a window
	window_param=0 #hidden window
	params=""
	shell = WIN32OLE.new('Shell.Application')
	shell.ShellExecute(bf_path, params,'', 'open', window_param)
	
	#now we have to give ShellExecute enough time to create and write to the file
	tries=0
	while (!FileTest.exist?(log_path))
		sleep(0.25)
		tries+=1
		return nil if tries==4  #something must have failed if it takes this long to create the file
	end

	#now read the file that should contain the output from indigo_console.exe -v
	f=File.new(log_path,"r")
	data=f.gets
	f.close()
	return data
rescue
	p "Errors getting Indigo version from PC"
	return nil
end

end


########method for users to set the indigo path
def SkIndigo.set_indigo_path

on_mac=SkIndigo.on_mac?
if on_mac
	path=SkIndigo.get_user_app_support_path
	path=File.join(path,"indigo_path.txt")
else
	path=File.join(SkIndigo.get_indigo_app_data_path,"indigo_path.txt")
end

indigo_path=UI.openpanel("Locate Indigo.exe","","*.exe") if not on_mac
indigo_path=UI.openpanel("Locate Indigo Application","","*.exe") if on_mac
if indigo_path
	path_file=File.new(path,"w") 
	path_file.write(indigo_path) 
	path_file.close 
else
	UI.messagebox("In order to render with Indigo, you must download and install the Indigo Renderer application.")
end

if (FileTest.exist?(indigo_path)) and (File.basename(indigo_path,".*").downcase=="indigo") #check if the path to indigo is valid
	#p indigo_path
	return indigo_path
else
	return nil
end

end

#####
def SkIndigo.get_user_app_support_path

on_mac=SkIndigo.on_mac?
if on_mac
	path=File.join(ENV['HOME'],"Library","Application Support","SkIndigo")
	Dir.mkdir(path) unless FileTest.exist?(path)
else
	path=SkIndigo.get_indigo_app_data_path
end
return path

end

#####
def SkIndigo.get_indigo_path_from_program_files(app_name)

program_files=ENV['ProgramFiles'] 
indigo_dir=File.join(program_files,app_name) if program_files
if (program_files) and FileTest.exist?(File.join(indigo_dir,"indigo.exe"))
	return File.join(indigo_dir,"indigo.exe")
else
		
	program_files=ENV['ProgramFiles(x86)'] 
	indigo_dir=File.join(program_files,app_name) if program_files
	if (program_files) and FileTest.exist?(File.join(indigo_dir,"indigo.exe"))
		return File.join(indigo_dir,"indigo.exe")
	else
		program_files=ENV['ProgramW6432'] 
		indigo_dir=File.join(program_files,app_name) if program_files
		if (program_files) and FileTest.exist?(File.join(indigo_dir,"indigo.exe"))
			return File.join(indigo_dir,"indigo.exe")
		end
	end
end

return nil

end


# Try and find the path to the Indigo Renderer or Indigo RT install by looking in some common locations on disk.
# Returns the full path to the Indigo executable if found, e.g. "C:\Program Files (x86)\Indigo Renderer\indigo.exe" or "/Applications/Indigo.app"
# If no Indigo executable can be found, returns nil.
def SkIndigo.detect_indigo_path
	# puts "detect_indigo_path()"
	if !SkIndigo.on_mac?
		path=SkIndigo.get_indigo_path_from_program_files("Indigo Renderer")
		path_rt=SkIndigo.get_indigo_path_from_program_files("Indigo RT")
		if path!=nil
			# puts "detect_indigo_path(): Found path #{path}"
			return path
		elsif path_rt!=nil
			return path_rt
		end
		
	else   #user is on Mac OSX
		
		# Try Indigo Renderer
		path = "/Applications/Indigo Renderer/Indigo.app"
		return path if FileTest.exist?(path)
		
		# Try Indigo RT
		path = "/Applications/Indigo RT/Indigo.app"
		return path if FileTest.exist?(path)
	end
	
	return nil
end


####################method to get path to indigo.exe or indigo.app (mac)
def SkIndigo.get_indigo_path

	# Sketchup.send_action('showRubyPanel:') # open the Ruby Console - for development only.
	
begin 

path=File.join(SkIndigo.get_user_app_support_path,"indigo_path.txt")

if FileTest.exist?(path)  #check if indigo_path.txt exists
	path_file=File.new((path),"r")
	indigo_path=path_file.read  #get the contents of indigo_path.txt file
	path_file.close
		
	if (FileTest.exist?(indigo_path)) and (File.basename(indigo_path,".*").downcase=="indigo") #check if the path to indigo is valid
		#p indigo_path
		return indigo_path
	else
		indigo_path=SkIndigo.set_indigo_path # Ask user to set the path
	end
else
	indigo_path = SkIndigo.detect_indigo_path() # Try and detect the path by looking in known locations on disk.
	if indigo_path == nil
		indigo_path=SkIndigo.set_indigo_path # Ask user to set the path
	end
end

if (FileTest.exist?(indigo_path)) and (File.basename(indigo_path,".*").downcase=="indigo") #check if the path to indigo is valid
	#p indigo_path
	return indigo_path
else
	return nil
end


rescue
end

UI.messagebox("SkIndigo is having trouble finding the Indigo Renderer Application.  If you are on a PC, please install Indigo Renderer in your default Program Files directory.  On a Mac, please install Indigo in the Applications folder.")

end
##############################################################
def SkIndigo.get_indigo_console_path

path=SkIndigo.get_indigo_path
return nil if not path

if SkIndigo.on_mac?
	c_path=File.join(path,"Contents","MacOS","IndigoConsole")
else
	root=File.dirname(path)
	c_path=File.join(root,"indigo_console.exe")
end
if FileTest.exist?(c_path)
	return c_path
else
	return path  #return the indigo GUI path if the console path is not found
end



end

#########
def SkIndigo.get_indigo_app_data_path

if SkIndigo.on_mac?
	indigo_app=SkIndigo.get_indigo_path
	app_data=File.join(indigo_app,"Contents","Resources")
	#app_data=File.join(ENV['HOME'],"Library","Application Support","SkIndigo")
	Dir.mkdir(app_data) unless FileTest.exist?(app_data)
	return app_data
else
	app_data=ENV['APPDATA']
	indigo_app_data=File.join(app_data,"Indigo Renderer")
	unless FileTest.exist?(indigo_app_data)
		Dir.mkdir(indigo_app_data)
	end
	return indigo_app_data
end

end
#######method to pack an igm
def SkIndigo.pack_igm(igm_path)

return nil if !FileTest.exist?(igm_path)
#p igm_path
icp=SkIndigo.get_indigo_console_path
return nil if not icp
root=File.dirname(icp)

#create the output path for the PIGM file
out=File.basename(igm_path,".igm")
out+=".pigm"
out_dir=File.dirname(igm_path)
out=File.join(out_dir,out)

Dir.chdir(root) #changes the working directory to the indigo directoy
cmd="#{icp} --pack \"#{igm_path}\" \"#{out}\""
status=system(cmd)  #executes the command
#p status
return nil if not status
UI.messagebox("IGM packed to #{out}")
return out

end

#############################################################
def SkIndigo.unpack_pigm(pigm)   #returns the path of the igm that was unpacked

begin

return nil if !FileTest.exist?(pigm)
icp=SkIndigo.get_indigo_console_path
return nil if not icp

root=File.dirname(icp)
igm_name=File.basename(pigm,".pigm")
igm=igm_name+".igm"

unless SkIndigo.on_mac?
	unpacked_igm_path=File.join(SkIndigo.get_indigo_app_data_path,"unpacked")
	unless FileTest.exist?(unpacked_igm_path)
		Dir.mkdir(unpacked_igm_path)
	end
	out=File.join(unpacked_igm_path,igm_name)
	unless FileTest.exist?(out)
		Dir.mkdir(out)
	end
	Dir.chdir(root)  #changes the working directory to the indigo directoy
	cmd="indigo_console.exe --unpack \"#{pigm}\" \"#{out}\""
	status=system(cmd)
	unpacked_pigm_dir=out
else
	indigo_console=File.expand_path(icp)
	unpacked_igm_path="\/tmp"
	unless FileTest.exist?(unpacked_igm_path)
		Dir.mkdir(unpacked_igm_path)
	end
	system("cp \"#{pigm}\" \/tmp")   #copies the PIGM file to temp dir
	Dir.chdir("\/tmp")
	#p Dir.pwd
	system("mkdir \"#{igm_name}\"")	
	cmd="\""<<indigo_console<<"\" --unpack \"#{File.basename(pigm)}\" \"#{igm_name}\""   #unpack the PIGM file to the temp directory
	#p cmd
	status=system(cmd)
	user_app_data=SkIndigo.get_user_app_support_path
	unpacked_pigm_dir=File.join(user_app_data,igm_name)
	unless FileTest.exist?(unpacked_pigm_dir)
		Dir.mkdir(unpacked_pigm_dir)
	end
	cmd="mv \"#{igm_name}\" \"#{user_app_data}\""
	system(cmd)
end

raise if not status

#try to find the IGM file in the unpacked PIGM
igm_dir=Dir.entries(unpacked_pigm_dir)
igm=igm_dir.find {|f| f.split(".")[-1]=="igm"}
return nil if igm==nil  #could not find an igm file in the unpacked file
igm_path=File.join(unpacked_pigm_dir,igm)
#p "igm path is #{igm_path}"
return igm_path

rescue

UI.messagebox("Errors occurred while attempting to unpack the PIGM file.")
raise

end

end

#############################################################
def SkIndigo.quick_export

if not $skindigo_exported_this_session
	UI.messagebox("You must first export your scene normally this session before you can use Quick Export.")
end
irs=IndigoRenderSettings.new
igs_path=irs.last_path
return if not igs_path
#results=UI.messagebox("Refresh Textures?",MB_YESNOCANCEL,"Quick Export")
#return if not results
results=7 #i.e. we will not refresh the exported textures for quick export mode
if results==6
	SkIndigo.export(1,true,igs_path,true, false)
elsif results==7
	SkIndigo.export(1,false,igs_path,true, false)
elsif results==2 #cancel
	return
end

end


######
def SkIndigo.set_layer_names()

irs=IndigoRenderSettings.new
layer_names=irs.light_layer_names
prompts=layer_names.collect {|layer| layer_names.index(layer).to_s}
values=layer_names
results=UI.inputbox(prompts,values,"Light Layer Names")
return if not results
irs.light_layer_names=results
$skindigo_mat_observer.refresh_dialog()
if $g_skindigo_settings
	$g_skindigo_settings.refresh_dialog()
end

end

##################dummy instancing function#############
def SkIndigo.dummy(object)

if object.class==Sketchup::Group
    return object.entities if object.name==""
    arr=object.name.split("_")
else
    arr=object.definition.name.split("_")
end
if (arr.length>1) and (arr.last.upcase=="DUMMY")
    arr.pop
    dummy_def_name=arr.join("_")
    if dummy_def_name
        def_list=Sketchup.active_model.definitions
        dummy_def=def_list[dummy_def_name]
    end
end
if dummy_def
    return dummy_def.entities
else
    if object.class==Sketchup::Group
        return object.entities
    else
        return object.definition.entities
    end
end

end #function end

##########################
def SkIndigo.purge_mats

materials=Sketchup.active_model.materials
materials.each do |m|
	dicts=m.attribute_dictionaries
	if dicts
		dicts.delete("indigo07_materials")
		dicts.delete("uvsets")
	end
next
end

end

def SkIndigo.purge_mat

materials=Sketchup.active_model.materials
m=materials.current
if m
	dicts=m.attribute_dictionaries
	if dicts
		dicts.delete("indigo07_materials")
		dicts.delete("uvsets")
	end
end

end
#####################################
def SkIndigo.purge_model

m=Sketchup.active_model
dicts=m.attribute_dictionaries
dicts.delete("indigo07_settings")
end

###########################copy a file to a different location
def SkIndigo.copy_file(source,dest)

begin

in_file=File.new((source),"r")
in_data=in_file.read
in_file.close

out_file=File.new((dest),"w")
out_file.write(in_data)
out_file.close

rescue

	UI.messagebox("SkIndigo file IO error.")
end

end


#########################Function to check if current view is a photomatch view
def SkIndigo.photomatch?(view)

pages=Sketchup.active_model.pages
return false if pages.count==0
page = pages.selected_page
return false if page==nil	 #gets the currently selected page
page_label=page.label
name_length=page.name.length
label_length=page_label.length

if label_length>name_length	#if using photomatch, label length will be greater than name length
	if page_label[name_length..label_length]==" (2D)"
		cam1=page.camera
                cam2=view.camera
                if (cam1.aspect_ratio==cam2.aspect_ratio) and (cam1.focal_length==cam2.focal_length) and (cam1.eye==cam2.eye) and (cam1.direction==cam2.direction) and (cam1.image_width==cam2.image_width) and (cam1.up==cam2.up)
			return true #current view is photomatch view
		end
	end
end

return false

end


###########
def SkIndigo.igm_rgb_gamma_convert(colour,in_gamma,out_gamma)

in_gamma=out_gamma if not in_gamma
return nil if not colour
rgb=colour.split(" ")
if rgb.length==3
	red=((rgb[0].to_f)**(in_gamma.to_f/out_gamma.to_f))
	green=((rgb[1].to_f)**(in_gamma.to_f/out_gamma.to_f))
	blue=((rgb[2].to_f)**(in_gamma.to_f/out_gamma.to_f))
	return [red,green,blue]
else
	return nil
end

end


###########
def SkIndigo.set_instancing(entities,instancing)

entities.each {|ent|
	e=nil
	if ent.class==Sketchup::ComponentInstance
		e=ent.definition
	elsif ent.class==Sketchup::Group
		e=ent.entities[0].parent
	end
	e.set_attribute('indigo07_settings','inst',instancing)
}

end


#########convert euler rotation matrix to rotation quaternion
def SkIndigo.get_rotation_quaternion(trans_old,trans_new)

old_x_axis=trans_old.xaxis
old_y_axis=trans_old.yaxis
old_z_axis=trans_old.zaxis

xaxis=trans_new.xaxis
yaxis=trans_new.yaxis
zaxis=trans_new.zaxis

#get the euler angles

bank=xaxis.angle_between(old_x_axis)
heading=zaxis.angle_between(old_y_axis)
attitude=yaxis.angle_between(old_z_axis)
#p bank.degrees
#p heading.degrees
#p attitude.degrees

c1=Math.cos(heading/2.0)
c2=Math.cos(attitude/2.0)
c3=Math.cos(bank/2.0)
s1=Math.sin(heading/2.0)
s2=Math.sin(attitude/2.0)
s3=Math.sin(bank/2.0)

#note: I modified this from the reference website and it might not work.
#switched y and z, also changed minus to plus due to axis direction
w=c1*c2*c3-s1*s2*s3
x=s1*s2*c3+c1*c2*s3
y=c1*s2*c3+s1*c2*s3
z=s1*c2*c3+c1*s2*s3

return [w,x,y,z]

end

#########convert euler rotation matrix to rotation quaternion
def SkIndigo.get_rotation_quaternion_void(transformation)

xaxis=transformation.xaxis
yaxis=transformation.yaxis
zaxis=transformation.zaxis

#get the euler angles

bank=xaxis.angle_between(Geom::Vector3d.new(1,0,0))
heading=zaxis.angle_between(Geom::Vector3d.new(0,0,1))
attitude=yaxis.angle_between(Geom::Vector3d.new(0,1,0))

c1=Math.cos(heading/2.0)
c2=Math.cos(attitude/2.0)
c3=Math.cos(bank/2.0)
s1=Math.sin(heading/2.0)
s2=Math.sin(attitude/2.0)
s3=Math.sin(bank/2.0)

#note: I modified this from the reference website and it might not work.
w=c1*c2*c3-s1*s2*s3
x=s1*s2*c3+c1*c2*s3
y=c1*s2*c3+s1*c2*s3
z=s1*c2*c3+c1*s2*s3

return [w,x,y,z]

end

#########represent a quaternion in terms of axis-angle
def SkIndigo.quaternion_as_axis_angle(quaternion)

q=quaternion
#p q
w=q[0]
i=q[1]
j=q[2]
k=q[3]

angle=Math.acos(w)*2
x=i/(Math.sin(angle/2.0))
y=j/(Math.sin(angle/2.0))
z=k/(Math.sin(angle/2.0))

return [angle,Geom::Vector3d.new(x,y,z)]

end

#######
def SkIndigo.get_axis_angle(transformation)

xaxis=transformation.xaxis.normalize
yaxis=transformation.yaxis.normalize
zaxis=transformation.zaxis.normalize

#calculate the angle
angle=Math.acos((xaxis.x+yaxis.y+zaxis.z-1.0)/2.0)

#calculate the axis
axis=Geom::Vector3d.new
#axis.x=yaxis.z-zaxis.y
#axis.y=zaxis.x-xaxis.z
#axis.z=xaxis.y-yaxis.x
axis.x=zaxis.y-yaxis.z
axis.z=yaxis.x-xaxis.y
axis.y=xaxis.z-zaxis.x

axis.normalize!

if axis.length==0.0
	axis=Geom::Vector3d.new(0.0,1.0,0.0)
	angle=0.0
end

return [axis,angle]

end

end ##########################################SkIndigo Module############################################


#---------------------------------------Custom Tool to select Focal distance----------------------
class IndigoFocusTool

def initialize
    @ip1 = nil
        
end

def activate
    @ip1 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    self.reset(nil)
end

def deactivate(view)
    view.invalidate 
end

def onMouseMove(flags, x, y, view)
            
	@ip.pick view, x, y
        if( @ip != @ip1 )
           
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
                        
            view.tooltip = @ip1.tooltip
        end

end

def onLButtonDown(flags, x, y, view)
    	
        @ip1.pick view, x, y
        if( @ip1.valid? )

            	model=Sketchup.active_model		
		model.set_attribute 'indigo07_settings','fp',@ip1.position
		UI.messagebox ("New Focal Point set to #{@ip1.position}")
		                    
        end
    
    view.lock_inference
    Sketchup.active_model.select_tool nil
end


def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        
    end
end

def onCancel(flag, view)
    self.reset(view)
end


# Reset the tool back to its initial state
def reset(view)
       
    Sketchup::set_status_text("Select Point on Focal Plane", SB_PROMPT)
    @ip1.clear
        
    if( view )
        view.tooltip = nil
        view.invalidate 
    end
  
end

end # class FocusTool

####todo - put method inside skindigo module




####################Advanced user functions#####################
class Indigo

def Indigo::adv

indigo_path=SkIndigo.get_indigo_path
return if indigo_path==nil
inifile_path=File.dirname(indigo_path)
inifile_path+="\/inifile.xml"
if FileTest.exist?(inifile_path)
	UI.openURL(inifile_path)
else
	puts "inifile.xml not found"
end

end

def Indigo::ren

indigo_path=SkIndigo.get_indigo_path
return if indigo_path==nil
renders_path=File.dirname(indigo_path)
renders_path+="\/renders"
if FileTest.exist?(renders_path)
	UI.openURL(renders_path) 
else
	puts "renders directory not found"
end

end

def Indigo::igs

export_path=Sketchup.active_model.get_attribute 'indigo07_settings','export_path'
return false if not export_path
settings_path=File.join(File.dirname(export_path),(File.basename(export_path,".igs") + "-settings.igs"))
if settings_path!=""
	if FileTest.exist?(settings_path)
		UI.openURL(settings_path)
	else
		puts "igs not found"
	end 
else
	if FileTest.exist?(export_path)
		UI.openURL(export_path) 
	else
		puts "igs not found"
	end
end

end

def Indigo::tm

indigo_path=SkIndigo.get_indigo_path
return if indigo_path==nil
tm_path=File.dirname(indigo_path)
tm_path+="\/IndigoToneMapper.exe"
if FileTest.exist?(tm_path)
	UI.openURL(tm_path) 
else
	puts "IndigoToneMapper.exe not found"
end

end

def Indigo::web

UI.openURL("http:\/\/www2.indigorenderer.com\/joomla\/")

end

def Indigo::mdb

UI.openURL("http:\/\/www.elqx.com\/indmatdb\/")

end


def Indigo::scale=(val)
	Sketchup.active_model.set_attribute 'indigo07_settings','scale',val.to_f
end

def Indigo::scale
	Sketchup.active_model.get_attribute 'indigo07_settings','scale',1.0
end

end #class Indigo end

###todo - put observer in separate file?
class SkindigoAppObserver <  Sketchup::AppObserver
 	def onOpenModel(m)
		if $skindigo_mat_observer
			$skindigo_mat_observer.stop_preview()
			#timer=$skindigo_mat_observer.timer
			#UI.stop_timer(timer) if timer
			$skindigo_mat_observer.mat_editor.close if $skindigo_mat_observer.mat_editor.visible?
		end
		$skindigo_mat_observer=nil
		$skindigo_mat_observer=SkindigoMatObserver.new
		if $g_skindigo_settings
			$g_skindigo_settings.dialog.close if $g_skindigo_settings.dialog.visible?
		end
		$g_skindigo_settings=nil	
		status=Sketchup.active_model.materials.add_observer($skindigo_mat_observer)
		$g_su2in_loaded_entities=nil if $g_su2in_loaded_entities
		if $skindigo_object_editor
			$skindigo_object_editor.dialog.close if $skindigo_object_editor.dialog.visible?
		end
		$skindigo_object_editor=nil
		$skindigo_object_editor=SkindigoObjectEditor.new
		Sketchup.active_model.selection.add_observer($skindigo_object_editor)
		mb=IndigoMeshBuilder.new.reset
		$skindigo_exported_this_session=false
	end

	def onNewModel(m)

		if $skindigo_mat_observer
			$skindigo_mat_observer.stop_preview()
			#timer=$skindigo_mat_observer.timer
			#UI.stop_timer(timer) if timer
			$skindigo_mat_observer.mat_editor.close if $skindigo_mat_observer.mat_editor.visible?
		end
		$skindigo_mat_observer=nil  
		$skindigo_mat_observer=SkindigoMatObserver.new
		if $g_skindigo_settings
			$g_skindigo_settings.dialog.close if $g_skindigo_settings.dialog.visible?
		end
		$g_skindigo_settings=nil
		Sketchup.active_model.materials.add_observer($skindigo_mat_observer)
		$g_su2in_loaded_entities=nil if $g_su2in_loaded_entities
		if $skindigo_object_editor
			$skindigo_object_editor.dialog.close if $skindigo_object_editor.dialog.visible?
		end
		$skindigo_object_editor=nil
		$skindigo_object_editor=SkindigoObjectEditor.new
		Sketchup.active_model.selection.add_observer($skindigo_object_editor)
		mb=IndigoMeshBuilder.new.reset
		$skindigo_exported_this_session=false
	end
	
	def onQuit()
		if $skindigo_mat_observer
			#UI.messagebox("test")
			$skindigo_mat_observer.stop_preview()
			Sketchup.active_model.materials.remove_observer($skindigo_mat_observer) 
			$skindigo_mat_observer=nil
		end
		$g_skindigo_settings=nil if $g_skindigo_settings
		$skindigo_object_editor=nil
		mb=IndigoMeshBuilder.new.reset
		$skindigo_exported_this_session=nil
	end

end

##################UV Class to Manage UV Sets####################
#TODO - put in separate file? i belive this class is no longer used - it has been superceded by moduel SkIndigo UVS
class UVS


def UVS::store_uvs(uvset)

face=0
case uvset
	when 0
		key="uv0"
	when 1
		key="uv1"
	when 2
		key="uv2"
	when 3
		key="uv3"
end

Sketchup.active_model.start_operation "Save UV set"

for et in $g_su2in_loaded_entities
	
	if (et[0].valid?) and (et[0].class==Sketchup::Face)
		
		polymesh=et[0].mesh 5
		uvs=polymesh.uvs 5
		for i in (0..uvs.length-1)
			uvs[i]=uvs[i].to_a  #converts the Point 3D objects to arrays so they won't be transformed
		end
		et[0].set_attribute 'uvs',key,uvs
	
	
	face=face+1
	Sketchup.set_status_text("Stored uvs for #{face} faces.")
	end
		
end

#UI.messagebox "UVs saved for #{face} faces."

Sketchup.active_model.commit_operation

end #function
##########################################load entities#####################
def UVS::load_mat_entities(mat)

Sketchup.active_model.start_operation "Load entities"

$g_su2in_loaded_entities=[]
entities=Sketchup.active_model.entities
load_entities(mat,entities,Geom::Transformation.new)

UI.messagebox "Loaded #{$g_su2in_loaded_entities.length} entities currently assigned material \"#{mat.name}\"."

Sketchup.active_model.commit_operation

end


##########################################load entities recursive######################
def UVS::load_entities(mat,object,trans)


if object.class == Sketchup::ComponentInstance
	entity_list=object.definition.entities
elsif object.class == Sketchup::Group
	entity_list=object.entities
else
	entity_list=object
end

Sketchup.set_status_text("Collected #{$g_su2in_loaded_entities.length} entities assigned material #{mat.name}")

for e in entity_list
		if (e.class == Sketchup::Group)
			
			if e.material == mat
				$g_su2in_loaded_entities.push([e,trans])
				Sketchup.set_status_text("Collected #{$g_su2in_loaded_entities.length} entities assigned material #{mat.name}")
			end
			UVS.load_entities(mat,e,trans*e.transformation)
			
		end
		if (e.class == Sketchup::ComponentInstance)
			if e.material == mat
				$g_su2in_loaded_entities.push([e,trans])
				Sketchup.set_status_text("Collected #{$g_su2in_loaded_entities.length} entities assigned material #{mat.name}")
			
			end
			UVS.load_entities(mat,e,trans*e.transformation)
							
		end
		if (e.class == Sketchup::Face)
			
			if e.material == mat
				$g_su2in_loaded_entities.push([e,trans])
				Sketchup.set_status_text("Collected #{$g_su2in_loaded_entities.length} entities assigned material #{mat.name}")
					
			end
			
		end
end

return nil

end #function

####################################switch the texture map#########################3

def UVS::switch_material(mat)

materials = Sketchup.active_model.materials
names = materials.collect {|m| m.name}
translatednames = materials.collect {|m| m.display_name}
    
# Display a dialog to pick a material

    prompts = ["Material Name"]
    values=[materials[0].display_name]
    enums = [translatednames.sort.join("|")]
    results = inputbox prompts, values, enums,("Assign all loaded entities to:")
    return if not results
	
current_mat=results[0]

# Get the selected Material

index = translatednames.index(current_mat)
material = index ? materials[names[index]] : nil

if( not material )
       UI.messagebox ("Could not find Material named") + " #{results[0]}"
       return
end

Sketchup.active_model.start_operation "Switch Material"
for et in $g_su2in_loaded_entities
	if et[0].valid?
		et[0].material=material 
	end
end

if material.texture!=nil
	prompts=["Select UV index"]
	values=[1]
	enums=[[1,2,3].join("|")]
	result = inputbox prompts, values, enums,("Select UV set")
	if result
		position_map(result[0],$g_su2in_loaded_entities)
	end
end

Sketchup.active_model.commit_operation

end


################################Function to position the material according to stored uvset########
def UVS::position_map(set,entities)

Sketchup.active_model.start_operation "Load UV set"

face=0
failed=0
 
case set
	
	when 0
		key="uv0"
	when 1
		key="uv1"
	when 2
		key="uv2"
	when 3
		key="uv3"
end

for et in entities
	if (et[0].valid?) and (et[0].class==Sketchup::Face)
		uvs=et[0].get_attribute 'uvs',key
		
		if uvs
			
			pos=[]
			polymesh=et[0].mesh 5
			polymesh.transform! et[1]
			polygons=polymesh.polygons
			poly_index=0
			begin
				for p in polygons[poly_index]
					point=polymesh.point_at(p.abs)
					if point
						pos.push(point) 
						pos.push(uvs[p.abs-1])
					end
				end

				et[0].position_material et[0].material, pos, true
				face=face+1
				Sketchup.set_status_text("Loaded uvs for #{face} faces")
			rescue  #this is required because SketchUp sometimes fails to position the texture properly
				poly_index=poly_index+1
				if polygons[poly_index]
					pos=[]
					retry  #try to postion texture again using next polygon in the face
				else
					failed=failed+1
				end
			end
	
			
		end
	end
end

if failed>0
	stext="Unable to load UVs for #{failed} faces.  Loaded UVs for #{face} faces."
else
	stext="Loaded UVs for #{face} faces."
end

UI.messagebox(stext)

Sketchup.active_model.commit_operation

end




########################Purge UV sets################
def UVS::purge_uvs

result=UI.messagebox "Would you like to purge all stored UV sets for the currently loaded entities?",MB_YESNO
return if result!=6

Sketchup.active_model.start_operation "Purge UV sets"

faces=0

for et in $g_su2in_loaded_entities
	if (et[0].valid?) and (et[0].class==Sketchup::Face)
		et[0].delete_attribute 'uvs'
		faces=faces+1
	end
end  


UI.messagebox "Purged UV sets for #{faces} faces."

Sketchup.active_model.commit_operation

end

####################################Explode Distorted Textures#########################
def UVS::explode_distorted_texture

face=Sketchup.active_model.selection.first
return if not face
return if face.class!=Sketchup::Face

Sketchup.active_model.start_operation "Explode Texture"
mat=face.material
materials=Sketchup.active_model.materials
new_mat=materials.add(mat.name)
new_name=new_mat.name+".jpg"

tw=Sketchup.create_texture_writer

if (mat.respond_to?(:texture) and mat.texture !=nil)
	status=tw.load face,true
else
	return nil
end

texture_path=UI.savepanel("Save Distorted Texture", "" ,new_name )

return if texture_path==nil		
if texture_path==texture_path.split(".")[0] #no extension given
	texture_path+=".jpg"
end


arr=[]

uvh=face.get_UVHelper true,false,tw

for p in (0..3)
	vert=face.vertices[p]
	pos=vert.position
	uvq = uvh.get_front_UVQ(pos)
	arr.push(pos)
	arr.push(uvq)
end

tw.write(face,true,texture_path)
new_mat.texture=(texture_path) if new_mat
face.position_material(new_mat,arr,true)

dict="indigo07_materials"
att=["type","exp","fres","ior","cau","precedence","sss","sssval","power","ole","bump_map","bump_value","temp","no_smooth","exp_map","expmap_value","clip_map"]

for a in att
	new_mat.set_attribute dict,a,(mat.get_attribute dict,a)
end

dict="indigo_blend"
att=[]
att=["matA","matB","blendfactor"]

for a in att
	new_mat.set_attribute dict,a,(mat.get_attribute dict,a)
end

new_mat.alpha=mat.alpha

Sketchup.active_model.commit_operation
UI.messagebox "Texture was exploded and applied to the selected face."

end


end #Class end


#####
 class SkIndigoFrameChangeObserver

   def initialize
      # @current page is used to make sure that the observer is only triggered by page changes
      # This is to workaround the PC bug that does not correctly populate the fromPage variable in the
      # callback to frameChange.
      @current_page = Sketchup.active_model.pages.selected_page
   end  # def initialize

   # Callback triggered by the Frame change observer.
   # fromPage is populated with the page we were coming from, toPage is
   # populated with the page we were transitioning to
   # and percent done contains the percentage of the transition that
   # has completed.

   def frameChange(fromPage, toPage, percent_done)      
     
	 
      
         if @current_page != toPage then
            #update the current page
			p "updating settings"
            @current_page = toPage
			$g_skindigo_settings.refresh_dialog(@current_page)
            
         end
     
   end   # def frameChange

end #class   

##########################################Render Settings Editor Class
#TODO - put in separate file
class IndigoSettingsEditor

def initialize
	@dlg=UI::WebDialog.new("SkIndigo Render Settings", false, "SkIndigoRenderSettings",500, 450, 370,100,true)
	@dlg.set_file(File.join(File.dirname(__FILE__),"Settings.html"))
	color = @dlg.get_default_dialog_color 
	@dlg.set_background_color(color)
	@irs=IndigoRenderSettings.new
	Sketchup::Pages.add_frame_change_observer(SkIndigoFrameChangeObserver.new)
	
	@on_mac=SkIndigo.on_mac?
	@dlg.add_action_callback("apply_settings") {|d,p|
		@irs.save_to_model()
		apply_settings(d,p)
		
	} #end apply settings callback
	@dlg.add_action_callback("apply_to_scene") {|d,p|
		
		pages=Sketchup.active_model.pages
		if pages.count>0
			page=pages.selected_page
			if page==nil
				page=pages[0]
			end
			@irs.save_to_scene(page)
			apply_settings(d,p)
		else
			UI.messagebox("You must first select a SketchUp scene.")
		end
	}
	@dlg.add_action_callback("getViewSize") {|d,p|
		width=(Sketchup.active_model.active_view.vpwidth).to_s
		height=(Sketchup.active_model.active_view.vpheight).to_s
		cmd="document.getElementById('width').value=\"#{width}\""
		d.execute_script(cmd)
		cmd="document.getElementById('height').value=\"#{height}\""
		d.execute_script(cmd)
	} #end get View Size callback
	@dlg.add_action_callback("ValueChanged") {|d,p|
		pair=p.split("=")
		id=pair[0]
		value=pair[1]
		case id
			when "inifile"
				Indigo.adv   #opens inifile.xml for editing advanced indigo settings
			when "pickFocalPoint"
				Sketchup.active_model.select_tool IndigoFocusTool.new     #launches to focus tool
				setCheckbox("autofocus","false")
			when "envMapBrowser"
				exr=UI.openpanel "Select Environment Map", "", ""
				if exr
					@irs.store_texture(exr)
					mats=Sketchup.active_model.materials
					mat=mats.add("EnvironentMap")
					mat=IndigoMaterial.new(mat)
					mat.layer=0
					bm=mat.base_emission
					bm.uni_value=1.0
					bm.spectrum_type="Uniform"
					mat.base_emission=bm
					mat.emission_scale=75000
					mat.emission_measure="luminous_emittance"
					em=mat.emission_map
					em.a=0.0
					em.b=1.0
					em.c=0.0
					em.exponent=1.0
					em.type="External"
					em.path=exr
					#em.path=exr.gsub!("\\","\/")
					mat.emission_map=em
					mat.use_emission=true
					refreshMaterialsList()
					setValue("MaterialName",mat.name)
					setRadioButton("EnvironmentMap","true")
					
				end
			when "apertureImageBrowser"
				path=UI.openpanel "Select Aperture Map", "", "*.png"
				if path
					path.gsub!("\\","\/")
					setValue("apertureImage",path)
				end
			when "obstacleMapBrowser"
				path=UI.openpanel "Select Obstacle Map", "", "*.png"
				if path
					path.gsub!("\\","\/")
					setValue("obstacleMap",path)
				end
		end
	} # end Value Changed callback
	@dlg.add_action_callback("cancel_changes") {|d,p|
		loadValuesFromSU()
	} 
	@dlg.add_action_callback("reset_camera") {|d,p|
		current_camera=Sketchup.active_model.active_view.camera
		current_aspect=current_camera.aspect_ratio
		if current_aspect!=0.0
			current_camera.aspect_ratio=0.0
			current_camera.image_width=0.0
			current_camera.focal_length=current_camera.focal_length*(current_aspect)
		end
	}   
	@dlg.add_action_callback("undo_changes") {|d,p|
		Sketchup.undo
		loadValuesFromSU()
		model=Sketchup.active_model
		view=model.active_view
		width=@irs.width  
		height=@irs.height
		current_camera=model.active_view.camera
		old_aspect=current_camera.aspect_ratio
		old_aspect=1.0 if old_aspect==0.0
		f=current_camera.focal_length
		current_camera.aspect_ratio=(width.to_f)/(height.to_f)
		current_camera.focal_length=f/(current_camera.aspect_ratio/old_aspect)
	}
	@dlg.add_action_callback("select_region") {|d,p|
		validate_render_region()
		#@irs.render_region=true
		Sketchup.active_model.select_tool SelectRegionTool.new
		setCheckbox("regionRenderCheckbox","true")
	}
	@dlg.add_action_callback("set_sun") {|d,p|
		#@irs.sun_pos_override=true
		Sketchup.active_model.select_tool SkIndigoSunTool.new
		setCheckbox("overrideSunCheckbox","true")
	}

	@dlg.add_action_callback("happy_easter") {|d,p|
		SkIndigo.export_animation(false)
		#SkIndigo.export_animation(true)   #sketchy physics testing
	} 
	@dlg.add_action_callback("refresh") {|d,p|
		refresh_dialog()
	}
	@dlg.add_action_callback("save_defaults") {|d,p|
		begin
			status=@irs.store_settings()
			if status
				UI.messagebox("Current render settings successfully saved as defaults.")
			else
				UI.messagebox("You must first apply your current settings.")
			end
		rescue
			UI.messagebox("Unable to save settings.")
			raise
		end
	}
	
end #initialize

#
def apply_settings(d,p)  #d is the dialog, p is string of parameters
		
		#extract checkbox and radio values
		params=p.split("|")
		save_igi=params[0].split("=")[1] 
		custom_name=params[1].split("=")[1]
		tm=params[2].split("=")[1]
		af=params[3].split("=")[1]
		env=params[4].split("=")[1]
		auto_threads=params[5].split("=")[1]
		low_priority=params[6].split("=")[1]
		watermark=params[7].split("=")[1]
		ad=params[8].split("=")[1]
		om=params[9].split("=")[1]
		rr=params[10].split("=")[1]
		io=params[11].split("=")[1]
		so=params[12].split("=")[1]
		ga=params[13].split("=")[1]
		vi=params[14].split("=")[1]
		bgs=params[15].split("=")[1]
		la=params[16].split("=")[1]
		gpu=params[17].split("=")[1]
		sky_model=params[18].split("=")[1]
		cull=params[19].split("=")[1]
		alpha=params[20].split("=")[1]
		
		model=Sketchup.active_model
		model.start_operation "Render Settings"

		@irs.watermark=true if (watermark=="true")
		@irs.watermark=false if (watermark=="false")
		@irs.save_igi=true if (save_igi=="true")
		@irs.save_igi=false if (save_igi=="false")
		@irs.custom_image_name=true if (custom_name=="true")
		@irs.custom_image_name=false if (custom_name=="false")
		@irs.tm=tm
		@irs.focal_point="auto" if (af=="true")
		@irs.environment=env
		@irs.auto_choose_threads=true if (auto_threads=="true")
		@irs.auto_choose_threads=false if (auto_threads=="false")
		@irs.low_priority=true if (low_priority=="true")
		@irs.low_priority=false if (low_priority=="false")
		@irs.aperture_diffraction=true if (ad=="true")
		@irs.aperture_diffraction=false if (ad=="false")
		@irs.use_obstacle_map=true if (om=="true")
		@irs.use_obstacle_map=false if (om=="false")
		@irs.render_region=true if (rr=="true") and (validate_render_region())
		@irs.render_region=false if (rr=="false")
		@irs.info_overlay=true if (io=="true")
		@irs.info_overlay=false if (io=="false")
		@irs.sun_pos_override=true if (so=="true")
		@irs.sun_pos_override=false if (so=="false")
		@irs.glass_acceleration=true if (ga=="true")
		@irs.glass_acceleration=false if (ga=="false")
		@irs.vignetting=true if (vi=="true")
		@irs.vignetting=false if (vi=="false")
		@irs.bg_sun=true if (bgs=="true")
		@irs.bg_sun=false if (bgs=="false")
		@irs.lock_aspect=true if (la=="true")
		@irs.lock_aspect=false if (la=="false")
		@irs.use_gpu=(gpu=="true") ? true : false
		@irs.sky_model=(sky_model=="true") ? "original" : "captured-simulation"
		@irs.cull_geometry=(cull=="true") ? true : false
		@irs.foreground_alpha=(alpha=="true") ? true : false
		
		@irs.width=d.get_element_value("width")
		@irs.height=d.get_element_value("height")
		@irs.image_name=d.get_element_value("image_name")
		@irs.aperture=d.get_element_value("aperture")
		@irs.white_balance=d.get_element_value("wb")
		@irs.turbidity=d.get_element_value("turbidity")
		@irs.method=d.get_element_value("method")
		@irs.mncr=d.get_element_value("MNCR")
		@irs.world_scale=d.get_element_value("worldScale")
		@irs.image_save_period=d.get_element_value("savePeriod")
		@irs.halt=d.get_element_value("halt")
		@irs.halt_unit=d.get_element_value("haltUnit")
		@irs.num_threads=d.get_element_value("threads")
		#@irs.thumbnail_path=d.get_element_value("thumbPath")
		@irs.pre_scale=d.get_element_value("preScale")
		@irs.post_scale=d.get_element_value("postScale")
		@irs.burn=d.get_element_value("burn")
		@irs.iso=d.get_element_value("iso")
		@irs.shutter_speed=d.get_element_value("shutterSpeed")
		@irs.ev_adjust=d.get_element_value("exposure")
		#@irs.exr=d.get_element_value("exr")
		#@irs.exr_type=d.get_element_value("envMapType")
		@irs.exr_shift=d.get_element_value("envMapRotation")
		#@irs.exr_width=d.get_element_value("envMapWidth")
		#@irs.exr_gain=d.get_element_value("envMapGain")
		@irs.aperture_shape=d.get_element_value("apertureShape")
		@irs.blade_offset=d.get_element_value("bladeOffset")
		@irs.blade_curvature_radius=d.get_element_value("curvatureRadius")
		@irs.start_angle=d.get_element_value("startAngle")
		@irs.aperture_image=d.get_element_value("apertureImage")
		@irs.obstacle_map=d.get_element_value("obstacleMap")
		@irs.shift_up=d.get_element_value("shiftUp")
		@irs.shift_right=d.get_element_value("shiftRight")
		@irs.supersample_factor=d.get_element_value("supersampleFactor")
		@irs.sun_layer=d.get_element_value("sunLayer")
		@irs.sky_layer=d.get_element_value("skyLayer")
		@irs.camera_type=d.get_element_value("cameraType")
		#@irs.env_map_layer=d.get_element_value("envMapLayer")
		#@irs.frame_upload_period=d.get_element_value("frameUploadPeriod")
		@irs.network_mode=d.get_element_value("networkMode")
		@irs.cam_resp_func=d.get_element_value("responseFunction")
		#@irs.bg_luminance=d.get_element_value("envMapGain")
		@irs.bg_weight=d.get_element_value("bgWeight")
		@irs.sunsky_weight=d.get_element_value("sunskyWeight")
		@irs.bg_material=d.get_element_value("MaterialName")
		#p @irs.bg_material
						
		current_camera=model.active_view.camera
		old_aspect=current_camera.aspect_ratio
		old_aspect=1.0 if old_aspect==0.0
		f=current_camera.focal_length
		current_camera.aspect_ratio=(@irs.width.to_f)/(@irs.height.to_f)
		current_camera.focal_length=f/(current_camera.aspect_ratio/old_aspect)
		
		model.commit_operation


end

def refresh_dialog(page=nil)
refreshCamResponseList()
refreshLayerList()
refreshMaterialsList()
loadValuesFromSU(page)
setApertureVisible()
#setExrWidth()
end

def show
	if @dlg.visible?
		self.refresh_dialog()
	else
		if @on_mac
			self.refresh_dialog()
			@dlg.show_modal{}
		else
			@dlg.show{}
		end
	end
end

def refreshMaterialsList()

cmd="clearMaterialList()"
@dlg.execute_script(cmd)
mat_list=Sketchup.active_model.materials.collect {|m| m.name}
mat_list.sort!
id="MaterialName"
mat_list.each {|mat_name|
	cmd="createMapOption(\"#{id}\",\"#{mat_name}\",\"#{mat_name}\")"
	@dlg.execute_script(cmd)
}

end

def validate_render_region()
	if @irs.tm=="Reinhard"   #region rendering only works with camera or linear tonemapping
		UI.messagebox "Tonemapping has been set to 'Camera' for use with region rendering."
		@irs.tm="Camera"
		setRadioButton("cameraTonemapping","true")
	end
	return true
end

def setExrWidth()
	#@dlg.execute_script("setExrWidthVisible()")
end

def select_region_tool()
	#Sketchup.active_model.select_tool(SelectRegionTool.new)
end

def setApertureVisible()
	@dlg.execute_script("setApertureAttributesVisible()")
end

def loadValuesFromSU(page=nil)
	#check if current scene contains indigo settings

	#disabling loading from scene due to bugs
	#if page!=nil
	#	@irs.save_to_scene(page)   #this will cause the settings stored to the current scene to be loaded
		#p "load froms scene"
	#else
	#	@irs.save_to_model()
		#p "load from model"
	#end
	#set checkboxes and radio buttons
	env=@irs.environment
	tm=@irs.tm
	
	if @irs.bg_sun? then setCheckbox("bgSun","true")
	else setCheckbox("bgSun","false")
	end
	if @irs.vignetting? then setCheckbox("vignetting","true")
	else setCheckbox("vignetting","false")
	end
	if @irs.glass_acceleration? then setCheckbox("glassAccel","true")
	else setCheckbox("glassAccel","false")
	end
	if @irs.watermark? then setCheckbox("watermark","true")
	else setCheckbox("watermark","false")
	end
	if @irs.sun_pos_override? then setCheckbox("overrideSunCheckbox","true")
	else setCheckbox("overrideSunCheckbox","false")
	end
	if @irs.render_region? then setCheckbox("regionRenderCheckbox","true")
	else setCheckbox("regionRenderCheckbox","false")
	end
	if @irs.info_overlay? then setCheckbox("infoOverlay","true")
	else setCheckbox("infoOverlay","false")
	end
	if @irs.save_igi? then setCheckbox("saveigi","true")
	else setCheckbox("saveigi","false")
	end
	if @irs.custom_image_name? then setCheckbox("customName","true")
	else setCheckbox("customName","false")
	end
	if @irs.focal_point=="auto" then setCheckbox("autofocus","true")
	else setCheckbox("autofocus","false")
	end
	if env=="Sun and Sky" then setRadioButton("SunSky","true")
	elsif env=="Background" then setRadioButton("Background","true")
	elsif env=="Black" 
		setRadioButton("Background","true")
		setCheckbox("black","true")
	elsif env=="Environment Map" then setRadioButton("EnvironmentMap","true")
	end
	if tm=="Reinhard" then setRadioButton("Reinhard","true")
	elsif tm=="Camera" then setRadioButton("cameraTonemapping","true")
	end
	if @irs.auto_choose_threads? then setCheckbox("autoThreads","true")
	else setCheckbox("autoThreads","false")
	end
	if @irs.low_priority? then setCheckbox("lowPriority","true")
	else setCheckbox("lowPriority","false")
	end
	if @irs.aperture_diffraction? then setCheckbox("apertureDiffractionCheckbox","true")
	else setCheckbox("apertureDiffractionCheckbox","false")
	end
	if @irs.use_obstacle_map? then setCheckbox("obstacleCheckbox","true")
	else setCheckbox("obstacleCheckbox","false")
	end
	if @irs.lock_aspect? then setCheckbox("lockAspect","true")
	else setCheckbox("lockAspect","false")
	end
	if @irs.use_gpu? then setCheckbox("useGPU","true")
	else setCheckbox("useGPU","false")
	end
	setCheckbox("cullGeometry",@irs.cull_geometry?)
	if @irs.method=="Alpha"
		@irs.foreground_alpha=true
		@irs.method="Bidir PT"
	end
	setCheckbox("foregroundAlpha",@irs.foreground_alpha?)
	
	if @irs.sky_model=="original" then setCheckbox("skyModel","true")
	else setCheckbox("skyModel","false")
	end
	
	#Set other dialog input values
	setValue("width",(@irs.width))  
	setValue("height",(@irs.height)) 
	setValue("image_name",(@irs.image_name)) 
	setValue("aperture",(@irs.aperture))
	setValue("wb",(@irs.white_balance))
	setValue("method",(@irs.method))
	setValue("MNCR",(@irs.mncr))
	setValue("worldScale",(@irs.world_scale))
	setValue("savePeriod",(@irs.image_save_period))
	setValue("halt",(@irs.halt))
	setValue("haltUnit",(@irs.halt_unit))
	#setValue("exr",(@irs.exr))
	#setValue("envMapType",(@irs.exr_type))
	#setValue("envMapWidth",(@irs.exr_width))
	setValue("imagePath",(@irs.image_path))
	setValue("thumbPath",@irs.thumbnail_path)
	setValue("apertureShape",@irs.aperture_shape)
	setValue("bladeOffset",@irs.blade_offset)
	setValue("curvatureRadius",@irs.blade_curvature_radius)
	setValue("startAngle",@irs.start_angle)
	setValue("apertureImage", @irs.aperture_image)
	setValue("obstacleMap", @irs.obstacle_map)
	setValue("shiftUp",@irs.shift_up)
	setValue("shiftRight",@irs.shift_right)
	setValue("supersampleFactor",@irs.supersample_factor)
	setValue("sunLayer",@irs.sun_layer)
	setValue("skyLayer",@irs.sky_layer)
	setValue("cameraType",@irs.camera_type)
	#setValue("envMapLayer",@irs.env_map_layer)
	setValue("networkMode",@irs.network_mode)
	#setValue("frameUploadPeriod",@irs.frame_upload_period)
	setValue("responseFunction",@irs.cam_resp_func)
	setValue("bgWeight",@irs.bg_weight)
	setValue("sunskyWeight",@irs.sunsky_weight)
	#p @irs.bg_material
	setValue("MaterialName",@irs.bg_material)
	
	
	#Set Slider Values
	pre_scale=(@irs.pre_scale).to_f*10
	@dlg.execute_script("s.setValue(#{pre_scale.to_s})")
	post_scale=(@irs.post_scale).to_f*10
	@dlg.execute_script("s2.setValue(#{post_scale.to_s})")
	burn=(@irs.burn).to_f*10
	@dlg.execute_script("s3.setValue(#{burn.to_s})")
	iso=(@irs.iso)
	@dlg.execute_script("s5.setValue(#{iso})")
	shutterSpeed=(@irs.shutter_speed).to_f*10
	@dlg.execute_script("s6.setValue(#{shutterSpeed})")
	exposure=(@irs.ev_adjust).to_f*10
	@dlg.execute_script("s7.setValue(#{exposure})")
	turbidity=(@irs.turbidity).to_f*10
	@dlg.execute_script("s8.setValue(#{turbidity.to_s})")
	#envMapGain=(@irs.bg_luminance)
	#@dlg.execute_script("s9.setValue(#{envMapGain})")
	envMapRotation=(@irs.exr_shift)
	@dlg.execute_script("s10.setValue(#{envMapRotation})")
	threads=(@irs.num_threads)
	@dlg.execute_script("s11.setValue(#{threads})")
		
end #loadValuesFromSU

def setValue(id,value)
	new_value=value.to_s
	cmd="document.getElementById('#{id}').value=\"#{new_value}\""
	@dlg.execute_script(cmd)
end

def setCheckbox(id,checked)
	checked=checked.to_s  #can take either a true or false parameter as a string or TrueClass/FalseClass
	cmd="document.getElementById('#{id}').checked=#{checked}"
	@dlg.execute_script(cmd)
end

def setRadioButton(id,checked)
	cmd="document.getElementById('#{id}').checked=\"#{checked}\""
	@dlg.execute_script(cmd)
end

def dialog   
	return @dlg
end

def refreshCamResponseList()

cmd="$(\"#responseFunction\").html(\"\")"
@dlg.execute_script(cmd)
func_list=@irs.get_camera_response_functions
func_list=func_list.collect {|item| File.basename(item,".*")}
#p func_list

func_list.each {|func|
	cmd="$(\"<option>\").attr(\"value\",\"#{func}\").html(\"#{func}\").appendTo(\"#responseFunction\");"
	#p cmd
	@dlg.execute_script(cmd)
}

end

def refreshLayerList()
layer_ids=["sunLayer","skyLayer","envMapLayer"]
layer_ids.each {|id|
	cmd="$(\"##{id}\").html(\"\")"
	@dlg.execute_script(cmd)
	layer_names=IndigoRenderSettings.new.light_layer_names
	layer_names.each_index {|i|
		cmd="$(\"<option>\").attr(\"value\",\"#{i}\").html(\"#{layer_names[i]}\").appendTo(\"##{id}\");"
		#p cmd
		@dlg.execute_script(cmd)
	}
}

end

end #class


######################
class IndigoColorPicker

@@picker_name="SkIndigo Color Picker"

def initialize(color)

mats=Sketchup.active_model.materials
picker_mat=mats[@@picker_name]

if picker_mat
	@picker_mat=picker_mat
else
	@picker_mat=mats.add(@@picker_name)
end
if @picker_mat
	@picker_mat.color=color
	mats.current=@picker_mat
end

end

######
def color

return @picker_mat.color

end

end  #end class color picker


################NKData class#####################
class NKdata

def initialize(path)
	@path=path
end

def valid?
	
	if (@path) and (FileTest.exist?(self.expanded_path))
		return true
	else
		return false
	end
end

def valid_reference?
	path=@path.to_s
	if ((FileTest.exist?(self.expanded_path)) or (path==""))
		return true
	else
		return false
	end
end

def expanded_path

if SkIndigo.on_mac?
	root=SkIndigo.get_indigo_app_data_path
else
	root=File.dirname(SkIndigo.get_indigo_path) 
end
path=@path.to_s
return File.join(root,"nkdata",File.basename(path))

end

def to_s
	return File.join("nkdata",File.basename(path)).gsub(/\\/,"\/")
	#return @path.gsub(/\\/,"\/")
end

def path
	return @path
end

def path=(new_path)
	@path=new_path
end

def filename
	File.basename(@path)
end

def name
	File.basename(@path,".*")
end

def export(export_path)
	if (not valid?)
		UI.messagebox "NKdata path \"#{@path}\" is not valid."
		return
	else
		if FileTest.exist?(export_path)
			export_path=File.join(export_path,filename)
			nkdata=File.new((@path),"r")
			file_contents=nkdata.read
			nkdata.close
			export_file=File.new((export_path),"w")
			export_file.write(file_contents)
			export_file.close
		else
			UI.messagebox "Unable to write nkdata file to path #{export_path}"
		end
	end
end


end #class NKdata

################IESProfile class#####################
class IESProfile < NKdata


def expanded_path

return @path

end

def to_s

return @path.gsub(/\\/,"\/")

end

def export(export_path)
	if (not valid?)
		UI.messagebox "IES path is not valid."
		return
	else
		if FileTest.exist?(export_path)
			export_path=File.join(export_path,filename)
			ies_file=File.new((@path),"r")
			file_contents=ies_file.read
			ies_file.close
			export_file=File.new((export_path),"w")
			export_file.write(file_contents)
			export_file.close
		else
			UI.messagebox "Unable to write IES file to path #{export_path}"
		end
	end
end


end #class IESProfile

###################Indigo Texture Class################
##this class used to only represent textures, now it has been expanded to support more parameters including other spectrum types
class IndigoTexture

include DMcolormath

@@preview_uvset=0  #this is the uvset index that will be exported for material previews

attr_reader :uv_set, :a, :b, :c, :exponent, :u_scale, :v_scale, :shader_code, :type, :path, :spectrum_type, :bb_temperature, :bb_gain, :rgb_r, :rgb_g, :rgb_b, :rgb_gain, :rgb_gamma, :reg_start, :reg_end, :reg_num_values, :reg_values, :uni_value, :smooth
attr_writer :uv_set, :a, :b, :c, :exponent, :u_scale, :v_scale, :shader_code, :type, :path, :spectrum_type, :bb_temperature, :bb_gain, :rgb_r, :rgb_g, :rgb_b, :rgb_gain, :rgb_gamma, :reg_start, :reg_end, :reg_num_values, :reg_values, :uni_value, :smooth

def initialize(attribute_string="Constant")    #The format of the attribute string must be "type|uv_set|a|b|c|exponent|shader_code|texture_path|spectrum_type|bb_temperature|bb_gain|rgb_r|rgb_g|rgb_b|rgb_multi|rgb_gamma|reg_start|reg_end|reg_num_values|reg_values|uniform_value"
	attributes=attribute_string.split("|")
	@type=attributes[0]
	case @type
	when "SketchUp","External","Shader"
		#do nothing
	when "None"
		@type="Constant"
	when "SU"
		@type="SketchUp"  #backward compat.
	else
		@type="Constant"  #same as constant
	end
	@uv_set=attributes[1] ? attributes[1]:"Default"
	@uv_set="Default" if @uv_set=="0"  #for backwards compatibility
	@a=attributes[2] ? attributes[2]:0.0
	@b=attributes[3] ? attributes[3]:1.0
	@c=attributes[4] ? attributes[4]:0.0
	@exponent=attributes[5] ? attributes[5]:1.0
	@shader_code=attributes[6] ? attributes[6]:"Enter Shader Code"
	@path=attributes[7] ? attributes[7]:""
	@spectrum_type=attributes[8] ? attributes[8]:"RGB"
	@bb_temperature=attributes[9] ? attributes[9]:5000
	@bb_gain=attributes[10] ? attributes[10]:1.0
	@rgb_r=attributes[11] ? attributes[11]:0.0
	@rgb_g=attributes[12] ? attributes[12]:0.0
	@rgb_b=attributes[13] ? attributes[13]:0.0
	@rgb_gain=attributes[14] ? attributes[14]:1.0
	@rgb_gamma=attributes[15] ? attributes[15]:2.2
	@reg_start=attributes[16] ? attributes[16]:385  #in nm
	@reg_end=attributes[17] ? attributes[17]:775  #in nm
	@reg_num_values=attributes[18] ? attributes[18]:10
	@reg_values=attributes[19] ? attributes[19]:"0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0"
	@uni_value=attributes[20] ? attributes[20]:0.0
	@smooth=attributes[21] ? attributes[21]:"Off"
	
end

def invert

b=@b.to_f
c=@c.to_f
@c=@b.to_f+c.to_f
@b=-1.0*b.to_f

return self

end
	
def procedural?

	case type
		when "Fractal","Shader","Noise"
			return true
		else
			return false
	end
	
end

def shader_code
	case type
		when "Fractal"
			return "def eval(vec3 pos) vec3:vec3(add(mul(fbm(mul(getTexCoords(0),5.0),10),1.0),0.0))"
		when "Noise"
			return "def eval(vec3 pos) vec3:vec3(add(mul(noise(mul(getTexCoords(0),5.0)),1.0),0.0))"
		else
			return @shader_code
	end

end


def preview_uvset
	@@preview_uvset
end

def filename
	return 	File.basename(@path)
end

def export_filename
	illegal="\\\/:*?\"<>|"   #characters not allowed in filenames
	return filename
	#return @su_mat.name.delete(illegal)+".jpg"
end

def valid?
	#p @type
	case @type
		when "SketchUp"
			return true
		when "External"
			path=@path.to_s
			if (FileTest.exist?(path))
				return true
			else
				return false
			end
		when "Shader"
			return true
		when "Constant"
			return true
	end
	
end

def valid_reference?

case @type
		when "SketchUp"
			return true
		when "External"
			path=@path.to_s
			if ((FileTest.exist?(path)) or (path==""))
				return true
			else
				return false
			end
		when "Shader"
			return true
		when "Constant"
			return true
	end

end


def uv_set_index()

return 0 if @uv_set=="Default"
return @uv_set.to_i

end

def to_s
	
[type, uv_set, a, b, c, exponent, shader_code, path, spectrum_type, bb_temperature, bb_gain, rgb_r, rgb_g, rgb_b, rgb_gain, rgb_gamma, reg_start, reg_end, reg_num_values, reg_values,uni_value,smooth].join("|")
		
end

def smooth?

if smooth=="On"
	return true
else
	return false
end

end
######Defaults
def default_sss_map
@type="Constant"
@spectrum_type="Uniform"
@uni_value=0.0
@rgb_gamma=1.0
return self
end
###
def default_g_spectrum
@type="Constant"
@spectrum_type="Uniform"
@uni_value=1.0
@rgb_gamma=1.0
return self
end
####
def default_base_emission
@type="Constant"
@spectrum_type="Blackbody"
@bb_temperature=5000
@bb_gain=1.0
return self
end

def default_bump
@type="External"
@shader_code=self.default_wi_shader
@b=0.01
@exponent=2.2
return self
end

def default_normal
@type="External"
@shader_code=self.default_wd_shader
@b=1.0
@exponent=1.0
return self
end

def default_albedo
@type="SketchUp"
@b=1.0
@exponent=2.2
@shader_code=self.default_wd_shader
return self
end

def default_exponent
@type="Constant"
@b=1000
@exponent=2.2
@shader_code=self.default_wi_shader
return self
end

def default_clip
@type="External"
@b=1.0
@exponent=2.2
@shader_code=self.default_wi_shader
return self
end

def default_displacement
@type="External"
@b=0.1
@exponent=2.2
@shader_code=self.default_wi_shader
return self
end

def default_emission
@type="External"
@b=1.0
@exponent=2.2
@shader_code=self.default_wd_shader
return self
end

def default_transmittance
@type="Constant"
@spectrum_type="Uniform"
@uni_value=1.0
@rgb_gamma=1.0
@shader_code=self.default_wd_shader
return self
end

def default_sigma
@type="Constant"
@b=0.0
@exponent=2.2
@shader_code=self.default_wi_shader
return self
end

def default_r_f
@type="Constant"
@b=0.5
@exponent=2.2
@shader_code=self.default_wi_shader
return self
end

def default_roughness
type="Constant"
@b=0.3
@exponent=2.2
@shader_code=self.default_wi_shader
return self
end

def default_fresnel
type="Constant"
@b=1.0
@exponent=2.2
@shader_code=self.default_wi_shader
return self
end

def default_thickness
type="Constant"
@b=0.001
@exponent=2.2
@shader_code=self.default_wi_shader
return self
end

def default_specular
@type="SketchUp"
@b=1.0
@exponent=2.2
@shader_code=self.default_wd_shader
return self
end

def default_abs_layer_trans
@type="External"
@b=1.0
@exponent=2.2
@shader_code=self.default_wd_shader
return self
end

def default_absorption
@type="SketchUp"
@shader_code=self.default_wd_shader
@rgb_gamma=1.0
return self
end

def default_blend
@type="SketchUp"
@b=1.0
@exponent=2.2
@shader_code=self.default_wi_shader
return self
end

####
def default_wi_shader  #default wavelength-indepdent shader

return "def eval() real : fbm(posOS()*4.0, 2)*0.02
	#This is a fractal noise shader.
	#4.0=Position scale
	#2=fbm octave
	#0.02=multiplier (eg. height for displacement or bump)"
end

def default_wd_shader
return "def eval(vec3 pos) vec3:vec3(0.4, 0.4, 0.4) # Return a grey colour"
end
###returns the color of the spectrum for setting a swatch color
def color

case @spectrum_type
when "RGB"
	red=@rgb_r.to_f.abs
	green=@rgb_g.to_f.abs
	blue=@rgb_b.to_f.abs
	color_vec=[red,green,blue]# .normalize!
	color_vec=color_vec.collect {|e| (e*255).to_i}
	return color_vec
when "Tabulated"
	return self.tabulated_to_rgb(@reg_values.split(" "),@reg_start.to_i,@reg_end.to_i)
when "Blackbody"
	color=self.blackbody_to_rgb(@bb_temperature.to_i)
	return color.collect {|c| (c*255).to_i}
when "Uniform"
	color=[]
	3.times {|i| color[i]=(@uni_value.to_f*255).to_i.abs}
	return color
end

end


end #end of class IndigoTexture


module SkIndigo

module Dialog

#this function clears web-dialog dropdown list by calling a javascript function that must be defined in the webdialog
def clear_dropdown(dlg,id)

cmd="clearDropDown('#{id}')"
dlg.execute_script(cmd)

end

#this function creates an option to be added to a dropdown list by calling a javascript function that must be defined inside the webdialog
def create_option(dlg,id,name,value)

cmd="createMapOption('#{id}','#{name}','#{value}')"
dlg.execute_script(cmd)

end

end #module

end #module
#######################################SkIndigo Material Editor############################3

class SkindigoMatObserver <  Sketchup::MaterialsObserver

include SkIndigoUVS
include SkIndigo::Dialog

def initialize
	@model=Sketchup.active_model
	@default_ui=Sketchup.read_default("SkIndigoDefaults",'ui',"normal")
	@dict="indigo07_settings"
	@ui="normal" # Force normal material UI mode.
	@on_mac=SkIndigo.on_mac?
	@preview_process_id=nil
	preview_folder="su_preview_scenes"
	@custom_preview_scene_path=File.join(SkIndigo.get_indigo_app_data_path,preview_folder)
	Dir.mkdir(@custom_preview_scene_path) unless FileTest.exist?(@custom_preview_scene_path)
	@standard_preview_scene_path=File.join(File.dirname(__FILE__),"SUpreviews")
	@start=nil
	@preset_dlg=nil
	@mdb_dlg=nil
	mats=Sketchup.active_model.materials
	#initialize the currently selected material
	if mats.length>0
		current_mat=mats.current
		if current_mat==nil
			mats.current=mats[0]  #select the first material
		end
		@sm=IndigoMaterial.new(mats.current)
	else  #no materials found so create one
		new_mat=mats.add("Material1")
		new_mat.color=[125,125,125]
		mats.current=new_mat
		@sm=IndigoMaterial.new(new_mat)
	end
	#@selection_stack=[Sketchup.active_model.materials.current]
	@selection_stack=[]
	@mat_editor=UI::WebDialog.new("SkIndigo Material Editor", false, "SkIndigoMaterialEditor",307, 750, 1,1,true)
	plugins=Sketchup.find_support_file("Plugins\/skindigo")
	fn=File.join(plugins,"MaterialEditor.html")
	@mat_editor.set_file fn
	color = @mat_editor.get_default_dialog_color 
	@mat_editor.set_background_color(color)
	@mat_editor.set_on_close {@preset_dlg.close if (@preset_dlg) and (@preset_dlg.visible?)}
	#############Primary Event Handler Callback Function for Material Editor##################

	@mat_editor.add_action_callback("ValueChanged") {|d,p|
		pair=p.split("=")
		id=pair[0]           
		value=pair[1]
		#p "#{id} changed to #{value}"
		#check if the selected material is still valid (not deleted)
		unless sm.mat.valid?
			UI.messagebox("Material not found.")
			#select the first material
			self.sm=IndigoMaterial.new(Sketchup.active_model.materials[0])
			refresh_dialog()
			return
		end
		
		type=sm.type
		case id
			when "AssignPresetButton"
				
				if @preset_dlg
					@preset_dlg.show{} unless @preset_dlg.visible?
					@preset_dlg.bring_to_front
					return
				end
					
				#Create the Preset Selection Dialog
				@preset_dlg=UI::WebDialog.new("Select Material Preset", 
					true, # scrollable
					"SkIndigoPresets", 300, 768, 272, 1, true)
				fn=File.join(File.dirname(__FILE__),"presets.html")
				@preset_dlg.set_file fn
				color = @preset_dlg.get_default_dialog_color 
				@preset_dlg.set_background_color(color)
				@preset_dlg.add_action_callback("assign_preset") {|d,p|
					params=p.split("=")
					preset=params[0]
					use_thumb=params[1]
					if use_thumb=="true"
						new_thumbnail=File.join(File.dirname(__FILE__),preset+".jpg")
						sm.thumbnail=new_thumbnail
					end
					case preset
   						when "glass"
							apply_glass(sm)
						when "frostedGlass"
							apply_frosted_glass(sm)
						when "mirror"
							apply_mirror(sm)
						when "ceramic"
							apply_ceramic(sm)
						when "dullPlastic"
							apply_dull_plastic(sm)
						when "shinyPlastic"
							apply_shiny_plastic(sm)
						when "leather"
							apply_leather(sm)
						when "carPaint"
							apply_car_paint(sm)
						when "lampshade"
							apply_lampshade(sm)
						when "gold"
							apply_nkdata(sm,"Au.nk")
						when "tintedGlass"
							apply_tinted_glass(sm)
						when "water"
							apply_water(sm)
						when "lambertian"
							apply_lambertian(sm)
						when "clay"
							apply_clay(sm)
						when "white_wax"
							apply_wax(sm)
						when "orangejuice"
							apply_oj(sm)
						when "incan100w"
							apply_100W(sm)
						when "fluor34wcool"
							apply_34WTFC(sm)
						when "fluor34wmid"
							apply_34WTFM(sm)
						when "fluor34wwarm"
							apply_34WTFW(sm)
					end
					loadValuesFromSU()   #updates the material editor with new material values
					setVisibility()
						
				}
					
				@preset_dlg.show{}
					
			when "SearchMDB"
				
				if @mdb_dlg
					fn=File.join(File.dirname(__FILE__),"mdbsearch.html")
					@mdb_dlg.set_file fn
					@mdb_dlg.show{} unless @mdb_dlg.visible?
					@mdb_dlg.bring_to_front
					return
				end
					
				#Create the Material DB search dialog
				@mdb_dlg=UI::WebDialog.new("Search Indigo Material Database", false,"SkIndigoMaterialSearch", 530, 768, 272, 1, true)
				fn=File.join(File.dirname(__FILE__),"mdbsearch.html")
				@mdb_dlg.set_file fn
				color = @mdb_dlg.get_default_dialog_color 
				@mdb_dlg.set_background_color(color)
				@mdb_dlg.add_action_callback("download") {|d,p|
					result=UI.messagebox("Download and load into selected material?",MB_YESNO,"Download material")
					if result==6
						begin
							igm=download_igm(p)
							#p "download completed"
							status=nil
							if igm!=nil
								f_name=File.basename(igm)
								if f_name.split(".")[-1]=="pigm"  #loading a PIGM file
									igm=SkIndigo.unpack_pigm(igm)
									return if igm==nil
									Dir.chdir(File.dirname(igm))  #set the working directory to the unpacked igm file
								end
								imi=IndigoMaterialImporter.new(igm)
								status=imi.load_igm(igm,sm)
								if status==0
									UI.messagebox("Material successfully loaded.")
								elsif status==1
									sm.type="Linked IGM"
									self.create_linked_igm(igm)
								end
								refresh_dialog()
							end
							
						rescue
							UI.messagebox("Material could not be loaded.")
							raise
						end
					elsif result==7
						igm_path=UI.savepanel("Save Material","",File.basename(p))
						if igm_path!=nil
							igm=download_igm(p,igm_path)
							if FileTest.exist?(igm)
								UI.messagebox("Material saved to #{igm}.")
							else
								UI.messagebox("Material could not be saved to #{igm}.  Plese check read/write permissions for this folder.")
							end
						end
					
					end #if result
						
					#UI.openURL(p)
				}
				@mdb_dlg.add_action_callback("search_mdb") {|d,p|
					@mdb_dlg.execute_script("searchMDB();")
				}
				@mdb_dlg.show{}
			when "SwitchUI"
				if @ui=="normal" 				 
					@ui=@model.set_attribute(@dict,'ui',"simple")
					@mat_editor.set_size(300,600)
				else	
					@ui=@model.set_attribute(@dict,'ui',"normal")
					@mat_editor.set_size(300,768)
  				end
				
				setVisibility()
				loadValuesFromSU()
			when "sampleMaterial"
				Sketchup.active_model.select_tool SkIndigoMaterialPicker.new
			when "previewScene"
				if value=="New"
					status=create_preview_scene()
				end
				if status
					refresh_dialog()
				end
			when "PreviewButton"
				#if SkIndigo.on_mac?
					#UI.messagebox("Material Previews are not yet available on Mac systems.")
					#return
				#end
							
				if ((type=="Blackbody Light") or (type=="RGB Light") or (type=="Exit Portal"))
					UI.messagebox "#{type} materials are currently not supported by this feature."
				else
					path=SkIndigo.get_indigo_path
					return if not path
					thumbnail_path=IndigoRenderSettings.new().thumbnail_path
					if not FileTest.exist?(thumbnail_path)
						UI.messagebox "Thumbnail path not found. Preview aborted."
						return
					end
					illegal="\\\/:*?\"<>|" 
					mat_name=sm.mat.name.delete(illegal)		
					new_thumbnail=File.join(thumbnail_path,mat_name+".png")
					File.delete(new_thumbnail) if FileTest.exist?(new_thumbnail)
					sm.thumbnail=new_thumbnail
					if @preview_process_id
					#we are currently rendering a preview so stop it before starting a new one
						stop_preview()
					end
					@preview_process_id=preview_material(sm,value,new_thumbnail)
					if @preview_process_id
						toggle_stop_button(d,true)
						if @timer
							UI.stop_timer(@timer)
						end
						@timer=UI.start_timer(1, true) do
							getThumbnail()
						end # Update material preview every 1 second.
					end						 
				end
			when "StopPreview"
				self.stop_preview()
			when "SaveIGMButton"
				if ((type=="Blackbody Light") or (type=="RGB Light") or (type=="Exit Portal") or (type=="Linked IGM"))
					UI.messagebox "#{type} materials are currently not supported by this feature."
			
				else
					igm=UI.savepanel("Export IGM/PIGM", "",((sm.name.delete("<>"))+".igm"))
					return if igm==nil
					file_name=File.basename(igm,".*")
					igm=File.join(File.dirname(igm),file_name+".igm")
					result=UI.messagebox("Would you like to pack this IGM? (Recommended for materials with textures)",MB_YESNO)
					if result==6
						pack=true
					else
						pack=false
					end		
					IndigoExporter.new(igm).export_igm(sm,pack)
				end
			when "LoadIGMButton"
				
					igm=UI.openpanel "Import IGM/PIGM", "", ""
					return if igm==nil
					status=nil
					if (FileTest.exist?(igm))
						f_name=File.basename(igm)
						if f_name.split(".")[-1]=="pigm"  #loading a PIGM file
							#if SkIndigo.on_mac?
								#UI.messagebox("PIGM Loading is not yet available on Mac systems.  You must first unpack/unzip the PIGM using another application (eg. Stuffit).  Then, load the IGM file.")
								#return
							#end
							igm=SkIndigo.unpack_pigm(igm)
							return if igm==nil
							Dir.chdir(File.dirname(igm))  #set the working directory to the unpacked igm file
						end
						#result=load_igm(igm,sm)
						#TODO error reporting
						imi=IndigoMaterialImporter.new(igm)
						status=imi.load_igm(igm,sm)
						if status==0
							UI.messagebox("Material succesfully loaded.")
						elsif status==1
							sm.type="Linked IGM"
							self.create_linked_igm(igm)
						end			
					end
					refresh_dialog()
				
			when "editIGMButton"
				if SkIndigo.on_mac?
					UI.messagebox("This feature is not yet available on Mac systems.")
				else
					igm=d.get_element_value("igm")
					if FileTest.exist?(igm)
						UI.openURL("file:///"+File.expand_path(igm))
					else
						UI.messagebox("IGM path not valid.")
					end
				end
			when "MaterialName"
				new_selected_mat=IndigoMaterial.find(value)
				if new_selected_mat
					self.sm=new_selected_mat
					Sketchup.active_model.materials.current=self.sm.mat  #set the current material in the Sketchup Materials Window
					refresh_dialog()
					@mat_editor.bring_to_front #ensure the focus stays on the material editor and does not switch to the Sketchup Materials window
				else
					UI.messagebox("Unable to find material named #{value}")
				end
			when "type"
				sm.type=value
				sm.alpha=0.2 if value=="Thin Glass"
				if value=="Specular" or value=="Glossy Transparent"
					if sm.alpha==1.0 and sm.absorption_map.type=="SketchUp"
						UI.messagebox("Warning: SketchUp opacity is currently set to 100%.  If you wish to create a transparent material, lower the opacity setting to near zero using the SketchUp material editor.")
					end
				end
				setVisibility()
			when "layer"
				sm.layer=value
			when "ior"
				sm.ior=value
			when "cau"
				sm.cauchy_b_coeff=value
			when "exp"
				sm.exponent=value
			when "precedence"
				sm.precedence=value
			when "temp"
				sm.temperature=value
			when "power"
				sm.power_drawn=value
			when "ole"
				sm.overall_luminous_efficacy=value
			when "g_spectrum"
				sm.g_spectrum=value
			when "sssval"
				sm.sss_spectrum=value
			when "fres"
				sm.fresnel_scale=value
			when "sigma"
				sm.sigma=value
			when "subSmoothingCheckbox"
				sm.smooth=true if value=="true"
				sm.smooth=false if value=="false"
			when "viewDependentCheckbox"
				sm.view_dependent_subdivision=true if value=="true"
				sm.view_dependent_subdivision=false if value=="false"
			when "EnableIESCheckbox"
				sm.ies=true if value=="true"
				sm.ies=false if value=="false"
			when "SSSCheckbox"
				sm.sss=true if value=="true"
				sm.sss=false if value=="false"
			when "bumpCheckbox"
				sm.use_bump=true if value=="true"
				sm.use_bump=false if value=="false"
			when "normalCheckbox"
				sm.use_normal=true if value=="true"
				sm.use_normal=false if value=="false"
			when "displaceCheckbox"
				sm.use_displace=true if value=="true"
				sm.use_displace=false if value=="false"
			when "emissionCheckbox"
				sm.use_emission=true if value=="true"
				sm.use_emission=false if value=="false"
			when "absLayerTransCheckbox"
				sm.use_abs_layer_trans=true if value=="true"
				sm.use_abs_layer_trans=false if value=="false"
			when "clipCheckbox"
				sm.use_clip=true if value=="true"
				sm.use_clip=false if value=="false"
			when "stepBlend"
				sm.step_blend=true if value=="true"
				sm.step_blend=false if value=="false"
			when "interferenceCheckbox"
				sm.interference=true if value=="true"
				sm.interference=false if value=="false"
			when "specularCheckbox"
				if value=="true"
					sm.use_specular=true
					sm.use_nkdata=false
					setCheckbox("EnableNKdataCheckbox","false")
				else 
					sm.use_specular=false
				 end				
			when "EnableNKdataCheckbox"
				if value=="true"
					sm.use_nkdata=true
					sm.use_specular=false
					setCheckbox("specularCheckbox","false")
				else
					sm.use_nkdata=false
				end
			when "Phase_Function"
				sm.phase_function=value
				if value=="Uniform"
					setVisible("g_spectrum_param",false)
				else
					setVisible("g_spectrum_param",true)
				end
			#when "UniformPF"
			#	sm.phase_function="Uniform"
			#when "Henyey-GreensteinPF"
			#	sm.phase_function="Henyey-Greenstein"
			when "bbES" #blackbody emission spectrum
				sm.base_emission_spectrum="Blackbody"
			when "rgbES" #rgb emission specrum
				sm.base_emission_spectrum="RGB"
				
			when "albedo_map"
				tex=sm.albedo_texture
				tex.type=value
				sm.albedo_texture=tex
				setColorBox("albedoSwatch",sm.color("albedo"))
			when "specular_map"
				tex=sm.specular_map
				tex.type=value
				sm.specular_map=tex
				setColorBox("specularSwatch",sm.color("specular"))
			when "blend_map"
				tex=sm.blend_map
				tex.type=value
				sm.blend_map=tex
				setValueBoxVisible(tex,"blendfactor")
			when "bump_map"
				tex=sm.bump_map   #get the current bump map
				tex.type=value        #assign the SU material name to the texture object
				sm.bump_map=tex   #assign the updated texture object back to the selected material
				setValueBoxVisible(tex,"bump_value")
			when "normal_map"
				tex=sm.normal_map   
				tex.type=value        
				sm.normal_map=tex   
				setValueBoxVisible(tex,"normalmap_value")
			when "exp_map"
				tex=sm.exponent_map   
				tex.type=value       
				sm.exponent_map=tex
				setValueBoxVisible(tex,"expmap_value")
			when "clip_map"
				tex=sm.clip_map   
				tex.type=value       
				sm.clip_map=tex
				setValueBoxVisible(tex,"clipValue")
			when "disp_map"
				tex=sm.disp_map
				tex.type=value
				sm.disp_map=tex
				setValueBoxVisible(tex,"dispmap_value")
			when "emission_map"
				tex=sm.emission_map
				tex.type=value
				sm.emission_map=tex
				setColorBox("emissionSwatch",sm.color("emission"))
			when "transmittance_map"
				tex=sm.transmittance_map
				tex.type=value
				sm.transmittance_map=tex
				setColorBox("transmittanceSwatch",sm.color("transmittance"))
			when "abs_layer_trans_map"
				tex=sm.abs_layer_trans_map
				tex.type=value
				sm.abs_layer_trans_map=tex
				setColorBox("abslayertransSwatch",sm.color("absorption_layer_transmittance"))
			when "sigma_map"
				tex=sm.sigma_map
				tex.type=value
				sm.sigma_map=tex
				setValueBoxVisible(tex,"sigma_value")
			when "roughness_map"
				tex=sm.roughness_map
				tex.type=value
				sm.roughness_map=tex
				setValueBoxVisible(tex,"roughness_value")
			when "fresnel_map"
				tex=sm.fresnel_map
				tex.type=value
				sm.fresnel_map=tex
				setValueBoxVisible(tex,"fresnel_value")
			when "thickness_map"
				tex=sm.thickness_map
				tex.type=value
				sm.thickness_map=tex
				setValueBoxVisible(tex,"thickness_value")
			when "front_roughness_map"
				tex=sm.front_roughness_map
				tex.type=value
				sm.front_roughness_map=tex
				setValueBoxVisible(tex,"front_roughness_value")
			when "back_roughness_map"
				tex=sm.back_roughness_map
				tex.type=value
				sm.back_roughness_map=tex
				setValueBoxVisible(tex,"back_roughness_value")
			when "front_fresnel_map"
				tex=sm.front_fresnel_scale_map
				tex.type=value
				sm.front_fresnel_scale_map=tex
				setValueBoxVisible(tex,"front_fresnel_value")
			when "back_fresnel_map"
				tex=sm.back_fresnel_scale_map
				tex.type=value
				sm.back_fresnel_scale_map=tex
				setValueBoxVisible(tex,"back_fresnel_value")
			when "r_f_map"
				tex=sm.r_f_map
				tex.type=value
				sm.r_f_map=tex
				setValueBoxVisible(tex,"r_f__value")
			when "abs_map"
				tex=sm.absorption_map
				tex.type=value
				sm.absorption_map=tex
				setColorBox("absorption",sm.color("absorption"))
			when "sss_map"
				tex=sm.sss_map
				tex.type=value
				sm.sss_map=tex
				setColorBox("sssSwatch",sm.color("sss"))
			when "bump_value"
				tex=sm.bump_map   
				tex.b=value       
				sm.bump_map=tex
			when "normalmap_value"
				tex=sm.normal_map   
				tex.b=value       
				sm.normal_map=tex
			when "expmap_value"
				tex=sm.exponent_map   
				tex.b=value       
				sm.exponent_map=tex
			when "dispmap_value"
				tex=sm.disp_map
				tex.b=value
				sm.disp_map=tex
			when "emission_value"
				tex=sm.emission_map
				tex.b=value
				sm.emission_map=tex
			when "abs_layer_trans_value"
				tex=sm.abs_layer_trans_map
				tex.b=value
				sm.abs_layer_trans_map=tex
			when "sigma_value"
				tex=sm.sigma_map
				tex.b=value
				sm.sigma_map=tex
			when "roughness_value"
				tex=sm.roughness_map
				tex.b=value
				sm.roughness_map=tex
			when "front_roughness_value"
				tex=sm.front_roughness_map
				tex.b=value
				sm.front_roughness_map=tex
			when "back_roughness_value"
				tex=sm.back_roughness_map
				tex.b=value
				sm.back_roughness_map=tex
			when "front_fresnel_value"
				tex=sm.front_fresnel_scale_map
				tex.b=value
				sm.front_fresnel_scale_map=tex
			when "back_fresnel_value"
				tex=sm.back_fresnel_scale_map
				tex.b=value
				sm.back_fresnel_scale_map=tex
			when "r_f_value"
				tex=sm.r_f_map
				tex.b=value
				sm.r_f_map=tex
			when "fresnel_value"
				tex=sm.fresnel_map
				tex.b=value
				sm.fresnel_map=tex
			when "thickness_value"
				tex=sm.thickness_map
				tex.b=value.to_f/1000000.0
				p tex.b
				sm.thickness_map=tex
			when "clipValue"
				tex=sm.clip_map
				tex.b=value
				sm.clip_map=tex
			when "advancedAlbedoMap","advancedEmissionMap","advancedAbsLayerTransMap","advancedSpecularMap","advancedAbsMap","advancedSSSMap","advancedG_spectrumMap","advancedBaseEmissionMap","advancedTransmittanceMap"
				params=[sm.albedo_texture,"albedo",0.0,1.0] if id=="advancedAlbedoMap"
				params=[sm.emission_map,"emission",0.0,1.0] if id=="advancedEmissionMap"
				params=[sm.abs_layer_trans_map,"abs_layer_trans",0.0,1.0] if id=="advancedAbsLayerTransMap"
				params=[sm.specular_map,"specular",0.0,1.0] if id=="advancedSpecularMap"
				params=[sm.absorption_map,"absorption",0.0,nil] if id=="advancedAbsMap"
				params=[sm.sss_map,"sss",0.0,nil] if id=="advancedSSSMap"
				params=[sm.g_spectrum_map,"g_spectrum",-1.0,1.0] if id=="advancedG_spectrumMap"
				params=[sm.base_emission,"base_emission",0.0,nil] if id=="advancedBaseEmissionMap"
				params=[sm.transmittance_map,"transmittance",0.0,1.0] if id=="advancedTransmittanceMap"
				case params[0].type
				when "Constant"
				ite=IndigoSpectrumPicker.new(params[0],sm,params[2],params[3],params[1])
				when "External","SketchUp"
					if params[0].type=="SketchUp" and params[1]=="absorption"
						UI.messagebox("Use the SketchUp color and opacity settings to modify the absorption.")
					else
						ite=IndigoTextureEditor.new(params[0],sm,params[1])
					end
				when "Shader"
				ite=IndigoShaderEditor.new(params[0],sm,params[1])
				end
				
			when "advancedBumpMap","advancedNormalMap","advancedDispMap","advancedExponentMap","advancedClipMap","advancedBlendMap","advancedSigmaMap","advancedRoughnessMap","advancedFresnelMap","advancedThicknessMap","advancedr_fMap","advancedFrontFresnelMap","advancedBackFresnelMap","advancedFrontRoughnessMap","advancedBackRoughnessMap"
				params=[sm.bump_map,"bump"] if id=="advancedBumpMap"
				params=[sm.disp_map,"displacement"] if id=="advancedDispMap"
				params=[sm.exponent_map,"exponent"] if id=="advancedExponentMap"
				params=[sm.clip_map,"clip"] if id=="advancedClipMap"
				params=[sm.blend_map,"blend"] if id=="advancedBlendMap"
				params=[sm.sigma_map,"sigma"] if id=="advancedSigmaMap"
				params=[sm.roughness_map,"roughness"] if id=="advancedRoughnessMap"
				params=[sm.fresnel_map,"fresnel"] if id=="advancedFresnelMap"
				params=[sm.thickness_map,"thickness"] if id=="advancedThicknessMap"
				params=[sm.normal_map,"normal"] if id=="advancedNormalMap"
				params=[sm.r_f_map,"r_f"] if id=="advancedr_fMap"
				params=[sm.front_fresnel_scale_map,"front fresnel"] if id=="advancedFrontFresnelMap"
				params=[sm.back_fresnel_scale_map,"back fresnel"] if id=="advancedBackFresnelMap"
				params=[sm.front_roughness_map,"front roughness"] if id=="advancedFrontRoughnessMap"
				params=[sm.back_roughness_map,"back roughness"] if id=="advancedBackRoughnessMap"
				case params[0].type
				when "Constant" #constant
					UI.messagebox("There are no additional parameters for a 'constant' #{params[1]} attribute.")
				when "External","SketchUp"
					ite=IndigoTextureEditor.new(params[0],sm,params[1])
				when "Shader"
					ite=IndigoShaderEditor.new(params[0],sm,params[1])
				end
			when "matA"
				sm.a_name=value
			when "matB"
				sm.b_name=value
			when "substrate"
				sm.substrate=value
			when "front_material_name"
				sm.set(id,value)
			when "back_material_name"
				sm.set(id,value)
			when "blendfactor"
				#sm.blend_factor=value
				tex=sm.blend_map
				tex.b=value
				sm.blend_map=tex
			when "nkdata"
				p value
				sm.nk_data=NKdata.new(value)
			when "ies_path"
				sm.ies_profile=IESProfile.new(value)
			when "metalsList"
				#root=File.dirname(SkIndigo.get_indigo_path) unless @on_mac
				#root=SkIndigo.get_indigo_app_data_path if @on_mac
				
				#nkdata=File.join(root,"nkdata",value+".nk")
				nkdata=File.join("nkdata",value+".nk")
				nk=NKdata.new(nkdata)
				sm.nk_data=nk
				sm.use_nkdata=true
				sm.use_specular=false
				setCheckbox("specularCheckbox","false")
				setCheckbox("EnableNKdataCheckbox","true")
				setValue("nkdata",nk.to_s)
			when "nkdataBrowse"
				UI.messagebox("All NKdata files must be located in the Indigo Renderer\/nkdata folder.")
				path=UI.openpanel "Load NKdata", "", "*.nk"
				if path
					path=File.join("nkdata",File.basename(path))
					nkdata=NKdata.new(path)
					sm.nk_data=nkdata
					setValue("nkdata",nkdata.to_s)
				end
			when "iesBrowse"
				path=UI.openpanel "Load IES Profile", "", "*.ies"
				if path
					ies=IESProfile.new(path)
					sm.ies_profile=ies
					setValue("ies_path",ies)
				end
			when "igmBrowseButton"
				self.create_linked_igm(nil)
			when "slider-1"  #reflection slider
				if (type!="RGB Light") and (type!="Blackbody Light")
					type=getTypeFromSliders()
					sm.type=type
				end
				if value=="0"
					ior="1.0"
				elsif value=="100"
					ior="1000"
				else
					ior=to_ior(value)
				end
				sm.ior=ior
			when "slider-2" #roughness slider
				#p "roughness modified"
				type=sm.type
				if (type!="RGB Light") and (type!="Blackbody Light")
					type=getTypeFromSliders()
					sm.type=type
				end
				if ((type=="Phong") or (type=="Glossy Transparent"))
					exp=to_exp(value)
					sm.exponent=exp
				end
			when "slider-3" #power slider
				power=to_power(value)
				#sm.power_drawn=power
				sm.emission_scale=power.to_i*100
			when "emitterCheckbox"
				if value=="true"
					sm.layer="0"  #set the layer to 0 to make the material an emitter
					be=sm.base_emission
					be.spectrum_type="Blackbody"
					be.bb_temperature=6500
					sm.base_emission=be
					em=sm.emission_map
					em.type="SketchUp"
					sm.use_emission=true
					sm.emission_map=em
					#setCheckbox("transparentCheckbox","false")
				else
					sm.layer="-1"
					sm.use_emission=false
					type=getTypeFromSliders()
					sm.type=type
				end
			when "transparentCheckbox"
				rough=d.get_element_value("roughness-value")
				if value=="true"
					if rough=="0"
						sm.type="Specular"
					else
						sm.type="Glossy Transparent"
					end
					#setCheckbox("emitterCheckbox","false")
				else
					if rough=="100"
						sm.type="Diffuse"
					else
						sm.type="Phong"
					end
				end
			when "emissionScale"
				sm.emission_scale=value
			when "emissionMeasure"
				sm.emission_measure=value
			when "maxSubdiv"
				sm.max_subdiv=value
			when "pixelThreshold"
				sm.pixel_threshold=value
			when "curveThreshold"
				sm.curvature_threshold=value
			when "matAButton"
				mata=d.get_element_value("matA")
				if mata!="None"
					@selection_stack.push(self.sm)
					self.sm=IndigoMaterial.find(mata)
					Sketchup.active_model.materials.current=sm.mat
					loadValuesFromSU()
					setVisibility()
				end
			when "matBButton"
				matb=d.get_element_value("matB")
				if matb!="None"
					@selection_stack.push(self.sm)
					self.sm=IndigoMaterial.find(matb)
					Sketchup.active_model.materials.current=sm.mat
					loadValuesFromSU()
					setVisibility()
				end
			when "substrateButton"
				substrate=d.get_element_value("substrate")
				if substrate!="None"
					@selection_stack.push(self.sm)
					self.sm=IndigoMaterial.find(substrate)
					Sketchup.active_model.materials.current=sm.mat
					loadValuesFromSU()
					setVisibility()
				end
			when "frontMatButton"
				front_material_name=d.get_element_value("front_material_name")
				if front_material_name!="None"
					@selection_stack.push(self.sm)
					self.sm=IndigoMaterial.find(front_material_name)
					Sketchup.active_model.materials.current=sm.mat
					loadValuesFromSU()
					setVisibility()
				end
			when "backMatButton"
				back_material_name=d.get_element_value("back_material_name")
				if back_material_name!="None"
					@selection_stack.push(self.sm)
					self.sm=IndigoMaterial.find(back_material_name)
					Sketchup.active_model.materials.current=sm.mat
					loadValuesFromSU()
					setVisibility()
				end
			when "backButton"
				valid=false
				#p @selection_stack
				while not valid
					unless @selection_stack.empty?
						self.sm=@selection_stack.pop()
						if self.sm.valid? 
							valid=true
							Sketchup.active_model.materials.current=sm.mat
						end
					else
						UI.messagebox("You cannot go back unless you click on the Material A or B buttons of a Blend material.")
						valid=true
					end
				end
				loadValuesFromSU()
				setVisibility()
			when "baseEmissionColorBox"
				#cp=ColorPicker.new(sm.base_emission,sm,"base_emission")
				#cp.show
				#current=Sketchup.active_model.materials.current
				#if current
				#	cp=IndigoMaterial.new(current)
				#	if cp.color_picker?  #if the color picker is the current material
				#		col=cp.color.to_a
				#		sm.base_emission=[col[0],col[1],col[2]]
				#		setColorBox(id,[col[0],col[1],col[2]])
				#	else
				#		old_col=sm.base_emission
				#		cp=IndigoColorPicker.new(old_col)
				#	end
				#end
			when "baseEmissionGain"
				sm.base_emission_gain=value.to_f
			when "VOID_absorption"
				prompts=["R (m^-1)","G (m^-1)","B (m^-1)"]
				values=sm.absorption_spectrum
				results=UI.inputbox(prompts,values,"Absorption RGB Spectrum")
				if results
					sm.absorption_spectrum=results.join(" ")
					self.refresh_dialog()
				end
			#spectrum picker teset
				
		end #case id

		ite=nil   #indigo texture editor
		
		} #end of Valuechanged callback function

	@mat_editor.add_action_callback("RefreshPreview") {|d,p| getThumbnail}
	@mat_editor.add_action_callback("SetVisibility") {|d,p| setVisibility()}
	@mat_editor.add_action_callback("showThumbnail"){|d,p|
		thumb_dlg=UI::WebDialog.new("SkIndigo Thumbnail", false,"SkIndigoThumbnail", 295, 320, 450, 125, true)
		thumb_dlg.set_html "<img src=\"#{p}\" style=\"cursor:pointer\" title=\"Double-click to load custom thumbnail\"ondblclick=\"window.location='skp:loadThumbnail';\">"
		thumb_dlg.add_action_callback("loadThumbnail") {|d,p|
			new_thumb=UI.openpanel("Load Thumbnail", nil,nil)
			if ((new_thumb) and (FileTest.exist?(new_thumb)))
				sm.thumbnail=new_thumb
				getThumbnail()
			end}
		thumb_dlg.show{}
	}
	@mat_editor.add_action_callback("RefreshValues") {|d,p| 
			#p "refreshing values"
			refresh_dialog()
			
	}
	@mat_editor.add_action_callback("check_current_version") {|d,p|
		version=p.to_s
		current=SkIndigo::VERSION.to_s
		if version>current
			text="A newer version of SkIndigo is available. Would you like to update?  (Click 'No' to be reminded in two weeks.)"
			result=UI.messagebox(text,MB_YESNO)
			if result==6 #yes
				UI.openURL("http://www.indigorenderer.com/documentation/sketchup")
			else
				time=Time.now.to_i
				Sketchup.write_default("SkIndigo","LastUpdateCheck",time)
			end
		end
	}
	@mat_editor.set_on_close{ self.stop_preview() }
		
end #initialize

##
def create_linked_igm(path=nil)

if path==nil
	path=UI.openpanel "Link IGM/PIGM", "", ""
end

igm=nil
if path
	name=File.basename(path)
	if name.split(".")[-1]=="pigm"
		igm=SkIndigo.unpack_pigm(path)
		if igm==nil  #problems unpacking file
			UI.messagebox("PIGM file could not be unpacked.")
			return
		end
	elsif name.split(".")[-1]=="igm"
		igm=path
	end
end
if igm
	igm=igm.gsub(/\\/,"\/")
	sm.linked_igm=igm
	setValue("igm",igm)
	result=UI.messagebox("Would you like to apply a new UV texture to this material?",MB_YESNO)
	if result==6  #yes
		#igm_dir=(File.dirname(igm)).gsub(/\//,"\\")
		tex_path=UI.openpanel("Load texture","","")
		if tex_path
			sm.texture=tex_path  #set the SU texture to the new texture
		end
	end
end

end

#####
def check_for_updates()

last_update_check=Sketchup.read_default("SkIndigo","LastUpdateCheck",0)
current_time=Time.now.to_i
update_check_period=1209600

if current_time-last_update_check>update_check_period  #check for updates every 2 weeks
	@mat_editor.execute_script("check_current_version();")
end

end


def sm
	@sm
end

def timer
	@timer
end
	
def sm=(new_mat)
	#p "newly selected material is #{new_mat.name}"
	@sm=new_mat
end

def refreshMaterialsList()

cmd="clearMaterialList()"
@mat_editor.execute_script(cmd)
mat_list=Sketchup.active_model.materials.collect {|m| m.name}
mat_list.sort!
id="MaterialName"
mat_list.each {|mat_name|
	cmd="createMapOption(\"#{id}\",\"#{mat_name}\",\"#{mat_name}\")"
	@mat_editor.execute_script(cmd)
}

end

def refreshTextureList()

	#function disabled - this was used for previous version internal textures
	return

end

def refreshBlendList()
	cmd="clearBlendLists()"
	@mat_editor.execute_script(cmd)
	blend_list=getBlendList()
	select_ids=["matA","matB"]
	select_ids.each do |id|
		blend_list.each do |tex|
			cmd="createMapOption(\"#{id}\",\"#{tex}\",\"#{tex}\")"
			@mat_editor.execute_script(cmd)
		next
		end
	next
	end
end


#generic dropdown list refreshing function that should be used for refreshing all dropdowns going forward
def refreshList(id,list)

	cmd="clearDropDown('#{id}')"
	@mat_editor.execute_script(cmd)
	list.each do |item|
		cmd="createMapOption('#{id}',\"#{item}\",\"#{item}\")"
		@mat_editor.execute_script(cmd)
	next
	end

end

def refreshLayerList()
	cmd="clearDropDown('layer')"
	@mat_editor.execute_script(cmd)
	layer_names=IndigoRenderSettings.new.light_layer_names
	cmd="createMapOption('layer','Off','-1')"
	@mat_editor.execute_script(cmd)
	layer_names.each_index  {|i|
		cmd="createMapOption('layer',\"#{layer_names[i]}\",\"#{i}\")"
		@mat_editor.execute_script(cmd)
	}
end

def refreshPreviewSceneList()
	clear_dropdown(@mat_editor,"previewScene")
	scenes=get_preview_scenes(@standard_preview_scene_path)
	scenes.each {|name,path| create_option(@mat_editor,"previewScene",name,path) }
	scenes=get_preview_scenes(@custom_preview_scene_path)
	scenes.each {|name,path| create_option(@mat_editor,"previewScene",name,path) }
	create_option(@mat_editor,"previewScene","New..","New")
end

#returns the list of preview scenes as an array of pairs of type [Display name, path to preview scene]
def get_preview_scenes(path)

begin
	# Names on disk, and 'nice' names to display in the UI.
	nice_names = { "mat_db" => "Mat DB", "mat_db1m" => "Mat DB 1 m", "mat_db10m" => "Mat DB 10 m" }

	path=path.gsub(/\\/,"\/")
	scenes=Dir.glob(path + "/**/*.igs")
	
	scenes=scenes.find_all {|f| File.basename(f)[0..2]=="IMP"}

	scene_hash={}
	scenes.each {|scene|
		bn=File.basename(scene,".igs")
		bn.slice!("IMP")  #remove the IMP from the name
		
		# Replace with a 'nice name' if present
		bn = nice_names[bn] if nice_names[bn]
		
		scene_hash[bn]=scene
	}

	# Turn the hash into an array, and then sort it. (alphabetically)
	res_array = scene_hash.to_a.sort()
	return res_array
rescue Exception => e
	# Failed to open the dir or somesuch
	puts "Exception raised in get_preview_scenes():"
	puts e
	return []
end

end


##create a new preview scene
def create_preview_scene()

begin
mats=Sketchup.active_model.materials
pm=mats["previewmaterial"]
if pm==nil
	UI.messagebox("You must have a material named 'previewmaterial' in your model and it must be applied to at least one face.")
end
result=UI.inputbox(["Name"],[""],"Enter Scene Name")
return nil if not result
scene_name=result[0]
preview_scene=File.join(@custom_preview_scene_path,"IMP"+scene_name+".igs")
if FileTest.exist?(preview_scene)
	result=UI.messagebox("Overwrite existing scene?",MB_OKCANCEL)
	if result==1 #OK
		File.delete(preview_scene)
	else
		return false
	end
end
export_path=SkIndigo.temp_dir
if not FileTest.exist?(export_path)
	Dir.mkdir(export_path)
end
export_path=File.join(export_path,scene_name+".igs")
status=SkIndigo.export(2,true,export_path,false,false)
if status
	#pack the scene
	console=SkIndigo.get_indigo_console_path
	temp_pigs=File.join(SkIndigo.temp_dir,"IMP"+scene_name+".pigs")
	params="--pack \"#{export_path}\" \"#{temp_pigs}\""
	SkIndigo.launch_indigo(console,"",params,true,"hidden")
	try=0
	while !FileTest.exist?(temp_pigs)
		raise "Packing Timed Out" if try==5  #allow 10 seconds for indigo to pack the scene
		try+=1
		sleep(1)
	end
	
	#now unpack the scene to the su_preview directory
	params="--unpack \"#{temp_pigs}\" \"#{@custom_preview_scene_path}\""
	SkIndigo.launch_indigo(console,"",params,true,"hidden")
	while !FileTest.exist?(preview_scene)
		raise "Unpacking Timed Out" if try==5  #allow 10 seconds for indigo to unpack the scene
		try+=1
		sleep(1)
	end
	File.delete(temp_pigs)
	UI.messagebox("New Preview Scene #{scene_name} created.")
	return true
end #if

rescue
	UI.messagebox("There was a problem creating the IGS file.")
	raise
end #begin

end   #function

		
def onMaterialRemove(mats, mat)
#p "#{mat.name} was deleted."
if mat.name==@model.get_attribute(@dict,'bg_material',"")
	new_mat=mats.add(mat.name)
	new_im=IndigoMaterial.new(new_mat)
	old_im=IndigoMaterial.new(mat)
	old_im.copy(new_im)
	@model.set_attribute(@dict,'bg_material',new_im.name)
end

end


#material is changed (eg. color changed by color wheel)
def onMaterialChange(material,skup_material)

#basically, we are creating a delay so that the Skindigo editor will update showing the new thumbnail but there is a two second delay so that the SketchUp UI does not lag
#due to the thumbnail texture being re-written

if @mat_editor.visible?
	if @mat_change_timer
		UI.stop_timer(@mat_change_timer)
	end
	@mat_change_timer=UI.start_timer(2,false) {loadValuesFromSU()}
end

end
# A new material was added to SketchUp.  
#def onMaterialAdd(materials, skup_material)
	
	# When a new material gets added to SketchUp, it copies all the attributes from the last selected material by design
	# We can't mess with this.  This is a core behavior of SketchUp.  To create a fresh material with no Indigo settings
	# just select the 'default' material (the square icon with the diagonal line through it) prior to creating a new material
	#  Another reason that we can't delete the indigo attributes when adding a new material is because this method 'onMaterialAdd'
	# fires also when you import an external SKP file into your scene.  If you have intentionally pre-made Indigo materials in
	# that SKP file, then all of the Indigo materials will have their settings reset by using the method below!
	#
	#IndigoMaterial.reset_indigo_attributes(skup_material)
#end


#for some reason, Mac and PC Material Editors behave quite differently.  This method fires on PC when a user clicks on a material
#in the Material Edit Window.  For Mac, you have to hold down the command button and click on a face in the model for this method to fire.
def onMaterialSetCurrent(mats,mat)
	#p "setting current material #{mat.name}"
	#p @timer
	if @timer
		UI.stop_timer(@timer)
	end
	new_mat=IndigoMaterial.new(mat)
	unless new_mat.color_picker? or new_mat.uv_helper?
		self.sm=IndigoMaterial.new(mat)
		#@selection_stack.push(self.sm)
		if @mat_editor.visible?
			refresh_dialog()
		end
		
		#testing for double-clicking the material in the paint bucket
		if @start and not @on_mac   #this doesn't work on Mac
			finish=Time.new
			if (finish-@start)<0.25     #check if user double-clicked a material in the Paint Bucket
				@mat_editor.show {refresh_dialog()} 
				#@mat_editor.bring_to_front
				#@mat_editor.show{}
			else
				@start=Time.new
			end
		else
			@start=Time.new
		end
	end

end

######refresh the dialog values
def refresh_dialog()

refreshPreviewSceneList()
refreshMaterialsList()
refreshTextureList()
refreshBlendList()
refreshList("substrate",getSubstrateList())
refreshList("front_material_name",getSubstrateList())
refreshList("back_material_name",getSubstrateList())
refreshLayerList()
setVisibility()
loadValuesFromSU()
validateReferences()

end

def validateReferences()

reference_ids=["advancedAlbedoMap","advancedBlendMap","advancedExponentMap","advancedSpecularMap","advancedSigmaMap","advancedRoughnessMap","advancedFresnelMap","advancedTransmittanceMap","advancedr_fMap","advancedFrontFresnelMap","advancedBackFresnelMap","advancedFrontRoughnessMap","advancedBackRoughnessMap","advancedBumpMap","advancedNormalMap","advancedDispMap","advancedEmissionMap","advancedAbsLayerTransMap","advancedClipMap","iesBrowse","nkdataBrowse","igmBrowseButton"]
reference_ids.each {|id| validate(id)}

invalidate("advancedAlbedoMap") if !sm.albedo_texture.valid_reference?
invalidate("advancedBlendMap") if !sm.blend_map.valid_reference?
invalidate("advancedExponentMap") if !sm.exponent_map.valid_reference?
invalidate("advancedSpecularMap") if !sm.specular_map.valid_reference?
invalidate("advancedSigmaMap") if !sm.sigma_map.valid_reference?
invalidate("advancedBumpMap") if !sm.bump_map.valid_reference?
invalidate("advancedNormalMap") if !sm.normal_map.valid_reference?
invalidate("advancedDispMap") if !sm.disp_map.valid_reference?
invalidate("advancedEmissionMap") if !sm.emission_map.valid_reference?
invalidate("advancedAbsLayerTransMap") if !sm.abs_layer_trans_map.valid_reference?
invalidate("advancedClipMap") if !sm.clip_map.valid_reference?
invalidate("iesBrowse") if !sm.ies_profile.valid_reference?
invalidate("nkdataBrowse") if !sm.nk_data.valid_reference?
invalidate("igmBrowseButton") if !sm.valid_igm_reference?
invalidate("advancedRoughnessMap") if !sm.roughness_map.valid_reference?
invalidate("advancedFresnelMap") if !sm.fresnel_map.valid_reference?
invalidate("advancedTransmittanceMap") if !sm.transmittance_map.valid_reference?
invalidate("advancedr_fMap") if !sm.r_f_map.valid_reference?
invalidate("advancedFrontFresnelMap") if !sm.front_fresnel_scale_map.valid_reference?
invalidate("advancedBackFresnelMap") if !sm.back_fresnel_scale_map.valid_reference?
invalidate("advancedFrontRoughnessMap") if !sm.front_roughness_map.valid_reference?
invalidate("advancedBackRoughnessMap") if !sm.back_roughness_map.valid_reference?
end

def validate(id)

cmd="$('##{id}').css('backgroundColor','');"
@mat_editor.execute_script(cmd)
cmd="$('##{id}').attr('title','');"
@mat_editor.execute_script(cmd)

end

def invalidate(id)

cmd="$('##{id}').css('backgroundColor','rgb(255,0,0)');"
@mat_editor.execute_script(cmd)
cmd="$('##{id}').attr('title','Invalid File Reference');"
@mat_editor.execute_script(cmd)

end
####display the material editor
def show_editor
	#p "show editor called"
	#p "selected material is #{sm.name}"
	if @timer
		UI.stop_timer(@timer)
	end
	if sm!=nil
		if @mat_editor.visible?
			#p "editor is visible"
			self.refresh_dialog()
			#refreshMaterialsList();refreshTextureList();refreshBlendList();loadValuesFromSU();setVisibility()
		else
			#p "editor is not visible"
			#@mat_editor.show{refreshMaterialsList();refreshTextureList();refreshBlendList();loadValuesFromSU();setVisibility()} if not @on_mac
			if @on_mac
				self.refresh_dialog()
				self.check_for_updates()
				@mat_editor.show_modal{}
			else
				@mat_editor.show{check_for_updates()}
			end
			#@mat_editor.show_modal{refreshMaterialsList();refreshTextureList();refreshBlendList();loadValuesFromSU();setVisibility()} if @on_mac
			#@mat_editor.show_modal{} if @on_mac  #test for bugfix
			#@mat_editor.show{}
			#@mat_editor.close
			#@mat_editor.show{}
		end
	else
		UI.messagebox "Please select a material using the Paint Bucket."
		#mats=Sketchup.active_model.materials
		#if mats.length==0  #no materials so we will create one
		#	new_mat=mats.add("Material1").color=[125,125,125]
		#	mats.current=new_mat
		#	sm=IndigoMaterial.new(new_mat)
		#else
		#	sm=IndigoMaterial.new(mats[0]) #select the first material for editing
		#end
		#self.show_editor
	end
end
	

def getThumbnail

	thumb=sm.thumbnail
	#p thumb
	if FileTest.exist?(thumb)
		thumb.gsub!(/\\/,"\/")
		#cmd="document.images[\"PreviewImage\"].src=\"nopreview.png\""  #This is required so that new image is not updated from the cache
		#@mat_editor.execute_script(cmd)
		cmd="document.images[\"PreviewImage\"].src=\"file:\/\/#{thumb}?#{(rand*10000).to_i}\""  #adding random data ensures the image is not loaded from the cache
		@mat_editor.execute_script(cmd)
	else
		if sm.texture!=nil
			tex_path=File.basename(sm.texture.filename)
			if tex_path  
				tex=tex_path.gsub(/\\/,"\/")
			else
				return
			end
			#p tex
			tex_path=File.join(SkIndigo.temp_dir,tex)
			#p tex_path
			tex_path=save_mat_texture(sm.mat,tex_path)
			if tex_path!=nil
				tex_path.gsub!(/\\/,"\/")
				cmd="document.images[\"PreviewImage\"].src=\"file:\/\/#{tex_path}?#{(rand*10000).to_i}\""
			end
		else
			cmd="document.images[\"PreviewImage\"].src=\"nopreview.png\""
		end
		@mat_editor.execute_script(cmd)
	end		
end


def loadValuesFromSU()
	#p "loading values from SU"
	getThumbnail()
	type=sm.type
	bm=sm.bump_map
	nm=sm.normal_map
	em=sm.exponent_map
	cm=sm.clip_map
	at=sm.albedo_texture
	dm=sm.disp_map
	sim=sm.sigma_map
	emm=sm.emission_map
	altm=sm.abs_layer_trans_map
	spm=sm.specular_map
	absm=sm.absorption_map
	sssm=sm.sss_map
	gsm=sm.g_spectrum_map
	rm=sm.roughness_map
	fm=sm.fresnel_map
	tm=sm.thickness_map
	transm=sm.transmittance_map
	frm=sm.front_roughness_map
	brm=sm.back_roughness_map
	ffsm=sm.front_fresnel_scale_map
	bfsm=sm.back_fresnel_scale_map
	r_fm=sm.r_f_map
	
	blm=sm.blend_map
	root=File.dirname(SkIndigo.get_indigo_path)


	###########################################Update the material editor dialog box
	setValue("MaterialName",sm.mat.name)
	setValue("type",sm.type)
	setValue("layer",sm.layer)
	setValue("ior",sm.ior)
	#setValue("exp",sm.exponent)
	#setValue("fres",sm.fresnel_scale)
	#setValue("sigma",sm.sigma)
	setValue("precedence",sm.precedence)
	setValue("cau",sm.cauchy_b_coeff)
	setValue("power",sm.power_drawn)
	setValue("ole",sm.overall_luminous_efficacy)
	###setValue("temp",sm.temperature)
	setValue("nkdata",sm.nk_data.to_s)
	setValue("ies_path",sm.ies_profile.to_s)
	###setValue("g_spectrum",sm.g_spectrum)
	###setValue("sssval",sm.sss_spectrum)
	setValue("albedo_map",at.type)
	setValue("specular_map",spm.type)
	setValue("blend_map",blm.type)
	setValue("bump_map",bm.type)
	setValue("bump_value",bm.b)
	setValue("normal_map",nm.type)
	setValue("normalmap_value",nm.b)
	setValue("disp_map",dm.type)
	setValue("dispmap_value",dm.b)
	setValue("exp_map",em.type)
	setValue("expmap_value",em.b)
	setValue("sigma_map",sim.type)
	setValue("sigma_value",sim.b)
	setValue("roughness_map",rm.type)
	setValue("roughness_value",rm.b)
	setValue("r_f_map",r_fm.type)
	setValue("r_f_value",r_fm.b)
	setValue("front_fresnel_map",ffsm.type)
	setValue("front_fresnel_value",ffsm.b)
	setValue("back_fresnel_map",bfsm.type)
	setValue("back_fresnel_value",bfsm.b)
	setValue("front_roughness_map",frm.type)
	setValue("front_roughness_value",frm.b)
	setValue("back_roughness_map",brm.type)
	setValue("back_roughness_value",brm.b)
	setValue("fresnel_map",fm.type)
	setValue("fresnel_value",fm.b)
	setValue("thickness_map",tm.type)
	setValue("thickness_value",tm.b.to_f*1000000.0)
	setValue("clipValue",cm.b)
	setValue("emission_map",emm.type)
	setValue("transmittance_map",transm.type)
	###setValue("emission_value",emm.b)
	setValue("abs_layer_trans_map",altm.type)
	###setValue("abs_layer_trans_value",altm.b)
	setValue("clip_map",cm.type)
	setValue("matA",sm.a_name)
	setValue("matB",sm.b_name)
	setValue("substrate",sm.substrate)
	setValue("front_material_name",sm.get("front_material_name"))
	setValue("back_material_name",sm.get("back_material_name"))
	#setValue("blendfactor",sm.blend_factor)
	setValue("blendfactor",blm.b)
	setValue("maxSubdiv",sm.max_subdiv)
	setValue("pixelThreshold",sm.pixel_threshold)
	setValue("curveThreshold",sm.curvature_threshold)
	setValue("igm",sm.linked_igm)
	###setValue("baseEmissionGain",sm.base_emission_gain)
	setValue("abs_map",absm.type)
	setValue("sss_map",sssm.type)
	setValue("Phase_Function",sm.phase_function)
	setValue("emissionScale",sm.emission_scale)
	setValue("emissionMeasure",sm.emission_measure)
	
	#set swatches
	setColorBox("PreviewImage",sm.mat.color.to_a)
	#set wavelength dependent param swatches
	setColorBox("absorption",sm.color("absorption"))
	setColorBox("baseEmissionColorBox",sm.color("base_emission"))
	setColorBox("emissionSwatch",sm.color("emission"))
	setColorBox("transmittanceSwatch",sm.color("transmittance"))
	setColorBox("albedoSwatch",sm.color("albedo"))
	setColorBox("abslayertransSwatch",sm.color("absorption_layer_transmittance"))
	setColorBox("specularSwatch",sm.color("specular"))
	setColorBox("sssSwatch",sm.color("sss"))
	setColorBox("g_spectrumSwatch",sm.color("g_spectrum"))
	
	#set value boxes visible or invisible
	setValueBoxesVisible()
	if sm.phase_function=="Uniform" then setVisible("g_spectrum_param",false)
	else setVisible("g_spectrum_param",true)
	end
	
	if sm.sss? then	setCheckbox("SSSCheckbox","true")
	else setCheckbox("SSSCheckbox","false")
	end
	#if  sm.smooth? then setCheckbox("subSmoothingCheckbox","true")
		#else setCheckbox("subSmoothingCheckbox","false")
	#end
	if  sm.view_dependent_subdivision? then setCheckbox("viewDependentCheckbox","true")
		else setCheckbox("viewDependentCheckbox","false")
	end
	if sm.ies? then setCheckbox("EnableIESCheckbox","true")
	else setCheckbox("EnableIESCheckbox","false")
	end
	if sm.use_nkdata? then setCheckbox("EnableNKdataCheckbox","true")
	else setCheckbox("EnableNKdataCheckbox","false")
	end
	if sm.use_bump? then setCheckbox("bumpCheckbox","true")
	else setCheckbox("bumpCheckbox","false")
	end
	if sm.use_normal? then setCheckbox("normalCheckbox","true")
	else setCheckbox("normalCheckbox","false")
	end
	if sm.use_displace? then setCheckbox("displaceCheckbox","true")
	else setCheckbox("displaceCheckbox","false")
	end
	if sm.use_emission? then setCheckbox("emissionCheckbox","true")
	else setCheckbox("emissionCheckbox","false")
	end
	if sm.use_abs_layer_trans? then setCheckbox("absLayerTransCheckbox","true")
	else setCheckbox("absLayerTransCheckbox","false")
	end
	if sm.use_clip? then setCheckbox("clipCheckbox","true")
	else setCheckbox("clipCheckbox","false")
	end
	if sm.use_specular? then setCheckbox("specularCheckbox","true")
	else setCheckbox("specularCheckbox","false")
	end
	if sm.step_blend? then setCheckbox("stepBlend","true")
	else setCheckbox("stepBlend","false")
	end
	if sm.interference? then setCheckbox("interferenceCheckbox","true")
	else setCheckbox("interferenceCheckbox","false")
	end
	###if sm.phase_function=="Uniform" then setRadioButton("UniformPF","true")
	###else setRadioButton("Henyey-GreensteinPF","true")
	###end
	###if sm.base_emission_spectrum=="Blackbody" then setRadioButton("bbES","true")
	###else setRadioButton("rgbES","true")
	###end

	# update simple ui values
	if @ui=="simple"
		if type=="Diffuse"
			s_value="0"
			s2_value="100"
		elsif type=="Specular"
			s_value=to_reflection(sm.ior)
			s2_value="0"
		else
			s_value=to_reflection(sm.ior)
			s2_value=to_roughness(sm.exponent_map.b)
		end
		#s3_value=to_power_slider(sm.power_drawn)
		s3_value=to_power_slider(sm.emission_scale)
		
		cmd="s.setValue(#{s_value})"
		@mat_editor.execute_script(cmd)
		
		cmd="s2.setValue(#{s2_value})"
		@mat_editor.execute_script(cmd)
		
		cmd="s3.setValue(#{s3_value})"
		@mat_editor.execute_script(cmd)
	end
	if (type=="Specular") or (type=="Glossy Transparent") then setCheckbox("transparentCheckbox","true")
	else setCheckbox("transparentCheckbox","false")
	end
	if (sm.emitter?) then setCheckbox("emitterCheckbox","true")
	else setCheckbox("emitterCheckbox","false")
	end
		
	
end

#this method sets the visibility of the material editor UI elements based on the current material type
def setVisibility()

	type=sm.type
	if @ui=="simple"
		cmd="setSimpleUIVisible()"
		@mat_editor.execute_script(cmd)
		return
	end
	case type
		when "Diffuse"
			@mat_editor.execute_script("setDiffuseVisible()")
		when "Phong"
			@mat_editor.execute_script("setPhongVisible()")
		when "Specular"
			@mat_editor.execute_script("setSpecularVisible()")
		when "Glossy Transparent"
			@mat_editor.execute_script("setGlossyTransparentVisible()")
		when "Blend"
			@mat_editor.execute_script("setBlendVisible()")
		when "Metals","Al","Cr","Cu","Au","Ag","Ti","W","Li","Ni","Pt"
			@mat_editor.execute_script("setMetalsVisible()")
		when "Thin Glass"
			@mat_editor.execute_script("setThinGlassVisible()")
		when "Oren-Nayar"
			@mat_editor.execute_script("setOrenNayarVisible()")
		when "Linked IGM"
			@mat_editor.execute_script("setLinkedIGMVisible()")
		when "RGB Light"
			@mat_editor.execute_script("setRGBLightVisible()")
		when "Blackbody Light"
			@mat_editor.execute_script("setBlackbodyLightVisible()")
		when "Exit Portal"
			@mat_editor.execute_script("setExitPortalVisible()")
		when "Coating"
			@mat_editor.execute_script("setCoatingVisible()")
		when "Double-Sided Thin"
			@mat_editor.execute_script("setDoubleSidedThinVisible()")
	end
	@mat_editor.execute_script("setEfficacyVisible()")   #disables efficacy if ies profile is enabled
end 
		
def setValue(id,value)
	new_value=value.to_s
	cmd="document.getElementById('#{id}').value=\"#{new_value}\""
	@mat_editor.execute_script(cmd)
end
def setCheckbox(id,checked)
	cmd="document.getElementById('#{id}').checked=#{checked}"
	@mat_editor.execute_script(cmd)
end
def setRadioButton(id,checked)
	cmd="document.getElementById('#{id}').checked=\"#{checked}\""
	@mat_editor.execute_script(cmd)
end
def setColorBox(id,color)

	if color!=nil
		r=color[0]
		g=color[1]
		b=color[2]
		cmd="$(\"##{id}\").show()"
		@mat_editor.execute_script(cmd)
		cmd="document.getElementById('#{id}').style.backgroundColor=\"rgb(#{r},#{g},#{b})\""
	else
		cmd="$(\"##{id}\").hide()"
	end
	@mat_editor.execute_script(cmd)

end

def setVisible(id,visible) #visible is boolean

if visible==true
	cmd="$(\"##{id}\").show()"
else
	cmd="$(\"##{id}\").hide()"
	#cmd="$(\"##{id}\").fadeTo('fast',0.3)"
end
@mat_editor.execute_script(cmd)

end

def setValueBoxesVisible()


textures=[sm.bump_map,sm.exponent_map,sm.clip_map,sm.disp_map,sm.sigma_map,sm.blend_map,sm.normal_map,sm.roughness_map,sm.fresnel_map,sm.thickness_map,sm.front_roughness_map,sm.back_roughness_map,sm.front_fresnel_scale_map,sm.back_fresnel_scale_map,sm.r_f_map]
dlg_ids=["bump_value","expmap_value","clipValue","dispmap_value","sigma_value","blendfactor","normalmap_value","roughness_value","fresnel_value","thickness_value","front_roughness_value","back_roughness_value","front_fresnel_value","back_fresnel_value","r_f_value"]

textures.each_index {|i|
	setValueBoxVisible(textures[i],dlg_ids[i])
}

end

def setValueBoxVisible(tex,id)

if tex.type=="Shader"
	setVisible(id,false)
else
	setVisible(id,true)
end

end

def getTextures()  #Returns array of all materials with textures
	texture_list=["None","SU"]
	materials=Sketchup.active_model.materials
	materials.each do |m|
	if (m.respond_to?(:texture) and m.texture !=nil)
		texture_list.push(m.name)
	end
	next
	end
	texture_list+=["Custom Shader"]
	return texture_list
end

def getSubstrateList()  #return an array of all valid materials for substrates
	materials=Sketchup.active_model.materials
	list=[]
	for mat in materials
		type=IndigoMaterial.new(mat).type
		case type
			when "Blackbody Light","RGB Light","Exit Portal"
			###
			else
				list.push(mat.name) if mat!=sm.mat
		end
	end
	return list.sort.unshift("None")
end

def getBlendList() #Returns array of all valid materials for blending 
	materials=Sketchup.active_model.materials
	#blend_list=["None"]
	blend_list=[]
	for mat in materials
		type=IndigoMaterial.new(mat).type
		if ((type) !="Blackbody Light") and ((type)!="RGB Light") and ((type)!="Exit Portal")
			#blend_list.push(mat.display_name) if mat!=sm.mat
			blend_list.push(mat.name) if mat!=sm.mat
		end
	end 
	return blend_list.sort.unshift("None")
end

##############################Material Presets
def apply_oj(material)
material.type="Specular"
material.ior=1.33
material.cauchy_b_coeff=0.0
material.sss=true
material.sss_spectrum="250"
material.alpha=0.41
material.color=[255,231,0]
material.phase_function="Henyey-Greenstein"
material.g_spectrum="0.55"
material.layer=-1
end

def apply_clay(mat)
mat.type="Oren-Nayar"
mat.alpha=1.0
mat.layer=-1
tex=mat.sigma_map
tex.b=0.3
mat.sigma_map=tex
mat.sigma=0.3
mat.color=[181, 158, 135]
end

def apply_wax(material)
material.type="Specular"
material.ior=1.1
material.cauchy_b_coeff=0.0
material.sss=true
material.sss_spectrum="300"
material.alpha=0.12
material.color=[230,230,5]
material.phase_function="Uniform"
material.layer=-1
end

def apply_lambertian(material)
material.type="Diffuse"
material.alpha=1.0
material.layer=-1
end

def apply_glass(material)
material.type="Specular"
material.ior=1.52
material.cauchy_b_coeff=0.0
material.sss=false
material.alpha=0.0
material.layer=-1
end

def apply_water(material)
material.type="Specular"
material.ior=1.33
material.cauchy_b_coeff=0.0
material.sss=true
material.sss_spectrum="0.0007 0.002 0.0026"
material.color=[26, 230, 246]
material.alpha=0.02
material.phase_function="Uniform"
material.layer=-1
end

def apply_frosted_glass(material)
material.type="Glossy Transparent"
material.exponent=100
material.ior=1.52
material.cauchy_b_coeff=0.0
material.sss=false
material.alpha=0.0
material.layer=-1
end

def apply_mirror(material)
material.type="Specular"
material.ior=20.0
material.cauchy_b_coeff=0.0
material.sss=false
material.alpha=1.0
material.layer=-1
end

def apply_ceramic(material)
material.type="Phong"
material.exponent=5000
material.fresnel_scale=1.0
material.ior=1.52
material.alpha=1.0
material.layer=-1
end

def apply_dull_plastic(material)
material.type="Phong"
material.exponent=300
material.fresnel_scale=1.0
material.ior=1.33
material.alpha=1.0
material.layer=-1
end

def apply_shiny_plastic(material)
material.type="Phong"
material.exponent=10000
material.fresnel_scale=1.0
material.ior=1.22
material.alpha=1.0
material.layer=-1
end

def apply_leather(material)
material.type="Phong"
material.exponent=50
material.fresnel_scale=1.0
material.ior=1.58
material.alpha=1.0
material.layer=-1
end

def apply_car_paint(material)
material.type="Phong"
material.exponent=30000
material.fresnel_scale=1.0
material.ior=1.72
material.alpha=1.0
material.layer=-1
end

def apply_tinted_glass(material)
material.type="Specular"
material.ior=1.52
material.cauchy_b_coeff=0.0
material.sss=false
material.alpha=0.1
material.layer=-1
end

def apply_nkdata(material,file_name)
root=File.dirname(SkIndigo.get_indigo_path)
return if not root
nkdata=File.join(root,"nkdata",file_name)
nkdata.gsub!(/\\/,"\/")
nk=NKdata.new(nkdata)
material.type="Metals"
material.nk_data=nk
material.use_nkdata=true
material.layer=-1
end

def apply_lampshade(material)
material.type="Diffuse"
material.alpha=0.5
material.layer=-1
end

def apply_100W(material)
material.layer=0 if material.layer==-1
material.emission_scale=1750.0
material.emission_measure="luminous_flux"
bm=material.base_emission
bm.bb_temperature=2800
bm.spectrum_type="Blackbody"
material.base_emission=bm
#material.base_emission_spectrum="Blackbody"
@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
end

def apply_60W(material)
material.layer=0 if material.layer==-1
material.emission_scale=60.0*14.0
material.emission_measure="luminous_flux"
material.base_emission_spectrum="Blackbody"
@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
end

def apply_7WCFW(material)
material.layer=0 if material.layer==-1
material.emission_scale=7.0*50.0
material.emission_measure="luminous_flux"
material.base_emission_spectrum="Blackbody"
@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
end

def apply_34WTFC(material)
material.layer=0 if material.layer==-1
material.emission_scale=34.0*50.0
material.emission_measure="luminous_flux"
bm=material.base_emission
bm.bb_temperature=6000
bm.spectrum_type="Blackbody"
material.base_emission=bm
@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
end

def apply_34WTFM(material)
material.layer=0 if material.layer==-1
material.emission_scale=34.0*50.0
material.emission_measure="luminous_flux"
bm=material.base_emission
bm.bb_temperature=4500
bm.spectrum_type="Blackbody"
material.base_emission=bm
@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
end

def apply_34WTFW(material)
material.layer=0 if material.layer==-1
material.emission_scale=34.0*50.0
material.emission_measure="luminous_flux"
bm=material.base_emission
bm.bb_temperature=3000
bm.spectrum_type="Blackbody"
material.base_emission=bm
@mat_editor.execute_script("document.getElementById('emitterTable').style.display='inline'")
end

###################calibration of simple UI sliders
def to_ior(value)
	value=value.to_f
	specular=0.001*Math.exp(value*0.0692)
	new_ior=-(Math.sqrt(specular)+1)/(Math.sqrt(specular)-1)
	new_ior=(new_ior*1000).round.to_f/1000    #round the value to three decimal places
	return new_ior.to_s
end
def to_exp(value)
	value=value.to_i
	exp=1000000*Math.exp(-0.14*value)
	exp=exp.round
	return exp.to_s
end
def to_reflection(value)
	value=value.to_f
	specular=((value-1)/(value+1))**2
	if specular<=0.0
		reflection=0
	else
		reflection=Math.log(1000*specular)/0.0692
		reflection=100 if reflection>100
	end
	return reflection.to_s
end
def to_roughness(value)
	value=value.to_f
	rough=Math.log(value/1000000)/(-0.14)
	rough=100 if rough>100
	rough=0 if rough<0
	return rough.to_s
end
def to_power(value)
	value=value.to_i
	power=value*1
	return power.to_s
end
def to_power_slider(value)
	value=value.to_f
	power_slider=value/100.0
	power_slider=100 if power_slider>100
	power_slider=1 if power_slider<1
	power_slider=power_slider.to_i
	return power_slider.to_s
end
def getTypeFromSliders()
	current_type=sm.type
	ref=@mat_editor.get_element_value("reflection-value").to_i
	rough=@mat_editor.get_element_value("roughness-value").to_i
	opacity=sm.alpha
	if (current_type=="Blackbody Light") or (current_type=="RGB Light") or (current_type=="Metals") or (current_type=="Exit Portal") or (current_type=="Thin Glass") or (current_type=="Blend")
		return current_type
	elsif (current_type=="Specular") or (current_type=="Glossy Transparent")
		if rough==0
			type="Specular"
		else
			type="Glossy Transparent"
		end
	else
		if (rough==100) and (ref==0)
			type="Diffuse"
		else
			type="Phong"
		end
	end
	return type

end

def mat_editor    #material editor accessor function
	@mat_editor
end

def preset_dlg
	@prset_dlg
end

#################################################Material Preview
def preview_material(mat,preview_model,thumbnail_path)

use_console=true

case File.basename(preview_model)
	when "IMPsphere.igs"
		if mat.emitter?
			UI.messagebox("The Sphere preview scene cannot be used for emitting materials.")
			return
		end
		if mat.type=="Linked IGM"
			UI.messagebox("The Sphere preview scene cannot be used for Linked IGM materials.")
			return
		end
end

igs_export_path=SkIndigo.default_export_path()

on_mac=SkIndigo.on_mac?
unless on_mac
	path=SkIndigo.get_indigo_console_path if use_console
	path=SkIndigo.get_indigo_path if not use_console
	indigo_dir=File.dirname(path)
	#preview_path=File.join(indigo_dir,"SUpreviews")
	#preview_path=File.join(Sketchup.find_support_file("Plugins\/skindigo"),"SUPreviews")
else
	path=SkIndigo.get_indigo_console_path #currently using the indigo console for previews
	#path=SkIndigo.get_indigo_path
	#preview_path=File.join(Sketchup.find_support_file("Plugins\/skindigo"),"SUPreviews")
end
return if not path

preview_igs_path=preview_model
preview_mat_igs=File.join(igs_export_path,"previewmats.igs")

if not FileTest.exist?(preview_igs_path)
	UI.messagebox "Preview IGS file not found in #{preview_igs_path}"
	return
end

#tex_path=File.join(preview_path,"textures")
tex_path=File.join(igs_export_path,"textures")

if not FileTest.exist? (tex_path)
	Dir.mkdir(tex_path)
end
ie=IndigoExporter.new(preview_mat_igs)
#ie.tex_path=tex_path
ie.tex_path="textures"
ie.preview_material=mat

ie.get_used_layers()
#Write the new material preview file
out=File.new((preview_mat_igs),"w")
#out = File.new(preview_path+"\\previewmats.igs",File::CREAT|File::TRUNC|File::RDWR, 0644)
out.puts "<scene>"
status=ie.export_material(mat,out)
out.puts "<include>"
out.puts "	<pathname>#{preview_igs_path}</pathname>"
out.puts "</include>"
out.puts "</scene>"
out.close

ie.export_textures(ie.textures)                     #exports the textures that were stored during the material export process
#for nk_path in ie.nkdata					#exports the nkdata that was collected during the material export process
	#nk=NKdata.new(nk_path)
	#nk.export(tex_path)  #exports the nkdata to the textures directory
#end


out_image_path=thumbnail_path
unless on_mac 
	params="-o \"#{out_image_path}\"" 
else
	params="-o \"#{out_image_path}\""
end
SkIndigo.launch_indigo(path,preview_mat_igs,params,true,"hidden")

end #function end

######
def stop_preview()

begin
	if @on_mac
		if @preview_process_id
			system("kill #{@preview_process_id}")
		end
	else
		if @preview_process_id
			# puts "Killing process, @preview_process_id: #{@preview_process_id}"
			Process.kill(9, @preview_process_id.to_i)
		end

	end

	#now kill the thumbnail refresh timer
	if @timer
		UI.stop_timer(@timer)
	end

	@preview_process_id=nil
	toggle_stop_button(@mat_editor,false)
rescue
	raise
end

end #function

#######dlg is the webdialog, id is the HTML id, status is either true or false
def toggle_stop_button(dlg,status)

if status  #enable the button
	cmd="document.getElementById('StopPreview').disabled=\"\""
	cmd2="document.getElementById('StopButtonImage').src=\"player_stop.png\""
else #disable the button
	cmd="document.getElementById('StopPreview').disabled=\"disabled\""
	cmd2="document.getElementById('StopButtonImage').src=\"player_stop_disabled.png\""
end
dlg.execute_script(cmd)
dlg.execute_script(cmd2)

end


#####
def preview_running?
wmi = WIN32OLE.connect("winmgmts://")
query = "select * from Win32_process"
processes = wmi.ExecQuery(query)
names=[]
processes.each{|p| names.push(p.Name)}
return true if names.include?("indigo_console.exe")

end #function

#########
def win_process_running?(p_name)
wmi = WIN32OLE.connect("winmgmts://")
query = "select * from Win32_process"
processes = wmi.ExecQuery(query)
names=[]
processes.each{|p| names.push(p.Name)}
return true if names.include?(p_name)

end #function
#####
def download_igm(url,dl_path=nil)

igm_filename=File.basename(url)
tempdir=SkIndigo.temp_dir
dl_path=File.join(tempdir,igm_filename) unless dl_path

if SkIndigo.on_mac?
	command_line="curl -o "<<dl_path<<" "<<url
	system(command_line)
		
else
	wget=File.join(File.dirname(__FILE__),"wget.exe")
	Dir.chdir(File.dirname(wget))
	command_line="start wget.exe -O "<<dl_path<<" "<<url
	#p command_line
	system(command_line)
	time=0.0
	while win_process_running?("wget.exe")
		sleep(1)
		time+=1
		if time>60
			break
		end
	end

end #if on mac

if FileTest.exist?(dl_path)
	return dl_path
else
	return nil
end

end  #end functin

end #material editor class


#################################Texture Editor Class#############################
#todo put in separate file
class IndigoTextureEditor

include SkIndigoUVS

def initialize(texture,material,map_type=nil)   #tex is an IndigoTexture object - material is an IndigoMaterial object - map_type is abledo,bump,exponent,etc.
	@on_mac=SkIndigo.on_mac?
	@tex=texture
	#p @tex
	@mat=material
	@su_mat=@mat.mat
	@map_type=map_type
	@irs=IndigoRenderSettings.new
	@dlg=UI::WebDialog.new("SkIndigo Texture Editor", true, "SkIndigoTextureEditor",404,304,419,243,true) #width,height,left,top
	fn=File.join(File.dirname(__FILE__),"TextureEditor.html")
	@dlg.set_file(fn)
	@temp_texture_path=ENV['temp']+'\texture_editor.jpg' if not @on_mac
	@temp_texture_path=File.join(ENV['TMPDIR'],"texture_editor.jpg") if @on_mac
	#p "temp texture path is #{@temp_texture_path}"
	@thumb=getThumbnail()
	#p "got thumbnail #{@thumb}"
	color = @dlg.get_default_dialog_color 
	@dlg.set_background_color(color)
	@dlg.add_action_callback("getValues") {|d,p|
		#p "Updating Values"
		getModelTextures()
		loadValuesFromSU()
		setVisibility()
	}
	@dlg.add_action_callback("load_uv_set") {|d,p|
		#uv_set=@dlg.get_element_value("uvset")
		#ents=Sketchup.active_model.selection
		#faces=ents.find_all {|e| e.class==Sketchup::Face}
		#if faces.length>0
		#	position_map(faces,uv_set)
		#end
		#defn=Sketchup.active_model.active_entities.parent
		#stored_set=defn.get_attribute('uvsets',uv_set)
		#if stored_set==true
			#UI.messagebox("UV set #{@tex.uv_set} loaded.")
		#else
		#	UI.messagebox("No UV data to load for UV set #{@tex.uv_set}.")
		#end
	}
	@dlg.add_action_callback("send_to_mapper") {|d,p|
		apply_settings(d)
		case @tex.type
			when "External"
				unless @tex.valid?
					UI.messagebox("You must first select a valid texture map.")
					break
				end
				if @tex.filename.slice(-3,3).downcase=="exr"
					UI.messagebox("EXR textures cannot be positioned.")
					break
				end
			when "SketchUp"
				if @su_mat.texture==nil
					UI.messagebox("You must first apply a texture to this material.")
					return
				end
			when "Constant"
				UI.messagebox("You must first load a texture.")
				return
		end
		uv_set=@tex.uv_set
		#if uv_set=="Default"
		#	result=UI.messagebox("By positioning this texture using the SketchUp UV set, you have chosen to apply this texture to the current material.  Continue?",MB_YESNO)
		#	if result==6 #yes
		#		@su_mat.texture=@temp_texture_path if @tex.type=="SketchUp"
		#		@su_mat.texture=@tex.path if @tex.type=="External"
		#		d.close()
		#		return
		#	else
		#		return
		#	end
		#end
		uv_mat=get_uv_mat()
		uv_mat.set_attribute('indigo_uv','current_set',@tex.uv_set.to_i)
		case @tex.type
			when "External"
				#uv_mat.texture=save_mat_texture(@su_mat,@temp_texture_path)  #store the currently applied texture of the material
				#@su_mat.texture=@tex.path
				uv_mat.texture=@tex.path
			when "SketchUp"
				uv_mat.texture=@temp_texture_path
				#uv_mat.texture=save_mat_texture(@su_mat,@temp_texture_path)
				#@su_mat.texture=@temp_texture_path
		end
		#ents=Sketchup.active_model.active_entities
		#faces=ents.find_all {|e| e.material==@su_mat}
		ents=Sketchup.active_model.selection
		faces=ents.find_all {|e| e.class==Sketchup::Face}
		if faces.length==0
			UI.messagebox("You must first select at least one face for positioning this texture.")
			return
		end
		Sketchup.active_model.start_operation("Position Map")
		#store_uvs(faces,0)  #save current uvs so they can be restored later
		switch_mat(faces,@mat.mat,uv_mat,@tex.uv_set.to_i)
		#try to load the stored uv set if the material has a texture
		#if uv_mat.texture!=nil
		#	position_map(faces,uv_set)
		#	$skindigo_current_uv_layer=uv_set.to_i
		#end
		d.close()
		defn=Sketchup.active_model.active_entities.parent
		stored_set=defn.get_attribute('uvsets',uv_set)
		#if stored_set==true
		#	UI.messagebox("UV set #{@tex.uv_set} loaded.")
		#else
		#	UI.messagebox("No UV data to load for UV set #{@tex.uv_set}.")
		#end
		Sketchup.active_model.commit_operation
	}
	@dlg.add_action_callback("apply_settings") {|d,p|
		apply_settings(d)
		$skindigo_mat_observer.refresh_dialog()
		d.close
	}
	@dlg.add_action_callback("update_thumbnail") {|d,p|
		textures=@irs.get_model_textures
		current_tex=d.get_element_value("TexturePath")
		new_thumb=textures.find {|path| path.gsub(/\\/,"\/")==current_tex}
		if new_thumb
			@thumb=new_thumb
			setThumbnail()
		end
	}
	@dlg.add_action_callback("cancel_changes") {|d,p| d.close}
	@dlg.add_action_callback("create_new_texture") {|d,p|
		filename=UI.openpanel("Load New Texture", nil,nil)
		return if not filename
		
		case @tex.type
			when "SketchUp"
				@mat.mat.texture=filename
			when "External","Constant"
				@tex.path=filename
				@tex.type="External"
				@irs.store_texture(filename)
				#add_map_option(File.basename(filename))
				add_map_option(filename.gsub(/\\/,"\/"))
		end
		@thumb=getThumbnail()
		loadValuesFromSU()
		
	}
	@dlg.add_action_callback("refresh_thumbnail") {|d,p|
		@thumb=@dlg.get_element_value("TextureName")
		#p @thumb
	}
	#@dlg.show_modal {getModelTextures();loadValuesFromSU();setVisibility()}
	@dlg.show_modal {}
end

def apply_settings(d) #d is the dialog

@tex.a=d.get_element_value("aValue")
@tex.b=d.get_element_value("bValue")
@tex.c=d.get_element_value("cValue")
@tex.exponent=d.get_element_value("textureExponent")
@tex.uv_set=d.get_element_value("uvset")
path=d.get_element_value("TexturePath")
smooth=d.get_element_value("smooth")
if smooth=="true"
	@tex.smooth="On"
else
	@tex.smooth="Off"
end
#p "setting texture path to #{path}"
textures=@irs.get_model_textures
current_tex=d.get_element_value("TexturePath")
new_tex=textures.find {|path| path.gsub(/\\/,"\/")==current_tex}
#p "new_tex set to #{new_tex}"
@tex.path=new_tex

#the map used to be applied by the material editor class.  however, this does not work on Mac because web dialogs are asynchronous.  We must apply the map inside this class for it to work on Mac
case @map_type
when "albedo"
	@mat.albedo_texture=@tex
when "bump"
	@mat.bump_map=@tex
	@mat.use_bump=true if @tex.valid?
when "normal"
	@mat.normal_map=@tex
	@mat.use_normal=true if @tex.valid?
when "exponent"
	@mat.exponent_map=@tex
when "displacement"
	@mat.disp_map=@tex
	@mat.use_displace=true if @tex.valid?
when "clip"
	@mat.clip_map=@tex
	@mat.use_clip=true if @tex.valid?
when "emission"
	@mat.emission_map=@tex
	@mat.use_emission=true if @tex.valid?
when "transmittance"
	@mat.transmittance_map=@tex
when "specular"
	@mat.specular_map=@tex
	@mat.use_specular=true if @tex.valid?
when "blend"
	@mat.blend_map=@tex
when "sigma"
	@mat.sigma_map=@tex
when "roughness"
	@mat.roughness_map=@tex
when "fresnel"
	@mat.fresnel_map=@tex
when "thickness"
	@mat.thickness_map=@tex
when "r_f"
	@mat.r_f_map=@tex
when "front fresnel"
	@mat.front_fresnel_scale_map=@tex
when "back fresnel"
	@mat.back_fresnel_scale_map=@tex
when "front roughness"
	@mat.front_roughness_map=@tex
when "back roughness"
	@mat.back_roughness_map=@tex
when "abs_layer_trans"
	@mat.abs_layer_trans_map=@tex
	@mat.use_abs_layer_trans=true if @tex.valid?
end

end

def show
	@dlg.show_modal
end

def setVisibility()
	type=@tex.type
	case type
		when "Constant","SketchUp"
			@dlg.execute_script("setSketchUpVisible()")
		when "External"
			@dlg.execute_script("setExternalVisible()")
	end
	case @map_type
		when "bump","exponent","displacement","clip","blend","sigma"
			@dlg.execute_script("$('#smoothTableData').attr('disabled', false);")
		else
			@dlg.execute_script("$('#smoothTableData').attr('disabled', true);")
	end
end

def getModelTextures()

@model_textures=@irs.get_model_textures
@model_textures.each {|path|
	add_map_option(path)
	#add_map_option(File.basename(path))
}
end

def add_map_option(path)
	path=path.gsub(/\\/,"\/")
	cmd="createMapOption(\"TexturePath\",\"#{path}\",\"#{path}\")"
	@dlg.execute_script(cmd)
end

def loadValuesFromSU()
	#p "getting values from su"
	tex_path=@tex.path.gsub(/\\/,"\/")
	p "setting texture path #{tex_path}"
	setValue("TexturePath",tex_path) if @tex.type=="External"
	setValue("TextureName",@mat.display_name) if @tex.type=="SketchUp"
	setValue("TextureName","") if @tex.type=="Constant"
	setValue("aValue",@tex.a)
	setValue("bValue",@tex.b)
	setValue("cValue",@tex.c)
	setValue("textureExponent",@tex.exponent)
	setValue("uvset",@tex.uv_set)
	if (@tex.smooth=="On") then setCheckbox("smooth","true")
		else setCheckbox("smooth","false")
	end
	setThumbnail()
	
end

def setThumbnail()

if @thumb
	@thumb.gsub!(/\\/,"\/")
	#cmd="document.images[\"TextureImage\"].src=\"nopreview.png\""  #this is required so the preview is refreshed
	#@dlg.execute_script(cmd)
	#p "setting thumbnail #{@thumb}"
	cmd="document.images[\"TextureImage\"].src=\"file:\/\/#{@thumb}?#{(rand*10000).to_i}\""
	@dlg.execute_script(cmd)
end

end

def setValue(id,value)
	new_value=value.to_s
	cmd="document.getElementById('#{id}').value=\"#{new_value}\""
	@dlg.execute_script(cmd)
end

def getThumbnail()
	case @tex.type
		when "Constant"
			return nil
		when "External"
			return @tex.path
		when "SketchUp"
			mat=@mat.mat  
			status=nil
			thumb=@temp_texture_path   #set the exported texture path to the user's temp directory
			#p thumb
			return save_mat_texture(mat,thumb)
	end
end

def setCheckbox(id,checked)
	cmd="document.getElementById('#{id}').checked=#{checked}"
	@dlg.execute_script(cmd)
end

end #indigo texture editor class



#################################Shader Editor Class#############################
#todo - put in separate file
class IndigoShaderEditor

def initialize(texture,material,map_type=nil)   #tex is an IndigoTexture object - material is an IndigoMaterial object
	@tex=texture
	@mat=material
	@map_type=map_type
	@dlg=UI::WebDialog.new("SkIndigo Shader Editor", true, "SkIndigoShaderEditor",630,350,419,243,true) #width,height,left,top
	@dlg.set_file(File.join(File.dirname(__FILE__),"ShaderEditor.html"))
	color = @dlg.get_default_dialog_color 
	@dlg.set_background_color(color)
	@dlg.add_action_callback("apply_settings") {|d,p|
		apply_settings(d)
		#@tex.shader_code=d.get_element_value("shaderCode")
		d.close
	}
	@dlg.add_action_callback("cancel_changes") {|d,p| d.close}
	@dlg.show_modal {loadValuesFromSU()}
end

def show
	@dlg.show_modal
end

def loadValuesFromSU()
	code=@tex.shader_code
	#the following substitutions are necessary due to a strange issue with html textarea elements and newline characters
	code.gsub!(/\n/,"\\n")
	code.gsub!(/\r/,"\\r")
	setValue("shaderCode",code)
end

def setValue(id,value)
	new_value=value.to_s
	cmd="document.getElementById('#{id}').value=\"#{new_value}\""
	@dlg.execute_script(cmd)
end

def apply_settings(d) #d is the dialog

@tex.shader_code=d.get_element_value("shaderCode")
#the map used to be applied by the material editor class.  however, this does not work on Mac because web dialogs are asynchronous.  We must apply the map inside this class for it to work on Mac
case @map_type
when "albedo"
	@mat.albedo_texture=@tex
when "bump"
	@mat.bump_map=@tex
when "normal"
	@mat.normal_map=@tex
when "exponent"
	@mat.exponent_map=@tex
when "displacement"
	@mat.disp_map=@tex
when "clip"
	@mat.clip_map=@tex
when "emission"
	@mat.emission_map=@tex
when "specular"
	@mat.specular_map=@tex
when "blend"
	@mat.blend_map=@tex
when "sigma"
	@mat.sigma_map=@tex
when "abs_layer_trans"
	@mat.abs_layer_trans_map=@tex
when "absorption"
	@mat.absorption_map=@tex
when "sss"
	@mat.sss_map=@tex
end


end

end #indigo shader editor class

class IndigoMaterial

include SkIndigoUVS

#These class variables are used as the material defaults
@@dict="indigo07_materials"  #do not change this dictionary (needed for backward compatibility)
@@type="Diffuse"
@@ior=1.52
@@cau=0.0
@@fres=1.0
@@sigma=0.5
@@power=100
@@ole=17.5
@@exp=1000
@@precedence=10
@@temp=5000
#@@g_spectrum=1.0
@@g_spectrum=IndigoTexture.new.default_g_spectrum.to_s
@@no_smooth="No"
@@ies="No"
@@ies_path=""
@@sss="No"
#@@sssval=0.0
#@@sssval=IndigoTexture.new.default_sss_val.to_s
@@phase_function="Uniform"
@@albedo_texture=IndigoTexture.new.default_albedo.to_s
@@blend_map=IndigoTexture.new.default_blend.to_s
#@@bump_map="None|Default|0.0|0.0|0.0|1.0"
@@bump_map=IndigoTexture.new.default_bump.to_s
@@normal_map=IndigoTexture.new.default_normal.to_s
@@exp_map=IndigoTexture.new.default_exponent.to_s
@@clip_map=IndigoTexture.new.default_clip.to_s
@@disp_map=IndigoTexture.new.default_displacement.to_s
@@emission_map=IndigoTexture.new.default_emission.to_s
@@sigma_map=IndigoTexture.new.default_sigma.to_s
@@roughness_map=IndigoTexture.new.default_roughness.to_s
@@fresnel_map=IndigoTexture.new.default_fresnel.to_s
@@thickness_map=IndigoTexture.new.default_thickness.to_s
@@specular_map=IndigoTexture.new.default_specular.to_s
@@abs_layer_trans=IndigoTexture.new.default_abs_layer_trans.to_s
@@absorption_map=IndigoTexture.new.default_absorption.to_s
@@sss_map=IndigoTexture.new.default_sss_map.to_s
@@g_spectrum_map=IndigoTexture.new.default_g_spectrum.to_s
#@@base_emission=[255,255,255]
@@base_emission=IndigoTexture.new.default_base_emission.to_s
@@base_emission_gain=0.0
@@base_emission_spectrum="Blackbody"
@@layer=-1
@@thumbnail=""
@@view_dependent=true
@@max_subdiv=10
@@pixel_threshold=4.0
@@curvature_threshold=0.1
@@sub_smooth=false
@@linked_igm=""
@@use_bump=false
@@use_normal=false
@@use_displace=false
@@use_emission=false
@@use_abs_layer_trans=false
@@use_clip=false
@@use_specular=true
@@step_blend=false
@@matA="None"
@@matB="None"
@@blendfactor=0.5
@@use_nkdata="No"
@@emission_scale=1750.0
@@emission_measure="luminous_flux"
@@substrate="None"
@@front_material="None"
@@back_material="None"
@@interference=false

@@defaults={'front_material_name'=>'None','back_material_name'=>'None'}
@@transmittance_map=IndigoTexture.new.default_transmittance.to_s
@@front_roughness=IndigoTexture.new.default_roughness.to_s
@@back_roughness=IndigoTexture.new.default_roughness.to_s
@@front_fresnel_scale=IndigoTexture.new.default_fresnel.to_s
@@back_fresnel_scale=IndigoTexture.new.default_fresnel.to_s
@@r_f=IndigoTexture.new.default_r_f.to_s


attr_reader :mat

def initialize(su_material)
	@mat=su_material
	#following code is for backward compatibility
	dicts=@mat.attribute_dictionaries if @mat
	return if not dicts
	if dicts["indigo_blend"]
		dict=dicts["indigo_blend"]
		self.blend_factor=dict["blendfactor"]
		self.a_name=dict["matA"]
		self.b_name=dict["matB"]
		dicts.delete(dict)
	end
	if dicts["indigo07_materials"]
		dict=dicts["indigo07_materials"]
		keys=dict.keys
		if keys.index("bump_value")
			b=dict["bump_value"]
			b=0.0 if not b
			dict.delete_key("bump_value")
			bm=self.bump_map
			bm.b=b
			self.bump_map=bm
		end
		if keys.index("expmap_value")
			b=dict["expmap_value"]
			b=1000 if not b
			dict.delete_key("expmap_value")
			em=self.exponent_map
			em.b=b
			self.exponent_map=em
		end
		if keys.index("exp")
			exp=dict["exp"]
			exp=1000 if not exp
			dict.delete_key("exp")
			em=self.exponent_map
			em.b=exp
			self.exponent_map=em
		end
		if keys.index("temp")  #apply temperature attribute to the base emission black body temperature
			temp=dict["temp"]
			temp=5000 if not temp
			dict.delete_key("temp")
			be=self.base_emission
			be.bb_temperature=temp
			self.base_emission=be
		end	
		if keys.index("g_spectrum") #new implementation of g spectrum in SkIndigo 2.4
			gs=dict["g_spectrum"]
			gs=1.0 if not gs
			gs=gs.to_s.split(' ')
			g_spectrum_map=self.g_spectrum_map
			if gs.length==3 #rgb spectrum
				g_spectrum_map.rgb_r=gs[0].to_f
				g_spectrum_map.rgb_g=gs[1].to_f
				g_spectrum_map.rgb_b=gs[2].to_f
				g_spectrum_map.spectrum_type="RGB"
			else
				g_spectrum_map.uni_value=gs[0].to_f
				g_spectrum_map.type="Uniform"
			end
			self.g_spectrum_map=g_spectrum_map
			dict.delete_key("g_spectrum")
		end
		if keys.index("sssval")  #new implementation of sss in SkIndigo 2.4
			sss=dict["sssval"]
			sss=0.0 if not sss
			sss=sss.to_s.split(' ')
			sss_map=self.sss_map
			if sss.length==3 #rgb spectrum
				sss_map.rgb_r=sss[0].to_f
				sss_map.rgb_g=sss[1].to_f
				sss_map.rgb_b=sss[2].to_f
				sss_map.spectrum_type="RGB"
			else
				sss_map.uni_value=sss[0].to_f
				sss_map.type="Uniform"
			end
			self.sss_map=sss_map
			dict.delete_key("sssval")
		end
		if keys.index("base_emission")
			be=dict["base_emission"]
			be=[0.0,0.0,0.0] if not be
			new_be=self.base_emission
			new_be.rgb_r=be[0]
			new_be.rgb_g=be[1]
			new_be.rgb_b=be[2]
			self.base_emission=new_be
			dict.delete_key("base_emission")
		end
		if keys.index("base_emission_spectrum")
			bes=dict["base_emission_spectrum"]
			bes="Blackbody" if not bes
			be=self.base_emission
			if bes=="Blackbody"
				be.spectrum_type="Blackbody"
			else
				be.spectrum_type="RGB"
			end
			self.base_emission=be
			dict.delete_key("base_emission_spectrum")
		end
		if keys.index("power")
			power=dict["power"]
			power=100.0 if not power
			ole=dict["ole"]  #the overall luminous efficacy
			ole=17.5 if not ole
			self.emission_scale=power.to_f*ole.to_f
			self.emission_measure="luminous_flux"
			dict.delete_key("power")
		end
		# backward compatiblity for the checkboxes added to certain material parameters in version 2.4
		#backward compatiblity code disabled for now - adds much complexity for probably little gain
=begin	
		unless keys.index("use_abs_layer_trans")
			altm=self.abs_layer_trans_map
			if altm.type=="None"
				self.use_abs_layer_trans=false
				altm.type="Constant"
				self.abs_layer_trans_map=altm
			end		
		end
		unless keys.index("use_specular")
			sm=self.specular_map
			if sm.type=="None"
				self.use_specular=false
				sm.type="Constant"
				self.specular_map=sm
			end
		end
		unless keys.index("use_bump")
			bm=self.bump_map
			case bm.type
				when "None"
					self.use_bump=false
					bm.type="Constant"
					self.bump_map=bm
				when "SketchUp","Shader"
					self.use_bump=true
				when "External"
					self.use_bump=true if FileTest.exist?(bm.path)
			end
		end
		unless keys.index("use_displace")
			dm=self.disp_map
			if dm.type=="None"
				self.use_displace=false
				dm.type="Constant"
				self.disp_map=dm
			end
		end
		unless keys.index("use_emission")
			em=self.emission_map
			if em.type=="None"
				self.use_emission=false
				em.type="Constant"
				self.emission_map=em
			end
		end
		unless keys.index("use_clip")
			cm=self.clip_map
			if cm.type=="None"
				self.use_clip=false
				cm.type="Constant"
				self.clip_map=sm
			end
		end
=end				
		#following code is due to deprecation of blackbody and rgb lights - now are converted to light-emitting materials
		t=self.type
		if (t=="Blackbody Light") or (t=="RGB Light")
			be=self.base_emission
			be.spectrum_type="Blackbody" if t =="Blackbody Light"
			be.spectrum_type="RGB" if t=="RGB Light"
			self.base_emission=be
			self.type="Diffuse"  #change the type to diffuse
			self.layer=0         #set the layer to 0 to ensure the material will emit light when rendering
		end
	end
	
	
	
	if dicts["uvsets"]
		dict=dicts["uvsets"]
		keys=dict.keys
		if keys.index("albedo")
			set=dict["albedo"]
			set="Default" if not set
			dict.delete_key("albedo")
			am=self.albedo_texture
			am.uv_set=set
			self.albedo_texture=am
		end
		if keys.index("bump")
			set=dict["bump"]
			set="Default" if not set
			dict.delete_key("bump")
			bm=self.bump_map
			bm.uv_set=set
			self.bump_map=bm
		end
		if keys.index("exponent")
			set=dict["exponent"]
			set="Default" if not set
			dict.delete_key("exponent")
			em=self.exponent_map
			em.uv_set=set
			self.exponent_map=em
		end
		if keys.index("blend")
			set=dict["blend"]
			set="Default" if not set
			dict.delete_key("blend")
			bm=self.blend_map
			bm.uv_set=set
			self.blend_map=bm
		end
		if keys.index("clip")
			set=dict["clip"]
			set="Default" if not set
			dict.delete_key("clip")
			cm=self.clip_map
			cm.uv_set=set
			self.clip_map=cm
		end
		dicts.delete(dict)
	end
	
end

def reset() #deletes all indigo attributes

	IndigoMaterial.reset_indigo_attributes(@mat)

end


# We'll make this a class method so it can operate on SketchUp materials without having to create an Indigo material first.
def IndigoMaterial.reset_indigo_attributes(skup_mat)
	
	dicts = skup_mat.attribute_dictionaries
	if dicts != nil
		dicts.delete(@@dict)
	end
end


def IndigoMaterial.print_out_attributes(skup_mat)
	puts "attributes for sketchup material #{skup_mat}..."
	puts "skup_mat.attribute_dictionaries: #{skup_mat.attribute_dictionaries.inspect}"
	if !skup_mat.attribute_dictionaries.nil?
		skup_mat.attribute_dictionaries.each do |dict|
			dict.each_pair do |key, value|
				puts [dict,key,value]
			end
		end
	end
end

###delete this material - code used from TIG's material-delete ruby snippet
def delete!

model=Sketchup.active_model
ents=model.entities; mats=model.materials
group=ents.add_group();gents=group.entities ### temporarily use other materials
temp=gents.add_group()
(mats.to_a-[@mat]).each{|m|tc=temp.copy;tc.material=m}
mats.current=nil if mats.current==@mat ### ensure self is not current material
mats.purge_unused ### purge self from browser
group.erase! ### erase! the temporary material user, use set as was.

end

###copy parameters from this Indigo material to another
def copy(im2)

im2.reset()
mat1=self.mat
mat2=im2.mat  #get the su material of the 
dicts=mat1.attribute_dictionaries
dicts.each {|dict|
	dict.each_pair {|key,value|
		mat2.set_attribute(dict.name,key,value)
		#p [dict,key,value]
	}
}
mat2.color=mat1.color
if mat1.texture!=nil
	tex_path=mat1.texture.filename
	if FileTest.exist?(tex_path)
		mat2.texture=mat1.texture.filename
	else
		temp=SkIndigo.temp_dir
		tex_path=File.join(temp,mat1.texture.filename)
		tex_path=save_mat_texture(mat1,tex_path)
		if tex_path!=nil and FileTest.exist?(tex_path)
			mat2.texture=tex_path
		end
	end
end

mat2.alpha=mat1.alpha

end

####################Method to find a SU material based on name
#note: prior to Indigo 2.0.X, this method was based on display_name, not name.
#this was changed to eliminate duplicate material naming conflicts
def IndigoMaterial.find(name) 
	materials=Sketchup.active_model.materials
	mat=materials[name]
	if mat
		return IndigoMaterial.new(mat)
	else
		return nil
	end
end

#####################################returns and existing material or creates a new material with the given texture image
#method was used for internal texture support of previous skindigo versions - not used currently
def IndigoMaterial.find_texture_file(path)

filename=File.basename(path)
mats=Sketchup.active_model.materials
textures=mats.collect {|m| (m.texture ? File.basename(m.texture.filename) : nil)}
names=mats.collect {|m| m.display_name}
index=textures.index(filename)
if index
	return IndigoMaterial.new(mats[index])
else
	new_mat=mats.add(File.basename(filename,".*"))
	if FileTest.exist?(path)
		new_mat.texture=path
		UI.messagebox "New texture material created name #{new_mat.display_name}"
		return IndigoMaterial.new(new_mat)
	else
		UI.messagebox "Could not find or create material with texture #{filename}"
		return nil
	end
end

end

#optimized method to check or emitters without creating an IndigoMaterial object
def IndigoMaterial.emitter?(su_mat)

return false if su_mat==nil
layer=su_mat.get_attribute(@@dict,'layer',-1)
type=su_mat.get_attribute(@@dict,'type',"Diffuse") 
return false if type=="Blend"
if layer!=(-1)
	return true
else
	return false
end
	
end

#####new attribute setting and retrievel methods test
def get(attr,texture=nil)

val=mat.get_attribute(@@dict,attr,@@defaults[attr])
if texture!=nil
	return IndigoTexture.new(val)
else
	return val
end

end

##
def set(attr,val,texture=nil)

if texture!=nil
	mat.set_attribute(@@dict,attr,val.to_s)
else
	mat.set_attribute(@@dict,attr,val)
end

end

def name
	#this method used to replace <> characters with a * but now just returns the SketchUp material name
	return @mat.name
end

def xml_name
	return @mat.name
	#method previously substituted for GT and LT signs but now these strings will be wrapped in CDATA tags by another method
	#return @mat.name.gsub(/</,'&lt;').gsub(/>/,'&gt;')
end


def rgb
	return mat.color.red,mat.color.green,mat.color.blue
end
########################Attribute read and writer functions########################33
def type
	t=mat.get_attribute(@@dict,'type')  #check if the type has been set
	if not t
		if mat.alpha<1.0
			return "Specular"
		else
			return @@type
		end
	else
		return t
	end
end

def indigo_type
	t=self.type
	case t
	when "Diffuse"
		if mat.alpha==0 
			xml="diffuse_transmitter"
		else
			xml="diffuse"
		end
	when "Specular","Phong","Blend","Coating"
		xml=t.downcase
	when "Metals"
		xml="phong"
	when "Glossy Transparent"
		xml="glossy_transparent"
	when "Oren-Nayar"
		xml="oren_nayar"
	when "Double-Sided Thin"
		xml="double_sided_thin"
	when "Exit Portal"
		xml="exit_portal\/"
	else
		xml=t
	end
end

def texture
	mat.texture
end

def texture=(value)
	mat.texture=(value)
end

def thumbnail
	mat.get_attribute(@@dict,'thumbnail',@@thumbnail)
end

def thumbnail=(path)
	mat.set_attribute(@@dict,'thumbnail',path)
end

def type=(value)
	mat.set_attribute(@@dict,'type',value)
end

def ior
	mat.get_attribute(@@dict,'ior',@@ior)
end

def ior=(value)
	mat.set_attribute(@@dict,'ior',value)
end

def fresnel_scale
	mat.get_attribute(@@dict,'fres',@@fres)
end

def fresnel_scale=(value)
	mat.set_attribute(@@dict,'fres',value)
end

def sigma
	mat.get_attribute(@@dict,'sigma',@@sigma)	
end

def sigma=(value)
	mat.set_attribute(@@dict,'sigma',value)
end

def cauchy_b_coeff
	mat.get_attribute(@@dict,'cau',@@cau)
end

def cauchy_b_coeff=(value)
	mat.set_attribute(@@dict,'cau',value)
end

def power_drawn
	mat.get_attribute(@@dict,'power',@@power)
end

def power_drawn=(value)
	mat.set_attribute(@@dict,'power',value)
end

def overall_luminous_efficacy
	mat.get_attribute(@@dict,'ole',@@ole)
end

def overall_luminous_efficacy=(value)
	mat.set_attribute(@@dict,'ole',value)
end
	
def exponent
	mat.get_attribute(@@dict,'exp',@@exp)
end

def exponent=(value)
	#mat.set_attribute(@@dict,'exp',value)
	em=self.exponent_map
	em.b=value
	self.exponent_map=em
end

def precedence
	mat.get_attribute(@@dict,'precedence',@@precedence)
end

def precedence=(value)
	mat.set_attribute(@@dict,'precedence',value)
end

def temperature
	mat.get_attribute(@@dict,'temp',@@temp)
end

def temperature=(value)
	mat.set_attribute(@@dict,'temp',value)
end

def g_spectrum
	mat.get_attribute(@@dict,'g_spectrum',@@g_spectrum)
end

def g_spectrum=(value)
	mat.set_attribute(@@dict,'g_spectrum',value)
end

def smooth?  #tas of version 1.1.1 this method returns whether subdivision smoothing is enabled (no longer used for normal smoothing)
	mat.get_attribute(@@dict,'sub_smooth',@@sub_smooth)
end

def smooth=(value)
	if value
		mat.set_attribute(@@dict,'sub_smooth',true)  #enables subdivision smoothing
	else
		mat.set_attribute(@@dict,'sub_smooth',false)  #disables subdivision smoothing
	end
end

def ies?
	use_ies=mat.get_attribute(@@dict,'ies',@@ies)
	return true if use_ies=="Yes"
	return false if use_ies=="No"
end

def ies=(value)
	if value
		mat.set_attribute(@@dict,'ies',"Yes")  #enable ies profile
	else
		mat.set_attribute(@@dict,'ies',"No")  #disables ies profile
	end
end

def use_nkdata?
	use_nkdata=mat.get_attribute(@@dict,'use_nkdata',@@use_nkdata)
	return true if use_nkdata=="Yes"
	return false if use_nkdata=="No"
end

def use_nkdata=(value)
	if value
		mat.set_attribute(@@dict,'use_nkdata',"Yes")
	else
		mat.set_attribute(@@dict,'use_nkdata',"No")
	end
end

def ies_profile
	path=mat.get_attribute(@@dict,'ies_path',@@ies_path)
	return IESProfile.new(path)
end

def ies_profile=(value)  #value is an IESProfile object
	mat.set_attribute(@@dict,'ies_path',value.path)
end

def linked_igm
	path=mat.get_attribute(@@dict,'linked_igm',@@linked_igm)
	return path
end

def linked_igm=(value)
	value=value.gsub(/\\/,"\/")
	mat.set_attribute(@@dict,'linked_igm',value)
end

def valid_igm_reference?
	path=self.linked_igm.to_s
	if ((FileTest.exist?(path)) or (path==""))
		return true
	else
		return false
	end
end

def absorption_spectrum=(value)

#p value
rgb_ab=value.split(" ")
#p rgb_ab
if rgb_ab.length==1  #uniform
	#todo
elsif rgb_ab.length==3 #rgb
	red_ab=rgb_ab[0].to_f
	green_ab=rgb_ab[1].to_f
	blue_ab=rgb_ab[2].to_f
	opacity=(Math.sqrt([red_ab,green_ab,blue_ab].max*10)).round
	opacity=99 if opacity>99
	opacity=1 if opacity<1
	red=((1-(10*red_ab)/(opacity**2))*255).round
	red=0 if red<0
	red=255 if red>255
	blue=((1-(10*blue_ab)/(opacity**2))*255).round
	blue=0 if blue<0
	blue=255 if blue>255
	green=((1-(10*green_ab)/(opacity**2))*255).round
	green=0 if green<0
	green=255 if green>255
	set_alpha=opacity.to_f/100
	self.alpha=set_alpha
	self.color=[red,green,blue]
end

end

def absorption_spectrum

opacity=mat.alpha
alpha_factor=opacity*opacity*1000
rgb=self.rgb
absorption="%.4f" %((1.0-rgb[0]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-rgb[1]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-rgb[2]/255.0)*alpha_factor)
#absorption=((1.0-rgb[0]/255.0)*alpha_factor),((1.0-rgb[1]/255.0)*alpha_factor),((1.0-rgb[2]/255.0)*alpha_factor)
return absorption.split(" ")

end

def absorption_color

rgb=self.rgb
return [255.0-rgb[0],255.0-rgb[1],255.0-rgb[2]]

end

def sss?
	use_sss=mat.get_attribute(@@dict,'sss',@@sss)
	return true if use_sss=="Yes"
	return false if use_sss=="No"
end

def sss=(value)
	if value
		mat.set_attribute(@@dict,'sss',"Yes")  #enable sub-surface scattering
	else
		mat.set_attribute(@@dict,'sss',"No")  #disables sub-surface scattering
	end
end

def sss_spectrum
	mat.get_attribute(@@dict,'sssval',@@sssval)
end

def sss_spectrum=(value)
	#TODO validation
	mat.set_attribute(@@dict,'sssval',value)
end	

def phase_function
	mat.get_attribute(@@dict,'phase_function',@@phase_function)
end

def phase_function=(value)
	mat.set_attribute(@@dict,'phase_function',value)
end

def albedo_texture?
	t=self.type
	case t
	when "Specular","Glossy Transparent","Blend","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM","Metals"
		return false
	else
		return texture?(self.albedo_texture)
	end
			
end

def albedo_shader?
	t=self.type
	case t
	when "Specular","Glossy Transparent","Blend","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM","Metals"
		return false
	else
		return self.albedo_texture.procedural?
	end
end


def albedo_texture
	tex=IndigoTexture.new(mat.get_attribute(@@dict,'albedo_map',@@albedo_texture))
	return tex
end

def albedo_texture=(tex)  
	mat.set_attribute(@@dict,'albedo_map',tex.to_s)	
end


def abs_layer_trans_map?
	return false if !use_abs_layer_trans?
	t=self.type
	case t
	when "Specular","Glossy Transparent"
		return texture?(self.abs_layer_trans_map)
	else
		return false
	end
			
end

def abs_layer_trans_shader?
	t=self.type
	case t
	when "Specular","Glossy Transparent"
		return self.abs_layer_trans_map.procedural?
	else
		return false
	end
end


def abs_layer_trans_map
	tex=IndigoTexture.new(mat.get_attribute(@@dict,'abs_layer_trans',@@abs_layer_trans))
	return tex
end

def abs_layer_trans_map=(tex)  
	mat.set_attribute(@@dict,'abs_layer_trans',tex.to_s)	
end

def use_abs_layer_trans?
	value=mat.get_attribute(@@dict,'use_abs_layer_trans',@@use_abs_layer_trans)
	value=false unless self.abs_layer_trans_map.valid?
	return value
end

def use_abs_layer_trans=(value)
	mat.set_attribute(@@dict,'use_abs_layer_trans',value)
end

def specular_map?
	return false if !use_specular?
	t=self.type
	case t
	when "Metals"
		return texture?(self.specular_map)
	else
		return false
	end
			
end

def specular_shader?
	t=self.type
	case t
	when "Metals"
		return self.specular_map.procedural?
	else
		return false
	end
end


def specular_map
	tex=IndigoTexture.new(mat.get_attribute(@@dict,'specular_map',@@specular_map))
	return tex
end

def specular_map=(tex)  
	mat.set_attribute(@@dict,'specular_map',tex.to_s)	
end

def use_specular?
	value=mat.get_attribute(@@dict,'use_specular',@@use_specular)
	value=false unless self.specular_map.valid?
	return value
end

def use_specular=(value)
	mat.set_attribute(@@dict,'use_specular',value)
end

def blend_map?
	t=self.type
	case t
	when "Blend"
		return texture?(self.blend_map)
	else
		return false
	end
end

def blend_shader?
	t=self.type
	case t
	when "Blend"
		return self.blend_map.procedural?
	else
		return false		
	end
end

def blend_map
	tex=IndigoTexture.new(mat.get_attribute(@@dict,'blend_map',@@blend_map))
	return tex
end

def blend_map=(tex)  
	mat.set_attribute(@@dict,'blend_map',tex.to_s)	
end

def step_blend?
	mat.get_attribute(@@dict,'step_blend',@@step_blend)
end

def step_blend=(value)
	mat.set_attribute(@@dict,'step_blend',value)
end

def interference?
	mat.get_attribute(@@dict,'interference',@@interference)
end

def interference=(value)
	mat.set_attribute(@@dict,'interference',value)
end

def bump_map
	tex=mat.get_attribute(@@dict,'bump_map',@@bump_map)
	return IndigoTexture.new(tex)
end

def bump_map=(tex)  #tex is an IndigoTexture object
	mat.set_attribute(@@dict,'bump_map',tex.to_s)	
end

def bump_map?
	return false if !use_bump?
	t=self.type
	case t
	when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		#return false if self.indigo_type=="diffuse_transmitter"
		return texture?(self.bump_map)
	end
end

def bump_shader?
	t=self.type
	case t
	when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		#return false if self.indigo_type=="diffuse_transmitter"
		return self.bump_map.procedural?
	end
end

def use_bump?
	value=mat.get_attribute(@@dict,'use_bump',@@use_bump)
	value=false unless self.bump_map.valid?
	value=false if self.type=="Blend"
	return value
end

def use_bump=(value)
	mat.set_attribute(@@dict,'use_bump',value)
end

def normal_map
	tex=mat.get_attribute(@@dict,'normal_map',@@normal_map)
	return IndigoTexture.new(tex)
end

def normal_map=(tex)  #tex is an IndigoTexture object
	mat.set_attribute(@@dict,'normal_map',tex.to_s)	
end

def normal_map?
	return false if !use_normal?
	t=self.type
	case t
	when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		#return false if self.indigo_type=="diffuse_transmitter"
		return texture?(self.normal_map)
	end
end

def normal_shader?
	t=self.type
	case t
	when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		#return false if self.indigo_type=="diffuse_transmitter"
		return self.normal_map.procedural?
	end
end

def use_normal?
	value=mat.get_attribute(@@dict,'use_normal',@@use_normal)
	value=false unless self.normal_map.valid?
	value=false if self.type=="Blend"
	return value
end

def use_normal=(value)
	mat.set_attribute(@@dict,'use_normal',value)
end

def disp_map
	tex=mat.get_attribute(@@dict,'disp_map',@@disp_map)
	return IndigoTexture.new(tex)
end

def disp_map=(tex)
	mat.set_attribute(@@dict,'disp_map',tex.to_s)
end

def disp_map?
	return false if !use_displace?
	t=self.type
	case t
	when "RGB Light","Blackbody Light","Exit Portal","Linked IGM", "Blend"
		return false
	else
		return texture?(self.disp_map)
	end
end

def disp_shader?
	t=self.type
	case t
	when "RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		return self.disp_map.procedural?
	end

end

def use_displace?
	value=mat.get_attribute(@@dict,'use_displace',@@use_displace)
	value=false unless self.disp_map.valid?
	value=false if self.type=="Blend"
	return value
end

def use_displace=(value)
	mat.set_attribute(@@dict,'use_displace',value)
end



def emitter?
	t=self.type
	case t
	when "Blend"
		return false
	else
		#gain=self.base_emission_gain
		#if gain.to_f>0.0 and self.layer!="-1"
		if self.layer!=(-1)
			return true
		else
			return false
		end
	end
end	

def base_emission
tex=mat.get_attribute(@@dict,'base_emission2',@@base_emission)
tex=IndigoTexture.new(tex)
return tex
end

def base_emission=(tex)
	mat.set_attribute(@@dict,'base_emission2',tex.to_s)
end
=begin
def base_emission  #returns a three element array representing the rgb color
	default=mat.color  #default base emission is the material color
	mat.get_attribute(@@dict,'base_emission',[default.red,default.green,default.blue])
end

def base_emission=(rgb)  #rgb is a three float array representing the rgb color
	mat.set_attribute(@@dict,'base_emission',rgb)
end

def base_emission_gain
	mat.get_attribute(@@dict,'base_emission_gain',@@base_emission_gain)
end

def base_emission_gain=(value)
	mat.set_attribute(@@dict,'base_emission_gain',value.to_f)
end

def base_emission_spectrum
	mat.get_attribute(@@dict,'base_emission_spectrum',@@base_emission_spectrum)
end

def base_emission_spectrum=(value)
	mat.set_attribute(@@dict,'base_emission_spectrum',value)
end
=end

def emission_map
	tex=mat.get_attribute(@@dict,'emission_map',@@emission_map)
	return IndigoTexture.new(tex)
end

def emission_map=(tex)
	mat.set_attribute(@@dict,'emission_map',tex.to_s)
end

def emission_map?
	return false if !use_emission?
	t=self.type
	case t
	when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		return texture?(self.emission_map)
	end
end

def emission_shader?
	t=self.type
	case t
	when "Blend","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		return self.emission_map.procedural?
	end

end

def use_emission?
	value=mat.get_attribute(@@dict,'use_emission',@@use_emission)
	value=false unless self.emission_map.valid?
	value=false if self.type=="Blend"
	return value
end

def use_emission=(value)
	mat.set_attribute(@@dict,'use_emission',value)
end

####
def transmittance_map
	tex=mat.get_attribute(@@dict,'transmittance_map',@@transmittance_map)
	return IndigoTexture.new(tex)
end

def transmittance_map=(tex)
	mat.set_attribute(@@dict,'transmittance_map',tex.to_s)
end

def transmittance_map?
	t=self.type
	case t
	when "Double-Sided Thin"
		return texture?(self.transmittance_map)
	else
		false
	end
end

def transmittance_shader?
	t=self.type
	case t
	when "Double-Sided Thin"
		return self.transmittance_map.procedural?
	else
		false
	end

end

####
def absorption_map
	tex=mat.get_attribute(@@dict,'absorption_map',@@absorption_map)
	return IndigoTexture.new(tex)
end

def absorption_map=(tex)
	mat.set_attribute(@@dict,'absorption_map',tex.to_s)
end

####
def sss_map
	tex=mat.get_attribute(@@dict,'sss_map',@@sss_map)
	return IndigoTexture.new(tex)
end

def sss_map=(tex)
	mat.set_attribute(@@dict,'sss_map',tex.to_s)
end

####
def g_spectrum_map
	tex=mat.get_attribute(@@dict,'g_spectrum_map',@@g_spectrum_map)
	return IndigoTexture.new(tex)
end

def g_spectrum_map=(tex)
	mat.set_attribute(@@dict,'g_spectrum_map',tex.to_s)
end
######
def sigma_map
	tex=mat.get_attribute(@@dict,'sigma_map',@@sigma_map)
	return IndigoTexture.new(tex)
end

def sigma_map=(tex)
	mat.set_attribute(@@dict,'sigma_map',tex.to_s)
end

def sigma_map?
	t=self.type
	case t
	when "Diffuse","Specular","Phong","Glossy Transparent","Blend","Metals","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		return texture?(self.sigma_map)
	end
end

def sigma_shader?

	t=self.type
	case t
	when "Diffuse","Specular","Phong","Glossy Transparent","Blend","Metals","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		return self.sigma_map.procedural?
	end

end
######
def roughness_map
	tex=mat.get_attribute(@@dict,'roughness_map',@@roughness_map)
	return IndigoTexture.new(tex)
end

def roughness_map=(tex)
	mat.set_attribute(@@dict,'roughness_map',tex.to_s)
end

def roughness_map?
	t=self.type
	case t
	when "Coating"
		return texture?(self.roughness_map)
	else
		return false
	end
end

def roughness_shader?

	t=self.type
	case t
	when "Coating"
		return self.roughness_map.procedural?
	else
		return false
	end
end

######
def front_roughness_map
	tex=mat.get_attribute(@@dict,'front_roughness_map',@@front_roughness)
	return IndigoTexture.new(tex)
end

def front_roughness_map=(tex)
	mat.set_attribute(@@dict,'front_roughness_map',tex.to_s)
end

def front_roughness_map?
	t=self.type
	case t
	when "Double-Sided Thin"
		return texture?(self.front_roughness_map)
	else
		return false
	end
end

def front_roughness_shader?

	t=self.type
	case t
	when "Double-Sided Thin"
		return self.front_roughness_map.procedural?
	else
		return false
	end
end

######
def back_roughness_map
	tex=mat.get_attribute(@@dict,'back_roughness_map',@@back_roughness)
	return IndigoTexture.new(tex)
end

def back_roughness_map=(tex)
	mat.set_attribute(@@dict,'back_roughness_map',tex.to_s)
end

def back_roughness_map?
	t=self.type
	case t
	when "Double-Sided Thin"
		return texture?(self.back_roughness_map)
	else
		return false
	end
end

def back_roughness_shader?

	t=self.type
	case t
	when "Double-Sided Thin"
		return self.back_roughness_map.procedural?
	else
		return false
	end
end

######
def front_fresnel_scale_map
	tex=mat.get_attribute(@@dict,'front_fresnel_scale_map',@@front_fresnel_scale)
	return IndigoTexture.new(tex)
end

def front_fresnel_scale_map=(tex)
	mat.set_attribute(@@dict,'front_fresnel_scale_map',tex.to_s)
end

def front_fresnel_scale_map?
	t=self.type
	case t
	when "Double-Sided Thin"
		return texture?(self.front_fresnel_scale_map)
	else
		return false
	end
end

def front_fresnel_scale_shader?

	t=self.type
	case t
	when "Double-Sided Thin"
		return self.front_fresnel_scale_map.procedural?
	else
		return false
	end
end

######
def back_fresnel_scale_map
	tex=mat.get_attribute(@@dict,'back_fresnel_scale_map',@@back_fresnel_scale)
	return IndigoTexture.new(tex)
end

def back_fresnel_scale_map=(tex)
	mat.set_attribute(@@dict,'back_fresnel_scale_map',tex.to_s)
end

def back_fresnel_scale_map?
	t=self.type
	case t
	when "Double-Sided Thin"
		return texture?(self.back_fresnel_scale_map)
	else
		return false
	end
end

def back_fresnel_scale_shader?

	t=self.type
	case t
	when "Double-Sided Thin"
		return self.back_fresnel_scale_map.procedural?
	else
		return false
	end
end

######
def r_f_map
	tex=mat.get_attribute(@@dict,'r_f_map',@@r_f)
	return IndigoTexture.new(tex)
end

def r_f_map=(tex)
	mat.set_attribute(@@dict,'r_f_map',tex.to_s)
end

def r_f_map?
	t=self.type
	case t
	when "Double-Sided Thin"
		return texture?(self.r_f_map)
	else
		return false
	end
end

def r_f_shader?

	t=self.type
	case t
	when "Double-Sided Thin"
		return self.r_f_map.procedural?
	else
		return false
	end
end
######
def fresnel_map
	tex=mat.get_attribute(@@dict,'fresnel_map',@@fresnel_map)
	return IndigoTexture.new(tex)
end

def fresnel_map=(tex)
	mat.set_attribute(@@dict,'fresnel_map',tex.to_s)
end

def fresnel_map?
	t=self.type
	case t
	when "Coating","Phong"
		return texture?(self.fresnel_map)
	else
		return false
	end
end

def fresnel_shader?

	t=self.type
	case t
	when "Coating","Phong"
		return self.fresnel_map.procedural?
	else
		false
	end
end
######
def thickness_map
	tex=mat.get_attribute(@@dict,'thickness_map',@@thickness_map)
	return IndigoTexture.new(tex)
end

def thickness_map=(tex)
	mat.set_attribute(@@dict,'thickness_map',tex.to_s)
end

def thickness_map?
	t=self.type
	case t
	when "Coating"
		return texture?(self.thickness_map)
	else
		return false
	end
end

def thickness_shader?

	t=self.type
	case t
	when "Coating"
		return self.thickness_map.procedural?
	else
		false
	end
end
def exponent_map
	tex=mat.get_attribute(@@dict,'exp_map',@@exp_map)
	return IndigoTexture.new(tex)
end

def exponent_map=(tex)
	mat.set_attribute(@@dict,'exp_map',tex.to_s)
end

def exponent_map?
	t=self.type
	case t
	when "Diffuse","Specular","Blend","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		return texture?(self.exponent_map)
	end
end

def exponent_shader?
	t=self.type
	case t
	when "Diffuse","Specular","Blend","Thin Glass","RGB Light","Blackbody Light","Exit Portal","Linked IGM"
		return false
	else
		return self.exponent_map.procedural?
	end

end

def clip_map
	tex=mat.get_attribute(@@dict,'clip_map',@@clip_map)
	return IndigoTexture.new(tex)
end

def clip_map=(tex)
	mat.set_attribute(@@dict,'clip_map',tex.to_s)
end

def clip_map?
	return false if !use_clip?
	t=self.type
	case t
	when "RGB Light","Blackbody Light","Exit Portal","Linked IGM", "Thin Glass"
		return false
	else
		return texture?(self.clip_map)
	end
end

def clip_shader?
	t=self.type
	case t
	when "RGB Light","Blackbody Light","Exit Portal","Linked IGM", "Thin Glass"
		return false
	else
		return self.clip_map.procedural?
	end
end

def use_clip?
	value=mat.get_attribute(@@dict,'use_clip',@@use_clip)
	value=false unless self.clip_map.valid?
	return value
end

def use_clip=(value)
	mat.set_attribute(@@dict,'use_clip',value)
end

def a_name
	mat.get_attribute(@@dict,'matA',@@matA)
end

def a_name=(value)
	mat.set_attribute(@@dict,'matA',value)
end

def b_name
	mat.get_attribute(@@dict,'matB',@@matB)
end

def b_name=(value)
	mat.set_attribute(@@dict,'matB',value)
end

def substrate
	mat.get_attribute(@@dict,'substrate',@@substrate)
end

def substrate=(value)
	mat.set_attribute(@@dict,'substrate',value)
end

def blend_factor
	mat.get_attribute(@@dict,'blendfactor',@@blendfactor)
end

def blend_factor=(value)
	mat.set_attribute(@@dict,'blendfactor',value)
end

def nk_data
	on_mac=SkIndigo.on_mac?
	def_nkdata=File.join(File.dirname(SkIndigo.get_indigo_path),"nkdata","Al.nk") unless on_mac
	def_nkdata=File.join(SkIndigo.get_indigo_app_data_path,"nkdata","Al.nk") if on_mac
	path=mat.get_attribute(@@dict,'nkdata', def_nkdata)
	return NKdata.new(path)
end

def nk_data=(value) #value is an NKdata ojbect
	mat.set_attribute(@@dict,'nkdata',value.path)  #stores the path to the nkdata
end

def alpha=(value)
	mat.alpha=(value)
end

def alpha
	mat.alpha
end

def display_name
	mat.display_name
end

def color=(value)
	mat.color=(value)
end	

#wdp is a string description of a wavelength dependent parameter (eg. albedo, emission)
def color(wdp=nil)
	return mat.color if wdp==nil
	t=self.albedo_texture if wdp=="albedo"
	t=self.emission_map if wdp=="emission"
	t=self.specular_map if wdp=="specular"
	t=self.abs_layer_trans_map if wdp=="absorption_layer_transmittance"
	t=self.absorption_map if wdp=="absorption"
	t=self.sss_map if wdp=="sss"
	t=self.g_spectrum_map if wdp=="g_spectrum"
	t=self.base_emission if wdp=="base_emission"
	t=self.transmittance_map if wdp=="transmittance"
	case t.type
		when "SketchUp"
			return self.absorption_color if wdp=="absorption"
			return self.color.to_a
		when "Constant"
			return t.color
		when "Texture"
			return nil
		when "Shader"
			return nil
	end
end
#####subdivision settings
def view_dependent_subdivision?
	mat.get_attribute(@@dict,'view_dep_sub',@@view_dependent)
end

def view_dependent_subdivision=(value)
	mat.set_attribute(@@dict,'view_dep_sub',value)
end

def max_subdiv
	mat.get_attribute(@@dict,'max_subdiv',@@max_subdiv)
end

def max_subdiv=(value)
	mat.set_attribute(@@dict,'max_subdiv',value)
end

def pixel_threshold
	mat.get_attribute(@@dict,'pixel_threshold',@@pixel_threshold)
end

def pixel_threshold=(value)
	mat.set_attribute(@@dict,'pixel_threshold',value)
end

def curvature_threshold
	#return -1  #this feature currently disabled
 	mat.get_attribute(@@dict,'curvature_threshold',@@curvature_threshold)
end

def curvature_threshold=(value)
	mat.set_attribute(@@dict,'curvature_threshold',value)
end

def layer
	mat.get_attribute(@@dict,'layer',@@layer)
end

def layer=(value)
	mat.set_attribute(@@dict,'layer',value.to_i)
end

def valid?
	materials=Sketchup.active_model.materials
	begin
		if materials[mat.name]
			return true
		else
			return false
		end
	rescue
		return false
	end
end

def color_picker?

if mat.display_name=="SkIndigo Color Picker"
	return true
end

return false

end

def uv_helper?

if mat.display_name=="SkIndigo UV Helper"
	return true
end

return false

end

def texture?(tex)

case tex.type
	when "SketchUp"
		return true if mat.texture!=nil
	when "External"
		return true #if tex.valid?
	else
		return false
end

end

def background_material?

name=@mat.name
if name[0..2]=="BG_"
	return true
else
	return false
end

end

####
def emission_scale
	mat.get_attribute(@@dict,'emission_scale',@@emission_scale)
end

def emission_scale=(value)
	mat.set_attribute(@@dict,'emission_scale',value)
end

def emission_measure
	mat.get_attribute(@@dict,'emission_measure',@@emission_measure)
end

def emission_measure=(value)
	mat.set_attribute(@@dict,'emission_measure',value)
end


end #Indigo material class end

#################################Color Picker Class#############################

class ColorPicker

def initialize(sucolor,indigo_material,property)   #color is a Sketchup color object
	
	@im=indigo_material
	@prop=property
	@rgb=sucolor
	@dlg=UI::WebDialog.new("Color Picker", true, "SkIndigoColorPicker",375,241,419,243,true) #width,height,left,top
	@dlg.set_file(File.join(File.dirname(__FILE__),"colorpicker.html"))
	color = @dlg.get_default_dialog_color 
	@dlg.set_background_color(color)
	@dlg.add_action_callback("apply") {|d,p|
		
		r=@dlg.get_element_value("red")
		g=@dlg.get_element_value("green")
		b=@dlg.get_element_value("blue")
		@rgb=[r.to_i,g.to_i,b.to_i]
		#p @rgb
		case @prop
			when "base_emission"
				@im.base_emission=@rgb
		end
		$skindigo_mat_observer.refresh_dialog
		d.close
	}
	@dlg.add_action_callback("send_color") {|d,p|
		
	}
	@dlg.add_action_callback("set_color") {|d,p|
		setColor(@rgb)
	}
	@dlg.add_action_callback("cancel") {|d,p| d.close}
	@dlg.show_modal {}
	
end

def show
	@dlg.show_modal{}
end

def setColor(sucolor)  #rucolor is an rgb triplet array
	r=sucolor[0]
	b=sucolor[2]
	g=sucolor[1]
	cmd="$('#colorpicker').ColorPickerSetColor({r:#{r},g:#{g},b:#{b}});"
	@dlg.execute_script(cmd)
end

def getColor()
	@rgb
end

end #color picker
#######################Indigo Exporter Class#####################
#todo - put in separate file
class IndigoExporter

attr_reader :preview_material, :indigo_version, :path, :exported_mats, :model, :irs, :start, :end, :tex_path, :nkdata, :textures, :meshlights, :ies, :meshbuilder, :batch_path, :physics, :clay_render, :light_source, :preview_scene
attr_writer :tex_path, :path, :preview_material, :start, :end, :meshbuilder, :batch_path, :physics, :clay_render, :preview_scene

SKY_LAYER_INDEX=1
SUN_LAYER_INDEX=0
BACKGROUND_LAYER_INDEX=2

def initialize(path)
	@path=path      #the path of the file that the Exporter will write to
	#@tex_path=File.dirname(path)   #the directory that the Exporter will write textures to
	@tex_path=""   #the directory that the Exporter will write textures to
	@exported_mats=[]
	@nkdata=[]
	@textures=[]
	@ies=[]
	@meshlights=0
	@null_exported=false
	@preview_material=nil
	@indigo_version=SkIndigo::VERSION
	@model=Sketchup.active_model
	@materials=@model.materials
	@irs=IndigoRenderSettings.new
	#disabling saving to scene due to bugs
	#if @irs.saved_to_current_scene?
	#	@irs.save_to_scene(Sketchup.active_model.pages.selected_page)
	#else
	#	@irs.save_to_model()
	#end
	@used_layers=Set.new  #keeps track of light layers that are actually used
	####
	@scale=0.0254*Indigo.scale
	@null_exported=false
	@instance_mat=nil
	@igm=false
	@frame=0  #the current animation export frame
	@meshbuilder=nil
	@physics=nil
	@clay_render=false
	@light_source=false
	@preview_scene=false
end

#detects whether a string has leading or trailing whitespace or has contiguous whitespace inside the string that needs to be collapsed - such strings are not parsed correctly by TinyXML (Indigo's parser)
def needCData?(string)

return false if string.empty?
return true if (string=~/</)
return true if (string=~/>/)
return true if (string=~/&/)
return true if (string=~/\A\s/)  #whitespace at start
return true if (string=~/\s\Z/)  #whitespace at end
return true if (string=~/\s\s/) #more than one whitespace at any point in string
return false

end

###adds CDATA tag to string if required
def write_name_string(out,depth,name,string)

depth.times{|i| out.print("\t")}  #writes the tabs

if needCData?(string)
	out.print "<#{name}>"+wrap_cdata(string)+"</#{name}>\n"
else
	out.print "<#{name}>#{string}</#{name}>\n"
end

end

####
def wrap_cdata(string)

return "<![CDATA[#{string}]]>"

end

def validate_light_source()

if @light_source==false
	UI.messagebox("Scene must contain a light source.  Export aborted.")
	raise
end

end


def reset_materials
	@exported_mats=[]
end

def settings
	return @irs
end


#############################################Material Exporter######################
def export_material(mat,out,generate_uvs=false)

	if mat.type=="Blend"   #Check if A material and B material exist and export them first
		write_null_mat(out) if ((mat.a_name=="None") or (mat.b_name=="None"))
		a_mat=IndigoMaterial.find(mat.a_name)
		b_mat=IndigoMaterial.find(mat.b_name)
		export_material(a_mat,out,generate_uvs) if a_mat
		export_material(b_mat,out,generate_uvs) if b_mat
	end
	
	if mat.type=="Coating"  #export the substrate first
		substrate=IndigoMaterial.find(mat.substrate)
		export_material(substrate,out,generate_uvs) if substrate
	end
	
	if mat.type=="Double-Sided Thin"  #export the front and back materials first
		front_mat=IndigoMaterial.find(mat.get("front_material_name"))
		back_mat=IndigoMaterial.find(mat.get("back_material_name"))
		export_material(front_mat,out,generate_uvs) if front_mat
		export_material(back_mat,out,generate_uvs) if back_mat
	end

	if not @exported_mats.index(mat.name)
		#if mat.type=="Exit Portal"  #deprecated material type - now a mesh type
		#	UI.messagebox("Material #{mat.name} is set to Exit Portal which is now deprecated.  Use Object Editor to convert a mesh to an exit portal.  Material will be converted to Diffuse.")
		#	mat.type="Diffuse"
		#end
		write_mat(mat,out,generate_uvs)
		@exported_mats.push(mat.name)
	end

end

######
def export_clay(out)

out.puts "<material>"
out.puts "			<name>SkIndigoClay</name>"
out.puts "		<oren_nayar>"
out.puts "			<albedo>"
out.puts "				<constant>"
out.puts "					<rgb>"
out.puts "						<rgb>0.8784 0.8353 0.6941</rgb>"
out.puts "						<gamma>2.2</gamma>"
out.puts "					</rgb>"
out.puts "				</constant>"
out.puts "			</albedo>"
out.puts "			<sigma>"
out.puts "		<constant>0.5</constant>"
out.puts "			</sigma>"
out.puts "		</oren_nayar>"
out.puts "		</material>"

end
#####################Material Writer############################
def write_mat(mat,out,generate_uvs=false)

begin 

export_name=mat.xml_name

if self.clay_render==true
	unless mat.emitter?
		out.puts "<alias_material>"
		out.puts "	<current_name>Default</current_name>"
		write_name_string(out,1,"new_name",export_name)
		#out.puts "	<new_name>#{export_name}</new_name>"
		out.puts "</alias_material>"
		return
	end
end
		

@tex_index=0
export_name="previewmaterial" if mat==preview_material
export_name="Default" if @instance_mat==mat.mat                  #this allows instances to use different materials - blend materials may not work with this
result=mat.rgb 			#get the color of the SketchUp material
export_name="C_"+export_name+"_C" if mat.clip_map? or mat.clip_shader?
opacity=mat.alpha
type=mat.type

albedo_tex=nil 			
bump_map=nil
exponent_map=nil
normal_map=nil
sigma_map=nil
disp_map=nil
emission_map=nil
blend_map=nil
specular_map=nil
abs_layer_trans_map=nil
roughness_map=nil
fresnel_map=nil
thickness_map=nil
transmittance_map=nil
r_f_map=nil
front_fresnel_scale_map=nil
back_fresnel_scale_map=nil
front_roughness_map=nil
back_roughness_map=nil

albedo_shader=nil
bump_shader=nil
normal_shader=nil
exponent_shader=nil
sigma_shader=nil
disp_shader=nil
blend_shader=nil
emission_shader=nil
specular_shader=nil
abs_layer_trans_shader=nil
abs_shader=nil
sss_shader=nil
roughness_shader=nil
fresnel_shader=nil
thickness_shader=nil
transmittance_shader=nil
r_f_shader=nil
front_fresnel_scale_shader=nil
back_fresnel_scale_shader=nil
front_roughness_shader=nil
back_roughness_shader=nil

	if (type=="Specular") or (type=="Glossy Transparent")
		#alpha_factor=opacity*opacity*1000
		#sss_spectrum=mat.sss_spectrum.to_s.split(' ')
		#if sss_spectrum[0] and sss_spectrum[1] and sss_spectrum[2]
			#rgb_spectrum=true
		#else
			#rgb_spectrum=false
		#end
		#g_spectrum=mat.g_spectrum.to_s.split(' ')
		#if g_spectrum[0] and g_spectrum[1] and g_spectrum[2]
		#	g_rgb_spectrum=true
		#else
		#	g_rgb_spectrum=false
		#end

		#Export the medium
		medium_name=export_name+"_medium"
		out.puts "		<medium>"
		write_name_string(out,2,"name",medium_name)
		#out.print "			<name>#{medium_name}</name>\n"
		out.print "			<precedence>#{mat.precedence}</precedence>\n"
		out.print "			<basic>\n"
		out.print "				<ior>#{mat.ior}</ior>\n"
		out.print "				<cauchy_b_coeff>#{mat.cauchy_b_coeff}</cauchy_b_coeff>\n"
		export_absorption(mat,out)
		if mat.sss?
			out.print "				<subsurface_scattering>\n"
			export_sss(mat,out)
			out.print "					<phase_function>\n"
			if mat.phase_function=="Uniform"
				out.print "						<uniform/>\n"
			else
				out.puts "						<henyey_greenstein>"
				out.puts "							<g_spectrum>"
				export_constant_wdp(mat.g_spectrum_map,out,false)
				out.puts "							</g_spectrum>"
				out.puts "						</henyey_greenstein>"
			end

			out.print "					</phase_function>\n"
			out.print "				</subsurface_scattering>\n"
			
		end
		out.print "			</basic>\n"
		out.print "		</medium>\n"
	
	end     #Finished exporting the medium

	##Material Export Starts Here
	
	case type
	
	when "Linked IGM"
		ligm=mat.linked_igm
		if not FileTest.exist?(ligm)
			UI.messagebox("Could not find linked igm material #{ligm} for material '#{mat.display_name}'")
			raise "InvalidLinkedIGM"
		end
		ligm_file=File.new((ligm),"r")
		data=ligm_file.read
		ligm_file.close
		xml_parser=DMxml::Parser.new(data)
		doc=xml_parser.parse_document()
		material_data=doc["materialdata"]
		material_data=doc["scenedata"] if not material_data
		materials=material_data.find_all("material") #returns array of all materials in the IGM file
		last_material_in_igm=materials[-1]
		linked_name=last_material_in_igm["name"].element_text
		out.puts "<include>"
		out.puts "	<pathname>#{ligm}</pathname>"
		out.puts "</include>"
		out.puts "<alias_material>"
		out.puts "	<current_name>#{linked_name}</current_name>"
		write_name_string(out,1,"new_name",export_name)
		#out.puts "	<new_name>#{export_name}</new_name>"
		out.puts "</alias_material>"
	when "Diffuse","Specular","Phong","Metals","Oren-Nayar","Glossy Transparent","Blend","Coating","Double-Sided Thin"
		####export blended diffuse transmitters
		if type=="Diffuse" or type=="Phong"
			if (opacity<1.0) and (opacity>0.0)
				export_blended_diffuse_transmitter(mat,export_name,out,generate_uvs)
				export_clip_map(mat,out,export_name,generate_uvs)  if mat.use_clip? #if mat.clip_map? or mat.clip_shader? #exports the clip map
				@exported_mats.push(mat)  #keep track of all exported materials
				return
			end
		end
		out.puts "			<material>"
		write_name_string(out,3,"name",export_name)
		indigo_type=mat.indigo_type
		out.puts "		<#{indigo_type}>"
		unless type=="Blend"  #what about coating?  - can you have emitting coating?
			if mat.emitter?
				out.puts "			<layer>#{@used_layers.index(mat.layer.to_i)}</layer>" if !(@igm)
				#@used_layers.insert(mat.layer)
			end
		end
		if type=="Specular" or type=="Glossy Transparent"
			if type=="Specular"
				out.print " 			<transparent>true</transparent>\n" if opacity<1.0
				out.print "			<transparent>false</transparent>\n" if opacity==1.0
			end
			write_name_string(out,3,"internal_medium_name",medium_name)
			#out.print "			<internal_medium_name>#{medium_name}</internal_medium_name>\n"
		end

		#export all of the texture xml and collect the indices
		albedo_tex=export_texture(mat,mat.albedo_texture,out,generate_uvs) if mat.albedo_texture?
		specular_map=export_texture(mat,mat.specular_map,out,generate_uvs) if mat.specular_map?
		blend_map=export_texture(mat,mat.blend_map,out,generate_uvs) if mat.blend_map? 			
		bump_map=export_texture(mat,mat.bump_map,out,generate_uvs)  if mat.bump_map?
		normal_map=export_texture(mat,mat.normal_map,out,generate_uvs)  if mat.normal_map?
		exponent_map=export_texture(mat,mat.exponent_map,out,generate_uvs) if mat.exponent_map?
		sigma_map=export_texture(mat,mat.sigma_map,out,generate_uvs) if mat.sigma_map?
		disp_map=export_texture(mat,mat.disp_map,out,generate_uvs)  if mat.disp_map? #and (!mat.clip_map?) #don't export the displacement map if this material has a clip map, export it with the blend material
		emission_map=export_texture(mat,mat.emission_map,out,generate_uvs) if mat.emission_map?
		abs_layer_trans_map=export_texture(mat,mat.abs_layer_trans_map,out,generate_uvs) if mat.abs_layer_trans_map?
		roughness_map=export_texture(mat,mat.roughness_map,out,generate_uvs) if mat.roughness_map?
		fresnel_map=export_texture(mat,mat.fresnel_map,out,generate_uvs) if mat.fresnel_map?
		thickness_map=export_texture(mat,mat.thickness_map,out,generate_uvs) if mat.thickness_map?
		transmittance_map=export_texture(mat,mat.transmittance_map,out,generate_uvs) if mat.transmittance_map?
		r_f_map=export_texture(mat,mat.r_f_map,out,generate_uvs) if mat.r_f_map?
		front_fresnel_scale_map=export_texture(mat,mat.front_fresnel_scale_map,out,generate_uvs) if mat.front_fresnel_scale_map?
		back_fresnel_scale_map=export_texture(mat,mat.back_fresnel_scale_map,out,generate_uvs) if mat.back_fresnel_scale_map?
		front_roughness_map=export_texture(mat,mat.front_roughness_map,out,generate_uvs) if mat.front_roughness_map?
		back_roughness_map=export_texture(mat,mat.back_roughness_map,out,generate_uvs) if mat.back_roughness_map?
		
		#collect all of the shaders
		albedo_shader=mat.albedo_texture.shader_code if mat.albedo_shader?
		specular_shader=mat.specular_map.shader_code if mat.specular_shader?
		blend_shader=mat.blend_map.shader_code if mat.blend_shader?
		bump_shader=mat.bump_map.shader_code if mat.bump_shader?
		normal_shader=mat.normal_map.shader_code if mat.normal_shader?
		exponent_shader=mat.exponent_map.shader_code if mat.exponent_shader?
		sigma_shader=mat.sigma_map.shader_code if mat.sigma_shader?
		disp_shader=mat.disp_map.shader_code if mat.disp_shader?
		emission_shader=mat.emission_map.shader_code if mat.emission_shader?
		abs_layer_trans_shader=mat.abs_layer_trans_map.shader_code if mat.abs_layer_trans_shader?
		roughness_shader=mat.roughness_map.shader_code if mat.roughness_shader?
		fresnel_shader=mat.fresnel_map.shader_code if mat.fresnel_shader?
		thickness_shader=mat.thickness_map.shader_code if mat.thickness_shader?
		transmittance_shader=mat.transmittance_map.shader_code if mat.transmittance_shader?
		r_f_shader=mat.r_f_map.shader_code if mat.r_f_shader?
		front_fresnel_scale_shader=mat.front_fresnel_scale_map.shader_code if mat.front_fresnel_scale_shader?
		back_fresnel_scale_shader=mat.back_fresnel_scale_map.shader_code if mat.back_fresnel_scale_shader?
		front_roughness_shader=mat.front_roughness_map.shader_code if mat.front_roughness_shader?
		back_roughness_shader=mat.back_roughness_map.shader_code if mat.back_roughness_shader?		
		
		if type=="Blend"
			matA=mat.a_name
			matB=mat.b_name
			names = Sketchup.active_model.materials.collect {|m| m.name}
			indexA = names.index(matA)
			indexB = names.index(matB)
			if (indexA==nil and matA!="None")
				UI.messagebox("Error: Blend material #{mat.name} tried to use non-existent material named #{matA}")
				raise "BlendMaterialUndefined"
			end
			if (indexB==nil and matB!="None")
				UI.messagebox("Error: Blend material #{mat.name} tried to use non-existent material named #{matB}")
				raise "BlendMaterialUndefined"
			end
			#matA=matA.gsub(/</,'&lt;').gsub(/>/,'&gt;')
			#matB=matB.gsub(/</,'&lt;').gsub(/>/,'&gt;')
			
			write_name_string(out,4,"a_name",matA)	
			write_name_string(out,4,"b_name",matB)			
			#out.print "				<a_name>#{matA}</a_name>\n"
			#out.print "				<b_name>#{matB}</b_name>\n"
			out.puts "				<blend>"
			if blend_map
				export_texture_index(blend_map,out)
			elsif blend_shader
				export_shader(blend_shader,out)
			else
				out.puts "		<constant>#{mat.blend_map.b}</constant>"
			end
			out.puts "				</blend>"
			out.puts "				<step_blend>#{mat.step_blend?}</step_blend>"
		end
		
		if type=="Diffuse" or type=="Oren-Nayar" or type=="Phong"
			#out.puts "			<random_triangle_colours>true</random_triangle_colours>" if type=="Diffuse"
			out.puts "			<albedo>" unless type=="Phong"
			out.puts "			<diffuse_albedo>" if type=="Phong"
			if albedo_tex
				export_texture_index(albedo_tex,out)
			elsif albedo_shader
				export_shader(albedo_shader,out)
			else
				export_constant_param(result,mat.albedo_texture,out) if not albedo_tex
				#export_rgb_spectrum(out,result) if not albedo_tex
			end
			out.puts "			</albedo>" unless type=="Phong"
			out.puts "			</diffuse_albedo>" if type=="Phong"
		end
		
		if type=="Phong" or type=="Coating" or type=="Double-Sided Thin"
			out.puts "			<ior>#{mat.ior}</ior>"
		end
		
		if type=="Metals"
			nkdata=mat.nk_data    #returns an NKdata object
			if mat.use_nkdata?
				#out.puts "				<nk_data>#{File.join(File.dirname(@path),tex_path,nkdata.filename)}</nk_data>" if not @igm
				#out.puts "				<nk_data>nkdata\/#{nkdata.filename}</nk_data>" if @igm
				out.puts "				<nk_data>nkdata\/#{nkdata.filename}</nk_data>"
				@nkdata.push(nkdata.path) if (not @nkdata.index(nkdata.path))    #copies stores the nkdata path for future export
			else
				out.puts "			<specular_reflectivity>"
				if specular_map
					export_texture_index(specular_map,out)
				elsif specular_shader
					export_shader(specular_shader,out)
				else
					export_constant_param(result,mat.specular_map,out)
					#export_rgb_spectrum(out,result) if not specular_map
				end
				out.puts "			</specular_reflectivity>"
			end
		end 
		if type=="Oren-Nayar"
			out.puts "			<sigma>"
			if sigma_map
				export_texture_index(sigma_map,out)
			elsif sigma_shader
				export_shader(sigma_shader,out)
			else
				out.puts "		<constant>#{mat.sigma_map.b}</constant>"
			end
			out.puts "			</sigma>"
		end
		if type=="Phong" or type=="Glossy Transparent" or type=="Metals"
			out.puts "			<exponent>"
			if exponent_map
				export_texture_index(exponent_map,out)
			elsif exponent_shader
				export_shader(exponent_shader,out)
			else
				out.puts "		<constant>#{mat.exponent_map.b}</constant>"
			end
			out.puts "			</exponent>"
		end
		if type=="Glossy Transparent" or type=="Specular"
			#TODO - validate absorption layer parameter
			if mat.use_abs_layer_trans? # or abs_layer_trans_shader
				out.puts "			<absorption_layer_transmittance>"
				if abs_layer_trans_map
					export_texture_index(abs_layer_trans_map,out)
				elsif abs_layer_trans_shader
					export_shader(abs_layer_trans_shader,out)
				else
					export_constant_param(result,mat.abs_layer_trans_map,out)
				end
				out.puts "			</absorption_layer_transmittance>"
			end
		end
		if type=="Coating" or type=="Phong"
			out.puts "				<fresnel_scale>"
			if fresnel_map
				export_texture_index(fresnel_map,out)
			elsif fresnel_shader
				export_shader(fresnel_shader,out)
			else
				out.puts "			<constant>#{mat.fresnel_map.b}</constant>"
			end
			out.puts "				</fresnel_scale>"
		end
		
		if type=="Coating"
			export_coating_absorption(mat,out)
			out.puts "				<thickness>"
			if thickness_map
				export_texture_index(thickness_map,out)
			elsif thickness_shader
				export_shader(thickness_shader,out)
			else
				out.puts "			<constant>#{mat.thickness_map.b}</constant>"
			end
			out.puts "				</thickness>"
			
			out.puts "				<roughness>"
			if roughness_map
				export_texture_index(roughness_map,out)
			elsif roughness_shader
				export_shader(roughness_shader,out)
			else
				out.puts "			<constant>#{mat.roughness_map.b}</constant>"
			end
			out.puts "				</roughness>"
			
			out.puts "				<interference>#{mat.interference?}</interference>"
			
			write_name_string(out,4,"substrate_material_name",mat.substrate)
		end
		
		if type=="Double-Sided Thin"
			write_name_string(out,4,"front_material_name",mat.get("front_material_name"))
			write_name_string(out,4,"back_material_name",mat.get("back_material_name"))
			out.puts "				<r_f>"
			if r_f_map
				export_texture_index(r_f_map,out)
			elsif r_f_shader
				export_shader(r_f_shader,out)
			else
				out.puts "			<constant>#{mat.r_f_map.b}</constant>"
			end
			out.puts "				</r_f>"
			
			out.puts "				<transmittance>"
			if transmittance_map
				export_texture_index(transmittance_map,out)
			elsif transmittance_shader
				export_shader(transmittance_shader,out)
			else
				export_constant_param(result,mat.transmittance_map,out)
			end
			out.puts "				</transmittance>"
			
			out.puts "				<front_roughness>"
			if front_roughness_map
				export_texture_index(front_roughness_map,out)
			elsif front_roughness_shader
				export_shader(front_roughness_shader,out)
			else
				out.puts "			<constant>#{mat.front_roughness_map.b}</constant>"
			end
			out.puts "				</front_roughness>"
			
			out.puts "				<back_roughness>"
			if back_roughness_map
				export_texture_index(back_roughness_map,out)
			elsif back_roughness_shader
				export_shader(back_roughness_shader,out)
			else
				out.puts "			<constant>#{mat.back_roughness_map.b}</constant>"
			end
			out.puts "				</back_roughness>"
			
			out.puts "				<front_fresnel_scale>"
			if front_fresnel_scale_map
				export_texture_index(front_fresnel_scale_map,out)
			elsif front_fresnel_scale_shader
				export_shader(front_fresnel_scale_shader,out)
			else
				out.puts "			<constant>#{mat.front_fresnel_scale_map.b}</constant>"
			end
			out.puts "				</front_fresnel_scale>"
			
			out.puts "				<back_fresnel_scale>"
			if back_fresnel_scale_map
				export_texture_index(back_fresnel_scale_map,out)
			elsif back_roughness_shader
				export_shader(back_roughness_shader,out)
			else
				out.puts "			<constant>#{mat.back_fresnel_scale_map.b}</constant>"
			end
			out.puts "				</back_fresnel_scale>"
			
		end
		####export the common texture attributes
		if mat.use_bump? #bump_map or bump_shader
			out.puts "			<bump>"
			export_constant_wip(result,mat.bump_map,out) if !(bump_map or bump_shader)
			export_texture_index(bump_map,out) if bump_map
			export_shader(bump_shader,out) if bump_shader
			out.puts "			</bump>"
		end
		if mat.use_normal? 
			out.puts "			<normal_map>"
			export_constant_wip(result,mat.normal_map,out) if !(normal_map or normal_shader)
			export_texture_index(normal_map,out) if normal_map
			export_shader(normal_shader,out) if normal_shader
			out.puts "			</normal_map>"
		end
		if mat.use_displace? #disp_map or disp_shader
			out.puts "			<displacement>"
			export_constant_wip(result,mat.disp_map,out) if !(disp_map or disp_shader)
			export_texture_index(disp_map,out) if disp_map
			export_shader(disp_shader,out) if disp_shader
			out.puts "			</displacement>"
		end
		if mat.emitter?
			
			#gain=mat.base_emission_gain
			#color=color.collect {|val| val*gain}
			out.puts "			<base_emission>"
			export_constant_param(result,mat.base_emission,out)
			#if mat.base_emission_spectrum=="Blackbody"
			#	out.puts "				<constant>"
			#	out.puts "					<blackbody>"
			#	out.puts "						<temperature>#{mat.temperature}</temperature>"
			#	out.puts "						<gain>1.0</gain>" if not @preview_material
			#	out.puts "						<gain>10.0</gain>" if @preview_material
			#	out.puts "					</blackbody>"
			#	out.puts "				</constant>"
			#else
			#	color=mat.base_emission if not @preview_material
			#	color=mat.base_emission.collect {|val| val.to_f*10000000} if @preview_material
			#	export_rgb_spectrum(out,color)		
			#end
			out.puts "			</base_emission>"
		end
		
		if mat.use_emission? #emission_map or emission_shader
			out.puts "			<emission>"
			if emission_map
				export_texture_index(emission_map,out)
			elsif emission_shader
				export_shader(emission_shader,out)
			else
				export_constant_param(result,mat.emission_map,out)
			end
			out.puts "			</emission>"
		end
			
		out.puts "		</#{indigo_type}>"
		out.print " 		</material>\n"
		
		export_clip_map(mat,out,export_name,generate_uvs)  if mat.use_clip? #if mat.clip_map? or mat.clip_shader? #exports the clip map

	when "Thin Glass"

		mat_A="SP_"+export_name+"_SP"
		mat_B="NULL_"+export_name+"_NULL"
				
		out.puts "		<material>"
		write_name_string(out,3,"name",mat_A)					
		#out.print "			<name>#{mat_A}</name>\n"
		out.print "			<phong>\n"
		bump_map=export_texture(mat,mat.bump_map,out,generate_uvs)  if mat.bump_map?
		bump_shader=mat.bump_map.shader_code if mat.bump_shader?
		out.puts  "				<diffuse_albedo>"
		export_rgb_spectrum(out,result)
		out.puts "				</diffuse_albedo>"
		out.puts "				<ior>1000</ior>\n"
		out.puts "				<exponent>"
		out.puts "				<constant>1000000</constant>"
		out.puts "				</exponent>"
			
		if mat.use_bump? #bump_map or bump_shader
			out.puts "			<bump>"
			export_constant_wip(result,mat.bump_map,out) if !(bump_map or bump_shader)
			export_texture_index(bump_map,out) if bump_map
			export_shader(bump_shader,out) if bump_shader
			out.puts "			</bump>"
		end
		
		out.print " 			</phong>\n"
		out.print " 		</material>\n"

		#Export Transmittance Layer
		out.puts "		<material>"
		#out.print "			<name>#{mat_B}</name>\n"
		write_name_string(out,3,"name",mat_B)
		out.print "			<null_material/>\n"
		out.print "		</material>\n"

		#Export Blended Material
		@tex_index=0    #reset the texture index variable since we are now exporting a new material
		out.puts "		<material>"
		write_name_string(out,3,"name",export_name)
		#out.print "			<name>#{export_name}</name>\n"
		out.print "			<blend>\n"

		disp_map=export_texture(mat,mat.disp_map,out,generate_uvs)  if mat.disp_map? and (!mat.clip_map?) #don't export the displacement map if this material has a clip map, export it with the blend material
		disp_shader=mat.disp_map.shader_code if mat.disp_shader?
		
		write_name_string(out,4,"a_name",mat_B)
		write_name_string(out,4,"b_name",mat_A)
		#out.print "				<a_name>#{mat_B}</a_name>\n" 
		#out.print "				<b_name>#{mat_A}</b_name>\n" 
		out.puts "				<blend>"
		out.puts "				<constant>#{mat.alpha}</constant>"
		out.puts "				</blend>"

		if mat.use_displace? #disp_map or disp_shader
			out.puts "			<displacement>"
			export_constant_wip(result,mat.disp_map,out) if !(disp_map or disp_shader)
			export_texture_index(disp_map,out) if disp_map
			export_shader(disp_shader,out) if disp_shader
			out.puts "			</displacement>"
		end

		out.print "			</blend>\n"
		out.print "		</material>\n"
	when "Exit Portal"
		out.puts "			<material>"
		write_name_string(out,3,"name",export_name)
		out.puts "				<exit_portal/>"
		out.puts "			</material>"
	end  #end case type
	
	@exported_mats.push(mat)  #keep track of all exported materials

rescue IndigoInvalidTabulatedException
	UI.messagebox("Material #{mat.name} contains invalid tabulated spectrum data.  Export aborted.")
	raise
end  #end begin

end  #end material writer


####
def export_constant_param(rgb,param,out)

case param.type
	when "SketchUp"
		export_rgb_spectrum(out,rgb)
	when "Constant"
		export_constant_wdp(param,out,true)
	else
		# Control flow can get here if the texture is invalid.  In this case just use the SketchUp colour.
		export_rgb_spectrum(out,rgb)
end

end

#####
def export_constant_wip(rgb,param,out)

constant=0.0
case param.type
	when "SketchUp"
		#export normalized rgb value multiplied by b parameter.
		avg_rgb=((rgb[0]+rgb[1]+rgb[2])/3.0)/255.0
		constant=avg_rgb*param.b.to_f
		constant="%.4f" %(constant)  #format to 4 decimal places
		##?
	when "Constant"
		constant=param.b.to_f
end
out.puts "				<constant>#{constant}</constant>"

end

###
def export_absorption(mat,out)

out.puts "				<absorption_coefficient>"
abs=mat.absorption_map
case abs.type
	when "SketchUp"
		result=mat.rgb
		opacity=mat.alpha
		alpha_factor=opacity*opacity*1000
		out.puts "				<constant>"
		out.puts "					<rgb>"
		out.print "						<rgb>" + "%.4f" %((1.0-result[0]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-result[1]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-result[2]/255.0)*alpha_factor) + "</rgb>\n"
		out.print "						<gamma>1.0</gamma>\n"
		out.print "					</rgb>\n"
		out.puts "				</constant>"
	when "Constant"
		export_constant_wdp(abs,out,true)
	when "Shader"
		export_shader(abs.shader_code,out,"false")
end
out.puts "				</absorption_coefficient>"

end

#this function just uses a different tag then the previous one
def export_coating_absorption(mat,out)

out.puts "				<absorption>"
abs=mat.absorption_map
case abs.type
	when "SketchUp"
		result=mat.rgb
		opacity=mat.alpha
		alpha_factor=opacity*opacity*1000
		out.puts "				<constant>"
		out.puts "					<rgb>"
		out.print "						<rgb>" + "%.4f" %((1.0-result[0]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-result[1]/255.0)*alpha_factor) + " "+ "%.4f" %((1.0-result[2]/255.0)*alpha_factor) + "</rgb>\n"
		out.print "						<gamma>1.0</gamma>\n"
		out.print "					</rgb>\n"
		out.puts "				</constant>"
	when "Constant"
		export_constant_wdp(abs,out,true)
	when "Shader"
		export_shader(abs.shader_code,out,"false")
end
out.puts "				</absorption>"

end
####
def export_sss(mat,out)

out.puts "				<scattering_coefficient>"
param=mat.sss_map
case param.type
	when "Constant"
		export_constant_wdp(param,out,true)
	when "Shader"
		export_shader(param.shader_code,out,"false")
end
out.puts "				</scattering_coefficient>"

end

####
def export_constant_wdp(param,out,new_syntax)

out.puts "					<constant>" if new_syntax #some parameters still use the old syntax (without the constant tag)
case param.spectrum_type
when "RGB"
	#p [param.rgb_r,param.rgb_g,param.rgb_b]
	gain=param.rgb_gain.to_f
	out.puts "					<rgb>"
	out.print "						<rgb>" + "%.4f" %(param.rgb_r.to_f*gain) + " "+ "%.4f" %(param.rgb_g.to_f*gain) + " "+ "%.4f" %(param.rgb_b.to_f*gain) + "</rgb>\n"
	out.print "						<gamma>#{param.rgb_gamma}</gamma>\n"
	out.print "					</rgb>\n"
when "Blackbody"
	out.puts "					<blackbody>"
	out.puts "						<temperature>#{param.bb_temperature}</temperature>"
	out.puts "						<gain>#{param.bb_gain}</gain>"
	out.puts "					</blackbody>"
when "Uniform"
	out.puts "					<uniform>"
	out.puts "						<value>#{param.uni_value}</value>"
	out.puts "					</uniform>"
when "Tabulated"
	values=param.reg_values
	num_values=values.split(" ").length
	if num_values<2
		raise IndigoInvalidTabulatedException
	end
	start_wl=param.reg_start.to_f/1000.0
	end_wl=param.reg_end.to_f/1000.0
	out.puts "					<regular_tabulated>"
	out.puts "						<start_wavelength>#{start_wl}E-06</start_wavelength>"
	out.puts "						<end_wavelength>#{end_wl}E-06</end_wavelength>"
	out.puts "						<num_values>#{num_values}</num_values>"
	out.puts"						<values>#{values}</values>"
	out.puts "					</regular_tabulated>"
end
out.puts "					</constant>" if new_syntax

end

#############################
def export_blended_diffuse_transmitter(mat,export_name,out,generate_uvs=false)

type=mat.type   
result=mat.rgb 	
mat_a="DT_"+export_name+"_DT"
mat_b="D_"+export_name+"_D"
out.puts "		<material>"
write_name_string(out,3,"name",mat_b)
#out.print "			<name>#{mat_b}</name>\n"
out.print "			<#{mat.indigo_type}>\n" 
if mat.emitter?
	out.puts "			<layer>#{@used_layers.index(mat.layer.to_i)}</layer>" if !(@igm)
	#@used_layers.insert(mat.layer)
end

#export all of the texture xml and collect the indices
albedo_tex=export_texture(mat,mat.albedo_texture,out,generate_uvs) if mat.albedo_texture?		
bump_map=export_texture(mat,mat.bump_map,out,generate_uvs)  if mat.bump_map?
normal_map=export_texture(mat,mat.normal_map,out,generate_uvs)  if mat.normal_map?
exponent_map=export_texture(mat,mat.exponent_map,out,generate_uvs) if mat.exponent_map?
disp_map=export_texture(mat,mat.disp_map,out,generate_uvs)  if mat.disp_map?
emission_map=export_texture(mat,mat.emission_map,out,generate_uvs) if mat.emission_map?

#collect all of the shaders
albedo_shader=mat.albedo_texture.shader_code if mat.albedo_shader?
bump_shader=mat.bump_map.shader_code if mat.bump_shader?
normal_shader=mat.normal_map.shader_code if mat.normal_shader?
exponent_shader=mat.exponent_map.shader_code if mat.exponent_shader?
disp_shader=mat.disp_map.shader_code if mat.disp_shader?
emission_shader=mat.emission_map.shader_code if mat.emission_shader?
		
out.puts  "				<albedo>" if type=="Diffuse"
out.puts "				<diffuse_albedo>" if type=="Phong"
if albedo_tex
	export_texture_index(albedo_tex,out)
elsif albedo_shader
	export_shader(albedo_shader,out)
else
	export_constant_param(result,mat.albedo_texture,out) if not albedo_tex
	#export_rgb_spectrum(out,result) if not albedo_tex
end
out.puts "			</albedo>" unless type=="Phong"
out.puts "			</diffuse_albedo>" if type=="Phong"
out.puts "			<ior>#{mat.ior}</ior>" if type=="Phong"

if type=="Phong"
	out.puts "			<exponent>"
	if exponent_map
		export_texture_index(exponent_map,out)
	elsif exponent_shader
		export_shader(exponent_shader,out)
	else
		out.puts "		<constant>#{mat.exponent_map.b}</constant>"
	end
	out.puts "			</exponent>"
end

####export the common texture attributes
if mat.use_bump? #bump_map or bump_shader
	out.puts "			<bump>"
	export_constant_wip(result,mat.bump_map,out) if !(bump_map or bump_shader)
	export_texture_index(bump_map,out) if bump_map
	export_shader(bump_shader,out) if bump_shader
	out.puts "			</bump>"
end
if mat.use_normal?
	out.puts "			<normal_map>"
	export_constant_wip(result,mat.normal_map,out) if !(normal_map or normal_shader)
	export_texture_index(normal_map,out) if normal_map
	export_shader(normal_shader,out) if normal_shader
	out.puts "			</normal_map>"
end
if mat.use_displace? #disp_map or disp_shader
	out.puts "			<displacement>"
	export_constant_wip(result,mat.disp_map,out) if !(disp_map or disp_shader)
	export_texture_index(disp_map,out) if disp_map
	export_shader(disp_shader,out) if disp_shader
	out.puts "			</displacement>"
end
if mat.emitter?
	out.puts "			<base_emission>"
	export_constant_param(result,mat.base_emission,out)
	out.puts "			</base_emission>"
end
if mat.use_emission? #emission_map or emission_shader
	out.puts "			<emission>"
	if emission_map
		export_texture_index(emission_map,out)
	elsif emission_shader
		export_shader(emission_shader,out)
	else
		export_constant_param(result,mat.emission_map,out)
	end
	out.puts "			</emission>"
end
			
out.puts "		</#{mat.indigo_type}>"
out.puts "		</material>"

#Blend the material with a diffuse transmitter material

@tex_index=0  #we are exporting a new material so reset the texture index			
out.puts "		<material>"
write_name_string(out,3,"name",mat_a)
#out.print "			<name>#{mat_a}</name>\n"
out.print "			<diffuse_transmitter>\n"
if mat.emitter?
	out.puts "			<layer>#{@used_layers.index(mat.layer.to_i)}</layer>" if !(@igm)
	#@used_layers.insert(mat.layer)
end
#export all of the texture xml and collect the indices
albedo_tex=disp_map=emission_map=bump_map=nil
albedo_shader=disp_shader=emission_shader=bump_shader=nil

albedo_tex=export_texture(mat,mat.albedo_texture,out,generate_uvs) if mat.albedo_texture?
bump_map=export_texture(mat,mat.bump_map,out,generate_uvs)  if mat.bump_map?
normal_map=export_texture(mat,mat.normal_map,out,generate_uvs)  if mat.normal_map?	
disp_map=export_texture(mat,mat.disp_map,out,generate_uvs)  if mat.disp_map?
emission_map=export_texture(mat,mat.emission_map,out,generate_uvs) if mat.emission_map?

#collect all of the shaders
albedo_shader=mat.albedo_texture.shader_code if mat.albedo_shader?
bump_shader=mat.bump_map.shader_code if mat.bump_shader?
normal_shader=mat.normal_map.shader_code if mat.normal_shader?
disp_shader=mat.disp_map.shader_code if mat.disp_shader?
emission_shader=mat.emission_map.shader_code if mat.emission_shader?

out.puts  "				<albedo>"
if albedo_tex
	export_texture_index(albedo_tex,out)
elsif albedo_shader
	export_shader(albedo_shader,out)
else
	export_rgb_spectrum(out,result) if not albedo_tex
end
out.puts "			</albedo>"

if mat.use_bump? #bump_map or bump_shader
	out.puts "			<bump>"
	export_constant_wip(result,mat.bump_map,out) if !(bump_map or bump_shader)
	export_texture_index(bump_map,out) if bump_map
	export_shader(bump_shader,out) if bump_shader
	out.puts "			</bump>"
end
if mat.use_normal?
	out.puts "			<normal_map>"
	export_constant_wip(result,mat.normal_map,out) if !(normal_map or normal_shader)
	export_texture_index(normal_map,out) if normal_map
	export_shader(normal_shader,out) if normal_shader
	out.puts "			</normal_map>"
end
if mat.use_displace? #disp_map or disp_shader
	out.puts "			<displacement>"
	export_constant_wip(result,mat.disp_map,out) if !(disp_map or disp_shader)
	export_texture_index(disp_map,out) if disp_map
	export_shader(disp_shader,out) if disp_shader
	out.puts "			</displacement>"
end
if mat.emitter?
	out.puts "			<base_emission>"
	export_constant_param(result,mat.base_emission,out)
	out.puts "			</base_emission>"
end
if mat.use_emission? #emission_map or emission_shader
	out.puts "			<emission>"
	if emission_map
		export_texture_index(emission_map,out)
	elsif emission_shader
		export_shader(emission_shader,out)
	else
		export_constant_param(result,mat.emission_map,out)
	end
	out.puts "			</emission>"
end
out.print "			</diffuse_transmitter>\n"
out.print "		</material>\n"

#Export the blended diffuse transmitter

out.puts "		<material>"
write_name_string(out,3,"name",export_name)
#out.print "			<name>#{export_name}</name>\n"
out.print "			<blend>\n"
write_name_string(out,4,"a_name",mat_a)
write_name_string(out,4,"b_name",mat_b)
#out.print "				<a_name>#{mat_a}</a_name>\n" #transmitter component
#out.print "				<b_name>#{mat_b}</b_name>\n" #normal diffuse component 	
out.puts "				<blend>"
out.puts "					<constant>#{mat.alpha}</constant>"
out.puts "				</blend>"
out.print "			</blend>\n"
out.puts "		</material>"

end

####################################Write the Texture Information#################
def export_texture(indigo_mat,tex,out,generate_uvs=false)

	su_mat=indigo_mat.mat  #get the SU material that we are exporting
	case tex.type
		when "SketchUp"
			tex_path=self.exported_texture_filename(su_mat)
		when "External"
			tex_path=tex.path
			#p tex_path
	end
	if not tex.valid?
		UI.messagebox("Could not find map for material named" + " #{su_mat.name}")
		return
	end	
	uvs=tex.uv_set_index()
	#uvs="default" if uvs=="Default" 
	out.puts "				<texture>"
	out.print "					<uv_set_index>"
	if @igm or @preview_material
		out.print "#{tex.preview_uvset}"
	else
		out.print "#{uvs}"
	end
	out.print "</uv_set_index>\n"
	
	if @igm
		out.print "					<path>#{File.basename(tex_path)}</path>\n"   #the path for the texture is the file basename when exporting an IGM
	else
		out.puts "					<path>#{File.join(@tex_path,tex_path)}</path>" if tex.type=="SketchUp"
		out.puts "					<path>#{tex_path}</path>" if tex.type=="External"
	end
	
	out.puts "					<a>#{tex.a}</a>"
	out.puts "					<b>#{tex.b}</b>"
	out.puts "					<c>#{tex.c}</c>"
	out.print "					<exponent>#{tex.exponent}</exponent>\n"
	out.puts "					<smooth>#{tex.smooth?}</smooth>"
	if generate_uvs!=false
		texture=su_mat.texture
		if texture
			tex_height=texture.height.to_f
			tex_width=texture.width.to_f
		else
			tex_height=12.0  
			tex_width=12.0
		end
		out.puts "				<tex_coord_generation>"
		if generate_uvs=="Spherical"
			out.puts "				<spherical>"
			out.puts "					<rotation>"
			out.puts "						<axis_rotation>"
			out.puts "							<axis>0 0 1</axis>"
			out.puts "							<angle>#{@irs.exr_shift.to_f.degrees}</angle>"
			out.puts "						</axis_rotation>"
			out.puts "					</rotation>"
			out.puts "				</spherical>"
		else
			out.puts "					<uv>"
			out.puts "					<matrix>"
			out.puts "					#{1.0/tex_width} 0 0 #{1.0/tex_height}"
			out.puts "					</matrix>"
			out.puts "					<translation>"
			out.puts "					0 0"
			out.puts "					</translation>"
			out.puts "					</uv>"
		end
		out.puts "				</tex_coord_generation>"
	end
	out.puts "				</texture>"
	@textures.push([su_mat,tex])  #keep track of all the exported textures
	tex_index=@tex_index
	@tex_index+=1
	return tex_index  #return the current index of this texture in the current material
	
end

#########################
def export_texture_index(index,out)

	out.puts "				<texture>"
	out.puts "					<texture_index>#{index}</texture_index>"
	out.puts "				</texture>"

end

#########################
def export_shader(shader_code,out,wld=nil)

	unless shader_code=~/CDATA/
		cdata=true
	else
		cdata=false
	end
	out.puts "				<shader>"
	out.puts "					<shader>"
	out.puts "					#{wrap_cdata(shader_code)}" if cdata
	out.puts "					#{shader_code}" unless cdata
	out.puts "					</shader>"
	out.puts "				<wavelength_dependent>#{wld}</wavelength_dependent>" if wld
	out.puts "				</shader>"

end

#############################Write Procedural Fractral Brownian Motion########
def write_fbm(tex,mat,out)
	out.puts "				<fbm>"
	out.puts "					<u_scale>#{tex.u_scale}</u_scale>"
	out.puts "					<v_scale>#{tex.v_scale}</v_scale>"
	out.puts "				</fbm>"
end
	
####################################Albedo Map Export#############################
def export_albedo_map(mat,out)
	@tex_index.push("albedo")
	albedo_texture=mat.albedo_texture
	out.print "				<texture>\n"
	write_texture(albedo_texture,mat,out)
	out.print " 				</texture>\n"
end

###################################Blend Map Export#############################
def export_blend_map(mat,out)
	@tex_index.push("blend")
	blend_map=mat.blend_map
	out.print "				<blend_map>\n"
	write_texture(blend_map,mat,out)
	out.print " 				</blend_map>\n"
end

###################################Emitter Map Export#############################
def export_emitter_map(mat,out)
	#@tex_index.push("emission")
	emitter_map=mat.albedo_texture
	export_texture(mat,emitter_map,out)
end

###################################Emission Map Export#############################
def export_emission_map(mat,out)
	@tex_index.push("emission")
	emission_map=mat.emission_map
	out.print "				<texture>\n"
	write_texture(emission_map,mat,out)
	out.print " 				</texture>\n"
end

###################################Sigma Map Export#############################
def export_sigma_map(mat,out)
	@tex_index.push("sigma")
	sigma_map=mat.sigma_map
	out.print "				<texture>\n"
	write_texture(sigma_map,mat,out)
	out.print " 				</texture>\n"
end

####################################Bump Map Export#############################
def export_bump_map(mat,out)
	@tex_index.push("bump")
	bump_map=mat.bump_map
	if not bump_map.valid?   
		UI.messagebox ("Could not find bump map for material named") + " #{mat.mat.name}"  
		return
	end
	out.print "				<texture>\n"
	write_texture(bump_map,mat,out)
	out.print " 				</texture>\n"
end

####################################Displacement Map Export#############################
def export_disp_map(mat,out)
	@tex_index.push("displacement")
	dm=mat.disp_map
	if not dm.valid?   
		UI.messagebox ("Could not find displacement map for material named") + " #{mat.mat.name}"  
		return
	end
	out.print "				<texture>\n"
	write_texture(dm,mat,out)
	out.print " 				</texture>\n"
end

####################################Exponent Map Export#############################
def export_exponent_map(mat,out)
	@tex_index.push("exponent")
	exponent_map=mat.exponent_map
	if not exponent_map.valid?   
		UI.messagebox ("Could not find exponent map for material named") + " #{mat.mat.name}"  
		return
	end
	out.print "				<texture>\n"
	write_texture(exponent_map,mat,out)
	out.print " 				</texture>\n"
end
####################################Clip Map Export
def export_clip_map(mat,out,clipped_name,generate_uvs)

	@tex_index=0    #reset the texture index variable since we are now exporting a new material
	clip_index=nil
	clip_shader=nil
	result=mat.rgb
	mat_name=mat.xml_name
	mat_name="previewmaterial" if mat==preview_material
	mat_name="Default" if @instance_mat==mat.mat
	clip_map=mat.clip_map
	type=clip_map.type
	if type=="External"
		if not clip_map.valid?
			UI.messagebox ("Could not find clip map for material named" + " #{mat.mat.name}" ) 
			return
		end	
	end
	write_null_mat(out) if (not @null_exported)
	out.print "		<material>\n"
	write_name_string(out,3,"name",mat_name)
	#out.print "			<name>#{mat_name}</name>\n"
	out.print "			<blend>\n"
	write_name_string(out,4,"a_name",clipped_name)
	#out.print "				<a_name>#{clipped_name}</a_name>\n"
	out.print "				<b_name>None</b_name>\n"
	clip_index=export_texture(mat,clip_map,out,generate_uvs) if mat.clip_map?
	clip_shader=clip_map.shader_code if mat.clip_shader?
				
	out.puts "				<blend>"
	
	export_constant_wip(result,mat.clip_map,out) if !(clip_index or clip_shader)
	export_texture_index(clip_index,out) if clip_index
	export_shader(clip_shader,out) if clip_shader
	
	out.puts "				</blend>"
	out.puts "				<step_blend>true</step_blend>"  if (mat.clip_map? and clip_map.b.to_f==1.0)#force step blend for clip maps
	out.print "			</blend>\n"
	out.print "		</material>\n"

end

###############################################Write Textures########################
def export_textures(textures)   #function takes an array of an array of SU materials and IndigoTexture objects
	
	Sketchup.active_model.start_operation("Export Textures")
	illegal="\\\/:*?\"<>|"   #characters not allowed in filenames
	exported=[]
	tw=Sketchup.create_texture_writer
	entities = @model.active_entities
	helper_group=entities.add_group
	#p "number of textures found = #{textures.length}"
for t in textures
	mat=t[0]  #get the SU material that references this texture
 	tex=t[1]  #get the IndigoTexture object

	case tex.type
		when "SketchUp"  #export the texture to the textures directory
			if (mat.respond_to?(:texture) and mat.texture !=nil)
				export_filename=self.exported_texture_filename(mat)
				helper_group.material=mat
				if not exported.index(export_filename)    #check if this texture has already been exported
					export_path=File.join(File.dirname(@path),tex_path,export_filename)
					tw.load(helper_group)   #load the texture if it has not yet been loaded
					status=tw.write(helper_group,export_path)
					exported.push(export_filename)     #keep track of all of the exported texture filenames
				end
			end		
		when "External"   #copy the file to the textures directory if we are exporting an IGM
			
			export_filename=File.basename(tex.path)
			export_path=File.join(File.dirname(@path),tex_path,export_filename)
			if @igm and (not exported.index(export_filename))
				
				read_file=File.new((tex.path),"rb")
				contents=read_file.read
				read_file.close()
				write_file=File.new((export_path),"wb")
				write_file.write(contents)
				write_file.close()
				exported.push(export_filename)   #keep track of all of the exported texture filenames
			end
		else
			puts("Texture not exported " + tex.inspect)
			 
	end

end

#entities.erase_entities helper_group
Sketchup.active_model.abort_operation
tw=nil

end

#method to provide a unique expoted texture filename for each material and keep the original filename extension (png, jpeg, tif)
def exported_texture_filename(su_mat)
	if su_mat.texture!=nil
		texture_filename=File.basename(su_mat.texture.filename)
		
		extension = texture_filename.split(".").last.downcase
		
		if not %w/jpg png gif tiff tif/.include?(extension)
			extension = "jpg"
		end
		
		return [su_mat.name.downcase.gsub(/[ \.\\\/:*#\[\]?"><\|]/, '_').slice(0..25), extension].join(".")
	else
		puts "Error! Could not get su_mat.texture"
		return nil
	end
end


###############Write the Null Material
def write_null_mat(out)
	if not @null_exported
		out.puts "		<material>"
		out.print "			<name>None</name>\n"
		out.print "			<null_material/>\n"
		out.print "		</material>\n"
		@null_exported=true
	end
end

############################################IGM Exporter
def export_igm(mat,pack)

	out=File.new((path),"w")
	#@tex_path=File.dirname(path)
	@tex_path=""
	@igm=true
	out.puts "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
	out.puts "<scenedata>"
	
	begin
		export_material(mat,out)
	rescue
		out.close
		UI.messagebox "Errors occured while exporting the IGM file."
		@igm=false
		raise
	end
	out.puts "</scenedata>"
	out.close
	
	begin
		
		export_textures(@textures)  #export textures collected during the material export process
		
	rescue
		UI.messagebox "Errors occured while exporting the material's textures."
		raise
	end
	
	@igm=false	
	#######the IGM spec does not require exporting of nkdata files to the IGM path
	#begin
		#for data in @nkdata
			#nk=NKdata.new(data)
			#nk.export(File.dirname(@path))  #exports the nkdata to the textures directory
		#end
	#rescue
		#UI.messagebox "Errors occured while exporting the material's nkdata."
		#raise
	#end
	
	if pack   #the user wishes to pack the material
		status=SkIndigo.pack_igm(path)
		if status==nil
			UI.messagebox("Unable to pack IGM file.")
		end
	else
		UI.messagebox "Material saved to #{path}"
	end

end

#########################################Camera export
def export_camera(v,out)

aperture=@irs.aperture
ap_shape=@irs.aperture_shape
focal_point=@irs.focal_point
exr_shift=@irs.exr_shift.to_f
env=@irs.environment
shutterSpeed=1/(@irs.shutter_speed.to_f)

h = v.vpheight   #get the dimensions of the viewport
w = v.vpwidth
user_camera = v.camera
f=user_camera.focal_length
user_eye = user_camera.eye
user_y = user_camera.yaxis #camera up vector
user_z = user_camera.zaxis #camera forward vector
sensor_width=user_camera.image_width.to_f/1000

if SkIndigo.photomatch?(v)  #check if the current view is photomatched
	aspect=user_camera.aspect_ratio     #view will have a set aspect ratio if in photo match mode
	if sensor_width==0.0                
		sensor_distance=(f.to_f/aspect)/1000
		sensor_width=0.036
	else
		sensor_distance=f.to_f/1000
	end
else
	sensor_width=0.036 if sensor_width==0.0
	old_aspect=user_camera.aspect_ratio
	if old_aspect==0.0  #if aspect ratio is not set, set it to the viewport aspect ratio
		old_aspect=(w.to_f)/(h.to_f)
		sensor_distance=(f.to_f/old_aspect)/1000
	else
		sensor_distance=f.to_f/1000
	end
	aspect=old_aspect
end

unless user_camera.perspective?
	camera_type="orthographic"
	pt=[0,0,0]
	viewport_aspect=(w.to_f)/(h.to_f)
	if viewport_aspect>aspect
		w=w*(aspect/viewport_aspect)
	end
	sensor_width=v.pixels_to_model(w,pt)*@scale
else
	camera_type=@irs.camera_type
end

stop=aperture.split("\/")[1].to_f
ap_radius=sensor_distance/(stop*2)

#Convert the stored focal point to a focus distance
if focal_point!="auto"
	vector=user_eye - focal_point if focal_point
	focus_distance=vector.length*@scale
else
	focus_distance=2.0
end

out.puts "		<camera>"
out.puts "			<camera_type>#{camera_type}</camera_type>"
out.puts "			<pos>" + "%.4f" %(user_eye.x*@scale)+ " "+ "%.4f" % (user_eye.y*@scale) + " " + "%.4f" %(cz=user_eye.z*@scale)+ "</pos>"
out.puts "			<up>" + "%.4f" %(user_y.x)+ " "+ "%.4f" % (user_y.y) + " " + "%.4f" %(user_y.z)+ "</up>"
out.puts "			<forwards>" + "%.4f" %(user_z.x)+ " "+ "%.4f" % (user_z.y) + " " + "%.4f" %(user_z.z)+ "</forwards>"
out.puts "			<aperture_radius>" + "%.4f" %(ap_radius)+"</aperture_radius>"
out.puts "			<focus_distance>"+ "%.4f" %(focus_distance)+"</focus_distance>"
out.puts "			<aspect_ratio>" + "%.4f" %(aspect) + "</aspect_ratio>"
out.puts "			<sensor_width>#{sensor_width}</sensor_width>"
out.puts "			<lens_sensor_dist>" + "%.4f" %(sensor_distance)+"</lens_sensor_dist>"
out.puts "			<white_balance>#{@irs.white_balance}</white_balance>"
out.puts "			<autofocus/>" if focal_point=="auto"
out.puts "			<exposure_duration>#{shutterSpeed}</exposure_duration>"
out.puts "			<aperture_shape>"
case ap_shape
	when "circular"
		out.puts "				<circular/>"
	when "image"
		if !FileTest.exist?(@irs.aperture_image)
			UI.messagebox("Error: Aperture image path is invalid.")
			raise
		end
		out.puts "				<image>"
		out.puts "					<path>#{@irs.aperture_image}</path>"
		out.puts "				</image>"
	else
		out.puts "				<generated>"
		out.puts "					<num_blades>#{ap_shape}</num_blades>"
		out.puts "					<blade_offset>#{@irs.blade_offset}</blade_offset>"
		out.puts "					<blade_curvature_radius>#{@irs.blade_curvature_radius}</blade_curvature_radius>"
		out.puts "					<start_angle>#{@irs.start_angle.to_f*3.14159/180}</start_angle>"
		out.puts "				</generated>"
	
end
out.puts "			</aperture_shape>"
if @irs.use_obstacle_map?
	if !FileTest.exist?(@irs.obstacle_map)
			UI.messagebox("Error: Aperture obstacle map path is invalid.")
			raise
	else
		out.puts "			<obstacle_map>"
		out.puts "				<path>#{@irs.obstacle_map}</path>"
		out.puts "			</obstacle_map>"
	end
end


out.puts "			<lens_shift_up_distance>#{@irs.shift_up.to_f/1000.0}</lens_shift_up_distance>"

out.puts "			<lens_shift_right_distance>#{@irs.shift_right.to_f/1000.0}</lens_shift_right_distance>" 							
out.puts "		</camera>"


end
####
def export_metadata(out)

out.puts "	<metadata>"
out.puts "		<exporter>SkIndigo version:#{SkIndigo::VERSION}</exporter>"
out.puts "		<platform>#{Object::RUBY_PLATFORM}</platform>"
author=ENV["USERNAME"]
if author==nil
	author=ENV["USER"]
end
out.puts "	    <author>#{author.to_s}</author>"
out.puts "		<created_date>#{Time.now}</created_date>"
out.puts "		<exporter_specific>"
out.puts "			<sketchup_version>#{Sketchup.version}</sketchup_version>"
out.puts "		</exporter_specific>"
out.puts "	</metadata>"

end
####
def export_section_planes(out)

ents=Sketchup.active_model.active_entities
planes=ents.find_all {|e| e.class==Sketchup::SectionPlane}

planes.each {|section_plane|

	if section_plane.get_attribute('indigo07_settings','active_section',false)
		plane=section_plane.get_plane
		point=Geom::Point3d.new(0,0,0)
		pp=point.project_to_plane(plane)
		normal=Geom::Vector3d.new(plane[0],plane[1],plane[2]).normalize
		out.puts "		<section_plane>"
		out.puts "		<point>#{pp.x.to_f*@scale} #{pp.y.to_f*@scale} #{pp.z.to_f*@scale}</point>"
		out.puts "		<normal>#{normal.x} #{normal.y} #{normal.z}</normal>"
		out.puts "		<enabled>true</enabled>"
		out.puts "		<cull_geometry>#{@irs.cull_geometry?}</cull_geometry>"
		out.puts "		</section_plane>"
	end #if
}

end
###################################Export Render Settings
def export_render_settings(out)

mat_id=false
alpha=false
gpu=@irs.use_gpu?

case @irs.method
	when "MLT"
		met=true
		bidir=false
		hybrid=false
	when "PT"
		met=false
		bidir=false
		hybrid=false
	when "Hybrid"
		met=false
		bidir=false
		hybrid=true
	when "Bidir MLT"
		met=true
		bidir=true
		hybrid=false
	when "Bidir PT"
		met=false
		bidir=true
		hybrid=false
	when "Bidir Hybrid"
		met=false
		bidir=true
		hybrid=true
	when "Alpha"  #for backwards compatibility with previous skindigo versions - alpha used to be a tracing method
		@irs.method="Bidir PT"
		@irs.foreground_alpha=true
	when "Material_ID"
		met=false
		bidir=false
		hybrid=false
		mat_id=true
		gpu=false
end

width=@irs.width.to_i
height=@irs.height.to_i
view=Sketchup.active_model.active_view
current_camera=view.camera
old_aspect=current_camera.aspect_ratio

if old_aspect==0.0 
	old_aspect=view.vpwidth.to_f/view.vpheight.to_f
	height=(width.to_f/old_aspect).to_i    #set the height to match the aspect ratio of the viewport
	#width=(height.to_f*old_aspect).to_i    #set the width to match the aspect ratio of the viewport
	
end

if SkIndigo.photomatch?(Sketchup.active_model.active_view)   
	current_aspect=current_camera.aspect_ratio   #this is the aspect ratio of the photomatch
	aspect=width.to_f/height.to_f
	if (current_aspect-aspect).abs>0.01  #check if rendered image aspect ratio matches photomatch aspct ratio
		height=(width/current_aspect).to_i   #set the height to match the aspect ratio of the photo
		UI.messagebox "Image dimensions set to #{width}x#{height} to match photo aspect ratio."
	end
end

out.puts "		<renderer_settings>"
out.puts "			<width>#{width}</width>"
out.puts "			<height>#{height}</height>"
out.puts "			<max_num_consec_rejections>#{@irs.mncr.to_i}</max_num_consec_rejections>"
out.puts "			<bidirectional>true</bidirectional>" if bidir
out.puts "			<bidirectional>false</bidirectional>" if not bidir
out.puts "			<metropolis>true</metropolis>" if met
out.puts "			<metropolis>false</metropolis>" if not met
out.puts "			<hybrid>true</hybrid>" if hybrid
out.puts "			<hybrid>false</hybrid>" if not hybrid
out.puts "			<halt_time>#{@irs.halt}</halt_time>" if @irs.halt_unit=="sec"
out.puts "			<halt_samples_per_pixel>#{@irs.halt}</halt_samples_per_pixel>" if @irs.halt_unit=="spp"
out.puts "			<image_save_period>3.0</image_save_period>" if @preview_scene
out.puts "			<image_save_period>#{@irs.image_save_period}</image_save_period>" if !@preview_scene
out.puts "			<auto_choose_num_threads>true</auto_choose_num_threads>" if @irs.auto_choose_threads?
out.puts "			<auto_choose_num_threads>false</auto_choose_num_threads>" if !@irs.auto_choose_threads?
out.puts "			<num_threads>#{@irs.num_threads}</num_threads>" if !@irs.auto_choose_threads?
out.puts "			<save_igi>true</save_igi>" if @irs.save_igi?
out.puts "			<save_igi>false</save_igi>" if !@irs.save_igi?
out.puts "			<watermark>true</watermark>" if @irs.watermark?
out.puts "			<watermark>false</watermark>" if !@irs.watermark?
out.puts "			<aperture_diffraction>#{@irs.aperture_diffraction?}</aperture_diffraction>"
out.puts "			<render_foreground_alpha>#{@irs.foreground_alpha?}</render_foreground_alpha>"
out.puts "			<material_id_tracer>true</material_id_tracer>" if mat_id
out.puts "			<material_id_tracer>false</material_id_tracer>" if not mat_id
out.puts "			<info_overlay>true</info_overlay>" if @irs.info_overlay?
out.puts "			<info_overlay>false</info_overlay>" if !@irs.info_overlay?
out.puts "			<glass_acceleration>#{@irs.glass_acceleration?}</glass_acceleration>"
out.puts "			<super_sample_factor>#{@irs.supersample_factor}</super_sample_factor>"
out.puts "			<vignetting>#{@irs.vignetting?}</vignetting>"
#out.puts "			<frame_upload_period>#{irs.frame_upload_period}</frame_upload_period>" #feature no longer used as of Indigo 3.0
out.puts "			<splat_filter>"
out.puts "				<fastbox/>"
out.puts "			</splat_filter>"
#out.puts "			<max_depth>2000</max_depth>"
#out.puts "			<post_process_diffraction>false</post_process_diffraction>"    #this feature currently disabled
out.puts "			<gpu>#{gpu}</gpu>"
irs_model=IndigoRenderSettings.new
if @irs.render_region?
	#ensure we get region dimensions from model attributes, not scene attributes
	region=irs_model.region
	if (region[1].to_i>0) and (region[3].to_i>0)
		out.puts "			<render_region>"
		out.puts "				<x1>#{[region[0].to_i,width].min}</x1>"
		out.puts "				<y1>#{[region[2].to_i,height].min}</y1>"
		out.puts "				<x2>#{[region[1].to_i,width].min}</x2>"
		out.puts "				<y2>#{[region[3].to_i,height].min}</y2>"
		out.puts "			</render_region>"
	end
end
#out.puts "				<verbose>false</verbose>"
out.puts "		</renderer_settings>"



end

######
def get_used_layers()
Sketchup.active_model.materials.each {|mat|
	im=IndigoMaterial.new(mat)
	layer=im.layer
	if layer!=-1
		@used_layers.insert(im.layer)
	end
}
irs=IndigoRenderSettings.new
case irs.environment
	when "Sun and Sky"
		@used_layers.insert(@irs.sun_layer.to_i)
		@used_layers.insert(@irs.sky_layer.to_i)
	when "Environment Map"
		@used_layers.insert(@irs.env_map_layer.to_i)
		if (@irs.sunsky_weight.to_f>0.0) and (@irs.bg_sun?)
			@used_layers.insert(@irs.sun_layer.to_i)
		end
		#@used_layers.insert(@irs.sky_layer.to_i) if @irs.bg_sun?
end
		
@used_layers=@used_layers.to_a

end


####
def export_light_layer_names(out)

irs_model=IndigoRenderSettings.new
#export only used layers
layer_names=irs_model.light_layer_names
@used_layers.each_index {|index|
	this_layer=@used_layers[index].to_i
	

	export_layer_name(out,layer_names[this_layer],index)
}

end
#################################################Export Tonemapping
def export_tonemapping(out)

out.puts "		<tonemapping>"
case @irs.tm
	when "Reinhard"
		out.puts "			<reinhard>"
		out.puts "				<pre_scale>#{@irs.pre_scale}</pre_scale>"
		out.puts "				<post_scale>#{@irs.post_scale}</post_scale>"
		out.puts "				<burn>#{@irs.burn}</burn>"
		out.puts "			</reinhard>"
	when "Linear"
		out.puts "			<linear>"
		out.puts "				<scale>#{@irs.linearScale}</scale>"
		out.puts "			</linear>"
	when "Camera"
		out.puts "			<camera>"
		out.puts "				<film_iso>#{@irs.iso}</film_iso>"
		out.puts "				<ev_adjust>#{@irs.ev_adjust}</ev_adjust>"
		out.puts "				<response_function_path>data/camera_response_functions/#{@irs.cam_resp_func}.txt</response_function_path>"
		out.puts "			</camera>"
			
end
out.puts "		</tonemapping>"

end

#################################Export Environment
def export_environment(out)

#get the sun direction
override=@irs.sun_pos_override?
if override
	#ensure we get the sun position from the model attributes, not the scene attributes
	irs_model=IndigoRenderSettings.new
	v=irs_model.sun_dir
	if v==[]
		UI.messagebox("Warning: Sun position override not set.  Exporting SketchUp sun position.")
		override=false
	end
end
if !override
	si = Sketchup.active_model.shadow_info
	v = si["SunDirection"]
end

sun_layer=@irs.sun_layer
sky_layer=@irs.sky_layer
		
case @irs.environment
	when "Sun and Sky"
		
		@light_source=true
		#export_layer_name(out,"Sun",SUN_LAYER_INDEX)
		#export_layer_name(out,"Sky",SKY_LAYER_INDEX)
	
		out.puts "		<skylight>"
		out.puts "			<sundir>"+ "%.4f" % (v.x) + " " + "%.4f" % (v.y) + " " + "%.4f" % (v.z) + "</sundir>"
		out.puts "			<turbidity>#{@irs.turbidity}</turbidity>"
		out.puts "			<sun_layer>#{@used_layers.index(sun_layer.to_i)}</sun_layer>"
		out.puts "			<sky_layer>#{@used_layers.index(sky_layer.to_i)}</sky_layer>"
		out.puts "			<model>#{@irs.sky_model}</model>"
		out.puts "		</skylight>"
		#@used_layers.insert(sun_layer)
		#@used_layers.insert(sky_layer)
	when "Background"
		@light_source=true
		#export_layer_name(out,"Background",BACKGROUND_LAYER_INDEX)
		bg_gain=20            #calibrated gain of background spectrum to balance meshlights of common efficacy scale
		result=Sketchup.active_model.rendering_options["BackgroundColor"]
		if result
			red=result.red*bg_gain
			green=result.green*bg_gain
			blue=result.blue*bg_gain
		else
			red=0.0
			green=0.0
			blue=0.0
		end
		out.puts "		<background>"
		out.puts "			<spectrum>"
		out.puts "				<rgb>"
		out.print "					<rgb>#{red} #{green} #{blue}</rgb>\n"
		out.print "					<gamma>2.2</gamma>\n"
		out.puts "				</rgb>"
		out.puts "			</spectrum>"
		out.puts "		</background>"
	when "Environment Map"
		
		envmap_angle=@irs.exr_shift.to_f
		mats=Sketchup.active_model.materials
		#locate the background material
		bg_mat=mats.find {|mat| mat.name==@irs.bg_material}
		if bg_mat==nil
			UI.messagebox "Warning: Background material not found.  You must specify a valid material name to use as an environment map."
			raise "Invalid_Background_Material"
		end
		bg_mat=IndigoMaterial.new(bg_mat)
		if bg_mat.layer==-1
				UI.messagebox("Warning: Environment map material light layer is set to off.")
		else
			@light_source=true
		end
		
		#export the sun_sky material
		if @irs.bg_sun?
			@light_source=true
			bg_mat_name="background"
			export_material(bg_mat,out,"Spherical") #first export the background material
			out.puts "<material>"
			out.puts "	<name>sunsky</name>"
			out.puts "		<sunsky>"
			out.puts "			<sundir>"+ "%.4f" % (v.x) + " " + "%.4f" % (v.y) + " " + "%.4f" % (v.z) + "</sundir>"
			out.puts "			<turbidity>#{@irs.turbidity}</turbidity>"
			out.puts "			<enable_sky>false</enable_sky>"
			out.puts "			<sun_layer>#{@used_layers.index(sun_layer.to_i)}</sun_layer>" if @irs.sunsky_weight.to_f>0.0
			#out.puts "			<sky_layer>#{@used_layers.index(sky_layer.to_i)}</sky_layer>" if @irs.bg_sun?
			out.puts "		</sunsky>"
			out.puts "</material>"
			out.puts "<background_settings>"
			out.puts "	<background_material>"
			out.puts "	<material>"
			#out.puts "	<name>#{bg_mat_name}</name>"
			write_name_string(out, 1, "name", bg_mat_name)
			out.puts "	<sum>"
			out.puts "		<mat>"
			out.puts "			<mat_name>sunsky</mat_name>"
			out.puts "			<weight>"
			out.puts "				<constant>#{@irs.sunsky_weight}</constant>"
			out.puts "			</weight>"
			out.puts "		</mat>"
			out.puts "		<mat>"
			#out.puts "			<mat_name>#{bg_mat.xml_name}</mat_name>"
			write_name_string(out, 3, "mat_name", bg_mat.xml_name)
			out.puts "			<weight>"
			out.puts "				<constant>#{@irs.bg_weight}</constant>"
			out.puts "			</weight>"
			out.puts "		</mat>"
			out.puts "	</sum>"
			out.puts "	</material>"
			out.puts "</background_material>"
		else
			
			bg_mat_name=bg_mat.xml_name
			out.puts "<background_settings>"
			out.puts "	<background_material>"
			export_material(bg_mat,out,"Spherical") #export the background material
			#out.puts "	<name>#{bg_mat_name}</name>"
			out.puts "</background_material>"
		end
			
		out.puts "<emission_scale>"
		# out.puts "	<material_name>#{bg_mat_name}</material_name>"
		write_name_string(out, 1, "material_name", bg_mat_name)
		out.puts "	<measure>#{bg_mat.emission_measure}</measure>"
		out.puts "	<value>#{bg_mat.emission_scale}</value>"
		out.puts "</emission_scale>"
		out.puts "</background_settings>"
		@exported_mats=[]  #clear the array of exported materials so that the bg material will be exported again as a regular material
end #end case


end

#####
def export_layer_name(out,name,index)

out.puts "<layer_name>"
#out.puts "	<layer_name>#{name}</layer_name>"
write_name_string(out, 1, "layer_name", name)
out.puts "	<layer_index>#{index}</layer_index>"
out.puts "</layer_name>"

end

##############################################Export RGB Spectrum
def export_rgb_spectrum(out,result)

out.puts "				<constant>"
out.puts "					<rgb>"
out.puts "						<rgb>" + "%.4f" %((result[0]/255.0)) + " "+ "%.4f" % ((result[1]/255.0)) + " "+ "%.4f" % ((result[2]/255.0)) + "</rgb>"
out.puts "						<gamma>2.2</gamma>"
out.puts "					</rgb>"
out.puts "				</constant>"

end

###############################################Export Default Material
def export_default_mat(out,mat)

@instance_mat=mat
if not mat 
	colour = Sketchup.active_model.rendering_options["FaceFrontColor"]
	result=colour.red * 0.8,colour.green * 0.8,colour.blue * 0.8
	result=[216,216,216] if @monochrome=="Clay"
	out.puts "		<material>"
	out.print "			<name>Default</name>\n"
	out.print "			<oren_nayar>\n"
	out.puts  "				<albedo>"
	export_rgb_spectrum(out,result)
	out.puts "				</albedo>"
	out.puts "			<sigma>"
	out.puts "				<constant>0.5</constant>"
	out.puts "			</sigma>"
	out.puts "			</oren_nayar>"
	out.print " 		</material>\n"
else							#this will export any material as the default material (useful for instancing)
	#@instance_mat=mat				#keeps track of the last instanced material
	#write_mat(IndigoMaterial.new(mat),out,true)
	export_material(IndigoMaterial.new(mat),out,true)
end

end


##############################################Fog Export
def export_fog(out)

options=model.rendering_options
model_extents=model.bounds
eye=model.active_view.camera.eye
model_extents.add eye #ensures that the camera is inside the fog sphere
center=model_extents.center

cx=center.x*@scale
cy=center.y*@scale
cz=center.z*@scale

depth=model_extents.depth*@scale
height=model_extents.height*@scale
width=model_extents.width*@scale

d=depth
d=height if height>depth
d=width if (width>height) and (width>depth)

r=d/2 #fog sphere radius

fog_end=options["FogEndDist"]
fog_end=fog_end*@scale
fog_color=options["FogColor"]
alpha_factor=1

sss=2.772/fog_end  #distance at which light intensity is halved is 0.693/scattering coefficient

out.puts "		<medium>"
out.print "			<name>Fog_medium</name>\n"
out.print "			<precedence>2</precedence>\n"
out.print "			<basic>\n"
out.print "				<ior>1.0</ior>\n"
out.print "				<cauchy_b_coeff>0.0</cauchy_b_coeff>\n"
out.puts "				<absorption_coefficient_spectrum>"
out.puts "					<rgb>"
out.print "						<rgb>#{(1.0-fog_color.red/255.0)*alpha_factor} #{(1.0-fog_color.green/255.0)*alpha_factor} #{(1.0-fog_color.blue/255.0)*alpha_factor}</rgb>\n"
out.print "						<gamma>1.0</gamma>\n"
out.print "					</rgb>\n"
out.print "				</absorption_coefficient_spectrum>\n"
out.print "				<subsurface_scattering>\n"
out.print "					<scattering_coefficient_spectrum>\n"
out.print "						<uniform>\n"
out.print "							<value>#{sss}</value>\n"
out.print "						</uniform>\n"
out.print "					</scattering_coefficient_spectrum>\n"
out.print "					<phase_function>\n"
out.print "						<uniform/>\n"
out.print "					</phase_function>\n"
out.print "				</subsurface_scattering>\n"
out.print "			</basic>\n"
out.print "		</medium>\n"
out.print "		<material>\n"
out.print " 		<name>Fog</name>\n"
out.print " 		<specular>\n"
out.print " 			<transparent>true</transparent>\n"
out.print "			<internal_medium_name>Fog_medium</internal_medium_name>\n"
out.print " 		</specular>\n"
out.print " 		</material>\n"

out.puts "		<sphere>"
out.puts "			<center>#{cx} #{cy} #{cz}</center>"
out.puts "			<radius>#{r}</radius>"
out.puts "			<material_name>Fog</material_name>"
out.puts "		</sphere>"

end

###########################################Export Sphere Primitivies
def export_sphere(inst,out)

trans=inst.transformation
sphere_mat=inst.material

if sphere_mat
	sphere_mat=IndigoMaterial.new(sphere_mat)
	sphere_mat_name=sphere_mat.xml_name
else
	sphere_mat_name="Default"
end

box=inst.su_instance.bounds
center=box.center.transform(trans) #get the sphere in world coordinates
cx=center.x*@scale
cy=center.y*@scale
cz=center.z*@scale

r=box.height*@scale/2 #set radius of sphere equal to the half the height of its bounding box

out.puts "	<sphere>"
out.puts "		<center>#{cx} #{cy} #{cz}</center>"
out.puts "		<radius>#{r}</radius>"
write_name_string(out, 2, "material_name", sphere_mat_name)
out.puts "	</sphere>"

end


$image_export_counter = 0

#####export sketchup images
def export_image(image,out)

su_image=image[0]
id=su_image.entityID.to_s
trans=image[1]
path=su_image.path
height=su_image.height.to_f*@scale
normal=su_image.normal
#normal=Geom::Vector3d.new(0,0,1)
width=su_image.width.to_f*@scale
origin=su_image.origin
zrot=su_image.zrotation

#trans=Geom::Transformation.new(origin,normal,zrot)
local_trans=Geom::Transformation.new(origin,normal)
rot_trans=Geom::Transformation.rotation([0,0,0],Z_AXIS,zrot)
trans=trans*(local_trans*rot_trans)

found_texture=true
unless FileTest.exist?(path)

	# Write the image to disk
	basename=File.basename(path)
	path=File.join(File.dirname(@path),tex_path,basename)

	# puts "Writing image to '#{path}'..."

	tw=Sketchup.create_texture_writer
	tw.load(su_image)	
	status=tw.write(su_image, path)

	# TODO: handle status != 0?
end
mat_name=File.basename(path)+id + "_" + $image_export_counter.to_s
$image_export_counter += 1

out.puts "<material>"
#out.puts "			<name>#{mat_name}</name>"
write_name_string(out, 3, "name", mat_name)
out.puts "		<diffuse>"
if found_texture==true
	out.puts "			<texture>"
	out.puts "					<uv_set>default</uv_set>"
	out.puts "					<path>#{path}</path>"
	out.puts "					<a>0.0</a>"
	out.puts "					<b>1.0</b>"
	out.puts "					<c>0.0</c>"
	out.puts "					<exponent>2.2</exponent>"
	out.puts "				</texture>"
end

out.puts "			<albedo>"
if found_texture==true
	out.puts "				<texture>"
	out.puts "					<texture_index>0</texture_index>"
	out.puts "				</texture>"
else
	out.puts "					<constant>"
	out.puts "					<uniform>"
	out.puts "						<value>0.5</value>"
	out.puts "					</uniform>"
	out.puts "					</constant>"
end
out.puts "			</albedo>"
out.puts "		</diffuse>"
out.puts "		</material>"
out.puts "<mesh>"
#out.puts "			<name>#{mat_name}</name>"
write_name_string(out, 3, "name", mat_name)
out.puts "			<normal_smoothing>true</normal_smoothing>"
out.puts " 			<embedded_2>"
out.puts "				<expose_uv_set>"
out.puts "					<index>0</index>"
out.puts "					<name>default</name>"
out.puts "				</expose_uv_set>"
#out.puts "				<used_material_name>#{mat_name}</used_material_name>"
write_name_string(out, 4, "used_material_name", mat_name)
#out.puts "				<v p=\"#{origin.x.to_f} #{origin.y.to_f} #{origin.z.to_f}\" n=\"#{normal.x.to_f} #{normal.y.to_f} #{normal.z.to_f}\"\/>"
#out.puts "				<v p=\"#{origin.x.to_f+width} #{origin.y.to_f} #{origin.z.to_f}\" n=\"#{normal.x.to_f} #{normal.y.to_f} #{normal.z.to_f}\"\/>"
#out.puts "				<v p=\"#{origin.x.to_f} #{origin.y.to_f+height} #{origin.z.to_f}\" n=\"#{normal.x.to_f} #{normal.y.to_f} #{normal.z.to_f}\"\/>"
#out.puts "				<v p=\"#{origin.x.to_f+width} #{origin.y.to_f+height} #{origin.z.to_f}\" n=\"#{normal.x.to_f} #{normal.y.to_f} #{normal.z.to_f}\"\/>"
out.puts "				<v p=\"0.0 0.0 0.0\" n=\"#{normal.x.to_f} #{normal.y.to_f} #{normal.z.to_f}\"\/>"
out.puts "				<v p=\"#{width} 0.0 0.0\" n=\"#{normal.x.to_f} #{normal.y.to_f} #{normal.z.to_f}\"\/>"
out.puts "				<v p=\"0.0 #{height} 0.0\" n=\"#{normal.x.to_f} #{normal.y.to_f} #{normal.z.to_f}\"\/>"
out.puts "				<v p=\"#{width} #{height} 0.0\" n=\"#{normal.x.to_f} #{normal.y.to_f} #{normal.z.to_f}\"\/>"

out.puts "				<uvs uv0=\"0.0000 0.0000\" \/>"
out.puts "				<uvs uv0=\"1.0000 0.0000\" \/>"
out.puts "				<uvs uv0=\"0.0000 1.0000\" \/>"
out.puts "				<uvs uv0=\"1.0000 1.0000\" \/>"
out.puts "				<t v=\"0 1 2\" uv=\"0 1 2\" m=\"0\"\/>"
out.puts "				<t v=\"1 3 2\" uv=\"1 3 2\" m=\"0\"\/>"
out.puts "			</embedded_2>"
out.puts "		</mesh>"
out.puts "		<model>"
self.write_transformation(trans,out)
#out.puts " 			<mesh_name>#{mat_name}</mesh_name>"
write_name_string(out, 4, "mesh_name", mat_name)
out.puts "		</model>"


end

end  #End IndigoExporter class

class Sketchup::ComponentDefinition

def indigo_instance?
	#return true
	instanced=self.get_attribute 'indigo07_settings','inst',false	
	return instanced
end

end

class IndigoExporter

def export_meshes2(meshbuilder,out)


mesh_dir=SkIndigo.temp_dir
num_meshes=meshbuilder.count_meshes()
count=0

meshbuilder.indigo_meshes.each_value {|imesh|


Sketchup.set_status_text("Exporting Meshes...")
out.puts "		<mesh>"
#out.puts "			<name>#{imesh.name}</name>"
write_name_string(out, 3, "name", imesh.name)
out.puts "			 <normal_smoothing>true</normal_smoothing>\n"  
out.puts "			 <max_num_subdivisions>#{imesh.max_subdiv}</max_num_subdivisions>"
out.puts "			<subdivide_pixel_threshold>#{imesh.pixel_threshold}</subdivide_pixel_threshold>"
out.puts "			<subdivide_curvature_threshold>#{imesh.curvature_threshold}</subdivide_curvature_threshold>"
out.puts "			<subdivision_smoothing>#{imesh.subdivision_smoothing?}</subdivision_smoothing>"
out.puts "			<displacement_error_threshold>0.0005</displacement_error_threshold>"
out.puts "			<merge_vertices_with_same_pos_and_normal>true</merge_vertices_with_same_pos_and_normal>"
out.puts " 			<external>"
if imesh.use_external?
	mesh_path=imesh.external_mesh_path
else
	mesh_path=File.join(mesh_dir,imesh.guid+".igmesh")
	imesh.write_to_file(mesh_path) unless imesh.valid_igmesh==true
	imesh.valid_igmesh=true if FileTest.exist?(mesh_path)
end
out.puts "				<path>#{mesh_path}</path>"
out.puts "			</external>"
out.puts "		</mesh>"
count=count+1
}
	
	
end

##################################Export all meshes
def export_meshes(meshbuilder,out)
	
	mm=meshbuilder.mesh_materials
	#Export the material meshes
	mm.each do |material,faces|
		mesh={material=>faces}    #create a new hash that contains only the faces from this material
		export_mesh(material,mesh,out)
		#export_mesh_bin(material,mesh,out)   #binary mesh export test
		if material==nil
			#export_model("Default",Geom::Transformation.new,out)
			export_model(mesh,"Default",Geom::Transformation.new,out)
		else
			im=IndigoMaterial.new(material)
			case im.type
				when "Blackbody Light","RGB Light"
					export_meshlight(im,Geom::Transformation.new,out)
				when "Exit Portal"
					export_exit_portal(im.name,Geom::Transformation.new,out)
				else
					#export_model(im.name,Geom::Transformation.new,out)
					export_model(mesh,im.name,Geom::Transformation.new,out)
			end
		end
	end

	#Export the component meshes
	meshbuilder.component_meshes.each {|mesh| export_mesh(mesh[0],mesh[1],out)}
	#meshbuilder.component_meshes.each {|mesh| export_mesh_bin(mesh[0],mesh[1],out)}
	
	#Export the instances
	#export_instances(meshbuilder,out)
	
end

####################################Export Instances
def export_instances(meshbuilder,out)
	
	#Export the instances
	indigo_instances=meshbuilder.component_instances
	num_instances=meshbuilder.instances
	count=0
	indigo_instances.each {|mat,instances|
		@exported_mats=[]
		export_default_mat(out,mat) if mat!=@instance_mat  #this function allows instances to use different materials
		for inst in instances
			imesh=meshbuilder.get_imesh(inst)
			
			# Work around issue #572 - Error when exporting scene.
			# For some reason imesh is nil here.  Need to work out why.
			if imesh.nil?
				next
			end
			
			trans=inst.transformation
			definition=imesh.definition
			mesh_name=imesh.name
			su_instance=inst.su_instance
							
			if su_instance and su_instance.has_indigo_keyframes?
				keyframes=su_instance.get_indigo_keyframes
				export_model(imesh,mesh_name,inst,trans,out,keyframes)
			else
				export_model(imesh,mesh_name,inst,trans,out)
			end
			count=count+1
		end
	}
	@exported_mats=[]
	@null_exported=false
end

########check if geometry has been flipped - i.e. not right-handed coordinate system
def is_mirrored?(trans)

flipped=false

xa=trans.xaxis
ya=trans.yaxis
za=trans.zaxis

xy=xa.cross(ya)  #should be same direction as z axis
xz=xa.cross(za)  #should be same direction as negative y
yz=ya.cross(za)   #should be same direction as x axis

if !xy.samedirection?(za)
	flipped=!flipped
end
if !xz.samedirection?(ya.reverse)
	flipped=!flipped
end
if !yz.samedirection?(xa)
	flipped=!flipped
end

return flipped


end

######################################### Model Exporter
def export_model(imesh,mesh_name,instance,trans,out,keyframes=[])

if imesh.exit_portal?
	export_exit_portal(mesh_name,trans,out)
	return
end

lem=imesh.get_lems()  #get the light emitting materials of the mesh
emitting_faces_material=imesh.emitter?
emitting_instance_material=false
if IndigoMaterial.emitter?(instance.material)
	if imesh.has_default_material?
		emitting_instance_material=true
	end
end

out.print "		<model>\n"

if keyframes.length>0
	write_keyframes(trans,keyframes,out)
else
	write_transformation(trans,out)
end

#out.print " 			<mesh_name>#{mesh_name}</mesh_name>\n"
write_name_string(out, 3, "mesh_name", mesh_name)


if emitting_faces_material
	lem.each {|im| export_emission_scale(im,im.xml_name,out)}
end

if emitting_instance_material
	export_emission_scale(IndigoMaterial.new(instance.material),"Default",out)
end

out.print "		</model>\n"
	
end

####
def export_emission_scale(im,lem_name,out)

@light_source=true
out.puts "			<emission_scale>"
write_name_string(out, 4, "material_name", lem_name)
out.puts "				<measure>#{im.emission_measure}</measure>" 
out.puts "				<value>#{im.emission_scale}</value>"
out.puts "			</emission_scale>"
if im.ies?  #check if lem using ies profile
	ies_data=im.ies_profile  #returns in IESProfile object
	if ies_data and ies_data.valid?
		out.puts "				<ies_profile>"
		write_name_string(out, 5, "material_name", lem_name)
		out.puts "					<path>#{File.join(self.tex_path,ies_data.filename)}</path>"
		out.puts "				</ies_profile>"
		@ies.push(ies_data.path) if (not @ies.index(ies_data.path))    #copies stores the ies path for future export
	end
end

end
##########################################Export the exit portal
def export_exit_portal(mesh_name,trans,out)
	
out.print "		<exit_portal>\n"
write_transformation(trans,out)
#out.print " 			<mesh_name>#{mesh_name}</mesh_name>\n"
write_name_string(out, 4, "mesh_name", mesh_name)
out.print "		</exit_portal>\n"
	
end

###############################Extract and write the transformation of the model
def write_transformation(trans,out)

#trans=trans*@exrshift if @exrshift  #deprecated in 1.1.16
trans=trans.to_a

#the following code may not be required.  It extracts the scale of the instance
#scalex=Math.sqrt(trans[0]**2+trans[1]**2+trans[2]**2)
#scaley=Math.sqrt(trans[4]**2+trans[5]**2+trans[6]**2)
#scalez=Math.sqrt(trans[8]**2+trans[9]**2+trans[10]**2)
			
#scale=scalex
#scale=scaley if scaley>scale
#scale=scalez if scalez>scale

out.print "			<pos>#{trans[12]*@scale} #{trans[13]*@scale} #{trans[14]*@scale}</pos>\n"
#out.print "			<scale>#{scale}</scale>\n"
out.print "			<scale>1.0</scale>\n"	
out.print "			<rotation>\n"
out.print "				<matrix>\n"
#out.print "					#{trans[0]/scalex} #{trans[4]/scaley} #{trans[8]/scalez} #{trans[1]/scalex} #{trans[5]/scaley} #{trans[9]/scalez} #{trans[2]/scalex} #{trans[6]/scaley} #{trans[10]/scalez}\n"
out.print "					#{trans[0]} #{trans[4]} #{trans[8]} #{trans[1]} #{trans[5]} #{trans[9]} #{trans[2]} #{trans[6]} #{trans[10]}\n"
out.print "				</matrix>\n"
out.print "			</rotation>\n"

end

########
def write_keyframe(time,trans,out)

origin=trans.origin
axis_angle=SkIndigo.get_axis_angle(trans)
axis=axis_angle[0]
angle=axis_angle[1]
out.puts "<keyframe>"
out.puts "		<time>#{time.to_f}</time>"
out.puts "		<pos>#{origin.x.to_f*@scale} #{origin.y.to_f*@scale} #{origin.z.to_f*@scale}</pos>"
out.puts "		<rotation_quaternion>"
out.puts "			<axis>#{axis.x} #{axis.y} #{axis.z}</axis>"
out.puts "			<angle>#{angle}</angle>"
out.puts "		</rotation_quaternion>"
out.puts "</keyframe>"

end

######
def write_keyframes(trans,keyframes,out)

out.print "			<scale>1.0</scale>\n"	

#export the first transformation keyframe
keyframes.each {|kf|
	time=kf[0]  #get the time
	trans=Geom::Transformation.new(kf[1])  #get the transformation
	write_keyframe(time,trans,out)
}

end

######
def write_sketchy_physics_keyframes(trans,keyframes,out)

out.print "			<scale>1.0</scale>\n"	

num_keyframes=trans.length.to_f  #get the number of keyframes
#export the first transformation keyframe
write_keyframe(0.0,trans.first,out)
intermediate_transformations=trans[1..-2] #we have already checked prior to calling this method that there are at least two transformations in the array, so this will never return nil
unless intermediate_transformations.empty? 
	timestep=1.0/(num_keyframes-1)
	time=0.0
	intermediate_transformations.each_index {|i|
		tr=intermediate_transformations[i]
		time=time+timestep
		write_keyframe(time,tr,out)
	}
end
write_keyframe(1.0,trans.last,out)

end

end  # end class

#####################################Region Selection Tool####################################
class SelectRegionTool

def initialize
    @ip1 = nil
    @ip2 = nil
    @xdown = 0
    @ydown = 0
	@x1=0
	@x2=0
	@y1=0
	@y2=0
end

def activate
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false    
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
    if( @state == 0 )
        # We are getting the first point of the region.  Call the pick method
        # on the InputPoint to get a 3D position from the 2D screen position
        # that is bassed as an argument to this method.
        @ip.pick view, x, y
        if( @ip != @ip1 )
            # if the point has changed from the last one we got, then
            # see if we need to display the point.  We need to display it
            # if it has a display representation or if the previous point
            # was displayed.  The invalidate method on the view is used
            # to tell the view that something has changed so that you need
            # to refresh the view.
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
		@x1=x
		@y1=y
            view.tooltip = @ip1.tooltip
        end
    else
        # Getting the second point of the region
        @ip2.pick view, x, y
        view.tooltip = @ip2.tooltip if( @ip2.valid? )
        view.invalidate
        
        # Update the length displayed in the VCB
        if( @ip2.valid? )
            length = @ip1.position.distance(@ip2.position)
            Sketchup::set_status_text length.to_s, SB_VCB_VALUE
		@x2=x
		@y2=y
        end
        
        # Check to see if the mouse was moved far enough to create a line.
        # This is used so that you can create a line by either draggin
        # or doing click-move-click
        if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
            @dragging = true
        end
    end
end

# The onLButtonDOwn method is called when the user presses the left mouse button.
def onLButtonDown(flags, x, y, view)
    # When the user clicks the first time, we switch to getting the
    # second point.  When they click a second time we create the line
    if( @state == 0 )
        @ip1.pick view, x, y
        if( @ip1.valid? )
            @state = 1
            Sketchup::set_status_text("Select region", SB_PROMPT)
            @xdown = x
            @ydown = y
		@x1=x
		@y1=y
        end
    else
        # create the line on the second click
        if( @ip2.valid? )
            self.save_region(view)
            self.reset(view)
        end
    end
    
    # Clear any inference lock
    view.lock_inference
end

# The onLButtonUp method is called when the user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
    # If we are doing a drag, then create the line on the mouse up event
    if( @dragging && @ip2.valid? )
        self.save_region(view)
        self.reset(view)
    end
end

# onKeyDown is called when the user presses a key on the keyboard.
# We are checking it here to see if the user pressed the shift key
# so that we can do inference locking
def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2, @ip1
        end
    end
end

# onKeyUp is called when the user releases the key
# We use this to unlock the interence
# If the user holds down the shift key for more than 1/2 second, then we
# unlock the inference on the release.  Otherwise, the user presses shift
# once to lock and a second time to unlock.
def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end


# The draw method is called whenever the view is refreshed.  It lets the
# tool draw any temporary geometry that it needs to.
def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        
        if( @ip2.valid? )
            @ip2.draw(view) if( @ip2.display? )
            
            # The set_color_from_line method determines what color
            # to use to draw a line based on its direction.  For example
            # red, green or blue.
            view.set_color_from_line(@ip1, @ip2)
            self.draw_geometry([@x1,@y1,0],[@x2,@y1,0],[@x2,@y2,0],[@x1,@y2,0], view)
            @drawn = true
        end
    end
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
end


# The following methods are not directly called from SketchUp.  They are
# internal methods that are used to support the other methods in this class.

# Reset the tool back to its initial state
def reset(view)
    # This variable keeps track of which point we are currently getting
    @state = 0
	@x1=0
	@x2=0
	@y1=0
	@y2=0
    
    # Display a prompt on the status bar
    Sketchup::set_status_text("Select region", SB_PROMPT)
    
    # clear the InputPoints
    @ip1.clear
    @ip2.clear
    
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    
    @drawn = false
    @dragging = false
end

# Create new geometry when the user has selected two points.
def save_region(view)

irs=IndigoRenderSettings.new
w=view.vpwidth.to_f
h=view.vpheight.to_f
width=irs.width
height=irs.height
cam=view.camera
aspect=cam.aspect_ratio
view_aspect=w/h
aspect=view_aspect if aspect==0.0
if view_aspect>=aspect   #black borders on left and right edges
	eff_width_pixels=aspect*h
	border_offset=((w-eff_width_pixels)/2).to_i
	width_fraction_x1=(@x1-border_offset)/eff_width_pixels
	width_fraction_x2=(@x2-border_offset)/eff_width_pixels
	height_fraction_y1=(@y1)/h
	height_fraction_y2=(@y2)/h
	x1=(width*width_fraction_x1).to_i
	x2=(width*width_fraction_x2).to_i
	y1=(height*height_fraction_y1).to_i
	y2=(height*height_fraction_y2).to_i

elsif view_aspect<aspect  #black borders on top and bottom edges
	eff_height_pixels=w/aspect
	border_offset=((h-eff_height_pixels)/2).to_i
	width_fraction_x1=(@x1)/w
	width_fraction_x2=(@x2)/w
	height_fraction_y1=(@y1-border_offset)/eff_height_pixels
	height_fraction_y2=(@y2-border_offset)/eff_height_pixels
	x1=(width*width_fraction_x1).to_i
	x2=(width*width_fraction_x2).to_i
	y1=(height*height_fraction_y1).to_i
	y2=(height*height_fraction_y2).to_i
	
end

#Validate region and store

min_x=[x1,x2].min
max_x=[x1,x2].max
min_y=[y1,y2].min
max_y=[y1,y2].max
min_x=0 if min_x<0
max_x=width if max_x>width
min_y=0 if min_y<0
max_y=height if max_y>height
reg=[min_x,max_x,min_y,max_y]
UI.messagebox "Region (#{min_x},#{min_y}) (#{max_x},#{max_y}) selected."
irs.region=reg
	
end

# Draw the geometry
def draw_geometry(pt1, pt2, pt3, pt4, view)
    view.draw2d(GL_LINE_LOOP,[pt1,pt2,pt3,pt4])
end

end #class


###indigo mesh class for binary exporting - based on code that nick made opensource
class IndigoMesh

attr_reader :num_uv_mappings, :used_materials, :uv_set_expositions, :vert_positions, :vert_normals, :uv_pairs, :triangles, :valid_igmesh
attr_writer :num_uv_mappings, :used_materials, :uv_set_expositions, :vert_positions, :vert_normals, :uv_pairs, :triangles, :valid_igmesh

MAX_STRING_LENGTH=1024
MAX_VECTOR_LENGTH=1000000
MAGIC_NUMBER=5456751
FORMAT_VERSION=1

@@dict="indigo07_materials"
@@max_subdiv=0
@@view_dependent=true
@@pixel_threshold=4.0
@@curvature_threshold=0.1
@@sub_smooth=false
@@normal_smoothing=true
@@external_mesh_path=""
@@use_external=false
@@exit_portal=false
@@uv_set=0

def IndigoMesh.external_skp?(defn)
	use_external=defn.get_attribute(@@dict,'use_external',@@use_external)
	path=defn.get_attribute(@@dict,'external_mesh_path',@@external_mesh_path)
	#p use_external
	#p path
	if use_external==true
		#p "external mesh"
		if FileTest.exist?(path)
			#p "file exists"
			if path[-4..-1]==".skp"
				#p "returning true"
				return true
			end
		end
	end
end

def initialize(defn=nil)


if defn
	@defn=defn  #store the component definition object
	@name=defn.name
	if @name==""
		@name="__model__"
	end
	if FileTest.exist?(defn.description) and defn.name[0..3]=="EXT_"
		self.external_mesh_path=defn.description
		self.use_external=true
	end
end

#self.add_uvset(0)
dict=@defn.attribute_dictionary('uvsets')
if dict
	num_uvsets=dict.length
	#p "uv sets #{num_uvsets}"
else
	num_uvsets=1
end
@num_uv_mappings=num_uvsets  #there will always be at least one
@used_materials=[]  #array of strings of material names
@uv_set_expositions=[]  #array of uv sets (string name and uint32 uv set index)
num_uvsets.times {|i|
	name=i.to_s
	name="Default" if i==0
	@uv_set_expositions.push([name,i])
}
#p @uv_set_expositions
@vert_positions=[]  #array of vertex positions (3 floats)
@vert_normals=[]  #array of normals (3 floats)
@uv0=[]
@uv1=[]
@uv2=[]
@uv3=[]
@uv_pairs=[]   #array of UVs (2 floats)
@triangles=[]  #array of triangles  (uint32 vertex indices[3], uint32 uv indices[3], uint32 material index)
@su_materials=[]
@lems=nil  #array of light emitting indigo materials
@emitter=nil
@valid_igmesh=false  #flag that determines if this mesh has been exported as an igmesh and the file is still valid
@has_default_material=nil

end

###
def uv_texture?

@defn.get_attribute(@@dict,'uv_texture',false)

end

def uv_texture=(value)

@defn.set_attribute(@@dict,'uv_texture',value)

end

def add_uvset(index)

index=index.to_s
@defn.set_attribute('uvsets',index,true)

end

def has_uvset(index)

return true if index==0

index=index.to_s
@defn.get_attribute('uvsets',index,false)

end


def exit_portal?
	@defn.get_attribute(@@dict,'exit_portal',@@exit_portal)
end

def exit_portal=(value)
	if value
		@defn.set_attribute(@@dict,'exit_portal',value) 
	else
		@defn.set_attribute(@@dict,'exit_portal',value)  
	end
end

####check if this mesh is an emitter
def has_default_material?

if @has_default_material==nil
	self.get_lems()
end
return @has_default_material

end

def emitter?

if @emitter==nil
	self.get_lems()
end
return @emitter

end
#returns all the light emitting materials in this mesh (as IndigoMaterial objects
def get_lems()


if @lems==nil
	@emitter=false
	@has_default_material=false
	@lems=[]
	mats=self.su_materials
	for mat in mats
		if mat!=nil
			if IndigoMaterial.emitter?(mat)
				@lems.push(IndigoMaterial.new(mat))
				@emitter=true
			end
		else
			@has_default_material=true
		end
	end
end

return @lems.dup

end
#######
#####subdivision settings
def view_dependent_subdivision?
	@defn.get_attribute(@@dict,'view_dep_sub',@@view_dependent)
end

def view_dependent_subdivision=(value)
	@defn.set_attribute(@@dict,'view_dep_sub',value)
end

def max_subdiv
	@defn.get_attribute(@@dict,'max_subdiv',@@max_subdiv)
end

def max_subdiv=(value)
	@defn.set_attribute(@@dict,'max_subdiv',value)
end

def pixel_threshold
	@defn.get_attribute(@@dict,'pixel_threshold',@@pixel_threshold)
end

def pixel_threshold=(value)
	@defn.set_attribute(@@dict,'pixel_threshold',value)
end

def curvature_threshold
 	@defn.get_attribute(@@dict,'curvature_threshold',@@curvature_threshold)
end

def curvature_threshold=(value)
	@defn.set_attribute(@@dict,'curvature_threshold',value)
end

def subdivision_smoothing?  
	@defn.get_attribute(@@dict,'sub_smooth',@@sub_smooth)
end

def subdivision_smoothing=(value)
	@defn.set_attribute(@@dict,'sub_smooth',value) 
end

def external_mesh_path
	@defn.get_attribute(@@dict,'external_mesh_path',@@external_mesh_path)
end

def external_mesh_path=(value)
	@defn.set_attribute(@@dict,'external_mesh_path',value)
end

def use_external_skp?
	use_external=self.use_external?
	if use_external
		if self.external_mesh_path[-4..1]==".skp"
			return true
		end
	end
	return false
end

def use_external_obj?
	use_external=self.use_external?
	if use_external
		#p self.external_mesh_path
		#p self.name
		if self.external_mesh_path[-4..-1].downcase==".obj"
			return true
		end
	end
	return false
end

def use_external? 
	if FileTest.exist?(self.external_mesh_path)
		@defn.get_attribute(@@dict,'use_external',@@use_external)
	else
		return false
	end
end

def use_external=(value)
	@defn.set_attribute(@@dict,'use_external',value) 
end

#######
def su_materials

return @su_materials.dup

end

def guid

return @defn.guid

end

def name

return @name.gsub(/[<>]/, '*')

end

def definition

return @defn

end

def definition=(value)

@defn=value

end

def uv_set

@defn.get_attribute(@@dict,'uv_set',@@uv_set)

end

def uv_set=(value)

@defn.set_attribute(@@dict,'uv_set',value)

end

def add_points(pt_array,scale=0.0254)


trans=Geom::Transformation.scaling(scale)

#pts=pt_array.collect {|p| p.transform(trans).to_a}
#pt_array=pts.collect {|p| p.to_a}

@vert_positions.concat(pt_array.collect {|p| p.transform!(trans).to_a})

end

def add_normals(nor_array)

#p nor_array
nor_array=nor_array.collect {|nor|
	nor=nor.to_a
	nor=nor.collect {|f| (f*10000).round.to_f/10000}
	nor
}
@vert_normals.concat(nor_array)
#p nor_array

end

def add_uvs(uv0,uv1,uv2,uv3)

#todo - add multiple uv sets
#uv0=uv0.collect {|uv| uv.to_a}
#p uv0
#TODO check if << is faster than concatenation
#p uv0,uv1,uv2,uv3
#@uv0.concat(uv0)
#uv1 ? @uv1.concat(uv1) : @uv1.concat(uv0)
#uv2 ? @uv2.concat(uv2) : @uv2.concat(uv0)
#uv3 ? @uv3.concat(uv3) : @uv3.concat(uv0)

uv0.each {|uv| normalize_uvs(uv)}
uv1.each {|uv| normalize_uvs(uv)}
uv2.each {|uv| normalize_uvs(uv)}
uv3.each {|uv| normalize_uvs(uv)}

has_uv1=!uv1.empty?
has_uv2=!uv2.empty?
has_uv3=!uv3.empty?

#now interleave the uvsets
uv0.each_index {|i|
	@uv0.push(uv0[i])
	@uv0.push(uv1[i]) if has_uv1
	@uv0.push(uv2[i]) if has_uv2
	@uv0.push(uv3[i]) if has_uv3
}

#uv1 ? @uv0.concat(uv1) : @uv0.concat(uv0)
#uv2 ? @uv0.concat(uv2) : @uv0.concat(uv0)
#uv3 ? @uv0.concat(uv3) : @uv0.concat(uv0)

end

def normalize_uvs(uvs)

uvz=uvs.z
uvs.x=uvs.x/uvz
uvs.y=uvs.y/uvz

end

def add_triangles(tris,mat)

mat_index=@su_materials.index(mat)
if mat_index==nil
	@su_materials.push(mat)
	mat_index=@su_materials.index(mat)
	if mat!=nil
		#@used_materials.push(IndigoMaterial.new(mat).name)
		@used_materials.push(mat.name)
	else
		@used_materials.push("Default")
	end
end

for tri in tris
	@triangles.push(IndigoTriangle.new(tri,mat_index))
end

end

####writing to file

def writeBasic(out,object)   #object is a binary string
	out.write(object)
end

def write_string(out,string)

	len=string.length
	len_bin=[len].pack("I")  #convert uint32
	#p len_bin.unpack("I")
	writeBasic(out,len_bin)  #write the length of the string
	str_arr=[string]
	#str_arr=[]
	#string.each {|c| str_arr.push(c)}  #build the string array (may not be required)
	string_bin=str_arr.pack("a#{len}")  #convert to null terminated string
	#string_bin=str_arr.pack("a#{len+1}")
	#p string_bin.unpack("a#{len}")
	writeBasic(out,string_bin)
end

def write_float(out,float)
	float_bin=[float].pack("f")  #convert to single precision float
	#p float_bin.unpack("f")
	writeBasic(out,float_bin)
end

def write_uint(out,uint)
	uint_bin=[uint].pack("I")  #convert to unsigned integer
	#p uint_bin.unpack("I")
	writeBasic(out,uint_bin)
end

def write_vec2f(out,vec2f)  #vec2f is an array of two floats
	if vec2f==nil
		vec2f=[0.0,0.0]
		p "nil uvs found"
	end
	vec2f_bin=vec2f.pack("ff")
	#p vec2f_bin.unpack("ff")
	writeBasic(out,vec2f_bin)
end

def write_vec3f(out,vec3f)  #vec3f is an array of three floats
	#vec3f=vec3f.collect {|f| (f*1000).round.to_f / 1000}
	vec3f=[vec3f.x,vec3f.y,vec3f.z]
	#vec3f=vec3f.to_a
	#p vec3f
	vec3f_bin=vec3f.pack("fff")
	#p vec3f_bin.unpack("fff")
	writeBasic(out,vec3f_bin)
end

def write_uvSetExposition(out,x)  #x is an array of a uv set name and an index number
	set_name=x[0]
	set_index=x[1]
	write_string(out,set_name)
	write_uint(out,set_index)
end

def write_triangle(out,tri)  #tri is an array of 3 vertex indices, 3 uv indices and a material index
	verts=tri[0]  #get the array of vertex indices
	uvs=tri[1]   #get the array of uv_indices
	mat_index=tri[2]
	verts_bin=verts.pack("III")
	#p verts_bin.unpack("III")
	uvs_bin=uvs.pack("III")
	#p uvs_bin.unpack("III")
	mat_index_bin=[mat_index].pack("I")
	#p mat_index_bin.unpack("I")
	writeBasic(out,verts_bin)
	writeBasic(out,uvs_bin)
	writeBasic(out,mat_index_bin)
end

def write_array(out,arr,type)  #intended to replace the C++ writeVector method
	len=arr.length    #get the number of items in the array
	#return if len==0
	len_bin=[len].pack("I")  #convert uint32
	#p len_bin.unpack("I")
	writeBasic(out,len_bin)  #write the length of the array
	case type
	when "tri"
		arr.each {|tri| tri.write(out)}
	when "vec3f"
		arr.each {|vec| write_vec3f(out,vec)}
	when "vec2f"
		arr.each {|vec| write_vec2f(out,vec)}
	when "uvse"
		arr.each {|uv| write_uvSetExposition(out,uv)}
	when "string"
		arr.each {|mat| write_string(out,mat)}
	end
end
	
def write_to_file(path)
	out=File.new((path),"wb")  #create a new file in binary write mode
	begin
	write_uint(out,MAGIC_NUMBER)
	#p "wrote magic number"
	write_uint(out,FORMAT_VERSION)
	#p "wrote format version"
	write_uint(out,@num_uv_mappings)
	#p "wrote num uv mappings"
	
	write_array(out,@used_materials, "string")
	#p @used_materials
	write_array(out,@uv_set_expositions, "uvse")
	#p "wrote uv set expositions"
	write_array(out,@vert_positions, "vec3f")
	#p "wrote vert positions"
	write_array(out,@vert_normals, "vec3f")
	
	#p "wrote normals"
	#write_array(out,@uv_pairs, "vec2f")
	write_array(out,@uv0, "vec2f")
	#p "wrote uvs"
	write_array(out,@triangles, "tri")
	#p @triangles
	#p "wrote triangles"
	ensure
	out.close()
	#p "closing file"
	end
	#out.close()
end

##file reading functions
def read_array(out,type)  #intended to replace the C++ writeVector method
	
	arr=[]
	len=read_uint(out)
	
	case type
	when "tri"
		#arr.each {|tri| tri.write(out)}
	when "vec3f"
		#arr.each {|vec| write_vec3f(out,vec)}
	when "vec2f"
		#arr.each {|vec| write_vec2f(out,vec)}
	when "uvse"
		#arr.each {|uv| write_uvSetExposition(out,uv)}
	when "string"
		len.times {|i| arr[i]=read_string(out)}
	end
	return arr
end
##
def read_string(out)
	str_len=read_uint(out)
	str=out.read(str_len)
	return str.unpack("a#{str_len}")[0]
	
end

def read_uint(out)
	num_bytes=4
	num=out.read(4)
	return num.unpack("I")[0]
end

###
def read_from_file(path)
	out=File.new((path),"rb")  #create a new file in binary read mode
	begin
	magic_number=read_uint(out)
	format_version=read_uint(out)
	@num_uv_mappings=read_uint(out)
	#p "wrote num uv mappings"
	
	@used_materials=read_array(out, "string")
	#p "wrote used materials"
	#read_array(out,@uv_set_expositions, "uvse")
	#p "wrote uv set expositions"
	#read_array(out,@vert_positions, "vec3f")
	#p "wrote vert positions"
	#read_array(out,@vert_normals, "vec3f")
	#p "wrote normals"
	#read_array(out,@uv_pairs, "vec2f")
	#p "wrote uvs"
	#read_array(out,@triangles, "tri")
	#p "wrote triangles"
	result=self.validate_and_load_su_materials()
	self.valid_igmesh=result
	
	ensure
	out.close()
	#p "closing file"
	end
	#out.close()
	
		
end


def validate_and_load_su_materials()

#check if the used material names are valid
@su_materials=[]
mats=Sketchup.active_model.materials
@used_materials.each {|m|
	@su_materials.push(mats[m]) unless m=="Default"
}
#p @used_materials
if @su_materials.index(nil)  #will occur if the named material does not exist
	#p "mesh has at least one invalid material"
	self.valid_igmesh=false
	@su_materials=[]
	@used_materials=[]
	return false
end

return true

end

def test()

path="C:\\Users\\Dale\\Desktop\\test3.igmesh"
self.read_from_file(path)
#p @used_materials
return @used_materials

end

end #class indigomesh

#####
class IndigoTriangle

def initialize(v_indices,mat_index)
	@v_indices=v_indices
	@uv_indices=v_indices
	@mat_index=mat_index
end

def write(out)
	verts_bin=@v_indices.pack("III")
	uvs_bin=@uv_indices.pack("III")
	mat_index_bin=[@mat_index].pack("I")
	out.write(verts_bin)
	out.write(uvs_bin)
	out.write(mat_index_bin)
end

end #end class triangle

####
class IndigoInvalidTabulatedException < RuntimeError
  
end

#-----------------------------------------------------------------------------
# This functions is just a shortcut for selecting the new tool
def select_region_tool
    Sketchup.active_model.select_tool SelectRegionTool.new
end


# ---------------------------------------- Menu item
if( not file_loaded?(__FILE__) )




	main_menu = UI.menu("Plugins").add_submenu("SkIndigo")
	main_menu.add_item("Render Scene") {(SkIndigo.export(0,true,nil,true,false))}
	main_menu.add_separator
	main_menu.add_item("Export Scene  as..") { (SkIndigo.export_as(0,true)) }
	main_menu.add_item("Export Sketchy Replay Animation") { (SkIndigo.export_animation(true)) }
	main_menu.add_item("Export Scene Tab Animation") { (SkIndigo.export_animation(false)) }
	main_menu.add_separator
	main_menu.add_item("Render Settings") { (SkIndigo.render_settings)}
	main_menu.add_item("Material Editor") { ($skindigo_mat_observer.show_editor if $skindigo_mat_observer) }
	main_menu.add_item("Query Material Types") { (SkIndigo.mat_query) }
	main_menu.add_separator
	main_menu.add_item("Set Light Layer Names") { SkIndigo.set_layer_names()}
	main_menu.add_item("Set Indigo Path") {SkIndigo.set_indigo_path}
	#main_menu.add_item("Resume a Render") { (Indigo.res) }
	main_menu.add_separator
	main_menu.add_item("About SkIndigo #{SkIndigo::VERSION}") { (SkIndigo.about) }
	
	Sketchup.add_observer(SkindigoAppObserver.new)

	$skindigo_mat_observer=SkindigoMatObserver.new
	Sketchup.active_model.materials.add_observer $skindigo_mat_observer
	
	$skindigo_object_editor=SkindigoObjectEditor.new
	Sketchup.active_model.selection.add_observer $skindigo_object_editor

	plugins = Sketchup.find_support_file("Plugins\/skindigo")
	icon1 = File.join(plugins,"IndigoLogo.gif")
	icon2 = File.join(plugins, "IndigoLogo3.gif")
	icon_3new = File.join(plugins, "ExportSelected.png") # Export selected entities only.
	icon3 = File.join(plugins, "IndigoLogo4.gif")
	icon4 = File.join(plugins, "IndigoLogo2.gif")
        tb = UI::Toolbar.new("SkIndigo")
		
	cmd1 = UI::Command.new("Render Scene"){(SkIndigo.export(0,true,nil,true, 
		false # Export selected entities only
	))} 
	cmd1.small_icon = icon1
	cmd1.large_icon = icon1
	cmd1.tooltip = cmd1.status_bar_text = "Render with Indigo"
	
	cmd2 = UI::Command.new("Quick Export"){(SkIndigo.quick_export)}
	cmd2.menu_text="   Quick Export"
	cmd2.set_validation_proc { (SkIndigo.validate_quick_export)}
	cmd2.small_icon = icon2
	cmd2.large_icon = icon2
	cmd2.tooltip = cmd2.status_bar_text = "Quick Export"
	
	cmd3_new = UI::Command.new("Render Selected Entities"){(SkIndigo.export(0,true,nil,true,
		true # Export selected entities only
	))}
	cmd3_new.menu_text="   Render Selected Entities"
	cmd3_new.set_validation_proc { SkIndigo.validate_export_selected_entities_only }
	cmd3_new.small_icon = icon_3new
	cmd3_new.large_icon = icon_3new
	cmd3_new.tooltip = cmd3_new.status_bar_text = "Render Selected Entities"
	
	cmd3 = UI::Command.new("Materal Editor"){($skindigo_mat_observer.show_editor if $skindigo_mat_observer)}
	cmd3.small_icon = icon3
	cmd3.large_icon = icon3
	cmd3.tooltip = cmd3.status_bar_text = "Material Editor"
	
	cmd4 = UI::Command.new("Render Settings"){(SkIndigo.render_settings)}
	cmd4.menu_text="Render Settings"
	cmd4.small_icon = icon4
	cmd4.large_icon = icon4
	cmd4.tooltip = cmd4.status_bar_text = "Render Settings"
	
	cmd5 = UI::Command.new("Object Editor"){($skindigo_object_editor.show if $skindigo_object_editor)}
	cmd5.menu_text="Object Editor"
	cmd5.small_icon=icon1
	cmd5.large_icon=icon1
	
	tb.add_item(cmd1)
	tb.add_item(cmd2)
	tb.add_item(cmd3_new)
	tb.add_item(cmd3)
	tb.add_item(cmd4)
	#tb.add_item(cmd5)
	tb.show
	
	
##############################UV Manager Context menu##########################
	
	UI.add_context_menu_handler do |menu|
	ent=Sketchup.active_model.selection.first
	mat=nil
	if ent
		if ent.material
			mat=IndigoMaterial.new(ent.material)
		elsif (ent.class==Sketchup::Face) and ent.back_material
			mat=IndigoMaterial.new(ent.back_material)
		end
	end
	menu.add_separator
	menu.add_item("SkIndigo"){(nil)}
        if mat
			unless mat.color_picker? or mat.uv_helper?
				menu.add_item("   Edit \"#{mat.name}\" (#{mat.type.upcase})") {($skindigo_mat_observer.sm=mat;$skindigo_mat_observer.show_editor)}
			end
		 end
   	end
	
=begin	
UI.add_context_menu_handler do |menu|
	sel=Sketchup.active_model.selection
	ent=sel.first
	mat=IndigoMaterial.new(ent.material) if (ent) and (ent.material!=nil)
	if mat and mat.texture!=nil
		#uv_menu=menu.add_submenu("   UV Mapping") 
		#TODO - fix so that user can triple click on faces
		#if mat.uv_helper?
		#	current_set=mat.mat.get_attribute('indigo_uv','current_set')  #get the uv set that is currently being edited
		#		if current_set!=nil
		#			uv_menu.add_item("Save UV set #{current_set} and Restore Material") {
		#			faces=Sketchup.active_model.active_entities.find_all {|e| e.class==Sketchup::Face} #find all faces in current context
		#			uv_faces=faces.find_all {|f| f.material==mat.mat}  #find all faces in current context that are using the UV helper material
		#			mat.store_uvs(uv_faces,current_set)                 #save the current uvs to the current set
		#			UI.messagebox("Saved current UVs to set #{current_set}")
		#			status=mat.restore_old_mat(uv_faces)               #revert back to the original material
		#			if status==false
		#				UI.messagebox("Error restoring material.")
		#			end
		#		}
		#		end
		#end
		
		#uv_menu.add_item("Save UVs") {mat.store_uvs(sel,$skindigo_current_uv_layer)} 
		#uv_menu.add_item("Restore Material") {mat.restore_material()}
		#uv_menu.add_item("Save UVs to set 1") {mat.store_uvs(sel,1)} 
		#uv_menu.add_item("Save UVs to set 2") {mat.store_uvs(sel,2)} 
		#uv_menu.add_item("Save UVs to set 3") {mat.store_uvs(sel,3)} 
		#uv_menu.add_item("Purge all UV sets") {mat.purge_uvs(sel)}
		#uv_menu.add_separator
		#uv_menu.add_item("Explode distorted texture") {(UVS.explode_distorted_texture)} if (mat.texture)
		
	end
		
end
=end
	
	UI.add_context_menu_handler do |menu|
		sel=Sketchup.active_model.selection
		ent=sel.first
		if ent.class==Sketchup::SectionPlane
			unless (ent.get_attribute('indigo07_settings','active_section',false))
				menu.add_item("   Enable Section Plane") {ent.set_attribute('indigo07_settings','active_section',true)}
			else
				menu.add_item("   Disable Section Plane") {ent.set_attribute('indigo07_settings','active_section',false)}
			end
		end
		
	end #menu
	
	UI.add_context_menu_handler do |menu|
	
	menu.add_item("   Edit Active Mesh") {($skindigo_object_editor.show(Sketchup.active_model.active_entities.parent) if $skindigo_object_editor)}
	
	sel=Sketchup.active_model.selection
	ent=sel.first
	e=nil
	if ent.class==Sketchup::ComponentInstance
		e=ent.definition
	elsif ent.class==Sketchup::Group
		e=ent.entities[0].parent
	end
	
	if e and sel.length==1
		#instanced=e.get_attribute 'indigo07_settings','inst',false
		#if not instanced
			#menu.add_item("   \"#{e.name}\" Enable Instancing") {(e.set_attribute 'indigo07_settings','inst',true)}
		#else
			#menu.add_item("   \"#{e.name}\" Disable Instancing") {(e.set_attribute 'indigo07_settings','inst',false)}
			#menu.add_item("   Edit Object") {($skindigo_object_editor.show(e) if $skindigo_object_editor)}
			menu.add_item("   Set Keyframe") {SkIndigo.set_keyframe([ent])}
			menu.add_item("   Clear Keyframes") {SkIndigo.clear_keyframes([ent])} if ent.has_indigo_keyframes?
		#end
	elsif e and sel.length>1
		#menu.add_item("   Enable Instancing") {(SkIndigo.set_instancing(sel,true))}
		#menu.add_item("   Disable Instancing") {(SkIndigo.set_instancing(sel,false))}
		menu.add_item("   Set Keyframes") {SkIndigo.set_keyframe(sel)}
		menu.add_item("   Clear Keyframes") {SkIndigo.clear_keyframes(sel)}
	end	
	end
	

SkIndigo.check_indigo_version


end 

#-----------------------------------------------------------
file_loaded(__FILE__)

